/************************************************************************
*
*                             Xg3d.h
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.2   5/92
*
*                Main header file for Xg3d plotting package
*
************************************************************************
*/
#define Xg3dPlot
#define Xg3dClass          "XgPlot"
#define Xg3dNSETS          20   /* total allowed number of data sets to plot */
#define Xg3dNWID           14

#define Xg3dSci_label      101
#define Xg3dNo_label       102
#define Xg3dPoint          101
#define Xg3dCircle         102
#define Xg3dSquare         103
#define Xg3dBar            104
#define Xg3dTriangle       105
#define Xg3dLine           201
#define Xg3dNoLine         202
#define Xg3dYLine          203
#define Xg3dMesh           204
#define Xg3dSig0           300
#define Xg3dSig1           301
#define Xg3dSig2           302
#define Xg3dSig3           303
#define Xg3dLast0          400
#define Xg3dLast1          401
#define Xg3dLast2          402
#define Xg3dLast5          405
#define Xg3dSize0          500
#define Xg3dSize3          503
#define Xg3dSize4          504
#define Xg3dSize5          505
#define Xg3dLog            601
#define Xg3dLinear         602

/***** bit masks *****/
#define Xg3dUmin           1
#define Xg3dUmax           2
#define Xg3dUinc           4
#define Xg3dUnsig          8
#define Xg3dUscale         16

/****** defaults ******/
#define Xg3dDefPointType        Xg3dBar
#define Xg3dDefLineType         Xg3dLine
#define Xg3dDefPtSize           Xg3dSize3
#define Xg3dDefLabelType        Xg3dNo_label
#define Xg3dDefMinXSize         100
#define Xg3dDefMinYSize         100
#define Xg3dDefXtextXCoord      50
#define Xg3dDefXtextYCoord      2
#define Xg3dDefYtextXCoord      2
#define Xg3dDefYtextYCoord      50
#define Xg3dDefTitletextXCoord  50
#define Xg3dDefTitletextYCoord  2
#define Xg3dDefSubtextXCoord    50
#define Xg3dDefSubtextYCoord    7
#define Xg3dDefSigFig           Xg3dSig1
#define Xg3dDefInc              Xg3dLast2
#define Xg3dDefFont             "6x13"
#define Xg3dDefXscaleXCoord     90
#define Xg3dDefXscaleYCoord     0
#define Xg3dDefYscaleXCoord     90
#define Xg3dDefYscaleYCoord     5
#define Xg3dDefFontDir          "/usr/lib/X11/fonts/misc/"
#define Xg3dDefTheta            30;
#define Xg3dDefScaleFac         0.1;
#define Xg3dDefHoff             1;
#define Xg3dDefFunc             Xg3dLinear;

/*******************************/
typedef struct Xg3dcoord
  {
    double
      x,                 /* normalized x coordinate (0 - 1) */
      y;                 /* normalized y coordinate (0 - 1) */
  } Xg3dCoord;

typedef struct Xg3daxis    /* Layout Prescription for each axis */
  {
    double
      min,               /* minimum value along this axis */
      max;               /* maximum value along this axis */
    int
      win_min,           /* transformed minimum value */
      win_max;           /* transformed maximum value */
    Xg3dCoord
      scale_loc;         /* location of axis scaling info */
    int
      increment,         /* increment of least sig. figure between labels */
      nsig_fig;          /* number of significant figures in labels */
    char
      user_set_flag;     /* which values are defaults or user set */
  } Xg3dAxis;

typedef struct Xg3ddatainfo /* One structure per data set to display */
  {
    short int
      *darray;           /* pointer to start of 1-Dx2 data set */
    int
      frame_num,         /* frame number (for multi-frame data */
      num_points,        /* number of data points in this data set */
      nx,                /* number of pixels per row (# of columns) */
      ny,                /* number of rows */
      point_type,        /* type of point to display for this data set */
      point_size,        /* size in pixels for Circle or Square */
      line_status,       /* Connect data points with line, or no line */
      s_func;            /* scaling function */
    char
      filename[80];      /* file name for data set */
    long int
      datacolor;         /* Color for data set */
  } Xg3dDataInfo;

typedef struct Xg3dfontinfo  /* Font name and XFontStruct for labels, etc. */
  {
    char
      name[80];
    XFontStruct
      *font;
  } Xg3dFontInfo;

typedef struct Xg3dwidstruct
  {
    caddr_t
      client,
      graph;
  } Xg3dWidStruct;

typedef struct Xg3dgraphinfo /* Main Data structure for Xg3d graphs */
  {
    char
      filename[80],      /* filename for this graph */
      label_type,        /* Scaling rules: none, scientific, ... */
      x_axis_text[80],   /* x-axis description text */
      y_axis_text[80],   /* y-axis description text */
      title_text[80],    /* Main title text */
      subtitle_text[80]; /* subtitle text */
    Xg3dFontInfo
      x_axis_font,      /* font for x-axis description */
      y_axis_font,      /* font for y-axis description */
      title_font,       /* font for title */
      subtitle_font,    /* font for subtitle */
      scale_font;       /* font for axis numbering and scale info */
    int
      num_sets;          /* number of data sets */
    double
      theta,             /* skew angle of y axis from vertical z axis (CW) */
      hoff,              /* vertical line spacing (indpendent of skew) */
      scale_fac;         /* z axis scaling factor */
    Xg3dAxis
      xaxis_layout,      /* layout prescription for x-axis */
      yaxis_layout;      /* layout prescription for y-axis */
    Xg3dDataInfo
      dataset[Xg3dNSETS];  /* Data & info for individual data sets */
    Xg3dCoord
      x_axis_coord,      /* position of x-axis text (normalized coord) */
      y_axis_coord,      /* position of y-axis text (normalized coord) */
      title_coord,       /* position of title text (normalized coord) */
      subtitle_coord;    /* position of subtitle text (normalized coord) */
    long int
      text_color,        /* x,y-axis text, title, subtitle color */
      axis_color,        /* x,y-axes, ticks, numbers, & scaling color */
      background_color,  /* overall graph background color */
      cursor_color;      /* cursor color */
    /************ realization variables *************/
    Widget
      app_shell,
      main,
      form,
      menu_bar,
      draw_area;
    int
      width,
      height,
      redraw,
      nxdraw,
      nydraw,
      cursor_status,
      cursWidth,
      cursHeight,
      cursDset,
      cursPos;
    double
      sw_xll,
      sw_xur,
      sw_yll,
      sw_yur,
      temp_theta,
      temp_hoff,
      temp_scale;
    int
      misc;
    Pixmap
      savePixmap,
      cursPixmap;
    GC
      theGC;
    Xg3dCoord
      temp_coord;
    Xg3dAxis
      temp_axis;
    Xg3dWidStruct
      *wid_array[Xg3dNWID];

  } Xg3dGraphInfo;

/********* external definitions ************/
int Xg3dCreateGraph
  (
   Xg3dGraphInfo * graphinfo
  );
void Xg3dInitializeStruct
  (                              /* initialize Xg3dGraphInfo data structure
				    to default values */
   Xg3dGraphInfo * graphinfo
  );

/********************** 3-D scaling factors ********************************
*
*  dx = hoff * tan(theta)
*  given a z value: value(x, y) in normal 2-D coordinates, this maps to:
*
*    ==> (x + dx*y, y*hoff + value(x, y)*scale_fac)
*
*  The displayed window takes min(x) and min(y) and transforms these to
*   obtain win_min(x) and win_min(y). The same is done for the maximum
*   window values.
*
*****************************************************************************/
