/**********************************************************************
*
*                          Xg3dcon.c
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.2   5/92
*
*  Usage: Declare a variable;
*
*    Xg3dGraphInfo
*      graph;
*
*  as defined in Xg3d.h. This structure may be initialized by a call to
*  Xg3dInitializeStruct which will supply default values. You must then set
*  the following: num_sets must be greater than zero, with the
*  corresponding datasets[i] structures initialized with *darray, and
*  num_points. A shell window must be supplied in app_shell to contain the
*  graph and menu bar. You may override any default values. Now call
*  XgCreateGraph(&graph) and the graph will be created using the default
*  values supplemented by yours. The graph is an independent widget.
*  Values may be altered interactively for each graph using the menu, or
*  they may be adjusted by the calling program. Note that this means that
*  once created, the graph can be updated dynamically.
*
***********************************************************************
*/
#ifdef sgi
#define USE_DIRENT
#endif
#ifdef __hpux
#define USE_DIRENT
#endif

#include "stdio.h"
#include "string.h"
#include "ctype.h"
/* either use the next line and then skip 2, or under strict system V,
   skip the next line and use the following 2 */
#ifndef USE_DIRENT
#include "sys/dir.h"
#else
#include "sys/types.h"
#include "dirent.h"
#endif
#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "Xm/BulletinB.h"
#include "Xm/RowColumn.h"
#include "Xm/PushB.h"
#include "Xm/PushBG.h"
#include "Xm/Form.h"
#include "Xm/Frame.h"
#include "Xm/ToggleBG.h"
#include "Xm/ToggleB.h"
#include "Xm/CascadeB.h"
#include "Xm/FileSB.h"
#include "Xm/MessageB.h"
#include "Xm/PanedW.h"
#include "Xm/Label.h"
#include "Xm/Text.h"
#include "Xm/MainW.h"
#include "Xm/SelectioB.h"
#include "Xm/ScrolledW.h"
#include "XgPlot.bmp"
#include "Xg3d.h"
#include "etsheadr.h"

/*************** help sources ****************/
#define Xg3dMAIN_MENU     101
#define Xg3dDATA_SELECT   102
#define Xg3dEDIT_LABEL    103
#define Xg3dADD_FILE      104
#define Xg3dFONTS_HELP    105
#define Xg3dDATA_OPTIONS  106
#define Xg3dLOAD_SAVE     107
#define Xg3dLAYOUT        108
#define Xg3dCOORDS        109
#define Xg3dCOLOR_HELP    110
#define Xg3dSKEW          111

/************* error sources ****************/
#define Xg3dNO_FILE           101
#define Xg3dDATA_CALLOC_FAIL  102
#define Xg3dTOO_MANY_FILES    103
#define Xg3dNO_DATA_SETS      104
#define Xg3dNO_DELETE_LAST    105
#define Xg3dNOT_IMPLEMENTED   106
#define Xg3dNO_FONT           107
#define Xg3dNO_SHELL          108
#define Xg3dBAD_COORDS        109
#define Xg3dBAD_MINMAX        110

/****************** general defines of current selections ***********/
#define Xg3dXaxisLabel  1
#define Xg3dYaxisLabel  2
#define Xg3dTitle       4
#define Xg3dSubtitle    8
#define Xg3dText        16
#define Xg3dFont        32
#define Xg3dColor       64
#define Xg3dCoords      128
#define Xg3dScale       256
#define Xg3dXScale      16
#define Xg3dYScale      32
#define Xg3dBkgnd       64
#define Xg3dCursor      128
#define Xg3dDSelDelete  1
#define Xg3dDSelColor   2
#define Xg3dDSelPoint   3
#define Xg3dLoad        1
#define Xg3dSave        2
#define Xg3dXaxis       1
#define Xg3dYaxis       2
#define Xg3dMin         0
#define Xg3dMax         1
#define Xg3dTheta       1
#define Xg3dHoff        2
#define Xg3dScale_fac   3

/************** function declarations *********/
void Xg3d_help_sys();
void Xg3d_error_sys(int, Xg3dGraphInfo *);
void Xg3d_cancel_CB();
void Xg3d_free_wid();
Xg3dWidStruct * Xg3dAllocStruct(int, caddr_t);

void Xg3d_load_save_graph();
void Xg3d_save_graph();
void Xg3d_load_graph();
void Xg3d_create_exit_warning();
void Xg3d_exit_graph();
void Xg3d_exit_program();

void Xg3d_do_edit_titles();
void Xg3d_get_new_text();
void Xg3d_do_font_titles();
void Xg3d_update_fonts();
void Xg3d_default_font();
void Xg3d_do_colors();
void Xg3d_update_colors();
void Xg3d_default_colors();
void Xg3d_get_coords();
void Xg3d_temp_coord();
void Xg3d_set_coord();
void Xg3d_default_coord();

void Xg3d_configure_axes();
void Xg3d_set_axes();
void Xg3d_default_axes();
void Xg3d_minmax();
void Xg3d_skew_factor();
void Xg3d_temp_skew();
void Xg3d_set_skew();
void Xg3d_default_skew();

void Xg3d_start_add_file();
void Xg3d_add_file();
void Xg3d_dataset_action();
void Xg3d_select_data_set();
void Xg3d_data_toggle();
void Xg3d_set_data_options();
void Xg3d_set_data_defaults();

void Xg3d_CursorToggle();
void Xg3dPrint();
void Xg3dDummy();
void Xg3d_redrawExpose();

  XmStringCharSet Xg3dCharset = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  char
    Xg3dFontdir[80];

/******************************************************************
*
*                        MAIN ENTRY POINT
*
********************************************************************/
int Xg3dCreateGraph
  (
   Xg3dGraphInfo * graphinfo
  )
{
  Arg
    args[10];
  int
    nargs;
  Pixmap
    icon_pixmap;
/****************** Set up Main Application Shell *************/
if (Xg3d_Initialize(graphinfo) < 0)
  return(0);
icon_pixmap = XCreateBitmapFromData(XtDisplay(graphinfo->app_shell),
	      RootWindowOfScreen(XtScreen(graphinfo->app_shell)),
	      bitmap_bits, bitmap_width, bitmap_height);
nargs = 0;
XtSetArg(args[nargs], XtNiconPixmap, icon_pixmap); ++nargs;
XtSetValues(graphinfo->app_shell, args, nargs);
Xg3d_createMenus(graphinfo);
/*Xg3d_redrawExpose(graphinfo->app_shell, graphinfo, graphinfo);*/
Xg3d_createGraph(graphinfo);
}

/*****************************************************************
*
*                     Initialization Stuff
*
******************************************************************/
Xg3d_Initialize(graphinfo)
     Xg3dGraphInfo
       *graphinfo;
{
  int
    i;
  long int
    fore_color,
    back_color;
if (graphinfo->num_sets == 0)
  {
    Xg3d_error_sys(Xg3dNO_DATA_SETS, graphinfo);
    return(-1);
  }
if (graphinfo->app_shell == NULL)
  {
    Xg3d_error_sys(Xg3dNO_SHELL, graphinfo);
    return(-1);
  }

/*************** fonts **************************/
strcpy(Xg3dFontdir, Xg3dDefFontDir);
if (graphinfo->x_axis_font.font == NULL)
  {
    graphinfo->x_axis_font.font
      = XLoadQueryFont(XtDisplay(graphinfo->app_shell), Xg3dDefFont);
    strcpy(graphinfo->x_axis_font.name, Xg3dDefFont);
  }
if (graphinfo->y_axis_font.font == NULL)
  {
    graphinfo->y_axis_font.font
      = XLoadQueryFont(XtDisplay(graphinfo->app_shell), Xg3dDefFont);
    strcpy(graphinfo->y_axis_font.name, Xg3dDefFont);
  }
if (graphinfo->title_font.font == NULL)
  {
    graphinfo->title_font.font
      = XLoadQueryFont(XtDisplay(graphinfo->app_shell), Xg3dDefFont);
    strcpy(graphinfo->title_font.name, Xg3dDefFont);
  }
if (graphinfo->subtitle_font.font == NULL)
  {
    graphinfo->subtitle_font.font
      = XLoadQueryFont(XtDisplay(graphinfo->app_shell), Xg3dDefFont);
    strcpy(graphinfo->subtitle_font.name, Xg3dDefFont);
  }
if (graphinfo->scale_font.font == NULL)
  {
    graphinfo->scale_font.font
      = XLoadQueryFont(XtDisplay(graphinfo->app_shell), Xg3dDefFont);
    strcpy(graphinfo->scale_font.name, Xg3dDefFont);
  }

graphinfo->cursor_status = 0;
graphinfo->redraw = 0;
graphinfo->draw_area = NULL;

graphinfo->width = 300;
graphinfo->height = 200;

fore_color = XBlackPixelOfScreen(XtScreen(graphinfo->app_shell));
back_color = XWhitePixelOfScreen(XtScreen(graphinfo->app_shell));

if (graphinfo->text_color == -1)
  graphinfo->text_color = fore_color;
if (graphinfo->axis_color == -1)
  graphinfo->axis_color = fore_color;
if (graphinfo->background_color == -1)
  graphinfo->background_color = back_color;
if (graphinfo->cursor_color == -1)
  graphinfo->cursor_color = fore_color;
for (i=0; i<Xg3dNSETS; ++i)
  if (graphinfo->dataset[i].datacolor == -1)
    graphinfo->dataset[i].datacolor = fore_color;

if (!(graphinfo->xaxis_layout.user_set_flag & Xg3dUmin))
  graphinfo->xaxis_layout.min = 0;
if (!(graphinfo->xaxis_layout.user_set_flag & Xg3dUmax))
  graphinfo->xaxis_layout.max = graphinfo->dataset[0].nx;
if (!(graphinfo->yaxis_layout.user_set_flag & Xg3dUmin))
  graphinfo->yaxis_layout.min = 0;
if (!(graphinfo->yaxis_layout.user_set_flag & Xg3dUmax))
  graphinfo->yaxis_layout.max = graphinfo->dataset[0].ny;
return(1);

}
/*************************************/
void Xg3dInitializeStruct
  (                              /* initialize Xg3dGraphInfo data structure
				    to default values */
   Xg3dGraphInfo * graphinfo
  )
{
  int
    i;
/***** need to set colors *****/
graphinfo->num_sets = 0;

/******* individual data sets *****/
for (i=0; i<Xg3dNSETS; ++i)
  {
    graphinfo->dataset[i].num_points = 0;
    graphinfo->dataset[i].point_type = Xg3dDefPointType;
    graphinfo->dataset[i].line_status = Xg3dDefLineType;
    graphinfo->dataset[i].point_size = Xg3dDefPtSize;
    graphinfo->dataset[i].datacolor = -1;
    graphinfo->dataset[i].s_func = Xg3dDefFunc;
  }

/******* axes & text layouts ******/
graphinfo->label_type = Xg3dSci_label;
graphinfo->x_axis_coord.x = Xg3dDefXtextXCoord;
graphinfo->x_axis_coord.y = Xg3dDefXtextYCoord;
graphinfo->y_axis_coord.x = Xg3dDefYtextXCoord;
graphinfo->y_axis_coord.y = Xg3dDefYtextYCoord;
graphinfo->title_coord.x = Xg3dDefTitletextXCoord;
graphinfo->title_coord.y = Xg3dDefTitletextYCoord;
graphinfo->subtitle_coord.x = Xg3dDefSubtextXCoord;
graphinfo->subtitle_coord.y = Xg3dDefSubtextYCoord;
graphinfo->xaxis_layout.nsig_fig = Xg3dDefSigFig;
graphinfo->xaxis_layout.increment = Xg3dDefInc;
graphinfo->xaxis_layout.min = -1.;
graphinfo->xaxis_layout.max = 1.;
graphinfo->xaxis_layout.scale_loc.x = Xg3dDefXscaleXCoord;
graphinfo->xaxis_layout.scale_loc.y = Xg3dDefXscaleYCoord;
graphinfo->xaxis_layout.user_set_flag = 0;
graphinfo->yaxis_layout.nsig_fig = Xg3dDefSigFig;
graphinfo->yaxis_layout.increment = Xg3dDefInc;
graphinfo->yaxis_layout.min = -1.;
graphinfo->yaxis_layout.max = 1.;
graphinfo->yaxis_layout.scale_loc.x = Xg3dDefYscaleXCoord;
graphinfo->yaxis_layout.scale_loc.y = Xg3dDefYscaleYCoord;
graphinfo->yaxis_layout.user_set_flag = 0;
graphinfo->theta = Xg3dDefTheta;
graphinfo->scale_fac = Xg3dDefScaleFac;
graphinfo->hoff = Xg3dDefHoff;

graphinfo->text_color = -1;
graphinfo->axis_color = -1;
graphinfo->background_color = -1;
graphinfo->cursor_color = -1;
}
/******************* Create Menu Bar ***********************/
Xg3d_createMenus(graph)
     Xg3dGraphInfo
       *graph;
{
  int
    nargs;
  Arg
    args[10];
  XmString
    label;
  Xg3dWidStruct
    *wid_addr;
  Widget
    main,
    pulldown,
    next_level,
    select_text,
    button;
nargs = 0;
XtSetArg(args[nargs], XmNscrollingPolicy, XmAPPLICATION_DEFINED); ++nargs;
graph->main = XmCreateMainWindow(graph->app_shell,"main",(ArgList) args,nargs);
XtManageChild(graph->main);

nargs = 0;
XtSetArg(args[nargs], XmNwidth, graph->width); ++nargs;
XtSetArg(args[nargs], XmNheight, graph->height); ++nargs;
XtSetArg(args[nargs], XmNresizeWidth, False); ++nargs;
XtSetArg(args[nargs], XmNresizeHeight, False); ++nargs;
graph->form = XmCreateForm(graph->main, "draw form", args, nargs);
XtManageChild(graph->form);

nargs = 0;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
graph->menu_bar = XmCreateMenuBar(graph->form, "menu_bar", args, nargs);
XtManageChild(graph->menu_bar);

/************* Create Menu Bar Buttons *************/
/***** help *****/
nargs = 0;    /* Menu Bar Button */
label = XmStringCreateLtoR("Help", Xg3dCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetArg(args[nargs], XmNmnemonic, 'H'); ++nargs;
button = XmCreateCascadeButton(graph->menu_bar, "Help", args, nargs);
XtAddCallback(button, XmNactivateCallback, Xg3d_help_sys,
	      (caddr_t) Xg3dMAIN_MENU);
XtManageChild(button);
XtFree(label);
XtSetArg(args[0], XmNmenuHelpWidget, button);
XtSetValues(graph->menu_bar, args, 1);

/************* pulldowns ****************/
/***** File *****/
pulldown = XmCreatePulldownMenu(graph->menu_bar, "File_pane", NULL, 0);

nargs = 0;   /* Menu Bar Button */
label = XmStringCreateLtoR("File", Xg3dCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetArg(args[nargs], XmNsubMenuId, pulldown); ++nargs;
XtSetArg(args[nargs], XmNmnemonic, 'F'); ++nargs;
button = XmCreateCascadeButton(graph->menu_bar, "File", args, nargs);
XtManageChild(button);
XtFree(label);

  nargs = 0;   /* 1st level menu button */
  label = XmStringCreateLtoR("Load Graph", Xg3dCharset);
  XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'L'); ++nargs;
  button = XmCreatePushButton(pulldown, "Load Graph", args, nargs);
  wid_addr = Xg3dAllocStruct(Xg3dLoad, (caddr_t) graph);
  XtAddCallback(button, XmNactivateCallback, Xg3d_load_save_graph, wid_addr);
  XtManageChild(button);
  XtFree(label);

  nargs = 0;   /* 1st level menu button */
  label = XmStringCreateLtoR("Save Graph", Xg3dCharset);
  XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'S'); ++nargs;
  button = XmCreatePushButton(pulldown, "Save Graph", args, nargs);
  wid_addr = Xg3dAllocStruct(Xg3dSave, (caddr_t) graph);
  XtAddCallback(button, XmNactivateCallback, Xg3d_load_save_graph, wid_addr);
  XtManageChild(button);
  XtFree(label);

  nargs = 0;   /* 1st level menu button */
  label = XmStringCreateLtoR("Exit", Xg3dCharset);
  XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'E'); ++nargs;
  button = XmCreatePushButton(pulldown, "Exit", args, nargs);
  XtAddCallback(button, XmNactivateCallback, Xg3d_create_exit_warning, graph);
  XtManageChild(button);
  XtFree(label);

/****** Edit ******/
pulldown = XmCreatePulldownMenu(graph->menu_bar, "Edit_pane", NULL, 0);

nargs = 0;    /* Menu Bar Button */
label = XmStringCreateLtoR("Edit", Xg3dCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetArg(args[nargs], XmNsubMenuId, pulldown); ++nargs;
XtSetArg(args[nargs], XmNmnemonic, 'E'); ++nargs;
button = XmCreateCascadeButton(graph->menu_bar, "Edit", args, nargs);
XtManageChild(button);
XtFree(label);

/*** Titles submenu ***/
next_level = XmCreatePulldownMenu(pulldown, "Text_pane", NULL, 0);

  nargs = 0;   /* 1st level menu button */
  XtSetArg(args[nargs], XmNsubMenuId, next_level); ++nargs;
  label = XmStringCreateLtoR("Titles", Xg3dCharset);
  XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'T'); ++nargs;
  button = XmCreateCascadeButton(pulldown, "Titles", args, nargs);
  XtManageChild(button);
  XtFree(label);

  /* select_text sub-sub menu */    /** Text **/
  select_text = XmCreatePulldownMenu(next_level, "select_text_pane", NULL, 0);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNsubMenuId, select_text); ++nargs;
    XtSetArg(args[nargs], XmNmnemonic, 'T'); ++nargs;
    button = XmCreateCascadeButton(next_level, "Text", args, nargs);
    XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'X'); ++nargs;
     button = XmCreatePushButton(select_text,"X-axis Description", args,nargs);
     wid_addr = Xg3dAllocStruct(Xg3dXaxisLabel, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_edit_titles, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'Y'); ++nargs;
     button = XmCreatePushButton(select_text, "Y-axis Description",args,nargs);
     wid_addr = Xg3dAllocStruct(Xg3dYaxisLabel, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_edit_titles, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'T'); ++nargs;
     button = XmCreatePushButton(select_text, "Title", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dTitle, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_edit_titles, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'S'); ++nargs;
     button = XmCreatePushButton(select_text, "Sub-title", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dSubtitle, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_edit_titles, wid_addr);
     XtManageChild(button);

    /* select_text sub-sub menu */    /** Font **/
    select_text = XmCreatePulldownMenu(next_level, "select_font_pane",NULL, 0);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNsubMenuId, select_text); ++nargs;
    XtSetArg(args[nargs], XmNmnemonic, 'F'); ++nargs;
    button = XmCreateCascadeButton(next_level, "Font", args, nargs);
    XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'X'); ++nargs;
     button = XmCreatePushButton(select_text, "X-axis Description",args,nargs);
     wid_addr = Xg3dAllocStruct(Xg3dXaxisLabel, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_font_titles, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'Y'); ++nargs;
     button = XmCreatePushButton(select_text, "Y-axis Description",args,nargs);
     wid_addr = Xg3dAllocStruct(Xg3dYaxisLabel, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_font_titles, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'T'); ++nargs;
     button = XmCreatePushButton(select_text, "Title", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dTitle, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_font_titles, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'S'); ++nargs;
     button = XmCreatePushButton(select_text, "Sub-title", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dSubtitle, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_font_titles, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'c'); ++nargs;
     button = XmCreatePushButton(select_text, "Scales", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dScale, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_font_titles, wid_addr);
     XtManageChild(button);
/*     XtManageChild(select_text);   /* kludge */

  /* select_text sub-sub menu */   /** Location **/
  select_text = XmCreatePulldownMenu(next_level, "select_locat_pane", NULL, 0);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNsubMenuId, select_text); ++nargs;
    XtSetArg(args[nargs], XmNmnemonic, 'L'); ++nargs;
    button = XmCreateCascadeButton(next_level, "Location", args, nargs);
    XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'X'); ++nargs;
     button = XmCreatePushButton(select_text, "X-axis Description",args,nargs);
     wid_addr = Xg3dAllocStruct(Xg3dXaxisLabel, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_get_coords, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'Y'); ++nargs;
     button = XmCreatePushButton(select_text, "Y-axis Description",args,nargs);
     wid_addr = Xg3dAllocStruct(Xg3dYaxisLabel, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_get_coords, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'T'); ++nargs;
     button = XmCreatePushButton(select_text, "Title", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dTitle, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_get_coords, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'S'); ++nargs;
     button = XmCreatePushButton(select_text, "Sub-title", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dSubtitle, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_get_coords, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'c'); ++nargs;
     button = XmCreatePushButton(select_text, "X Scaling", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dXScale, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_get_coords, wid_addr);
     XtManageChild(button);
     XtManageChild(select_text);   /* kludge */

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'a'); ++nargs;
     button = XmCreatePushButton(select_text, "Y Scaling", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dYScale, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_get_coords, wid_addr);
     XtManageChild(button);
/*     XtManageChild(select_text);   /* kludge */

/*** Layout Submenu ***/
  next_level = XmCreatePulldownMenu(pulldown, "Layout_pane", NULL, 0);

  nargs = 0;   /* 1st level menu button */
  XtSetArg(args[nargs], XmNsubMenuId, next_level); ++nargs;
  label = XmStringCreateLtoR("Layout", Xg3dCharset);
  XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'L'); ++nargs;
  button = XmCreateCascadeButton(pulldown, "Layout", args, nargs);
  XtManageChild(button);
  XtFree(label);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'X'); ++nargs;
    button = XmCreatePushButton(next_level, "X-axis Limits", args, nargs);
    wid_addr = Xg3dAllocStruct(Xg3dXaxis, (caddr_t) graph);
    XtAddCallback(button, XmNactivateCallback, Xg3d_configure_axes, wid_addr);
    XtManageChild(button);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'Y'); ++nargs;
    button = XmCreatePushButton(next_level, "Y-axis Limits", args, nargs);
    wid_addr = Xg3dAllocStruct(Xg3dYaxis, (caddr_t) graph);
    XtAddCallback(button, XmNactivateCallback, Xg3d_configure_axes, wid_addr);
    XtManageChild(button);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'S'); ++nargs;
    button = XmCreatePushButton(next_level, "Skew Factors", args, nargs);
    XtAddCallback(button, XmNactivateCallback, Xg3d_skew_factor, graph);
    XtManageChild(button);

  /* select_text sub-sub menu */   /** Color **/
  select_text = XmCreatePulldownMenu(next_level, "select_locat_pane", NULL, 0);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNsubMenuId, select_text); ++nargs;
    XtSetArg(args[nargs], XmNmnemonic, 'C'); ++nargs;
    button = XmCreateCascadeButton(next_level, "Colors", args, nargs);
    XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'T'); ++nargs;
     button = XmCreatePushButton(select_text, "Text", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dText, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_colors, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'A'); ++nargs;
     button = XmCreatePushButton(select_text, "Axes", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dScale, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_colors, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'B'); ++nargs;
     button = XmCreatePushButton(select_text, "Background", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dBkgnd, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_colors, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 3rd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'C'); ++nargs;
     button = XmCreatePushButton(select_text, "Cursor", args, nargs);
     wid_addr = Xg3dAllocStruct(Xg3dCursor, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg3d_do_colors, wid_addr);
     XtManageChild(button);

/*** Data Submenu ***/
  next_level = XmCreatePulldownMenu(pulldown, "Data_pane", NULL, 0);

  nargs = 0;   /* 1st level menu button */
  XtSetArg(args[nargs], XmNsubMenuId, next_level); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'D'); ++nargs;
  button = XmCreateCascadeButton(pulldown, "Data", args, nargs);
  XtManageChild(button);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'N'); ++nargs;
    button = XmCreatePushButton(next_level, "New Plot", args, nargs);
    XtAddCallback(button, XmNactivateCallback, Xg3d_start_add_file, graph);
    XtManageChild(button);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'C'); ++nargs;
    button = XmCreatePushButton(next_level, "Color", args, nargs);
    wid_addr = Xg3dAllocStruct(Xg3dDSelColor, (caddr_t) graph);
    XtAddCallback(button, XmNactivateCallback, Xg3d_select_data_set, wid_addr);
    XtManageChild(button);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'P'); ++nargs;
    button = XmCreatePushButton(next_level, "Points/Line", args, nargs);
    wid_addr = Xg3dAllocStruct(Xg3dDSelPoint, (caddr_t) graph);
    XtAddCallback(button, XmNactivateCallback, Xg3d_select_data_set, wid_addr);
    XtManageChild(button);
/*    XtManageChild(next_level);   /* kludge */

/****** Options ******/
pulldown = XmCreatePulldownMenu(graph->menu_bar, "Option_pane", NULL, 0);
XtManageChild(pulldown);

nargs = 0;   /* Menu Bar Button */
XtSetArg(args[nargs], XmNsubMenuId, pulldown); ++nargs;
XtSetArg(args[nargs], XmNmnemonic, 'O'); ++nargs;
button = XmCreateCascadeButton(graph->menu_bar, "Options", args, nargs);
XtManageChild(button);

  nargs = 0;   /* 1st level menu button */
  XtSetArg(args[nargs], XmNmnemonic, 'C'); ++nargs;
  button = XmCreatePushButton(pulldown, "Cursor On/Off", args, nargs);
  XtAddCallback(button, XmNactivateCallback, Xg3d_CursorToggle, graph);
  XtManageChild(button);

  nargs = 0;   /* 1st level menu button */
  XtSetArg(args[nargs], XmNmnemonic, 'P'); ++nargs;
  button = XmCreatePushButton(pulldown, "Print", args, nargs);
  XtAddCallback(button, XmNactivateCallback, Xg3dPrint, graph);
  XtManageChild(button);

}

/***********************************************************************
*
*                           Help & Error System
*
************************************************************************/
void Xg3d_help_sys(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    nargs;
  Arg
    args[5];
  XmString
    okaymsg,
    title,
    help_message;
  Widget
    help_box,
    button;
switch((int) client_data)
  {
  case Xg3dMAIN_MENU:
    help_message = XmStringCreateLtoR("               Xg3dPlot\n\
This is the main command menu. You can load or save\n\
graphs, edit text or layouts, and change data.\n\
   Use Edit, then Titles to change text, location,\n\
font, or color of all text items.\n\
   Use Edit/Layout to adjust the layout factors\n\
(scaling, skew factor, vertical offset) as well\n\
as the minimum and maximum for each axis.\n\
   Use Edit/Data to change data sets and to set\n\
the line type (horizontal, vertical, mesh).\n\
    Use Options/Cursor to toggle a data point\n\
cursor on/off. Move the cursor from point to\n\
point and obtain a read out of x-y values using\n\
the arrow keys.\n\
    The Options/Print button produces a postscript\n\
output file named Xg3dPlt.psc\n", Xg3dCharset);
    break;
  case Xg3dEDIT_LABEL:
    help_message = XmStringCreateLtoR("Edit the label text", Xg3dCharset);
    break;
  case Xg3dADD_FILE:
    help_message = XmStringCreateLtoR("Choose a file from the list to add\n\
to the list of data sets. Use the\nfilter option to view selected file\n",
Xg3dCharset);
    break;
  case Xg3dDATA_SELECT:
    help_message = XmStringCreateLtoR("Choose a data set to molest", Xg3dCharset);
    break;
  case Xg3dFONTS_HELP:
    help_message = XmStringCreateLtoR("Select a font for this label",
				      Xg3dCharset);
    break;
  case Xg3dDATA_OPTIONS:
    help_message = XmStringCreateLtoR("Select Point Type and Line Type\n\
for this dataset, or reset to defaults", Xg3dCharset);
    break;
  case Xg3dLOAD_SAVE:
    help_message = XmStringCreateLtoR("Choose a graph file to load or\n\
a name to save this graph to.\nAs a convention, the extension *.grf\n\
is suggested.", Xg3dCharset);
    break;
  case Xg3dLAYOUT:
    help_message = XmStringCreateLtoR("Select the values dorfus", Xg3dCharset);
    break;
  case Xg3dCOORDS:
    help_message = XmStringCreateLtoR("Choose the % coordintes", Xg3dCharset);
    break;
  case Xg3dCOLOR_HELP:
    help_message = XmStringCreateLtoR("Click on a color", Xg3dCharset);
    break;
  case Xg3dSKEW:
    help_message = XmStringCreateLtoR("  The skew angle (in degrees) is the\n\
clockwise angle of the y-axis from the vertical (Z) axis\n\
  The vertical offset is the line spacing, in pixels,\n\
between lines in the display\n\
  The Scaling factor maps the Z(x,y) image values\n\
into pixel values for plotting", Xg3dCharset);
    break;
  default:
    help_message = XmStringCreateLtoR("Internal Error: No such help", Xg3dCharset);
    break;
  }
title = XmStringCreateLtoR("Xg3d: help", Xg3dCharset);
okaymsg = XmStringCreateLtoR("I get it", Xg3dCharset);
nargs = 0;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNmessageString, help_message); ++nargs;
XtSetArg(args[nargs], XmNokLabelString, okaymsg); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
help_box = XmCreateInformationDialog(w, "Xg3d: help", args, nargs);
button = XmMessageBoxGetChild(help_box, XmDIALOG_CANCEL_BUTTON);
XtUnmanageChild(button);
button = XmMessageBoxGetChild(help_box, XmDIALOG_HELP_BUTTON);
XtUnmanageChild(button);
XtFree(help_message);
XtFree(okaymsg);
XtFree(title);
XtManageChild(help_box);
}
/******************* error system *************************/
void Xg3d_error_sys(error_type, graph)
     int
       error_type;
     Xg3dGraphInfo
       *graph;
{
  int
    nargs;
  Arg
    args[10];
  XmString
    message,
    title,
    ok_message;
  Widget
    error_box,
    button;
switch (error_type)
  {
  case Xg3dNO_FILE:
    message = XmStringCreateLtoR("No Such File", Xg3dCharset);
    break;
  case Xg3dDATA_CALLOC_FAIL:
    message = XmStringCreateLtoR("Internal Error: Data calloc failure",
				 Xg3dCharset);
    break;
  case Xg3dTOO_MANY_FILES:
    message = XmStringCreateLtoR("You are already using the maximum\n\
number of files. To add\nnew files you must delete\nsome existing ones",
Xg3dCharset);
    break;
  case Xg3dNO_DATA_SETS:
    message = XmStringCreateLtoR("Graphing Routine begun with no Data!\n\
Exit at start!", Xg3dCharset);
    break;
  case Xg3dNO_DELETE_LAST:
    message = XmStringCreateLtoR("You cannot delete the last data set\n\
Add another one, then delete this", Xg3dCharset);
    break;
  case Xg3dNOT_IMPLEMENTED:
    message = XmStringCreateLtoR("Function not implemented yet", Xg3dCharset);
    break;
  case Xg3dNO_FONT:
    message = XmStringCreateLtoR("Unable to load this font", Xg3dCharset);
    break;
  case Xg3dNO_SHELL:
    message = XmStringCreateLtoR("No application shell supplied in\n\
Xg3dGraphInfo structure. Application cannot\nrun without a shell. Please\n\
supply a shell window in calling procedure", Xg3dCharset);
    break;
  case Xg3dBAD_COORDS:
    message = XmStringCreateLtoR("       Invalid Coordinates\n\n\
Coordinates Must be between 0 and 100\nNo change made", Xg3dCharset);
    break;
  case Xg3dBAD_MINMAX:
    message = XmStringCreateLtoR("       Min >= Max\n\n\
Minimum must be less than Maximum\nNo change made", Xg3dCharset);
    break;
  default:
    message = XmStringCreateLtoR("Internal Error: No such error!", Xg3dCharset);
    break;
  }
nargs = 0;
XtSetArg(args[nargs], XmNmessageString, message); ++nargs;
ok_message = XmStringCreateLtoR("Continue", Xg3dCharset);
title = XmStringCreateLtoR("Xg3d: Error System", Xg3dCharset);
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNokLabelString, ok_message); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
error_box = XmCreateErrorDialog(graph->app_shell, "Xg3d: Error System",
				args, nargs);
button = XmMessageBoxGetChild(error_box, XmDIALOG_HELP_BUTTON);
XtUnmanageChild(button);
button = XmMessageBoxGetChild(error_box, XmDIALOG_CANCEL_BUTTON);
XtUnmanageChild(button);
XtManageChild(error_box);
XtFree(message);
XtFree(title);
XtFree(ok_message);
}
/***************************/
void Xg3d_cancel_CB(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
XtDestroyWidget(w);
if ((Widget) client_data != NULL)
  XtDestroyWidget((Widget) client_data);
}
/***************************/
void Xg3d_free_wid(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i;
  Xg3dGraphInfo
    *graph;
graph = (Xg3dGraphInfo *) client_data;
for (i=0; i<Xg3dNWID; ++i)
  if (graph->wid_array[i] != NULL)
    free(graph->wid_array[i]);
}
/****************************************/
Xg3dWidStruct * Xg3dAllocStruct(client, graph)
     int
       client;
     caddr_t
       graph;
{
  Xg3dWidStruct
    *wid_addr;
wid_addr = (Xg3dWidStruct *) calloc(1, sizeof(Xg3dWidStruct));
wid_addr->client = (caddr_t) client;
wid_addr->graph = graph;
return(wid_addr);
}
/******************************************************************
*
*                      File Button
*
*******************************************************************/
/*************** Exit *************/
void Xg3d_create_exit_warning(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    nargs;
  Arg
    args[5];
  XmString
    title,
    okaymsg,
    helpmsg,
    warning_message;
  Widget
    warning_box,
    button;
warning_message = XmStringCreateLtoR("                        Xg3dPlot\n\
You are about to exit the graph. Be sure anything you\n\
  want to save is saved.\n\
Press Exit to leave the graph and return to a\n\
  parent program.\n\
Press Kill to leave the program.\n\
Press Cancel to return", Xg3dCharset);
okaymsg = XmStringCreateLtoR("Exit", Xg3dCharset);
helpmsg = XmStringCreateLtoR("Kill", Xg3dCharset);
title = XmStringCreateLtoR("Xg3d: Warning!", Xg3dCharset);
nargs = 0;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNmessageString, warning_message); ++nargs;
XtSetArg(args[nargs], XmNokLabelString, okaymsg); ++nargs;
XtSetArg(args[nargs], XmNhelpLabelString, helpmsg); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
warning_box = XmCreateWarningDialog(w, "Xg3d: Warning!", args, nargs);
/*button = XmMessageBoxGetChild(warning_box, XmDIALOG_HELP_BUTTON);
XtUnmanageChild(button);*/
/*button = XmMessageBoxGetChild(warning_box, XmDIALOG_OK_BUTTON);
XtAddCallback(button, XmNactivateCallback, Xg3d_exit_program, NULL);*/
XtAddCallback(warning_box, XmNokCallback, Xg3d_exit_graph, client_data);
XtAddCallback(warning_box, XmNhelpCallback, Xg3d_exit_program, client_data);
XtFree(warning_message);
XtFree(okaymsg);
XtFree(helpmsg);
XtFree(title);
XtManageChild(warning_box);
}
/**************************/
void Xg3d_exit_graph(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
graph = (Xg3dGraphInfo *) client_data;
XtDestroyWidget(graph->app_shell);
if (graph->cursor_status == 1)  /* cursor is on, turn it off */
  {
    Xg3d_delcurs(graph);
  }
/*exit(0);*/
}
/**************************/
void Xg3d_exit_program(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
graph = (Xg3dGraphInfo *) client_data;
XtDestroyWidget(graph->app_shell);
if (graph->cursor_status == 1)  /* cursor is on, turn it off */
  {
    Xg3d_delcurs(graph);
  }
exit(0);
}
/************************ Load / Save Graph *********************/
void Xg3d_load_save_graph(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Widget
    file_select;
  int
    nargs;
  Arg
    args[10];
  char
    title[80];
  XmString
    stitle,
    dirmask;
  Xg3dWidStruct
    *wid_addr;
  Xg3dGraphInfo
    *graph;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
dirmask = XmStringCreateLtoR("*.grf", Xg3dCharset);
if ((int) wid_addr->client == Xg3dLoad)
  {
    strcpy(title, "Xg3d: Load Graph");
  }
else
  {
    strcpy(title, "Xg3d: Save Graph");
  }
nargs = 0;
stitle = XmStringCreateLtoR(title, Xg3dCharset);
XtSetArg(args[nargs], XmNdialogTitle, stitle); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
XtSetArg(args[nargs], XmNdirMask, dirmask); ++nargs;
file_select = XmCreateFileSelectionDialog(graph->app_shell,
		title, args, nargs);
if ((int) wid_addr->client == Xg3dLoad)
  XtAddCallback(file_select, XmNokCallback, Xg3d_load_graph, graph);
else
  XtAddCallback(file_select, XmNokCallback, Xg3d_save_graph, graph);
XtAddCallback(file_select, XmNcancelCallback, Xg3d_cancel_CB, NULL);
XtAddCallback(file_select, XmNhelpCallback, Xg3d_help_sys,
	      (caddr_t) Xg3dLOAD_SAVE);
XtManageChild(file_select);
XtFree(dirmask);
XtFree(stitle);
}
/**************************************/
void Xg3d_load_graph(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    *buf;
  int
    i,
    val;
  long int
    forecolor,
    backcolor;
  XmFileSelectionBoxCallbackStruct
    *dialog_rslt;
  FILE
    *fp;
  Xg3dGraphInfo
    *graph;
graph = (Xg3dGraphInfo *) client_data;
dialog_rslt = (XmFileSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(dialog_rslt->value, Xg3dCharset, &buf);
fp = fopen(buf, "r");
if (fp == NULL)
  {
    Xg3d_error_sys(Xg3dNO_FILE, graph);
    return;
  }
XtDestroyWidget(w);
if (graph->cursor_status == 1)
  Xg3d_delcurs(graph);
for (i=0; i<graph->num_sets; ++i)
  free(graph->dataset[i].darray);
Xg3dInitializeStruct(graph);
forecolor = XBlackPixelOfScreen(XtScreen(graph->app_shell));
backcolor = XWhitePixelOfScreen(XtScreen(graph->app_shell));
graph->text_color = forecolor;
graph->axis_color = forecolor;
graph->background_color = backcolor;
graph->cursor_color = forecolor;
for (i=0; i<Xg3dNSETS; ++i)
  graph->dataset[i].datacolor = forecolor;
graph->num_sets = 1;  /* minimal defaults */
Xg3dReadGrf(fp, graph);

fclose(fp);
if (!(graph->xaxis_layout.user_set_flag & Xg3dUmin))
  graph->xaxis_layout.min = 0;
if (!(graph->xaxis_layout.user_set_flag & Xg3dUmax))
  graph->xaxis_layout.max = graph->dataset[0].nx;
if (!(graph->yaxis_layout.user_set_flag & Xg3dUmin))
  graph->yaxis_layout.min = 0;
if (!(graph->yaxis_layout.user_set_flag & Xg3dUmax))
  graph->yaxis_layout.max = graph->dataset[0].ny;
Xg3d_createGraph(graph);
return;
}
/********************************/
Xg3dReadGrf(fp, graph)
     FILE
       *fp;
     Xg3dGraphInfo
       *graph;
{
  char
    input_text[200];
  int
    i,
    val;
  int2
    *pix_array;
  struct fileinfo
    datafile;
/*** read in graph setup values ***/
while (Xg3dGetline(fp, input_text) != EOF)
  {
    strcat(input_text, "\n");

    if (!strncmp(input_text, "num_sets: ", 10))
      {
	sscanf(&input_text[10], "%d", &graph->num_sets);
	graph->dataset[0].point_type = Xg3dDefPointType;
	graph->dataset[0].line_status = Xg3dDefLineType;
	graph->dataset[0].point_size = Xg3dDefPtSize;
	graph->dataset[0].s_func = Xg3dDefFunc;
      }

    if (!strncmp(input_text, "x-axis text: ", 13))
      sscanf(&input_text[13], "%[^\n]", graph->x_axis_text);
    if (!strncmp(input_text, "y-axis text: ", 13))
      sscanf(&input_text[13], "%[^\n]", graph->y_axis_text);
    if (!strncmp(input_text, "title text: ", 12))
      sscanf(&input_text[12], "%[^\n]", graph->title_text);
    if (!strncmp(input_text, "subtitle text: ", 15))
      sscanf(&input_text[15], "%[^\n]", graph->subtitle_text);

    if (!strncmp(input_text, "x-axis font: ", 13))
      sscanf(&input_text[13], "%[^\n]", graph->x_axis_font.name);
    if (!strncmp(input_text, "y-axis font: ", 13))
      sscanf(&input_text[13], "%[^\n]", graph->y_axis_font.name);
    if (!strncmp(input_text, "title font: ", 12))
      sscanf(&input_text[12], "%[^\n]", graph->title_font.name);
    if (!strncmp(input_text, "subtitle font: ", 15))
      sscanf(&input_text[15], "%[^\n]", graph->subtitle_font.name);
    if (!strncmp(input_text, "scale font: ", 12))
      sscanf(&input_text[12], "%[^\n]", graph->scale_font.name);

    if (!strncmp(input_text, "x-axis text location x: ", 24))
      sscanf(&input_text[24], "%le", &graph->x_axis_coord.x);
    if (!strncmp(input_text, "x-axis text location y: ", 24))
      sscanf(&input_text[24], "%le", &graph->x_axis_coord.y);
    if (!strncmp(input_text, "y-axis text location x: ", 24))
      sscanf(&input_text[24], "%le", &graph->y_axis_coord.x);
    if (!strncmp(input_text, "y-axis text location y: ", 24))
      sscanf(&input_text[24], "%le", &graph->y_axis_coord.y);
    if (!strncmp(input_text, "title text location x: ", 23))
      sscanf(&input_text[23], "%le", &graph->title_coord.x);
    if (!strncmp(input_text, "title text location y: ", 23))
      sscanf(&input_text[23], "%le", &graph->title_coord.y);
    if (!strncmp(input_text, "subtitle text location x: ", 26))
      sscanf(&input_text[26], "%le", &graph->subtitle_coord.x);
    if (!strncmp(input_text, "subtitle text location y: ", 26))
      sscanf(&input_text[26], "%le", &graph->subtitle_coord.y);

/*    if (!strncmp(input_text, "labels: ", 8))
      {
	sscanf(&input_text[8], "%d", &val);
	graph->label_type = (char) val;
      }*/

    if (!strncmp(input_text, "x-axis min: ", 12))
      sscanf(&input_text[12], "%le", &graph->xaxis_layout.min);
    if (!strncmp(input_text, "x-axis max: ", 12))
      sscanf(&input_text[12], "%le", &graph->xaxis_layout.max);
    if (!strncmp(input_text, "x-axis scale location x: ", 25))
      sscanf(&input_text[25], "%le", &graph->xaxis_layout.scale_loc.x);
    if (!strncmp(input_text, "x-axis scale location y: ", 25))
      sscanf(&input_text[25], "%le", &graph->xaxis_layout.scale_loc.y);
    if (!strncmp(input_text, "x-axis increment digit: ", 24))
      sscanf(&input_text[24], "%d", &graph->xaxis_layout.increment);
    if (!strncmp(input_text, "x-axis significant figures: ", 28))
      sscanf(&input_text[28], "%d", &graph->xaxis_layout.nsig_fig);
    if (!strncmp(input_text, "x-axis user_set_flag: ", 22))
      {
	sscanf(&input_text[22], "%d", &val);
	graph->xaxis_layout.user_set_flag = (char) val;
      }

    if (!strncmp(input_text, "y-axis min: ", 12))
      sscanf(&input_text[12], "%le", &graph->yaxis_layout.min);
    if (!strncmp(input_text, "y-axis max: ", 12))
      sscanf(&input_text[12], "%le", &graph->yaxis_layout.max);
    if (!strncmp(input_text, "y-axis scale location x: ", 25))
      sscanf(&input_text[25], "%le", &graph->yaxis_layout.scale_loc.x);
    if (!strncmp(input_text, "y-axis scale location y: ", 25))
      sscanf(&input_text[25], "%le", &graph->yaxis_layout.scale_loc.y);
    if (!strncmp(input_text, "y-axis increment digit: ", 24))
      sscanf(&input_text[24], "%d", &graph->yaxis_layout.increment);
    if (!strncmp(input_text, "y-axis significant figures: ", 28))
      sscanf(&input_text[28], "%d", &graph->yaxis_layout.nsig_fig);
    if (!strncmp(input_text, "y-axis user_set_flag: ", 22))
      {
	sscanf(&input_text[22], "%d", &val);
	graph->yaxis_layout.user_set_flag = (char) val;
      }

    if (!strncmp(input_text, "theta: ", 7))
      sscanf(&input_text[7], "%le", &graph->theta);
    if (!strncmp(input_text, "hoff: ", 6))
      sscanf(&input_text[6], "%le", &graph->hoff);
    if (!strncmp(input_text, "scale factor: ", 14))
      sscanf(&input_text[14], "%le", &graph->scale_fac);

    if (!strncmp(input_text, "dataset ", 8))
      {
	sscanf(&input_text[8], "%d", &val);
	if (val < 10) /* set offset for rest of string */
	  i = 10;
	else
	  i = 11;
	if (!strncmp(&input_text[i], "filename: ", 10))
	  sscanf(&input_text[i+10], "%s", graph->dataset[val].filename);
	if (!strncmp(&input_text[i], "points: ", 8))
	  sscanf(&input_text[i+8], "%d", &graph->dataset[val].num_points);
	if (!strncmp(&input_text[i], "frame: ", 7))
	  sscanf(&input_text[i+7], "%d", &graph->dataset[val].frame_num);
	if (!strncmp(&input_text[i], "point type: ", 12))
	  sscanf(&input_text[i+12], "%d", &graph->dataset[val].point_type);
	if (!strncmp(&input_text[i], "line style: ", 12))
	  sscanf(&input_text[i+12], "%d", &graph->dataset[val].line_status);
	if (!strncmp(&input_text[i], "scaling: ", 9))
	  sscanf(&input_text[i+9], "%d", &graph->dataset[i].s_func);
      }
  }

/*********** finish up *****/
strcpy(datafile.filename, graph->dataset[0].filename);
fileopen(&datafile);
/*showhead(&datafile);*/
pix_array = readframe(&datafile, graph->dataset[0].frame_num);
if (pix_array == NULL)
  {
    fprintf(stderr, "No data returned! Aborting\n");
    exit(0);
  }
/**********************************/
graph->dataset[0].darray = pix_array;
graph->dataset[0].nx = datafile.file_header.row_length;
graph->dataset[0].ny = datafile.file_header.col_length;
fileclose(datafile.dat_file);
}
/********************************/
Xg3dGetline(fp, buf)
     FILE
       *fp;
     char
       *buf;
{
  int
    rtrn,
    i;
  char
    c;
i = 0;
buf[0] = '\0';
c = getc(fp);
while ((c != (char) EOF) && (c != '\n'))
  {
    buf[i] = c;
    ++i;
    c = getc(fp);
  }
buf[i] = '\0';
rtrn = i;
if (c == (char) EOF)
  rtrn = EOF;
return(rtrn);
}
/**************************************/
void Xg3d_save_graph(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    *buf,
    comment[80];
  int
    i;
  XmFileSelectionBoxCallbackStruct
    *dialog_rslt;
  FILE
    *fp;
  Xg3dGraphInfo
    *graph;
graph = (Xg3dGraphInfo *) client_data;
dialog_rslt = (XmFileSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(dialog_rslt->value, Xg3dCharset, &buf);
fp = fopen(buf, "w");
if (fp == NULL)
  {
    Xg3d_error_sys(Xg3dNO_FILE, graph);
    return;
  }
XtDestroyWidget(w);
fprintf(fp, "graph name: %s\n", buf);
/*fprintf(fp, "description: %s\n", comment);*/
fprintf(fp, "num_sets: %d\n", graph->num_sets);

fprintf(fp, "x-axis text: %s\n", graph->x_axis_text);
fprintf(fp, "y-axis text: %s\n", graph->y_axis_text);
fprintf(fp, "title text: %s\n", graph->title_text);
fprintf(fp, "subtitle text: %s\n", graph->subtitle_text);

fprintf(fp, "x-axis font: %s\n", graph->x_axis_font.name);
fprintf(fp, "y-axis font: %s\n", graph->y_axis_font.name);
fprintf(fp, "title font: %s\n", graph->title_font.name);
fprintf(fp, "subtitle font: %s\n", graph->subtitle_font.name);
fprintf(fp, "scale font: %s\n", graph->scale_font.name);

/*fprintf(fp, "labels: %d\n", (int) graph->label_type);*/

fprintf(fp, "x-axis min: %lg\n", graph->xaxis_layout.min);
fprintf(fp, "x-axis max: %lg\n", graph->xaxis_layout.max);
fprintf(fp, "x-axis scale location x: %lg\n", graph->xaxis_layout.scale_loc.x);
fprintf(fp, "x-axis scale location y: %lg\n", graph->xaxis_layout.scale_loc.y);
fprintf(fp, "x-axis increment digit: %d\n", graph->xaxis_layout.increment);
fprintf(fp, "x-axis significant figures: %d\n", graph->xaxis_layout.nsig_fig);
fprintf(fp, "x-axis user_set_flag: %d\n",
	(int) graph->xaxis_layout.user_set_flag);

fprintf(fp, "y-axis min: %lg\n", graph->yaxis_layout.min);
fprintf(fp, "y-axis min: %lg\n", graph->yaxis_layout.min);
fprintf(fp, "y-axis scale location x: %lg\n", graph->yaxis_layout.scale_loc.x);
fprintf(fp, "y-axis scale location y: %lg\n", graph->yaxis_layout.scale_loc.y);
fprintf(fp, "y-axis increment digit: %d\n", graph->yaxis_layout.increment);
fprintf(fp, "y-axis significant figures: %d\n", graph->yaxis_layout.nsig_fig);
fprintf(fp, "y-axis user_set_flag: %d\n",
	(int) graph->yaxis_layout.user_set_flag);

fprintf(fp, "x-axis text location x: %lg\n", graph->x_axis_coord.x);
fprintf(fp, "x-axis text location y: %lg\n", graph->x_axis_coord.y);
fprintf(fp, "y-axis text location x: %lg\n", graph->y_axis_coord.x);
fprintf(fp, "y-axis text location y: %lg\n", graph->y_axis_coord.y);
fprintf(fp, "title text location x: %lg\n", graph->title_coord.x);
fprintf(fp, "title text location y: %lg\n", graph->title_coord.y);
fprintf(fp, "subtitle text location x: %lg\n", graph->subtitle_coord.x);
fprintf(fp, "subtitle text location y: %lg\n", graph->subtitle_coord.y);

fprintf(fp, "theta: %lg\n", graph->theta);
fprintf(fp, "hoff: %lg\n", graph->hoff);
fprintf(fp, "scale factor: %lg\n", graph->scale_fac);

for (i=0; i<graph->num_sets; ++i)
  {
    fprintf(fp, "\ndataset %d filename: %s\n", i, graph->dataset[i].filename);
    fprintf(fp, "dataset %d frame: %d\n", i, graph->dataset[i].frame_num);
    fprintf(fp, "dataset %d points: %d\n", i, graph->dataset[i].num_points);
    fprintf(fp, "dataset %d point type: %d\n",i,graph->dataset[i].point_type);
    fprintf(fp, "dataset %d line style: %d\n",i,graph->dataset[i].line_status);
    fprintf(fp, "dataset %d scaling: %d\n", i, graph->dataset[i].s_func);
  }

fclose(fp);
}
/****************************************************************
*
*                    Edit/Titles Button
*
*****************************************************************/
/********************* Edit Text *********************/
void Xg3d_do_edit_titles(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    nargs;
  Arg
    args[10];
  Widget
    dialog;
  XmString
    text,
    title,
    label;
  Xg3dWidStruct
    *wid_addr;
  Xg3dGraphInfo
    *graph;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
nargs = 0;
switch ((int) wid_addr->client)
  {
  case Xg3dXaxisLabel:
    text = XmStringCreateLtoR(graph->x_axis_text, Xg3dCharset);
    label = XmStringCreateLtoR("X axis Description:", Xg3dCharset);
    break;
  case Xg3dYaxisLabel:
    text = XmStringCreateLtoR(graph->y_axis_text, Xg3dCharset);
    label = XmStringCreateLtoR("Y axis Description:", Xg3dCharset);
    break;
  case Xg3dTitle:
    text = XmStringCreateLtoR(graph->title_text, Xg3dCharset);
    label = XmStringCreateLtoR("Title:", Xg3dCharset);
    break;
  case Xg3dSubtitle:
    text = XmStringCreateLtoR(graph->subtitle_text, Xg3dCharset);
    label = XmStringCreateLtoR("Subtitle:", Xg3dCharset);
    break;
  default:
    break;
  }
title = XmStringCreateLtoR("Xg3d: New Text", Xg3dCharset);
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNtextString, text); ++nargs;
XtSetArg(args[nargs], XmNselectionLabelString, label); ++nargs;
XtSetArg(args[nargs], XmNautoUnmanage, False); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
dialog = XmCreatePromptDialog(w, "Xg3d: New Text", args, nargs);
XtAddCallback(dialog, XmNokCallback, Xg3d_get_new_text, client_data);
XtAddCallback(dialog, XmNcancelCallback, Xg3d_cancel_CB, NULL);
XtAddCallback(dialog, XmNhelpCallback, Xg3d_help_sys,
	      (caddr_t) Xg3dEDIT_LABEL);
XtManageChild(dialog);
XtFree(text);
XtFree(title);
XtFree(label);
}
/****************************/
void Xg3d_get_new_text(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XmSelectionBoxCallbackStruct
    *dialog_rslt;
  char
    *buf;
  Xg3dWidStruct
    *wid_addr;
  Xg3dGraphInfo
    *graph;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
XtDestroyWidget(w);
dialog_rslt = (XmSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(dialog_rslt->value, Xg3dCharset, &buf);
switch((int) wid_addr->client)
  {
  case Xg3dXaxisLabel:
    strcpy(graph->x_axis_text, buf);
    break;
  case Xg3dYaxisLabel:
    strcpy(graph->y_axis_text, buf);
    break;
  case Xg3dTitle:
    strcpy(graph->title_text, buf);
    break;
  case Xg3dSubtitle:
    strcpy(graph->subtitle_text, buf);
    break;
  default:
    break;
  }
XtFree(buf);
/*Xg3d_drawGraph(graph);*/
}
/************************ Select Font **************************/
void Xg3d_do_font_titles(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Widget
    file_select;
  int
    nargs;
  Arg
    args[10];
  char
    buf[80],
    cur_font[80],
    title[80];
  XmString
    stitle,
    scur_font,
    dirmask;
  Xg3dWidStruct
    *wid_addr;
  Xg3dGraphInfo
    *graph;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
sprintf(buf, "%s*", Xg3dFontdir);
dirmask = XmStringCreateLtoR(buf, Xg3dCharset);
switch ((int) wid_addr->client)
  {
  case Xg3dXaxisLabel:
    strcpy(title, "Xg3d: X-axis Font");
    strcpy(cur_font, graph->x_axis_font.name);
    break;
  case Xg3dYaxisLabel:
    strcpy(title, "Xg3d: Y-axis Font");
    strcpy(cur_font, graph->y_axis_font.name);
    break;
  case Xg3dTitle:
    strcpy(title, "Xg3d: Title Font");
    strcpy(cur_font, graph->title_font.name);
    break;
  case Xg3dSubtitle:
    strcpy(title, "Xg3d: Subtitle Font");
    strcpy(cur_font, graph->subtitle_font.name);
    break;
  case Xg3dScale:
    strcpy(title, "Xg3d: Scale Font");
    strcpy(cur_font, graph->scale_font.name);
    break;
  default:
    break;
  }

nargs = 0;
stitle = XmStringCreateLtoR(title, Xg3dCharset);
scur_font = XmStringCreateLtoR(cur_font, Xg3dCharset);
XtSetArg(args[nargs], XmNdialogTitle, stitle); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
XtSetArg(args[nargs], XmNdirMask, dirmask); ++nargs;
file_select = XmCreateFileSelectionDialog(graph->app_shell,
		title, args, nargs);
XtAddCallback(file_select, XmNokCallback, Xg3d_update_fonts, client_data);
XtAddCallback(file_select, XmNcancelCallback, Xg3d_cancel_CB, NULL);
XtAddCallback(file_select, (String) XmNhelpCallback,
	      (XtCallbackProc) Xg3d_help_sys, (caddr_t) Xg3dFONTS_HELP);
XtManageChild(file_select);
XtFree(dirmask);
XtFree(stitle);
XtFree(scur_font);
}
/**************************************/
void Xg3d_update_fonts(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    nargs;
  Arg
    args[10];
  XmString
    name_string;
  char
    buf[80],
    *charpt,
    *font_name;
  XFontStruct
    *font;
  XmFileSelectionBoxCallbackStruct
    *dialog_rslt;
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;;
dialog_rslt = (XmFileSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(dialog_rslt->value, Xg3dCharset, &font_name);
/******* drop suffix and path name ********/
/*printf("full font_name: **%s**\n", font_name);*/
while ((charpt = strrchr(font_name, '.')) != NULL)
  *charpt = '\0';
charpt = strrchr(font_name, '/');
if (charpt != NULL)
  {
    strcpy(buf, charpt+1);
    strcpy(font_name, buf);
  }
/*printf("shortened font_name: **%s**\n", font_name);*/

font = XLoadQueryFont(XtDisplay(graph->app_shell), font_name);
XtDestroyWidget(w);
if (!font)
  {
    Xg3d_error_sys(Xg3dNO_FONT, graph);
    return;
  }
switch((int) wid_addr->client)
  {
  case Xg3dXaxisLabel:
    XFreeFont(XtDisplay(graph->app_shell),
	      graph->x_axis_font.font);
    graph->x_axis_font.font = font;
    strcpy(graph->x_axis_font.name, font_name);
    break;
  case Xg3dYaxisLabel:
    XFreeFont(XtDisplay(graph->app_shell),
	      graph->y_axis_font.font);
    graph->y_axis_font.font = font;
    strcpy(graph->y_axis_font.name, font_name);
    break;
  case Xg3dTitle:
    XFreeFont(XtDisplay(graph->app_shell),
	      graph->title_font.font);
    strcpy(graph->title_font.name, font_name);
    graph->title_font.font = font;
    break;
  case Xg3dSubtitle:
    XFreeFont(XtDisplay(graph->app_shell),
	      graph->subtitle_font.font);
    graph->subtitle_font.font = font;
    strcpy(graph->subtitle_font.name, font_name);
    break;
  case Xg3dScale:
    XFreeFont(XtDisplay(graph->app_shell),
	      graph->scale_font.font);
    graph->scale_font.font = font;
    strcpy(graph->scale_font.name, font_name);
    break;
  default:
    break;
  }
XtFree(font_name);
/*Xg3d_drawGraph(graph);*/
}
/**************************************/
void Xg3d_default_font(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XFontStruct
    *font;
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;;
XtDestroyWidget(w);
font = XLoadQueryFont(XtDisplay(graph->app_shell), Xg3dDefFont);
switch((int) wid_addr->client)
  {
  case Xg3dXaxisLabel:
    XFreeFont(XtDisplay(graph->app_shell),
	      graph->x_axis_font.font);
    graph->x_axis_font.font = font;
    strcpy(graph->x_axis_font.name, Xg3dDefFont);
    break;
  case Xg3dYaxisLabel:
    XFreeFont(XtDisplay(graph->app_shell),
	      graph->y_axis_font.font);
    graph->y_axis_font.font = font;
    strcpy(graph->y_axis_font.name, Xg3dDefFont);
    break;
  case Xg3dTitle:
    XFreeFont(XtDisplay(graph->app_shell),
	      graph->title_font.font);
    strcpy(graph->title_font.name, Xg3dDefFont);
    graph->title_font.font = font;
    break;
  case Xg3dSubtitle:
    XFreeFont(XtDisplay(graph->app_shell),
	      graph->subtitle_font.font);
    graph->subtitle_font.font = font;
    strcpy(graph->subtitle_font.name, Xg3dDefFont);
    break;
  case Xg3dScale:
    XFreeFont(XtDisplay(graph->app_shell),
	      graph->scale_font.font);
    graph->scale_font.font = font;
    strcpy(graph->scale_font.name, Xg3dDefFont);
    break;
  default:
    break;
  }
/*Xg3d_drawGraph(graph);*/
}
/******************* Text Location Coordinates **********************/
void Xg3d_get_coords(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XmString
    defaults,
    frame_label,
    title;
  Widget
    parent,
    form,
    label_frame,
    x_frame,
    y_frame,
    label,
    text_wid,
    button;
  Arg
    args[10];
  int
    nargs;
  char
    buf[80];
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;;
for (nargs=0; nargs<Xg3dNWID; ++nargs)
  graph->wid_array[nargs] = NULL;
switch ((int) wid_addr->client)
  {
  case Xg3dXaxisLabel:
    frame_label = XmStringCreateLtoR("X Axis Description: % Coordinates",
				     Xg3dCharset);
    graph->temp_coord.x = graph->x_axis_coord.x;
    graph->temp_coord.y = graph->x_axis_coord.y;
    break;
  case Xg3dYaxisLabel:
    frame_label = XmStringCreateLtoR("Y Axis Description: % Coordinates",
				     Xg3dCharset);
    graph->temp_coord.x = graph->y_axis_coord.x;
    graph->temp_coord.y = graph->y_axis_coord.y;
    break;
  case Xg3dTitle:
    frame_label = XmStringCreateLtoR("Graph Title: % Coordinates", Xg3dCharset);
    graph->temp_coord.x = graph->title_coord.x;
    graph->temp_coord.y = graph->title_coord.y;
    break;
  case Xg3dSubtitle:
    frame_label = XmStringCreateLtoR("Graph Subtitle: %Coordinates",
				     Xg3dCharset);
    graph->temp_coord.x = graph->subtitle_coord.x;
    graph->temp_coord.y = graph->subtitle_coord.y;
    break;
  case Xg3dXScale:
    frame_label = XmStringCreateLtoR("X Scaling Info: % Coordinates",
				     Xg3dCharset);
    graph->temp_coord.x = graph->xaxis_layout.scale_loc.x;
    graph->temp_coord.y = graph->xaxis_layout.scale_loc.y;
    break;
  case Xg3dYScale:
    frame_label = XmStringCreateLtoR("Y Scaling Info: % Coordinates",
				     Xg3dCharset);
    graph->temp_coord.x = graph->yaxis_layout.scale_loc.x;
    graph->temp_coord.y = graph->yaxis_layout.scale_loc.y;
    break;
  default:
    break;
  }
title = XmStringCreateLtoR("Xg3d: Set Coordinates", Xg3dCharset);
defaults = XmStringCreateLtoR("Defaults", Xg3dCharset);
nargs = 0;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNautoUnmanage, False); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
XtSetArg(args[nargs], XmNapplyLabelString, defaults); ++nargs;
parent = XmCreatePromptDialog(w, "coords", args, nargs);
button = XmSelectionBoxGetChild(parent, XmDIALOG_TEXT);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(parent, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(parent, XmDIALOG_APPLY_BUTTON);
XtManageChild(button);

nargs = 0;
form = XmCreateForm(parent, "form", args, nargs);
XtManageChild(form);

/** Label Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
label_frame = XmCreateFrame(form, "label frame", args, nargs);
XtManageChild(label_frame);
nargs = 0;
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(label_frame, "text Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

/** X Coord Text Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, label_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
x_frame = XmCreateFrame(form, "x frame", args, nargs);
XtManageChild(x_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(x_frame, "x window", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("X Coordinate", Xg3dCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "x Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
text_wid = XmCreateText(button, "x text", args, nargs);
sprintf(buf, "%lg", graph->temp_coord.x);
XmTextSetString(text_wid, buf);
XtManageChild(text_wid);
graph->wid_array[0] = Xg3dAllocStruct(Xg3dXaxis, (caddr_t) graph);
XtAddCallback(text_wid, XmNvalueChangedCallback, Xg3d_temp_coord,
	      graph->wid_array[0]);

/** Y Coord Text Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, x_frame); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 10); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, label_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
y_frame = XmCreateFrame(form, "y frame", args, nargs);
XtManageChild(y_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(y_frame, "y window", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("Y Coordinate", Xg3dCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "y Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
text_wid = XmCreateText(button, "y text", args, nargs);
sprintf(buf, "%lg", graph->temp_coord.y);
XmTextSetString(text_wid, buf);
XtManageChild(text_wid);
graph->wid_array[1] = Xg3dAllocStruct(Xg3dYaxis, (caddr_t) graph);
XtAddCallback(text_wid, XmNvalueChangedCallback, Xg3d_temp_coord,
	      graph->wid_array[1]);

XtManageChild(parent);
XtAddCallback(parent, XmNcancelCallback, Xg3d_cancel_CB, NULL);
XtAddCallback(parent, XmNcancelCallback, Xg3d_free_wid, graph);
XtAddCallback(parent, XmNhelpCallback, Xg3d_help_sys, (caddr_t) Xg3dCOORDS);
XtAddCallback(parent, XmNokCallback, Xg3d_set_coord, client_data);
XtAddCallback(parent, XmNapplyCallback, Xg3d_default_coord, client_data);
XtFree(defaults);
XtFree(title);
}
/**************************************/
void Xg3d_temp_coord(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    *buf;
  int
    i;
  double
    value;
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
buf = XmTextGetString(w);
for (i=0; i<strlen(buf); ++i)
  if (!(isdigit(buf[i]) || isspace(buf[i]) || (buf[i] == '-')))
      return;
sscanf(buf, "%le", &value);
if ((int) wid_addr->client == Xg3dXaxis)
  {
    graph->temp_coord.x = value;
  }
else
  {
    graph->temp_coord.y = value;
  }
}
/**************************************/
void Xg3d_set_coord(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
  int
    i;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
XtDestroyWidget(w);
if ((graph->temp_coord.x < 0.) || (graph->temp_coord.x > 100.))
  {
    Xg3d_error_sys(Xg3dBAD_COORDS, graph);
    return;
  }
if ((graph->temp_coord.y < 0.) || (graph->temp_coord.y > 100.))
  {
    Xg3d_error_sys(Xg3dBAD_COORDS, graph);
    return;
  }
switch ((int) wid_addr->client)
  {
  case Xg3dXaxisLabel:
    graph->x_axis_coord.x = graph->temp_coord.x;
    graph->x_axis_coord.y = graph->temp_coord.y;
    break;
  case Xg3dYaxisLabel:
    graph->y_axis_coord.x = graph->temp_coord.x;
    graph->y_axis_coord.y = graph->temp_coord.y;
    break;
  case Xg3dTitle:
    graph->title_coord.x = graph->temp_coord.x;
    graph->title_coord.y = graph->temp_coord.y;
    break;
  case Xg3dSubtitle:
    graph->subtitle_coord.x = graph->temp_coord.x;
    graph->subtitle_coord.y = graph->temp_coord.y;
    break;
  case Xg3dXScale:
    graph->xaxis_layout.scale_loc.x = graph->temp_coord.x;
    graph->xaxis_layout.scale_loc.y = graph->temp_coord.y;
    break;
  case Xg3dYScale:
    graph->yaxis_layout.scale_loc.x = graph->temp_coord.x;
    graph->yaxis_layout.scale_loc.y = graph->temp_coord.y;
    break;
  default:
    break;
  }
for (i=0; i<Xg3dNWID; ++i)
  if (graph->wid_array[i] != NULL)
    free(graph->wid_array[i]);
/*Xg3d_drawGraph(graph);*/
}
/**************************************/
void Xg3d_default_coord(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
  int
    i;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
XtDestroyWidget(w);
switch ((int) wid_addr->client)
  {
  case Xg3dXaxisLabel:
    graph->x_axis_coord.x = Xg3dDefXtextXCoord;
    graph->x_axis_coord.y = Xg3dDefXtextYCoord;
    break;
  case Xg3dYaxisLabel:
    graph->y_axis_coord.x = Xg3dDefYtextXCoord;
    graph->y_axis_coord.y = Xg3dDefYtextYCoord;
    break;
  case Xg3dTitle:
    graph->title_coord.x = Xg3dDefTitletextXCoord;
    graph->title_coord.y = Xg3dDefTitletextYCoord;
    break;
  case Xg3dSubtitle:
    graph->subtitle_coord.x = Xg3dDefSubtextXCoord;
    graph->subtitle_coord.y = Xg3dDefSubtextYCoord;
    break;
  case Xg3dXScale:
    graph->xaxis_layout.scale_loc.x = Xg3dDefXscaleXCoord;
    graph->xaxis_layout.scale_loc.y = Xg3dDefXscaleYCoord;
    break;
  case Xg3dYScale:
    graph->yaxis_layout.scale_loc.x = Xg3dDefYscaleXCoord;
    graph->yaxis_layout.scale_loc.y = Xg3dDefYscaleYCoord;
    break;
  default:
    break;
  }
for (i=0; i<Xg3dNWID; ++i)
  if (graph->wid_array[i] != NULL)
    free(graph->wid_array[i]);
/*Xg3d_drawGraph(graph);*/
}
/**************************************************************
*
*                     Edit/Layout Button
*
***************************************************************/
/****************** Configure X or Y Axis Layout ****************/
void Xg3d_configure_axes(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i,
    nargs;
  Arg
    args[10];
  XmString
    defaults,
    frame_label,
    text,
    title;
  Widget
    parent,
    form,
    sigDigit_frame,
    lastSig_frame,
    min_frame,
    max_frame,
    label,
    radio,
    text_wid,
    button;
  char
    buf[80];
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
for (i=0; i<Xg3dNWID; ++i)
  graph->wid_array[i] = NULL;
nargs = 0;
if ((int) wid_addr->client == Xg3dXaxis)
  {
    graph->temp_axis.min = graph->xaxis_layout.min;
    graph->temp_axis.max = graph->xaxis_layout.max;
    graph->temp_axis.increment = graph->xaxis_layout.increment;
    graph->temp_axis.nsig_fig = graph->xaxis_layout.nsig_fig;
    graph->temp_axis.user_set_flag = graph->xaxis_layout.user_set_flag;
    title = XmStringCreateLtoR("Xg3d: X axis", Xg3dCharset);
  }
else
  {
    graph->temp_axis.min = graph->yaxis_layout.min;
    graph->temp_axis.max = graph->yaxis_layout.max;
    graph->temp_axis.increment = graph->yaxis_layout.increment;
    graph->temp_axis.nsig_fig = graph->yaxis_layout.nsig_fig;
    graph->temp_axis.user_set_flag = graph->yaxis_layout.user_set_flag;
    title = XmStringCreateLtoR("Xg3d: Y axis", Xg3dCharset);
  }
defaults = XmStringCreateLtoR("Defaults", Xg3dCharset);
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNautoUnmanage, False); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
XtSetArg(args[nargs], XmNapplyLabelString, defaults); ++nargs;
parent = XmCreatePromptDialog(w, "cfg axes", args, nargs);
button = XmSelectionBoxGetChild(parent, XmDIALOG_TEXT);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(parent, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(parent, XmDIALOG_APPLY_BUTTON);
XtManageChild(button);

nargs = 0;
form = XmCreateForm(parent, "form", args, nargs);
XtManageChild(form);

/** Minimum Value Text Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
min_frame = XmCreateFrame(form, "min frame", args, nargs);
XtManageChild(min_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(min_frame, "min window", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("Minimum Value", Xg3dCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "min Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
text_wid = XmCreateText(button, "min text", args, nargs);
sprintf(buf, "%lg", graph->temp_axis.min);
XmTextSetString(text_wid, buf);
XtManageChild(text_wid);
graph->wid_array[6] = Xg3dAllocStruct(Xg3dMin, (caddr_t) graph);
XtAddCallback(text_wid, XmNvalueChangedCallback,Xg3d_minmax,graph->wid_array[6]);

/** Maximum Value Text Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, min_frame); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 10); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
max_frame = XmCreateFrame(form, "max frame", args, nargs);
XtManageChild(max_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(max_frame, "max window", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("Maximum Value", Xg3dCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "max Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
text_wid = XmCreateText(button, "max text", args, nargs);
sprintf(buf, "%lg", graph->temp_axis.max);
XmTextSetString(text_wid, buf);
XtManageChild(text_wid);
graph->wid_array[7] = Xg3dAllocStruct(Xg3dMax, (caddr_t) graph);
XtAddCallback(text_wid, XmNvalueChangedCallback,Xg3d_minmax,graph->wid_array[7]);

XtManageChild(parent);
XtAddCallback(parent, XmNhelpCallback, Xg3d_help_sys, (caddr_t) Xg3dLAYOUT);
XtAddCallback(parent, XmNokCallback, Xg3d_set_axes, client_data);
XtAddCallback(parent, XmNcancelCallback, Xg3d_cancel_CB, NULL);
XtAddCallback(parent, XmNcancelCallback, Xg3d_free_wid, graph);
XtAddCallback(parent, XmNapplyCallback, Xg3d_default_axes, client_data);
XtFree(title);
XtFree(defaults);
}
/*******************************************/
void Xg3d_set_axes(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
  int
    i;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
if (graph->temp_axis.min >= graph->temp_axis.max)
  {
    Xg3d_error_sys(Xg3dBAD_MINMAX, graph);
    XtDestroyWidget(w);
    return;
  }
if ((int) wid_addr->client == Xg3dXaxis)
  {
    graph->xaxis_layout.min = graph->temp_axis.min;
    graph->xaxis_layout.max = graph->temp_axis.max;
    graph->xaxis_layout.increment = graph->temp_axis.increment;
    graph->xaxis_layout.nsig_fig = graph->temp_axis.nsig_fig;
    graph->xaxis_layout.user_set_flag = graph->temp_axis.user_set_flag;
  }
else
  {
    graph->yaxis_layout.min = graph->temp_axis.min;
    graph->yaxis_layout.max = graph->temp_axis.max;
    graph->yaxis_layout.increment = graph->temp_axis.increment;
    graph->yaxis_layout.nsig_fig = graph->temp_axis.nsig_fig;
    graph->yaxis_layout.user_set_flag = graph->temp_axis.user_set_flag;
  }
for (i=0; i<Xg3dNWID; ++i)
  if (graph->wid_array[i] != NULL)
    free(graph->wid_array[i]);
XtDestroyWidget(w);
/*Xg3d_createGraph(graph);*/
}
/**************************************/
void Xg3d_default_axes(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
  int
    i;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
if ((int) wid_addr->client == Xg3dXaxis)
  {
    graph->xaxis_layout.min = 0;
    graph->xaxis_layout.max = graph->dataset[0].nx;
    graph->xaxis_layout.increment = Xg3dDefInc;
    graph->xaxis_layout.nsig_fig = Xg3dDefSigFig;
    graph->xaxis_layout.user_set_flag = 0;
  }
else
  {
    graph->yaxis_layout.min = 0;
    graph->yaxis_layout.max = graph->dataset[0].ny;
    graph->yaxis_layout.increment = Xg3dDefInc;
    graph->yaxis_layout.nsig_fig = Xg3dDefSigFig;
    graph->yaxis_layout.user_set_flag = 0;
  }
for (i=0; i<Xg3dNWID; ++i)
  if (graph->wid_array[i] != NULL)
    free(graph->wid_array[i]);
XtDestroyWidget(w);
/*Xg3d_createGraph(graph);*/
}
/**************************************/
void Xg3d_minmax(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    *buf;
  int
    i;
  double
    value;
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
buf = XmTextGetString(w);
for (i=0; i<strlen(buf); ++i)
  if (!(isdigit(buf[i]) || isspace(buf[i]) || (buf[i] == '-')))
      return;
sscanf(buf, "%le", &value);
if ((int) wid_addr->client == Xg3dMin)
  {
    graph->temp_axis.min = value;
    graph->temp_axis.user_set_flag = graph->temp_axis.user_set_flag | Xg3dUmin;
  }
else
  {
    graph->temp_axis.max = value;
    graph->temp_axis.user_set_flag = graph->temp_axis.user_set_flag | Xg3dUmax;
  }
}
/******************* 3-d skew factors **********************/
void Xg3d_skew_factor(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XmString
    defaults,
    frame_label,
    title;
  Widget
    parent,
    form,
    label_frame,
    theta_frame,
    hoff_frame,
    scale_frame,
    label,
    text_wid,
    button;
  Arg
    args[10];
  int
    nargs;
  char
    buf[80];
  Xg3dGraphInfo
    *graph;
graph = (Xg3dGraphInfo *) client_data;
for (nargs=0; nargs<Xg3dNWID; ++nargs)
  graph->wid_array[nargs] = NULL;
frame_label = XmStringCreateLtoR("Graph Skew & Scaling Factors",Xg3dCharset);
graph->temp_theta = graph->theta;
graph->temp_hoff = graph->hoff;
graph->temp_scale = graph->scale_fac;

title = XmStringCreateLtoR("Xg3d: Skew Factors", Xg3dCharset);
defaults = XmStringCreateLtoR("Defaults", Xg3dCharset);
nargs = 0;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNautoUnmanage, False); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
XtSetArg(args[nargs], XmNapplyLabelString, defaults); ++nargs;
parent = XmCreatePromptDialog(w, "skew", args, nargs);
button = XmSelectionBoxGetChild(parent, XmDIALOG_TEXT);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(parent, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(parent, XmDIALOG_APPLY_BUTTON);
XtManageChild(button);

nargs = 0;
form = XmCreateForm(parent, "form", args, nargs);
XtManageChild(form);

/** Label Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
label_frame = XmCreateFrame(form, "label frame", args, nargs);
XtManageChild(label_frame);
nargs = 0;
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(label_frame, "text Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

/** Theta Text Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, label_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
theta_frame = XmCreateFrame(form, "theta frame", args, nargs);
XtManageChild(theta_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(theta_frame, "theta window", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("Skew Angle (degrees)", Xg3dCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "theta Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
text_wid = XmCreateText(button, "theta text", args, nargs);
sprintf(buf, "%lg", graph->temp_theta);
XmTextSetString(text_wid, buf);
XtManageChild(text_wid);
graph->wid_array[0] = Xg3dAllocStruct(Xg3dTheta, (caddr_t) graph);
XtAddCallback(text_wid, XmNvalueChangedCallback, Xg3d_temp_skew,
	      graph->wid_array[0]);

/** Hoff Text Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, theta_frame); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 10); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, label_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
hoff_frame = XmCreateFrame(form, "hoff frame", args, nargs);
XtManageChild(hoff_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(hoff_frame, "hoff window", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("Vertical Offset", Xg3dCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "hoff Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
text_wid = XmCreateText(button, "hoff text", args, nargs);
sprintf(buf, "%lg", graph->temp_hoff);
XmTextSetString(text_wid, buf);
XtManageChild(text_wid);
graph->wid_array[1] = Xg3dAllocStruct(Xg3dHoff, (caddr_t) graph);
XtAddCallback(text_wid, XmNvalueChangedCallback, Xg3d_temp_skew,
	      graph->wid_array[1]);

/** Scale Text Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
/*XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;*/
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, theta_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
scale_frame = XmCreateFrame(form, "scale frame", args, nargs);
XtManageChild(scale_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(scale_frame, "scale window", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("Z Scaling", Xg3dCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "scale Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
text_wid = XmCreateText(button, "scale text", args, nargs);
sprintf(buf, "%lg", graph->temp_scale);
XmTextSetString(text_wid, buf);
XtManageChild(text_wid);
graph->wid_array[2] = Xg3dAllocStruct(Xg3dScale_fac, (caddr_t) graph);
XtAddCallback(text_wid, XmNvalueChangedCallback, Xg3d_temp_skew,
	      graph->wid_array[2]);

XtManageChild(parent);
XtAddCallback(parent, XmNcancelCallback, Xg3d_cancel_CB, NULL);
XtAddCallback(parent, XmNcancelCallback, Xg3d_free_wid, graph);
XtAddCallback(parent, XmNhelpCallback, Xg3d_help_sys, (caddr_t) Xg3dSKEW);
XtAddCallback(parent, XmNokCallback, Xg3d_set_skew, client_data);
XtAddCallback(parent, XmNapplyCallback, Xg3d_default_skew, client_data);
XtFree(defaults);
XtFree(title);
}
/**************************************/
void Xg3d_temp_skew(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    *buf;
  int
    i;
  double
    value;
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
buf = XmTextGetString(w);
for (i=0; i<strlen(buf); ++i)
  if (!(isdigit(buf[i]) || isspace(buf[i])
	|| (buf[i] == '-') || (buf[i] == '.')))
      return;
sscanf(buf, "%le", &value);
if ((int) wid_addr->client == Xg3dTheta)
  {
    graph->temp_theta = value;
  }
else if ((int) wid_addr->client == Xg3dHoff)
  {
    graph->temp_hoff = value;
  }
else if ((int) wid_addr->client == Xg3dScale_fac)
  {
    graph->temp_scale = value;
  }
}
/**************************************/
void Xg3d_set_skew(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
  int
    i;
graph = (Xg3dGraphInfo *) client_data;
XtDestroyWidget(w);
if (graph->temp_hoff < 0.)
  {
    Xg3d_error_sys(Xg3dBAD_COORDS, graph);
    return;
  }
graph->theta = graph->temp_theta;
graph->hoff = graph->temp_hoff;
graph->scale_fac = graph->temp_scale;
for (i=0; i<Xg3dNWID; ++i)
  if (graph->wid_array[i] != NULL)
    free(graph->wid_array[i]);
/*Xg3d_drawGraph(graph);*/
}
/**************************************/
void Xg3d_default_skew(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
  int
    i;
graph = (Xg3dGraphInfo *) client_data;
XtDestroyWidget(w);
graph->theta = Xg3dDefTheta;
graph->hoff = Xg3dDefHoff;
graph->scale_fac = Xg3dDefScaleFac;
for (i=0; i<Xg3dNWID; ++i)
  if (graph->wid_array[i] != NULL)
    free(graph->wid_array[i]);
/*Xg3d_drawGraph(graph);*/
}
/************************ Select Colors **************************/
void Xg3d_do_colors(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    len,
    nargs;
  long int
    color,
    ncolors;
  Arg
    args[10];
  Widget
    color_sel,
    form,
    label,
    row_column,
    swindow,
    hsb,
    vsb,
    button;
  char
    name[80];
  XmString
    title,
    name_string,
    ok_label,
    label_string;
  Xg3dWidStruct
    *wid_addr;
  Xg3dGraphInfo
    *graph;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
graph = (Xg3dGraphInfo *) client_data;
nargs = 0;
title = XmStringCreateLtoR("Xg3d: Color Selector", Xg3dCharset);
ok_label = XmStringCreateLtoR("Default", Xg3dCharset);
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNautoUnmanage, False); ++nargs;
XtSetArg(args[nargs], XmNokLabelString, ok_label); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
color_sel = XmCreatePromptDialog(w, "color_sel", args, nargs);
button = XmSelectionBoxGetChild(color_sel, XmDIALOG_TEXT);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(color_sel, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(button);

nargs = 0;
form = XmCreateForm(color_sel, "form", args, nargs);
XtManageChild(form);

nargs = 0;
switch ((int) wid_addr->client)
  {
  case Xg3dText:
    strcpy(name, "Select Text Color");
    break;
  case Xg3dScale:
    strcpy(name, "Select Axis Color");
    break;
  case Xg3dBkgnd:
    strcpy(name, "Select Background Color");
    break;
  case Xg3dCursor:
    strcpy(name, "Select Cursor Color");
    break;
  case Xg3dDSelColor:
    strcpy(name, "Select a Color for Data");
    break;
  default:
    break;
  }
name_string = XmStringCreateLtoR(name, Xg3dCharset);
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNlabelString, name_string); ++nargs;
label = XmCreateLabel(form, "color label", args, nargs);
XtManageChild(label);
XtFree(name_string);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, label); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNscrollBarDisplayPolicy, XmAS_NEEDED); ++nargs;
XtSetArg(args[nargs], XmNscrollingPolicy, XmAUTOMATIC); ++nargs;
swindow = XmCreateScrolledWindow(form, "swindow", args, nargs);
XtManageChild(swindow);
nargs = 0;
XtSetArg(args[nargs], XmNpacking, XmPACK_COLUMN); ++nargs;
XtSetArg(args[nargs], XmNnumColumns, 6); ++nargs;
row_column = XmCreateRowColumn(swindow, "row_column", args, nargs);
XtManageChild(row_column);
nargs = 0;
XtSetArg(args[nargs], XmNhorizontalScrollBar, &hsb); ++nargs;
XtSetArg(args[nargs], XmNverticalScrollBar, &vsb); ++nargs;
/*XtGetValues(font_sel, args, nargs);
XmAddTabGroup(row_column);
if (hsb)
  XmAddTabGroup(hsb);
if (vsb)
  XmAddTabGroup(vsb);*/

ncolors = DisplayCells(XtDisplay(color_sel),
		       DefaultScreen(XtDisplay(color_sel)));
for (color=0; color<ncolors; ++color)
  {
    nargs = 0;
    XtSetArg(args[nargs], XmNbackground, color); ++nargs;
    button = XmCreatePushButton(row_column, " ", args, nargs);
    XtManageChild(button);
    XtAddCallback(button, XmNarmCallback, Xg3d_update_colors, client_data);
    XtAddCallback(button, XmNarmCallback, Xg3d_cancel_CB, color_sel);
  }
XtManageChild(color_sel);
XtAddCallback(color_sel, XmNhelpCallback, Xg3d_help_sys,
	      (caddr_t) Xg3dCOLOR_HELP);
XtAddCallback(color_sel, XmNokCallback, Xg3d_default_colors, client_data);
XtAddCallback(color_sel, XmNcancelCallback, Xg3d_cancel_CB, NULL);
XtFree(title);
XtFree(ok_label);
}
/**************************************/
void Xg3d_update_colors(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    nargs;
  long int
    color;
  Arg
    args[10];
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;;
nargs = 0;
XtSetArg(args[nargs], XmNbackground, &color); ++nargs;
XtGetValues(w, args, nargs);
XtDestroyWidget(w);
switch((int) wid_addr->client)
  {
  case Xg3dText:
    graph->text_color = color;
    break;
  case Xg3dScale:
    graph->axis_color = color;
    break;
  case Xg3dBkgnd:
    graph->background_color = color;
    break;
  case Xg3dCursor:
    graph->cursor_color = color;
    break;
  case Xg3dDSelColor:
    graph->dataset[graph->misc].datacolor = color;
    break;
  default:
    break;
  }
/*Xg3d_drawGraph(graph);*/
}
/**************************************/
void Xg3d_default_colors(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
  long int
    fore_color,
    back_color;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;;
XtDestroyWidget(w);
fore_color = XBlackPixelOfScreen(XtScreen(graph->app_shell));
back_color = XWhitePixelOfScreen(XtScreen(graph->app_shell));

switch((int) wid_addr->client)
  {
  case Xg3dText:
    graph->text_color = fore_color;
    break;
  case Xg3dScale:
    graph->axis_color = fore_color;
    break;
  case Xg3dBkgnd:
    graph->background_color = back_color;
    break;
  case Xg3dCursor:
    graph->cursor_color = fore_color;
    break;
  case Xg3dDSelColor:
    graph->dataset[graph->misc].datacolor = fore_color;
    break;
  default:
    break;
  }
/*Xg3d_drawGraph(graph);*/
}
/************************************************************
*
*                       Edit/Add Button
*
*************************************************************/
/********************** Add new data file *****************/
void Xg3d_start_add_file(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Widget
    subwidget,
    data_select,
    file_selector;
  Arg
    args[5];
  int
    i,
    nargs;
  XmString
    title,
    datasets_label,
    DataList[Xg3dNSETS];
  char
    buf[80];
  Xg3dGraphInfo
    *graph;
graph = (Xg3dGraphInfo *) client_data;
if (graph->num_sets >= Xg3dNSETS)
  {
    Xg3d_error_sys(Xg3dTOO_MANY_FILES, graph);
    return;
  }
if (graph->cursor_status == 1)  /* turn off cursor */
  Xg3d_delcurs(graph);
for (nargs=0; nargs<graph->num_sets; ++nargs)
  {
    sprintf(buf, "%d %s", nargs, graph->dataset[nargs].filename);
    DataList[nargs] = XmStringCreateLtoR(buf, Xg3dCharset);
  }
if (graph->num_sets > 5)
  i = 5;
else
  i = graph->num_sets;
datasets_label = XmStringCreateLtoR("Current Data Files:", Xg3dCharset);
title = XmStringCreateLtoR("Xg3d: Active Data Sets", Xg3dCharset);
nargs = 0;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNlistItems, DataList); ++nargs;
XtSetArg(args[nargs], XmNlistItemCount, graph->num_sets); ++nargs;
XtSetArg(args[nargs], XmNlistLabelString, datasets_label); ++nargs;
XtSetArg(args[nargs], XmNlistVisibleItemCount, i); ++nargs;
data_select = XmCreateSelectionDialog(graph->app_shell,
	      "Xg3d: Active Data Sets", args, nargs);
subwidget = XmSelectionBoxGetChild(data_select, XmDIALOG_TEXT);
XtUnmanageChild(subwidget);
subwidget = XmSelectionBoxGetChild(data_select, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(subwidget);
subwidget = XmSelectionBoxGetChild(data_select, XmDIALOG_CANCEL_BUTTON);
XtUnmanageChild(subwidget);
subwidget = XmSelectionBoxGetChild(data_select, XmDIALOG_HELP_BUTTON);
XtUnmanageChild(subwidget);
subwidget = XmSelectionBoxGetChild(data_select, XmDIALOG_OK_BUTTON);
XtUnmanageChild(subwidget);
XtManageChild(data_select);
XtFree(title);
XtFree(datasets_label);
for (nargs=0; nargs<graph->num_sets; ++nargs)
  XtFree(DataList[nargs]);

nargs = 0;
title = XmStringCreateLtoR("Xg3d: Add Data File", Xg3dCharset);
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNdefaultPosition, False); ++nargs;
XtSetArg(args[nargs], XmNx, 400); ++nargs;
XtSetArg(args[nargs], XmNy, 100); ++nargs;
file_selector = XmCreateFileSelectionDialog(graph->app_shell,
					   "Xg3d: Add Data File", args, nargs);
XtAddCallback(file_selector, XmNhelpCallback, Xg3d_help_sys,
	      (caddr_t) Xg3dADD_FILE);
XtAddCallback(file_selector, XmNokCallback, Xg3d_cancel_CB, data_select);
XtAddCallback(file_selector, XmNokCallback, Xg3d_add_file, graph);
XtAddCallback(file_selector, XmNcancelCallback, Xg3d_cancel_CB, data_select);
XtManageChild(file_selector);
XtFree(title);
}
/***************************/
void Xg3d_add_file(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XmFileSelectionBoxCallbackStruct
    *dialog_rslt;
  char
    comment[80],
    *buf;
  int
    i,
    n,
    frame_num;
  int2
    *pix_array;
  struct fileinfo
    datafile;
  Xg3dGraphInfo
    *graph;
graph = (Xg3dGraphInfo *) client_data;
--graph->num_sets;
dialog_rslt = (XmFileSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(dialog_rslt->value, Xg3dCharset, &buf);
strcpy(datafile.filename, buf);
fileopen(&datafile);
/*showhead(&datafile);*/
do
  {
    printf("There are %d frames in this file.\n",
	   datafile.file_header.frames_good);
    printf("Enter frame number (base 1): ");
    scanf("%d", &frame_num);
  }
while ((frame_num < 1) || (frame_num > datafile.file_header.frames_good));
free(graph->dataset[graph->num_sets].darray);
graph->dataset[graph->num_sets].frame_num = frame_num;
pix_array = readframe(&datafile, frame_num);
if (pix_array == NULL)
  {
    fprintf(stderr, "No data returned! Aborting\n");
    exit(0);
  }
/**********************************/
graph->dataset[graph->num_sets].darray = pix_array;
strcpy(graph->dataset[graph->num_sets].filename, datafile.filename);
graph->dataset[graph->num_sets].nx = datafile.file_header.row_length;
graph->dataset[graph->num_sets].ny = datafile.file_header.col_length;
graph->xaxis_layout.min = 0;
graph->xaxis_layout.max = datafile.file_header.row_length;
graph->yaxis_layout.min = 0;
graph->yaxis_layout.max = datafile.file_header.col_length;
++graph->num_sets;
fileclose(datafile.dat_file);
XtFree(buf);
/*Xg3d_createGraph(graph);*/
}
/*********************** Select Data Set for Delete/Color/Points ************/
void Xg3d_select_data_set(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Widget
    subwidget,
    data_select;
  Arg
    args[10];
  int
    i,
    nargs;
  XmString
    title,
    datasets_label,
    DataList[Xg3dNSETS];
  char
    buf[80];
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
if ((graph->num_sets < 2) && ((int) wid_addr->client == Xg3dDSelDelete))
  {
    Xg3d_error_sys(Xg3dNO_DELETE_LAST, graph);
    return;
  }
if (graph->cursor_status == 1)  /* turn off cursor */
  Xg3d_delcurs(graph);
for (nargs=0; nargs<graph->num_sets; ++nargs)
  {
    sprintf(buf, "%d %s", nargs, graph->dataset[nargs].filename);
    DataList[nargs] = XmStringCreateLtoR(buf, Xg3dCharset);
  }
if (graph->num_sets > 5)
  i = 5;
else
  i = graph->num_sets;
datasets_label = XmStringCreateLtoR("Available Data Files:", Xg3dCharset);
title = XmStringCreateLtoR("Xg3d: Active Data Sets", Xg3dCharset);
nargs = 0;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNlistItems, DataList); ++nargs;
XtSetArg(args[nargs], XmNlistItemCount, graph->num_sets); ++nargs;
XtSetArg(args[nargs], XmNlistLabelString, datasets_label); ++nargs;
XtSetArg(args[nargs], XmNlistVisibleItemCount, i); ++nargs;
XtSetArg(args[nargs], XmNtextString, DataList[0]); ++nargs;
XtSetArg(args[nargs], XmNautoUnmanage, False); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
data_select = XmCreateSelectionDialog(graph->app_shell,
              "Xg3d: Active Data Sets", args, nargs);
subwidget = XmSelectionBoxGetChild(data_select, XmDIALOG_TEXT);
XtUnmanageChild(subwidget);
subwidget = XmSelectionBoxGetChild(data_select, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(subwidget);
XtManageChild(data_select);
XtAddCallback(data_select, XmNhelpCallback, Xg3d_help_sys,
	      (caddr_t) Xg3dDATA_SELECT);
XtAddCallback(data_select, XmNokCallback, Xg3d_dataset_action, client_data);
XtAddCallback(data_select, XmNcancelCallback, Xg3d_cancel_CB, NULL);
XtFree(datasets_label);
XtFree(title);
for (nargs=0; nargs<graph->num_sets; ++nargs)
  XtFree(DataList[nargs]);
}
/***************************/
void Xg3d_dataset_action(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XmSelectionBoxCallbackStruct
    *select_rslt;
  char
    buf[80],
    *filename;
  int
    i,
    dataset,
    nargs;
  Arg
    args[10];
  XmString
    defaults,
    dataname,
    radio_label,
    title;
  Widget
    parent,
    form,
    label,
    data_frame,
    point_frame,
    line_frame,
    size_frame,
    scale_frame,
    radio,
    button;
  Xg3dGraphInfo
    *graph;
  Xg3dWidStruct
    *wid_addr;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
for (i=0; i<Xg3dNWID; ++i)
  graph->wid_array[i] = NULL;
select_rslt = (XmSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(select_rslt->value, Xg3dCharset, &filename);
sscanf(filename, "%d", &dataset);
switch ((int) wid_addr->client)
  {
  case Xg3dDSelDelete:
    --graph->num_sets;
    XtDestroyWidget(w);
    graph->dataset[dataset].darray
      = graph->dataset[graph->num_sets].darray;
    strcpy(graph->dataset[dataset].filename,
	   graph->dataset[graph->num_sets].filename);
    graph->dataset[dataset].num_points
      = graph->dataset[graph->num_sets].num_points;
    graph->dataset[dataset].point_type
      = graph->dataset[graph->num_sets].point_type;
    graph->dataset[dataset].line_status
      = graph->dataset[graph->num_sets].line_status;
    graph->dataset[dataset].datacolor
      = graph->dataset[graph->num_sets].datacolor;
/*    Xg3d_createGraph(graph);*/
    break;
  case Xg3dDSelPoint:  /***********************/
    graph->misc = dataset;
    nargs = 0;
    title = XmStringCreateLtoR("Xg3d: Dataset", Xg3dCharset);
    defaults = XmStringCreateLtoR("Defaults", Xg3dCharset);
    XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
    XtSetArg(args[nargs], XmNautoUnmanage, False); ++nargs;
    XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
    XtSetArg(args[nargs], XmNcancelLabelString, defaults); ++nargs;
    parent = XmCreatePromptDialog(graph->app_shell, "pt_line select",
				  args, nargs);
    button = XmSelectionBoxGetChild(parent, XmDIALOG_TEXT);
    XtUnmanageChild(button);
    button = XmSelectionBoxGetChild(parent, XmDIALOG_SELECTION_LABEL);
    XtUnmanageChild(button);

    nargs = 0;
    form = XmCreateForm(parent, "form", args, nargs);
    XtManageChild(form);

    /** Dataset name frame **/
    nargs = 0;
    XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
    XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
    XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
    XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
    data_frame = XmCreateFrame(form, "data name frame", args, nargs);
    XtManageChild(data_frame);
    nargs = 0;
    sprintf(buf, "dataset: %s", filename);
    dataname = XmStringCreateLtoR(buf, Xg3dCharset);
    XtSetArg(args[nargs], XmNlabelString, dataname); ++nargs;
    XtSetArg(args[nargs], XmNalignment, XmALIGNMENT_BEGINNING); ++nargs;
    XtSetArg(args[nargs], XmNmarginLeft, 15); ++nargs;
    label = XmCreateLabel(data_frame, "data label", args, nargs);
    XtManageChild(label);
    XtFree(dataname);

    /** Point Style Selector Radio Box **/
    nargs = 0;
    XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
    XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
    XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
    XtSetArg(args[nargs], XmNtopWidget, data_frame); ++nargs;
    XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
    point_frame = XmCreateFrame(form, "point frame", args, nargs);
    XtManageChild(point_frame);

    nargs = 0;
    XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
    XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
    button = XmCreatePanedWindow(point_frame, "point window", args, nargs);
    XtManageChild(button);
    nargs = 0;
    dataname = XmStringCreateLtoR("Point Type", Xg3dCharset);
    XtSetArg(args[nargs], XmNlabelString, dataname); ++nargs;
    label = XmCreateLabel(button, "point label", args, nargs);
    XtManageChild(label);
    XtFree(dataname);

    nargs = 0;
    XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
    radio = XmCreateRadioBox(button, "Point Type", args, nargs);
    XtManageChild(radio);
    XmAddTabGroup(radio);

      nargs = 0;
      if (graph->dataset[dataset].point_type == Xg3dPoint)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Point", args, nargs);
      graph->wid_array[0] = Xg3dAllocStruct(Xg3dPoint, (caddr_t) graph);
      XtAddCallback(button, XmNarmCallback,Xg3d_data_toggle,graph->wid_array[0]);
      XtManageChild(button);
      nargs = 0;
      if (graph->dataset[dataset].point_type == Xg3dCircle)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Circle", args, nargs);
      graph->wid_array[1] = Xg3dAllocStruct(Xg3dCircle, (caddr_t) graph);
      XtAddCallback(button, XmNarmCallback,Xg3d_data_toggle,graph->wid_array[1]);
      XtManageChild(button);
      nargs = 0;
      if (graph->dataset[dataset].point_type == Xg3dSquare)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Square    ", args, nargs);
      graph->wid_array[2] = Xg3dAllocStruct(Xg3dSquare, (caddr_t) graph);
      XtAddCallback(button, XmNarmCallback,Xg3d_data_toggle,graph->wid_array[2]);
      XtManageChild(button);
      nargs = 0;
      if (graph->dataset[dataset].point_type == Xg3dBar)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Blank", args, nargs);
      graph->wid_array[3] = Xg3dAllocStruct(Xg3dBar, (caddr_t) graph);
      XtAddCallback(button, XmNarmCallback,Xg3d_data_toggle,graph->wid_array[3]);
      XtManageChild(button);
      nargs = 0;
      if (graph->dataset[dataset].point_type == Xg3dTriangle)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Triangle", args, nargs);
      graph->wid_array[6] = Xg3dAllocStruct(Xg3dTriangle, (caddr_t) graph);
      XtAddCallback(button, XmNarmCallback,Xg3d_data_toggle,graph->wid_array[6]);
      XtManageChild(button);

    /** Line Selector Radio Box **/
    nargs = 0;
    XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
    XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
    XtSetArg(args[nargs], XmNleftWidget, point_frame); ++nargs;
    XtSetArg(args[nargs], XmNleftOffset, 10); ++nargs;
    XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
    XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
    XtSetArg(args[nargs], XmNtopWidget, data_frame); ++nargs;
    XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
    line_frame = XmCreateFrame(form, "line frame", args, nargs);
    XtManageChild(line_frame);

    nargs = 0;
    XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
    XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
    button = XmCreatePanedWindow(line_frame, "line window", args, nargs);
    XtManageChild(button);
    nargs = 0;
    dataname = XmStringCreateLtoR("Line Type", Xg3dCharset);
    XtSetArg(args[nargs], XmNlabelString, dataname); ++nargs;
    label = XmCreateLabel(button, "line label", args, nargs);
    XtManageChild(label);
    XtFree(dataname);

    nargs = 0;
    XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
    radio = XmCreateRadioBox(button, "Line Type", args, nargs);
    XtManageChild(radio);
    XmAddTabGroup(radio);

      nargs = 0;
      if (graph->dataset[dataset].line_status == Xg3dLine)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Horizontal", args, nargs);
      graph->wid_array[4] = Xg3dAllocStruct(Xg3dLine, (caddr_t) graph);
      XtAddCallback(button, XmNarmCallback,Xg3d_data_toggle,graph->wid_array[4]);
      XtManageChild(button);
      nargs = 0;
      if (graph->dataset[dataset].line_status == Xg3dYLine)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Vertical Line", args, nargs);
      graph->wid_array[10] = Xg3dAllocStruct(Xg3dYLine, (caddr_t) graph);
      XtAddCallback(button,XmNarmCallback,Xg3d_data_toggle,graph->wid_array[10]);
      XtManageChild(button);
      nargs = 0;
      if (graph->dataset[dataset].line_status == Xg3dMesh)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Mesh", args, nargs);
      graph->wid_array[11] = Xg3dAllocStruct(Xg3dMesh, (caddr_t) graph);
      XtAddCallback(button,XmNarmCallback,Xg3d_data_toggle,graph->wid_array[11]);
      XtManageChild(button);
      nargs = 0;
      if (graph->dataset[dataset].line_status == Xg3dNoLine)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Line Off", args, nargs);
      graph->wid_array[5] = Xg3dAllocStruct(Xg3dNoLine, (caddr_t) graph);
      XtAddCallback(button, XmNarmCallback,Xg3d_data_toggle,graph->wid_array[5]);
      XtManageChild(button);

    /** Point Size Selector Radio Box **/
    nargs = 0;
    XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
    XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
    XtSetArg(args[nargs], XmNleftWidget, point_frame); ++nargs;
    XtSetArg(args[nargs], XmNleftOffset, 10); ++nargs;
    XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
    XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
    XtSetArg(args[nargs], XmNtopWidget, line_frame); ++nargs;
    XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
    size_frame = XmCreateFrame(form, "size frame", args, nargs);
    XtManageChild(size_frame);

    nargs = 0;
    XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
    XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
    button = XmCreatePanedWindow(size_frame, "size window", args, nargs);
    XtManageChild(button);
    nargs = 0;
    dataname = XmStringCreateLtoR("Point Size", Xg3dCharset);
    XtSetArg(args[nargs], XmNlabelString, dataname); ++nargs;
    label = XmCreateLabel(button, "size label", args, nargs);
    XtManageChild(label);
    XtFree(dataname);

    nargs = 0;
    XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
    radio = XmCreateRadioBox(button, "Size Type", args, nargs);
    XtManageChild(radio);
    XmAddTabGroup(radio);

      nargs = 0;
      if (graph->dataset[dataset].point_size == Xg3dSize3)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "3 Pixels", args, nargs);
      graph->wid_array[7] = Xg3dAllocStruct(Xg3dSize3, (caddr_t) graph);
      XtAddCallback(button, XmNarmCallback,Xg3d_data_toggle,graph->wid_array[7]);
      XtManageChild(button);
      nargs = 0;
      if (graph->dataset[dataset].point_size == Xg3dSize4)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "4 Pixels", args, nargs);
      graph->wid_array[8] = Xg3dAllocStruct(Xg3dSize4, (caddr_t) graph);
      XtAddCallback(button, XmNarmCallback,Xg3d_data_toggle,graph->wid_array[8]);
      XtManageChild(button);
      nargs = 0;
      if (graph->dataset[dataset].point_size == Xg3dSize5)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "5 Pixels", args, nargs);
      graph->wid_array[9] = Xg3dAllocStruct(Xg3dSize5, (caddr_t) graph);
      XtAddCallback(button, XmNarmCallback,Xg3d_data_toggle,graph->wid_array[9]);
      XtManageChild(button);

    /** Scale Function Selector Radio Box **/
    nargs = 0;
    XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
    XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
    XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
    XtSetArg(args[nargs], XmNtopWidget, point_frame); ++nargs;
    XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
    scale_frame = XmCreateFrame(form, "scale frame", args, nargs);
    XtManageChild(scale_frame);

    nargs = 0;
    XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
    XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
    button = XmCreatePanedWindow(scale_frame, "scale window", args, nargs);
    XtManageChild(button);
    nargs = 0;
    dataname = XmStringCreateLtoR("Z Scale Func", Xg3dCharset);
    XtSetArg(args[nargs], XmNlabelString, dataname); ++nargs;
    label = XmCreateLabel(button, "scale label", args, nargs);
    XtManageChild(label);
    XtFree(dataname);

    nargs = 0;
    XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
    radio = XmCreateRadioBox(button, "scale Type", args, nargs);
    XtManageChild(radio);
    XmAddTabGroup(radio);

      nargs = 0;
      if (graph->dataset[dataset].s_func == Xg3dLinear)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Linear", args, nargs);
      graph->wid_array[12] = Xg3dAllocStruct(Xg3dLinear, (caddr_t) graph);
      XtAddCallback(button, XmNarmCallback,Xg3d_data_toggle,graph->wid_array[12]);
      XtManageChild(button);
      nargs = 0;
      if (graph->dataset[dataset].s_func == Xg3dLog)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Logarithmic", args, nargs);
      graph->wid_array[13] = Xg3dAllocStruct(Xg3dLog, (caddr_t) graph);
      XtAddCallback(button, XmNarmCallback, Xg3d_data_toggle,
		    graph->wid_array[13]);
      XtManageChild(button);

    XtManageChild(parent);
    XtAddCallback(parent, XmNhelpCallback, Xg3d_help_sys,
		  (caddr_t) Xg3dDATA_OPTIONS);
    XtAddCallback(parent, XmNokCallback, Xg3d_set_data_options, graph);
    XtAddCallback(parent, XmNokCallback, Xg3d_cancel_CB, w);
    XtAddCallback(parent, XmNcancelCallback, Xg3d_set_data_defaults, graph);
    XtAddCallback(parent, XmNcancelCallback, Xg3d_cancel_CB, w);
    XtFree(title);
    XtFree(defaults);

    break;
  case Xg3dDSelColor:
    graph->misc = dataset;
    XtUnmanageChild(w);
    Xg3d_do_colors(graph->app_shell, client_data, NULL);
    break;
  default:
    break;
  }
XtFree(filename);
}
/**************************************/
void Xg3d_data_toggle(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    dataset;
  Xg3dWidStruct
    *wid_addr;
  Xg3dGraphInfo
    *graph;
wid_addr = (Xg3dWidStruct *) client_data;
graph = (Xg3dGraphInfo *) wid_addr->graph;
dataset = graph->misc;
switch((int) wid_addr->client)
  {
  case Xg3dPoint:
    graph->dataset[dataset].point_type = Xg3dPoint;
    break;
  case Xg3dCircle:
    graph->dataset[dataset].point_type = Xg3dCircle;
    break;
  case Xg3dSquare:
    graph->dataset[dataset].point_type = Xg3dSquare;
    break;
  case Xg3dBar:
    graph->dataset[dataset].point_type = Xg3dBar;
    break;
  case Xg3dTriangle:
    graph->dataset[dataset].point_type = Xg3dTriangle;
    break;
  case Xg3dLine:
    graph->dataset[dataset].line_status = Xg3dLine;
    break;
  case Xg3dYLine:
    graph->dataset[dataset].line_status = Xg3dYLine;
    break;
  case Xg3dMesh:
    graph->dataset[dataset].line_status = Xg3dMesh;
    break;
  case Xg3dNoLine:
    graph->dataset[dataset].line_status = Xg3dNoLine;
    break;
  case Xg3dSize3:
    graph->dataset[dataset].point_size = Xg3dSize3;
    break;
  case Xg3dSize4:
    graph->dataset[dataset].point_size = Xg3dSize4;
    break;
  case Xg3dSize5:
    graph->dataset[dataset].point_size = Xg3dSize5;
    break;
  case Xg3dLinear:
    graph->dataset[dataset].s_func = Xg3dLinear;
    break;
  case Xg3dLog:
    graph->dataset[dataset].s_func = Xg3dLog;
    break;
  default:
    break;
  }
}
/**************************************/
void Xg3d_set_data_options(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
  int
    i;
graph = (Xg3dGraphInfo *) client_data;
for (i=0; i<Xg3dNWID; ++i)
  if (graph->wid_array[i] != NULL)
    free(graph->wid_array[i]);
/*Xg3d_drawGraph(graph);*/
}
/**************************************/
void Xg3d_set_data_defaults(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
  int
    i;
graph = (Xg3dGraphInfo *) client_data;
graph->dataset[graph->misc].point_type = Xg3dDefPointType;
graph->dataset[graph->misc].line_status = Xg3dDefLineType;
graph->dataset[graph->misc].point_size = Xg3dDefPtSize;
graph->dataset[graph->misc].s_func = Xg3dDefFunc;
/*graph->dataset[graph->misc].datacolor = */
for (i=0; i<Xg3dNWID; ++i)
  if (graph->wid_array[i] != NULL)
    free(graph->wid_array[i]);
/*Xg3d_drawGraph(graph);*/
}
/***************************************************************
*
*                        Options Button
*
****************************************************************/
void Xg3d_CursorToggle(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
graph = (Xg3dGraphInfo *) client_data;
if (graph->cursor_status == 0)  /* cursor is off, turn it on */
  {
    Xg3d_initcurs(graph);
  }
else  /* cursor is on, turn it off */
  {
    Xg3d_delcurs(graph);
  }
}
/************************************/
void Xg3dDummy(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  Xg3dGraphInfo
    *graph;
graph = (Xg3dGraphInfo *) client_data;
Xg3d_drawGraph(graph);
}
/************************************/
