/**********************************************************************
*
*                            XgAxis.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*    axis layout descriptions
*
***********************************************************************
*/

#include "stdio.h"
#include "string.h"
#ifndef Xg_NO_CTYPE
#include "ctype.h"
#endif

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "Xm/Form.h"
#include "Xm/Frame.h"
#include "Xm/SelectioB.h"
#include "Xm/SeparatoG.h"
#include "Xm/RowColumn.h"
#include "Xm/ToggleB.h"
#include "Xm/ToggleBG.h"
#include "Xm/PanedW.h"
#include "Xm/Text.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"

/*******************************************************************/
void Xg_configure_axes(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    nargs;
  char
    buf[80];
  Arg
    args[10];
  Widget
    sigDigit_frame,
    lastSig_frame,
    tic_frame,
    min_frame,
    max_frame,
    sci_frame,
    scl_frame,
    radio,
    label,
    text_wid,
    parent,
    separ,
    button,
    form;
  XmString
    frame_label,
    title,
    defaults;

  XgWidStruct
    *wid_addr;
  XgGraphInfo
    *graph;

wid_addr = (XgWidStruct *) client_data;
graph = (XgGraphInfo *) wid_addr->graph;

switch ((int) wid_addr->client)
  {
  case XgXaxis:
    graph->temp_axis.min = graph->xaxis_layout.min;
    graph->temp_axis.max = graph->xaxis_layout.max;
    graph->temp_axis.increment = graph->xaxis_layout.increment;
    graph->temp_axis.nsig_fig = graph->xaxis_layout.nsig_fig;
    graph->temp_axis.div_marks = graph->xaxis_layout.div_marks;
    graph->temp_axis.scaling = graph->xaxis_layout.scaling;
    graph->temp_axis.label_type = graph->xaxis_layout.label_type;
    graph->temp_axis.user_set_flag = graph->xaxis_layout.user_set_flag;
    title = XmStringCreateLtoR("Xg: X Axis Layout", XgCharset);
    break;
  case XgYLaxis:
    graph->temp_axis.min = graph->yaxis_layout.min;
    graph->temp_axis.max = graph->yaxis_layout.max;
    graph->temp_axis.increment = graph->yaxis_layout.increment;
    graph->temp_axis.nsig_fig = graph->yaxis_layout.nsig_fig;
    graph->temp_axis.div_marks = graph->yaxis_layout.div_marks;
    graph->temp_axis.scaling = graph->yaxis_layout.scaling;
    graph->temp_axis.label_type = graph->yaxis_layout.label_type;
    graph->temp_axis.user_set_flag = graph->yaxis_layout.user_set_flag;
    title = XmStringCreateLtoR("Xg: Y Axis Layout", XgCharset);
    break;
  case XgYRaxis:
    graph->temp_axis.min = graph->y_rt_axis_layout.min;
    graph->temp_axis.max = graph->y_rt_axis_layout.max;
    graph->temp_axis.increment = graph->y_rt_axis_layout.increment;
    graph->temp_axis.nsig_fig = graph->y_rt_axis_layout.nsig_fig;
    graph->temp_axis.div_marks = graph->y_rt_axis_layout.div_marks;
    graph->temp_axis.scaling = graph->y_rt_axis_layout.scaling;
    graph->temp_axis.label_type = graph->y_rt_axis_layout.label_type;
    graph->temp_axis.user_set_flag = graph->y_rt_axis_layout.user_set_flag;
    title = XmStringCreateLtoR("Xg: Y (Right) Axis Layout", XgCharset);
    break;
  default:
    break;
  }

/********** main popup dialog shell ************/
nargs = 0;
defaults = XmStringCreateLtoR("Defaults", XgCharset);
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNapplyLabelString, defaults); ++nargs;
XtSetArg(args[nargs], XmNautoUnmanage, False); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
parent = XmCreatePromptDialog(graph->app_shell, "layouts", args, nargs);
button = XmSelectionBoxGetChild(parent, XmDIALOG_TEXT);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(parent, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(parent, XmDIALOG_APPLY_BUTTON);
XtManageChild(button);

nargs = 0;
form = XmCreateForm(parent, "form", args, nargs);
XtManageChild(form);

/***** Number of Significant Digits Radio Box *****/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
/*XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNrightWidget, separ); ++nargs;
XtSetArg(args[nargs], XmNrightOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
sigDigit_frame = XmCreateFrame(form, "sigDigit frame", args, nargs);
XtManageChild(sigDigit_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(sigDigit_frame, "num sig digits", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("Significant Digits", XgCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "SigDigits Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
radio = XmCreateRadioBox(button, "sigDigits", args, nargs);
XtManageChild(radio);
/*XmAddTabGroup(radio);*/

  nargs = 0;
  if (graph->temp_axis.nsig_fig == XgSig1)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "1", args, nargs);
  graph->wid_array[0].client = (caddr_t) XgSig1;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[0]);
  XtManageChild(button);

  nargs = 0;
  if (graph->temp_axis.nsig_fig == XgSig2)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "2", args, nargs);
  graph->wid_array[1].client = (caddr_t) XgSig2;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[1]);
  XtManageChild(button);

  nargs = 0;
  if (graph->temp_axis.nsig_fig == XgSig3)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "3", args, nargs);
  graph->wid_array[2].client = (caddr_t) XgSig3;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[2]);
  XtManageChild(button);

/***** Last Significant Digit Radio Box *****/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
/*XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNrightWidget, separ); ++nargs;
XtSetArg(args[nargs], XmNrightOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, sigDigit_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
lastSig_frame = XmCreateFrame(form, "lastSig frame", args, nargs);
XtManageChild(lastSig_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(lastSig_frame, "last sig digits", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("Last Sig. Digit", XgCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "LastSigDigit Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
radio = XmCreateRadioBox(button, "lastSig", args, nargs);
XtManageChild(radio);
/*XmAddTabGroup(radio);*/

  nargs = 0;
  if (graph->temp_axis.increment == XgLast1)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "1", args, nargs);
  graph->wid_array[3].client = (caddr_t) XgLast1;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[3]);
  XtManageChild(button);

  nargs = 0;
  if (graph->temp_axis.increment == XgLast2)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "2", args, nargs);
  graph->wid_array[4].client = (caddr_t) XgLast2;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[4]);
  XtManageChild(button);

  nargs = 0;
  if (graph->temp_axis.increment == XgLast5)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "5", args, nargs);
  graph->wid_array[5].client = (caddr_t) XgLast5;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[5]);
  XtManageChild(button);

/******** Scaling Rule Toggle Buttons ******/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, lastSig_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
/*XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNrightWidget, separ); ++nargs;
XtSetArg(args[nargs], XmNrightOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
scl_frame = XmCreateFrame(form, "scaling frame", args, nargs);
XtManageChild(scl_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(scl_frame, "scaling rules", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("Scaling Rules", XgCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "Scaling Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
radio = XmCreateRadioBox(button, "scale radio", args, nargs);
XtManageChild(radio);

  nargs = 0;
  if (graph->temp_axis.scaling == XgLinear)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "Linear", args, nargs);
  graph->wid_array[6].client = (caddr_t) XgLinear;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[6]);
  XtManageChild(button);

  nargs = 0;
  if (graph->temp_axis.scaling == XgLogarithm)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "Logarithmic", args, nargs);
  graph->wid_array[7].client = (caddr_t) XgLogarithm;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[7]);
  XtManageChild(button);

  if ( ((int) wid_addr->client == XgYLaxis)
      || ((int) wid_addr->client == XgYRaxis) )
    {
      nargs = 0;
      if (graph->temp_axis.scaling == XgProbability)
	{ XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Probability", args, nargs);
      graph->wid_array[8].client = (caddr_t) XgProbability;
      XtAddCallback(button, XmNarmCallback, Xg_toggle_axes,
		    &graph->wid_array[8]);
      XtManageChild(button);
  }

/************ Separator **********/
nargs = 0;
XtSetArg(args[nargs], XmNorientation, XmVERTICAL); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNbottomAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, sigDigit_frame); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNseparatorType, XmNO_LINE); ++nargs;
/*XtSetArg(args[nargs], XmNseparatorType, XmDOUBLE_DASHED_LINE); ++nargs;*/
separ = XmCreateSeparatorGadget(form, "separator", args, nargs);
XtManageChild(separ);

nargs = 0;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNrightWidget, separ); ++nargs;
XtSetArg(args[nargs], XmNrightOffset, 5); ++nargs;
XtSetValues(lastSig_frame, args, nargs);

nargs = 0;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNrightWidget, separ); ++nargs;
XtSetArg(args[nargs], XmNrightOffset, 5); ++nargs;
XtSetValues(scl_frame, args, nargs);

/** Minimum Value Text Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, separ); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 5); ++nargs;
min_frame = XmCreateFrame(form, "min frame", args, nargs);
XtManageChild(min_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(min_frame, "min window", args, nargs);
XtManageChild(button);

nargs = 0;
frame_label = XmStringCreateLtoR("Minimum Value", XgCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "min Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
text_wid = XmCreateText(button, "min text", args, nargs);
sprintf(buf, "%lg", graph->temp_axis.min);
XmTextSetString(text_wid, buf);
XtManageChild(text_wid);
graph->wid_array[9].client = (caddr_t) XgMin;
XtAddCallback(text_wid, XmNvalueChangedCallback, Xg_minmax,
	      &graph->wid_array[9]);

/** Maximum Value Text Box **/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, separ); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, min_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
max_frame = XmCreateFrame(form, "max frame", args, nargs);
XtManageChild(max_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(max_frame, "max window", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("Maximum Value", XgCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "max Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
text_wid = XmCreateText(button, "max text", args, nargs);
sprintf(buf, "%lg", graph->temp_axis.max);
XmTextSetString(text_wid, buf);
XtManageChild(text_wid);
graph->wid_array[10].client = (caddr_t) XgMax;
XtAddCallback(text_wid, XmNvalueChangedCallback, Xg_minmax,
	      &graph->wid_array[10]);

/******** Scientific Notation Toggle Button ******/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, max_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, separ); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
sci_frame = XmCreateFrame(form, "sci. not. frame", args, nargs);
XtManageChild(sci_frame);

nargs = 0;
XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
radio = XmCreateRadioBox(sci_frame, "radio box", args, nargs);
XtManageChild(radio);

  nargs = 0;
  if (graph->temp_axis.label_type == XgNo_label)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "Sci/Eng Notation Off",
				      args, nargs);
  graph->wid_array[11].client = (caddr_t) XgNo_label;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[11]);
  XtManageChild(button);

  nargs = 0;
  if (graph->temp_axis.label_type == XgSci_label)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "Scientific Notation",args,nargs);
  graph->wid_array[12].client = (caddr_t) XgSci_label;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[12]);
  XtManageChild(button);

  nargs = 0;
  if (graph->temp_axis.label_type == XgEng_label)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "Engineering Notation",
				      args, nargs);
  graph->wid_array[13].client = (caddr_t) XgEng_label;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[13]);
  XtManageChild(button);

/******** Tick Mark Toggle Button ******/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, sci_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, separ); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
tic_frame = XmCreateFrame(form, "tick line frame", args, nargs);
XtManageChild(tic_frame);

nargs = 0;
XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
button = XmCreatePanedWindow(tic_frame, "tic line window", args, nargs);
XtManageChild(button);
nargs = 0;
frame_label = XmStringCreateLtoR("Internal Div Marks", XgCharset);
XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
label = XmCreateLabel(button, "Internal Marks Label", args, nargs);
XtManageChild(label);
XtFree(frame_label);

nargs = 0;
XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
radio = XmCreateRadioBox(button, "tick radio box", args, nargs);
XtManageChild(radio);

  nargs = 0;
  if (graph->temp_axis.div_marks == XgDivNone)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "No Div. Marks", args, nargs);
  graph->wid_array[14].client = (caddr_t) XgDivNone;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[14]);
  XtManageChild(button);

  nargs = 0;
  if (graph->temp_axis.div_marks == XgDivMajorTic)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio, "Major Div Tics", args, nargs);
  graph->wid_array[15].client = (caddr_t) XgDivMajorTic;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[15]);
  XtManageChild(button);

  nargs = 0;
  if (graph->temp_axis.div_marks == XgDivMinorTic)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio,"Minor Div Tics",args,nargs);
  graph->wid_array[16].client = (caddr_t) XgDivMinorTic;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[16]);
  XtManageChild(button);

  nargs = 0;
  if (graph->temp_axis.div_marks == XgDivMajorLin)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio,"Major Div Lines",args,nargs);
  graph->wid_array[17].client = (caddr_t) XgDivMajorLin;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[17]);
  XtManageChild(button);

  nargs = 0;
  if (graph->temp_axis.div_marks == XgDivLin_Tic)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio,"Lines & Tics",args,nargs);
  graph->wid_array[18].client = (caddr_t) XgDivLin_Tic;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[18]);
  XtManageChild(button);

  nargs = 0;
  if (graph->temp_axis.div_marks == XgDivMinorLin)
    { XtSetArg(args[nargs], XmNset, True); ++nargs; }
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  button = XmCreateToggleButtonGadget(radio,"Minor Div Lines",args,nargs);
  graph->wid_array[19].client = (caddr_t) XgDivMinorLin;
  XtAddCallback(button, XmNarmCallback, Xg_toggle_axes, &graph->wid_array[19]);
  XtManageChild(button);

/**** add main popup callbacks ****/
XtManageChild(parent);
XtAddCallback(parent, XmNcancelCallback, Xg_cancel_CB, NULL);
XtAddCallback(parent, XmNhelpCallback, Xg_help_sys, (caddr_t) XgHELP_LAYOUT);
XtAddCallback(parent, XmNokCallback, Xg_set_axes, client_data);
XtAddCallback(parent, XmNapplyCallback, Xg_default_axes, client_data);
XtFree(title);
XtFree(defaults);
}

/**************************************/
void Xg_toggle_axes(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    nargs;
  Arg
    args[5];
  XmString
    label;

  XgGraphInfo
    *graph;
  XgWidStruct
    *wid_addr;
  XmToggleButtonCallbackStruct
    *toggle_info;

toggle_info = (XmToggleButtonCallbackStruct *) call_data;
wid_addr = (XgWidStruct *) client_data;
graph = (XgGraphInfo *) wid_addr->graph;

switch ((int) wid_addr->client)
  {
  case XgSig1:
  case XgSig2:
  case XgSig3:
    graph->temp_axis.nsig_fig = (int) wid_addr->client;
    graph->temp_axis.user_set_flag = graph->temp_axis.user_set_flag | XgUnsig;
    break;

  case XgLast1:
  case XgLast2:
  case XgLast5:
    graph->temp_axis.increment = (int) wid_addr->client;
    graph->temp_axis.user_set_flag = graph->temp_axis.user_set_flag | XgUinc;
    break;

  case XgDivNone:
  case XgDivMajorTic:
  case XgDivMinorTic:
  case XgDivMajorLin:
  case XgDivLin_Tic:
  case XgDivMinorLin:
    graph->temp_axis.div_marks = (int) wid_addr->client;
    break;

  case XgNo_label:
  case XgSci_label:
  case XgEng_label:
    graph->temp_axis.label_type = (int) wid_addr->client;
    break;

  case XgLinear:
  case XgLogarithm:
  case XgProbability:
    graph->temp_axis.scaling = (int) wid_addr->client;
    break;

  default:
    fprintf(stderr, "ERROR: bad client type in Xg_toggle_axes. Aborted!\n");
    exit(1);
    break;
  }
}

/**************************************/
void Xg_minmax(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    *cptr,
    *buf;
  int
    i;
  double
    value;
  XgGraphInfo
    *graph;
  XgWidStruct
    *wid_addr;

wid_addr = (XgWidStruct *) client_data;
graph = (XgGraphInfo *) wid_addr->graph;

buf = XmTextGetString(w);
#ifndef Xg_NO_CTYPE
for (i=0; i<strlen(buf); ++i)  /** e for sci. not. **/
  if (!(isdigit(buf[i]) || isspace(buf[i]) || (buf[i] == '-')
	|| (buf[i] == '.') || (buf[i] == 'e') || (buf[i] == '+') ))
      return;
#endif
cptr = strchr(buf, '+');
while (cptr != NULL)  /** some systems write e+05 but can't read the + **/
  {
    *cptr = '0';
    cptr = strchr(buf, '+');
  }
sscanf(buf, "%le", &value);

if ((int) wid_addr->client == XgMin)
  {
    graph->temp_axis.min = value;
    graph->temp_axis.user_set_flag = graph->temp_axis.user_set_flag | XgUmin;
  }

else if ((int) wid_addr->client == XgMax)
  {
    graph->temp_axis.max = value;
    graph->temp_axis.user_set_flag = graph->temp_axis.user_set_flag | XgUmax;
  }

else
  {
    fprintf(stderr, "Bad client type in Xg_minmax. Aborted!\n");
    exit(1);
  }

}

/*******************************************/
void Xg_set_axes(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgGraphInfo
    *graph;
  XgWidStruct
    *wid_addr;
  int
    i;

wid_addr = (XgWidStruct *) client_data;
graph = (XgGraphInfo *) wid_addr->graph;
XtDestroyWidget(w);

if (graph->temp_axis.min >= graph->temp_axis.max)
  graph->temp_axis.user_set_flag
    = graph->temp_axis.user_set_flag & (~(XgUmin | XgUmax));

if ((int) wid_addr->client == XgXaxis)
  {
    graph->xaxis_layout.increment = graph->temp_axis.increment;
    graph->xaxis_layout.nsig_fig = graph->temp_axis.nsig_fig;
    graph->xaxis_layout.div_marks = graph->temp_axis.div_marks;
    graph->xaxis_layout.scaling = graph->temp_axis.scaling;
    graph->xaxis_layout.label_type = graph->temp_axis.label_type;
    graph->xaxis_layout.user_set_flag = graph->temp_axis.user_set_flag;
  }
else if ((int) wid_addr->client == XgYLaxis)
  {
    graph->yaxis_layout.increment = graph->temp_axis.increment;
    graph->yaxis_layout.nsig_fig = graph->temp_axis.nsig_fig;
    graph->yaxis_layout.div_marks = graph->temp_axis.div_marks;
    graph->yaxis_layout.scaling = graph->temp_axis.scaling;
    graph->yaxis_layout.label_type = graph->temp_axis.label_type;
    graph->yaxis_layout.user_set_flag = graph->temp_axis.user_set_flag;
  }
else if ((int) wid_addr->client == XgYRaxis)
  {
    graph->y_rt_axis_layout.increment = graph->temp_axis.increment;
    graph->y_rt_axis_layout.nsig_fig = graph->temp_axis.nsig_fig;
    graph->y_rt_axis_layout.div_marks = graph->temp_axis.div_marks;
    graph->y_rt_axis_layout.scaling = graph->temp_axis.scaling;
    graph->y_rt_axis_layout.label_type = graph->temp_axis.label_type;
    graph->y_rt_axis_layout.user_set_flag = graph->temp_axis.user_set_flag;
  }

if (graph->temp_axis.min >= graph->temp_axis.max)
  {
    Xg_error_sys(XgERR_BAD_MINMAX, graph);
    return;
  }

if ((int) wid_addr->client == XgXaxis)
  {
    graph->xaxis_layout.min = graph->temp_axis.min;
    graph->xaxis_layout.max = graph->temp_axis.max;
  }
else if ((int) wid_addr->client == XgYLaxis)
  {
    graph->yaxis_layout.min = graph->temp_axis.min;
    graph->yaxis_layout.max = graph->temp_axis.max;
  }
else if ((int) wid_addr->client == XgYRaxis)
  {
    graph->y_rt_axis_layout.min = graph->temp_axis.min;
    graph->y_rt_axis_layout.max = graph->temp_axis.max;
  }

/*Xg_createGraph(graph);*/
}

/**************************************/
void Xg_default_axes(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgGraphInfo
    *graph;
  XgWidStruct
    *wid_addr;
  int
    i;

wid_addr = (XgWidStruct *) client_data;
graph = (XgGraphInfo *) wid_addr->graph;

if ((int) wid_addr->client == XgXaxis)
  {
    /*graph->xaxis_layout.min = 
      graph->xaxis_layout.max = */
    graph->xaxis_layout.increment = XgDefInc;
    graph->xaxis_layout.nsig_fig = XgDefSigFig;
    graph->xaxis_layout.div_marks = XgDefDivMark;
    graph->xaxis_layout.scaling = XgDefScale;
    graph->xaxis_layout.label_type = XgDefLabelType;
    graph->xaxis_layout.user_set_flag = XgUNothing;
  }
else if ((int) wid_addr->client == XgYLaxis)
  {
    /*graph->yaxis_layout.min = 
      graph->yaxis_layout.max = */
    graph->yaxis_layout.increment = XgDefInc;
    graph->yaxis_layout.nsig_fig = XgDefSigFig;
    graph->yaxis_layout.div_marks = XgDefDivMark;
    graph->yaxis_layout.scaling = XgDefScale;
    graph->yaxis_layout.label_type = XgDefLabelType;
    graph->yaxis_layout.user_set_flag = XgUNothing;
  }
else if ((int) wid_addr->client == XgYRaxis)
  {
    /*graph->y_rt_axis_layout.min = 
      graph->y_rt_axis_layout.max = */
    graph->y_rt_axis_layout.increment = XgDefInc;
    graph->y_rt_axis_layout.nsig_fig = XgDefSigFig;
    graph->y_rt_axis_layout.div_marks = XgDefDivMark;
    graph->y_rt_axis_layout.scaling = XgDefScale;
    graph->y_rt_axis_layout.label_type = XgDefLabelType;
    graph->y_rt_axis_layout.user_set_flag = XgUNothing;
  }

XtDestroyWidget(w);
/*Xg_createGraph(graph);*/
}
