/**********************************************************************
*
*                            XgData_Add.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*    Add Data sets for plotting
*
***********************************************************************
*/

#include "stdio.h"
#include "string.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "Xm/FileSB.h"
#include "Xm/Form.h"
#include "Xm/Frame.h"
#include "Xm/SelectioB.h"
#include "Xm/RowColumn.h"
#include "Xm/ToggleB.h"
#include "Xm/ToggleBG.h"
#include "Xm/PanedW.h"
#include "Xm/Text.h"
#include "Xm/List.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"

/*******************************************************************/
void Xg_start_add_file(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    vis,
    nargs;
  char
    buf[80];
  Widget
    file_selector,
    frame,
    form,
    form1,
    label,
    radio,
    data_set_list;
  Arg
    args[10];
  XmString
    title,
    datasets_label,
    frame_label,
    DataList[XgNSETS];

  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
if (graph->num_sets >= XgNSETS)
  {
    Xg_error_sys(XgERR_TOO_MANY_FILES, graph);
    return;
  }
if (graph->cursor_status == XgCursOn)  /* turn off cursor */
  Xg_delcurs(graph);

nargs = 0;
title = XmStringCreateLtoR("Xg: Add Data File", XgCharset);
/*XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;*/
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNdefaultPosition, False); ++nargs;
XtSetArg(args[nargs], XmNx, 400); ++nargs;
XtSetArg(args[nargs], XmNy, 100); ++nargs;
file_selector = XmCreateFileSelectionDialog(graph->app_shell,
					   "Xg: Add Data File", args, nargs);

/********** Current files ************/
for (nargs=0; nargs<graph->num_sets; ++nargs)
  {
    sprintf(buf, "%d %s", nargs, graph->dataset[nargs].filename);
    DataList[nargs] = XmStringCreateLtoR(buf, XgCharset);
  }
if (graph->num_sets > 5)  /** set number of files to show w/out scrollbars **/
  vis = 5;
else
  vis = graph->num_sets;

nargs = 0;
form = XmCreateForm(file_selector, "form", args, nargs);
XtManageChild(form);

nargs = 0;
datasets_label = XmStringCreateLtoR("Current Data Files", XgCharset);
XtSetArg(args[nargs], XmNlabelString, datasets_label); ++nargs;
XtSetArg(args[nargs], XmNalignment, XmALIGNMENT_BEGINNING); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
label = XmCreateLabel(form, "current files label", args, nargs);
XtManageChild(label);
XtFree(datasets_label);

nargs = 0;
XtSetArg(args[nargs], XmNitems, DataList); ++nargs;
XtSetArg(args[nargs], XmNitemCount, graph->num_sets); ++nargs;
XtSetArg(args[nargs], XmNvisibleItemCount, vis); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, label); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 2); ++nargs;
data_set_list = XmCreateScrolledList(form,
	      "current files list", args, nargs);
XtManageChild(data_set_list);

/*********** new file type *************/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, data_set_list); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
frame = XmCreateFrame(form, "current file frame", args, nargs);
XtManageChild(frame);

nargs = 0;
form1 = XmCreateForm(frame, "form", args, nargs);
XtManageChild(form1);

nargs = 0;
datasets_label = XmStringCreateLtoR("File Type", XgCharset);
XtSetArg(args[nargs], XmNlabelString, datasets_label); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
label = XmCreateLabel(form1, "current files label", args, nargs);
XtManageChild(label);
XtFree(datasets_label);

nargs = 0;
XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, label); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 2); ++nargs;
XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
radio = XmCreateRadioBox(form1, "file type radio", args, nargs);
XtManageChild(radio);

  nargs = 0;
  graph->ftype = Xg_Add_Normal;
  XtSetArg(args[nargs], XmNset, True); ++nargs;
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  graph->but1 = XmCreateToggleButtonGadget(radio, "Normal File", args, nargs);
  XtAddCallback(graph->but1, XmNarmCallback, Xg_toggle_ftype, graph);
  XtManageChild(graph->but1);

  nargs = 0;
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  graph->but2 = XmCreateToggleButtonGadget(radio, "Ascii Pairs", args, nargs);
  XtAddCallback(graph->but2, XmNarmCallback, Xg_toggle_ftype, graph);
  XtManageChild(graph->but2);

  nargs = 0;
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  graph->but3 = XmCreateToggleButtonGadget(radio, "Single Vals", args, nargs);
  XtAddCallback(graph->but3, XmNarmCallback, Xg_toggle_ftype, graph);
  XtManageChild(graph->but3);

/*********** Error Bars? *************/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
frame = XmCreateFrame(form, "error bars frame", args, nargs);
XtManageChild(frame);

nargs = 0;
form1 = XmCreateForm(frame, "form", args, nargs);
XtManageChild(form1);

/*nargs = 0;
datasets_label = XmStringCreateLtoR("Error Bars With Data", XgCharset);
XtSetArg(args[nargs], XmNlabelString, datasets_label); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
label = XmCreateLabel(form1, "error bars label", args, nargs);
XtManageChild(label);
XtFree(datasets_label);*/

nargs = 0;
XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
/*XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, label); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 2); ++nargs;*/
XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
radio = XmCreateRadioBox(form1, "error bars radio", args, nargs);
XtManageChild(radio);

  nargs = 0;
  graph->err_bar = XgNoErrorPts;
  XtSetArg(args[nargs], XmNset, True); ++nargs;
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  graph->but4 = XmCreateToggleButtonGadget(radio, "No Error Bars", args,nargs);
  XtAddCallback(graph->but4, XmNarmCallback, Xg_toggle_ftype, graph);
  XtManageChild(graph->but4);

  nargs = 0;
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  graph->but5 = XmCreateToggleButtonGadget(radio, "Has Error Bars",args,nargs);
  XtAddCallback(graph->but5, XmNarmCallback, Xg_toggle_ftype, graph);
  XtManageChild(graph->but5);

/********* finish up & realize box *********/
XtManageChild(file_selector);
XtAddCallback(file_selector, XmNhelpCallback, Xg_help_sys,
	      (caddr_t) XgHELP_ADD_FILE);
XtAddCallback(file_selector, XmNokCallback, Xg_add_file, graph);
XtAddCallback(file_selector, XmNcancelCallback, Xg_cancel_CB, NULL);
XtFree(title);
for (nargs=0; nargs<graph->num_sets; ++nargs)
  XtFree(DataList[nargs]);
}

/*******************************************************************/
void Xg_toggle_ftype(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;

if (w == graph->but1)
  graph->ftype = Xg_Add_Normal;
else if (w == graph->but2)
  graph->ftype = Xg_Add_Pair;
else if (w == graph->but3)
  graph->ftype = Xg_Add_Single;
else if (w == graph->but4)
  graph->err_bar = XgNoErrorPts;
else if (w == graph->but5)
  graph->err_bar = XgHasErrorPts;
}

/***************************/
void Xg_add_file(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    *buf;
  FILE
    *fp;
  XmFileSelectionBoxCallbackStruct
    *dialog_rslt;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;

dialog_rslt = (XmFileSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(dialog_rslt->value, XgCharset, &buf);
strcpy(graph->dataset[graph->num_sets].filename, buf);
XtFree(dialog_rslt);
free(buf);

fp = fopen(graph->dataset[graph->num_sets].filename, "r");
if (fp == NULL)
  {
    Xg_error_sys(XgERR_NO_FILE, graph);
    return;
  }
XtDestroyWidget(w);
graph->dataset[graph->num_sets].error_pts = graph->err_bar;
graph->dataset[graph->num_sets].file_type = graph->ftype;

Xg_read_datafile(fp, graph, graph->num_sets);

graph->dataset[graph->num_sets].point_type = XgDefPointType;
graph->dataset[graph->num_sets].line_status = XgDefLineType;
graph->dataset[graph->num_sets].point_size = XgDefPtSize;
graph->dataset[graph->num_sets].filled_open = XgDefFillOpen;
graph->dataset[graph->num_sets].error_bars = XgDefErrorBarStatus;
graph->dataset[graph->num_sets].left_right = XgLeft;
graph->dataset[graph->num_sets].in_legend = XgDefLegendStatus;
strcpy(graph->dataset[graph->num_sets].legend_text,
       graph->dataset[graph->num_sets].filename);
graph->dataset[graph->num_sets].description.text[0] = '\0';
graph->dataset[graph->num_sets].description.coord.x = XgDefDataDescXCoord;
graph->dataset[graph->num_sets].description.coord.y = XgDefDataDescYCoord;
graph->dataset[graph->num_sets].description.dimen.x = 0;
graph->dataset[graph->num_sets].description.dimen.y = 0;
++graph->num_sets;
fclose(fp);
/*Xg_createGraph(graph);*/
}

/*********************************************/
void Xg_read_datafile(FILE *fp, XgGraphInfo * graph, int ds_num)
{
  int
    i,
    n;
  double
    *data,
    *err_array;
  char
    comment[80];

err_array = NULL;
switch (graph->dataset[ds_num].file_type)
  {
  case Xg_Add_Normal:
    fscanf(fp, "%[^\n]", comment);
    getc(fp);
    fscanf(fp, "%[^\n]", comment);
    getc(fp);
    fscanf(fp, "%d %[^\n]", &n, comment);
    if ((n <= 0) || (n > 32000))
      {
	Xg_error_sys(XgERR_NO_FILE, graph);
	return;
      }

    data = (double *) calloc(n*2, sizeof(double));
    if (data == NULL)
      {
	Xg_error_sys(XgERR_DATA_CALLOC, graph);
	return;
      }
    if (graph->dataset[ds_num].error_pts == XgHasErrorPts)
      {
	err_array = (double *) calloc(n, sizeof(double));
	if (err_array == NULL)
	  {
	    Xg_error_sys(XgERR_DATA_CALLOC, graph);
	    return;
	  }
	for (i=0; i<n; ++i)
	  fscanf(fp, "%le %le %le", (data + 2*i), (data + 2*i + 1),
		 (err_array + i));
      }
    else  /** just normal data, no error points **/
      for (i=0; i<n; ++i)
	fscanf(fp, "%le %le", (data + 2*i), (data + 2*i + 1));
    break;

  case Xg_Add_Pair:
    data = (double *) calloc(XgFILE_MAX*2, sizeof(double));
    if (data == NULL)
      {
	Xg_error_sys(XgERR_DATA_CALLOC, graph);
	return;
      }
    if (graph->dataset[ds_num].error_pts == XgHasErrorPts)
      {
	err_array = (double *) calloc(XgFILE_MAX, sizeof(double));
	if (err_array == NULL)
	  {
	    Xg_error_sys(XgERR_DATA_CALLOC, graph);
	    return;
	  }
	n = 0;
	while ((fscanf(fp, "%le %le %le", (data + 2*n), (data + 2*n + 1),
		       (err_array +n)) != EOF)
	       && (n < XgFILE_MAX))
	  {
	    ++n;
	  }
      }
    else /** no error points, just pairs **/
      {
	n = 0;
	while ((fscanf(fp, "%le %le", (data + 2*n), (data + 2*n + 1)) != EOF)
	       && (n < XgFILE_MAX))
	  {
	    ++n;
	  }
      }
    break;

  case Xg_Add_Single:
    data = (double *) calloc(XgFILE_MAX*2, sizeof(double));
    if (data == NULL)
      {
	Xg_error_sys(XgERR_DATA_CALLOC, graph);
	return;
      }
    n = 0;
    while ((fscanf(fp, "%le", (data + 2*n + 1)) != EOF) && (n < XgFILE_MAX))
      {
	*(data + 2*n) = n;
	++n;
      }
    break;

  default:
    fprintf(stderr, "ERROR: invalid file type in Xg_read_datafile\n");
    break;
  }

graph->dataset[ds_num].num_points = n;
graph->dataset[ds_num].darray = data;
graph->dataset[ds_num].err_array = err_array;
}
