/**********************************************************************
*
*                              XgInit.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*  Usage:
*
*    #include "XgPlot.h"
*
*    XgGraphInfo
*      *graph;
*
*    graph = XgCreateStruct();
*
*  as defined in XgPlot.h. This structure may be initialized by a call to
*  XgInitializeStruct which will supply default values. You must then set
*  the following: num_sets must be greater than zero, with the
*  corresponding datasets[i] structures initialized with *darray, and
*  num_points. A shell window must be supplied in app_shell to contain the
*  graph and menu bar. You may override any default values. Now call
*  XgCreateGraph(graph) and the graph will be created using the default
*  values supplemented by yours. The graph is an independent widget.
*  Values may be altered interactively for each graph using the menu, or
*  they may be adjusted by the calling program. Note that this means that
*  once created, the graph can be updated dynamically.
*
***********************************************************************
*/
#define XgInit

#include "stdio.h"
#include "string.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "Xm/MainW.h"
#include "Xm/Form.h"
#include "Xm/PushB.h"
#include "Xm/RowColumn.h"
#include "Xm/CascadeB.h"

#include "XgPlot.bmp"
#include "XgPlot.h"
#include "XgPlot_loc.h"

XmStringCharSet XgCharset = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
double XgProb_array[] = { .00001, .0001, .001, .01, .05, .1, .3, .5, .7,
			    .9, .95, .99, .999, .9999, .99999 };

/******************************************************************
*
*                        MAIN ENTRY POINT
*
********************************************************************/
int XgCreateGraph
  (
   XgGraphInfo * graphinfo
  )
{
  Arg
    args[10];
  int
    nargs;
  Pixmap
    icon_pixmap;

/****************** Set up Main Application Shell *************/
if (Xg_finishInitialize(graphinfo) < 0)
  return(0);
icon_pixmap = XCreateBitmapFromData(graphinfo->display,
	      RootWindowOfScreen(XtScreen(graphinfo->app_shell)),
	      bitmap_bits, bitmap_width, bitmap_height);
nargs = 0;
XtSetArg(args[nargs], XtNiconPixmap, icon_pixmap); ++nargs;
XtSetValues(graphinfo->app_shell, args, nargs);
Xg_createMenus(graphinfo);
Xg_createGraph(graphinfo);
}

/*****************************************************************
*
*                     Initialization Stuff
*
******************************************************************/

/** This routine makes sure a minimal amount of information has been set **/
Xg_finishInitialize(graphinfo)
     XgGraphInfo
       *graphinfo;
{
  int
    i;
  long int
    fore_color,
    back_color;

/************/
if (graphinfo->num_sets == 0)
  {
    fprintf(stderr, "XgPlot ERROR: No data sets to plot!\n");
    return(-1);
  }
if (graphinfo->app_shell == NULL)
  {
    fprintf(stderr, "XgPlot ERROR: No Application Shell to run in!\n");
    return(-1);
  }

graphinfo->display = XtDisplay(graphinfo->app_shell);

for (i=0; i<XgNWID; ++i)
  graphinfo->wid_array[i].graph = (caddr_t) graphinfo;

graphinfo->cursor_status = XgCursOff;
graphinfo->redraw = 0;
graphinfo->draw_area = NULL;

fore_color = XBlackPixelOfScreen(XtScreen(graphinfo->app_shell));
back_color = XWhitePixelOfScreen(XtScreen(graphinfo->app_shell));

strcpy(graphinfo->helpFile, XgDefHelpFile);
strcpy(graphinfo->fontDir, XgDefFontDir);

/*************** fonts & colors **************************/

/***** titles and such *****/
if (graphinfo->x_axis_desc.font_info.font_struct == NULL)  /** no font yet **/
  {
    if (graphinfo->x_axis_desc.font_info.name[0] == '\0')/*try font_info.name*/
      strcpy(graphinfo->x_axis_desc.font_info.name, XgDefFont);
    graphinfo->x_axis_desc.font_info.font_struct
      = XLoadQueryFont(graphinfo->display,
		       graphinfo->x_axis_desc.font_info.name);
    if (graphinfo->x_axis_desc.font_info.font_struct == NULL)/*force def font*/
      {
	graphinfo->x_axis_desc.font_info.font_struct
	  = XLoadQueryFont(graphinfo->display, XgDefFont);
	strcpy(graphinfo->x_axis_desc.font_info.name, XgDefFont);
      }
    if (graphinfo->x_axis_desc.font_info.font_struct == NULL)
      {
	fprintf(stderr, "WARNING: XgPlot unable to load default fonts!!\n");
      }
  }
if (graphinfo->x_axis_desc.color == -1)
  graphinfo->x_axis_desc.color = fore_color;

if (graphinfo->y_axis_desc.font_info.font_struct == NULL)
  {
    if (graphinfo->y_axis_desc.font_info.name[0] == '\0')
      strcpy(graphinfo->y_axis_desc.font_info.name, XgDefFont);
    graphinfo->y_axis_desc.font_info.font_struct
      = XLoadQueryFont(graphinfo->display,
		       graphinfo->y_axis_desc.font_info.name);
    if (graphinfo->y_axis_desc.font_info.font_struct == NULL)
      {
	graphinfo->y_axis_desc.font_info.font_struct
	  = XLoadQueryFont(graphinfo->display, XgDefFont);
	strcpy(graphinfo->y_axis_desc.font_info.name, XgDefFont);
      }
  }
if (graphinfo->y_axis_desc.color == -1)
  graphinfo->y_axis_desc.color = fore_color;

if (graphinfo->y_rt_axis_desc.font_info.font_struct == NULL)
  {
    if (graphinfo->y_rt_axis_desc.font_info.name[0] == '\0')
      strcpy(graphinfo->y_rt_axis_desc.font_info.name, XgDefFont);
    graphinfo->y_rt_axis_desc.font_info.font_struct
      = XLoadQueryFont(graphinfo->display,
		       graphinfo->y_rt_axis_desc.font_info.name);
    if (graphinfo->y_rt_axis_desc.font_info.font_struct == NULL)
      {
	graphinfo->y_rt_axis_desc.font_info.font_struct
	  = XLoadQueryFont(graphinfo->display, XgDefFont);
	strcpy(graphinfo->y_rt_axis_desc.font_info.name, XgDefFont);
      }
  }
if (graphinfo->y_rt_axis_desc.color == -1)
  graphinfo->y_rt_axis_desc.color = fore_color;

if (graphinfo->title_desc.font_info.font_struct == NULL)
  {
    if (graphinfo->title_desc.font_info.name[0] == '\0')
      strcpy(graphinfo->title_desc.font_info.name, XgDefFont);
    graphinfo->title_desc.font_info.font_struct
      = XLoadQueryFont(graphinfo->display,
		       graphinfo->title_desc.font_info.name);
    if (graphinfo->title_desc.font_info.font_struct == NULL)
      {
	graphinfo->title_desc.font_info.font_struct
	  = XLoadQueryFont(graphinfo->display, XgDefFont);
	strcpy(graphinfo->title_desc.font_info.name, XgDefFont);
      }
  }
if (graphinfo->title_desc.color == -1)
  graphinfo->title_desc.color = fore_color;

if (graphinfo->subtitle_desc.font_info.font_struct == NULL)
  {
    if (graphinfo->subtitle_desc.font_info.name[0] == '\0')
      strcpy(graphinfo->subtitle_desc.font_info.name, XgDefFont);
    graphinfo->subtitle_desc.font_info.font_struct
      = XLoadQueryFont(graphinfo->display,
		       graphinfo->subtitle_desc.font_info.name);
    if (graphinfo->subtitle_desc.font_info.font_struct == NULL)
      {
	graphinfo->subtitle_desc.font_info.font_struct
	  = XLoadQueryFont(graphinfo->display, XgDefFont);
	strcpy(graphinfo->subtitle_desc.font_info.name, XgDefFont);
      }
  }
if (graphinfo->subtitle_desc.color == -1)
  graphinfo->subtitle_desc.color = fore_color;

/******** data sets ********/
for (i=0; i<XgNSETS; ++i)
  {
    if (graphinfo->dataset[i].description.font_info.font_struct == NULL)
      {
	if (graphinfo->dataset[i].description.font_info.name[0] == '\0')
	  strcpy(graphinfo->dataset[i].description.font_info.name, XgDefFont);
	graphinfo->dataset[i].description.font_info.font_struct
	  = XLoadQueryFont(graphinfo->display,
			   graphinfo->dataset[i].description.font_info.name);
	if (graphinfo->dataset[i].description.font_info.font_struct == NULL)
	  {
	    graphinfo->dataset[i].description.font_info.font_struct
	      = XLoadQueryFont(graphinfo->display, XgDefFont);
	    strcpy(graphinfo->dataset[i].description.font_info.name,XgDefFont);
	  }
      }
    if (graphinfo->dataset[i].description.color == -1)
      graphinfo->dataset[i].description.color = fore_color;
  }

/***** axes ********/
if (graphinfo->xaxis_layout.scale_desc.font_info.font_struct == NULL)
  {
    if (graphinfo->xaxis_layout.scale_desc.font_info.name[0] == '\0')
      strcpy(graphinfo->xaxis_layout.scale_desc.font_info.name, XgDefFont);
    graphinfo->xaxis_layout.scale_desc.font_info.font_struct
      = XLoadQueryFont(graphinfo->display,
		       graphinfo->xaxis_layout.scale_desc.font_info.name);
    if (graphinfo->xaxis_layout.scale_desc.font_info.font_struct == NULL)
      {
	graphinfo->xaxis_layout.scale_desc.font_info.font_struct
	  = XLoadQueryFont(graphinfo->display, XgDefFont);
	strcpy(graphinfo->xaxis_layout.scale_desc.font_info.name, XgDefFont);
      }
    /** use axis_color **/
  }

if (graphinfo->yaxis_layout.scale_desc.font_info.font_struct == NULL)
  {
    if (graphinfo->yaxis_layout.scale_desc.font_info.name[0] == '\0')
      strcpy(graphinfo->yaxis_layout.scale_desc.font_info.name, XgDefFont);
    graphinfo->yaxis_layout.scale_desc.font_info.font_struct
      = XLoadQueryFont(graphinfo->display,
		       graphinfo->yaxis_layout.scale_desc.font_info.name);
    if (graphinfo->yaxis_layout.scale_desc.font_info.font_struct == NULL)
      {
	graphinfo->yaxis_layout.scale_desc.font_info.font_struct
	  = XLoadQueryFont(graphinfo->display, XgDefFont);
	strcpy(graphinfo->yaxis_layout.scale_desc.font_info.name, XgDefFont);
      }
    /** use axis_color **/
  }

if (graphinfo->y_rt_axis_layout.scale_desc.font_info.font_struct == NULL)
  {
    if (graphinfo->y_rt_axis_layout.scale_desc.font_info.name[0] == '\0')
      strcpy(graphinfo->y_rt_axis_layout.scale_desc.font_info.name, XgDefFont);
    graphinfo->y_rt_axis_layout.scale_desc.font_info.font_struct
      = XLoadQueryFont(graphinfo->display,
		       graphinfo->y_rt_axis_layout.scale_desc.font_info.name);
    if (graphinfo->y_rt_axis_layout.scale_desc.font_info.font_struct == NULL)
      {
	graphinfo->y_rt_axis_layout.scale_desc.font_info.font_struct
	  = XLoadQueryFont(graphinfo->display, XgDefFont);
	strcpy(graphinfo->y_rt_axis_layout.scale_desc.font_info.name,
	       XgDefFont);
      }
    /** use axis_color **/
  }

/******* legend *******/
if (graphinfo->legend.description.font_info.font_struct == NULL)
  {
    if (graphinfo->legend.description.font_info.name[0] == '\0')
      strcpy(graphinfo->legend.description.font_info.name, XgDefFont);
    graphinfo->legend.description.font_info.font_struct
      = XLoadQueryFont(graphinfo->display,
		       graphinfo->legend.description.font_info.name);
    if (graphinfo->legend.description.font_info.font_struct == NULL)
      {
	graphinfo->legend.description.font_info.font_struct
	  = XLoadQueryFont(graphinfo->display, XgDefFont);
	strcpy(graphinfo->legend.description.font_info.name, XgDefFont);
      }
  }
if (graphinfo->legend.description.color == -1)
  graphinfo->legend.description.color = fore_color;

/***** other stuff *****/
if (graphinfo->axis_color == -1)
  graphinfo->axis_color = fore_color;
if (graphinfo->background_color == -1)
  graphinfo->background_color = back_color;
if (graphinfo->cursor_color == -1)
  graphinfo->cursor_color = fore_color;

return(1);
}

/*************************************/
XgGraphInfo * XgCreateStruct() /* Create XgGraphInfo data structure &
				      initialize it to default values */
{
  XgGraphInfo
    *graphinfo;

graphinfo = (XgGraphInfo *) calloc(1, sizeof(XgGraphInfo));
if (graphinfo == NULL)
  {
    fprintf(stderr, "Failed to calloc XgGraphInfo structure. Aborted!\n");
    exit(1);
  }

graphinfo->kill_flag = XgKillOnExit;
graphinfo->width = 300;    /** set initial startup graph dimensions **/
graphinfo->height = 200;

Xg_initializeStruct(graphinfo);
return(graphinfo);
}

/************************/
void Xg_initializeStruct(graphinfo) /* Initialize Structure with defaults */
     XgGraphInfo
       *graphinfo;
{
  int
    i;

graphinfo->num_sets = 0;
graphinfo->filename[0] = '\0';

/******* individual data sets *****/
for (i=0; i<XgNSETS; ++i)
  {
    graphinfo->dataset[i].num_points = 0;
    graphinfo->dataset[i].error_pts = XgNoErrorPts;
    graphinfo->dataset[i].err_array = NULL;
    graphinfo->dataset[i].error_bars = XgDefErrorBarStatus;
    graphinfo->dataset[i].point_type = XgDefPointType;
    graphinfo->dataset[i].point_size = XgDefPtSize;
    graphinfo->dataset[i].filled_open = XgDefFillOpen;
    graphinfo->dataset[i].line_status = XgDefLineType;
    graphinfo->dataset[i].left_right = XgLeft;
    graphinfo->dataset[i].in_legend = XgDefLegendStatus;
    graphinfo->dataset[i].legend_text[0] = '\0';
    graphinfo->dataset[i].filename[0] = '\0';
    graphinfo->dataset[i].description.text[0] = '\0';
    graphinfo->dataset[i].description.font_info.font_struct = NULL;
    graphinfo->dataset[i].description.font_info.name[0] = '\0';
    graphinfo->dataset[i].description.color = -1;
    graphinfo->dataset[i].description.coord.x = XgDefDataDescXCoord;
    graphinfo->dataset[i].description.coord.y = XgDefDataDescYCoord;
    graphinfo->dataset[i].description.dimen.x = 0;
    graphinfo->dataset[i].description.dimen.y = 0;
  }

/***** Set General Colors *****/
graphinfo->axis_color = -1;
graphinfo->background_color = -1;
graphinfo->cursor_color = -1;

/********** legend defaults ***********/
graphinfo->legend.use_legend = XgDefLegendOnOff;
strcpy(graphinfo->legend.description.text, "Legend");
graphinfo->legend.description.coord.x = XgDefLegendXCoord;
graphinfo->legend.description.coord.y = XgDefLegendYCoord;
graphinfo->legend.description.color = -1;
graphinfo->legend.description.font_info.font_struct = NULL;
graphinfo->legend.description.font_info.name[0] = '\0';
graphinfo->legend.description.dimen.x = 0;
graphinfo->legend.description.dimen.x = 0;

/******* axes & title text ******/
graphinfo->x_axis_desc.coord.x = XgDefXtextXCoord;
graphinfo->x_axis_desc.coord.y = XgDefXtextYCoord;
graphinfo->x_axis_desc.dimen.x = 0;
graphinfo->x_axis_desc.dimen.y = 0;
graphinfo->x_axis_desc.font_info.font_struct = NULL;
graphinfo->x_axis_desc.font_info.name[0] = '\0';
graphinfo->x_axis_desc.text[0] = '\0';
graphinfo->x_axis_desc.color = -1;

graphinfo->y_axis_desc.coord.x = XgDefYtextXCoord;
graphinfo->y_axis_desc.coord.y = XgDefYtextYCoord;
graphinfo->y_axis_desc.dimen.x = 0;
graphinfo->y_axis_desc.dimen.y = 0;
graphinfo->y_axis_desc.font_info.font_struct = NULL;
graphinfo->y_axis_desc.font_info.name[0] = '\0';
graphinfo->y_axis_desc.text[0] = '\0';
graphinfo->y_axis_desc.color = -1;

graphinfo->y_rt_axis_desc.coord.x = XgDefYrtTextXCoord;
graphinfo->y_rt_axis_desc.coord.y = XgDefYrtTextYCoord;
graphinfo->y_rt_axis_desc.dimen.x = 0;
graphinfo->y_rt_axis_desc.dimen.y = 0;
graphinfo->y_rt_axis_desc.font_info.font_struct = NULL;
graphinfo->y_rt_axis_desc.font_info.name[0] = '\0';
graphinfo->y_rt_axis_desc.text[0] = '\0';
graphinfo->y_rt_axis_desc.color = -1;

graphinfo->title_desc.coord.x = XgDefTitletextXCoord;
graphinfo->title_desc.coord.y = XgDefTitletextYCoord;
graphinfo->title_desc.dimen.x = 0;
graphinfo->title_desc.dimen.y = 0;
graphinfo->title_desc.font_info.font_struct = NULL;
graphinfo->title_desc.font_info.name[0] = '\0';
graphinfo->title_desc.text[0] = '\0';
graphinfo->title_desc.color = -1;

graphinfo->subtitle_desc.coord.x = XgDefSubtextXCoord;
graphinfo->subtitle_desc.coord.y = XgDefSubtextYCoord;
graphinfo->subtitle_desc.dimen.x = 0;
graphinfo->subtitle_desc.dimen.y = 0;
graphinfo->subtitle_desc.font_info.font_struct = NULL;
graphinfo->subtitle_desc.font_info.name[0] = '\0';
graphinfo->subtitle_desc.text[0] = '\0';
graphinfo->subtitle_desc.color = -1;

/********** Axis Layouts ***********/
graphinfo->xaxis_layout.nsig_fig = XgDefSigFig;
graphinfo->xaxis_layout.increment = XgDefInc;
graphinfo->xaxis_layout.div_marks = XgDefDivMark;
graphinfo->xaxis_layout.scaling = XgDefScale;
graphinfo->xaxis_layout.label_type = XgDefLabelType;
graphinfo->xaxis_layout.min = -1.;
graphinfo->xaxis_layout.max = 1.;
graphinfo->xaxis_layout.user_set_flag = XgUNothing;
graphinfo->xaxis_layout.scale_desc.font_info.font_struct = NULL;
graphinfo->xaxis_layout.scale_desc.font_info.name[0] = '\0';
graphinfo->xaxis_layout.scale_desc.text[0] = '\0';
graphinfo->xaxis_layout.scale_desc.coord.x = XgDefXscaleXCoord;
graphinfo->xaxis_layout.scale_desc.coord.y = XgDefXscaleYCoord;
graphinfo->xaxis_layout.scale_desc.dimen.x = 0;
graphinfo->xaxis_layout.scale_desc.dimen.y = 0;

graphinfo->yaxis_layout.nsig_fig = XgDefSigFig;
graphinfo->yaxis_layout.increment = XgDefInc;
graphinfo->yaxis_layout.div_marks = XgDefDivMark;
graphinfo->yaxis_layout.scaling = XgDefScale;
graphinfo->yaxis_layout.label_type = XgDefLabelType;
graphinfo->yaxis_layout.min = -1.;
graphinfo->yaxis_layout.max = 1.;
graphinfo->yaxis_layout.user_set_flag = XgUNothing;
graphinfo->yaxis_layout.scale_desc.font_info.font_struct = NULL;
graphinfo->yaxis_layout.scale_desc.font_info.name[0] = '\0';
graphinfo->yaxis_layout.scale_desc.text[0] = '\0';
graphinfo->yaxis_layout.scale_desc.coord.x = XgDefYscaleXCoord;
graphinfo->yaxis_layout.scale_desc.coord.y = XgDefYscaleYCoord;
graphinfo->yaxis_layout.scale_desc.dimen.x = 0;
graphinfo->yaxis_layout.scale_desc.dimen.y = 0;

graphinfo->y_rt_axis_layout.nsig_fig = XgDefSigFig;
graphinfo->y_rt_axis_layout.increment = XgDefInc;
graphinfo->y_rt_axis_layout.div_marks = XgDefDivMark;
graphinfo->y_rt_axis_layout.scaling = XgDefScale;
graphinfo->y_rt_axis_layout.label_type = XgDefLabelType;
graphinfo->y_rt_axis_layout.min = -1.;
graphinfo->y_rt_axis_layout.max = 1.;
graphinfo->y_rt_axis_layout.user_set_flag = XgUNothing;
graphinfo->y_rt_axis_layout.scale_desc.font_info.font_struct = NULL;
graphinfo->y_rt_axis_layout.scale_desc.font_info.name[0] = '\0';
graphinfo->y_rt_axis_layout.scale_desc.text[0] = '\0';
graphinfo->y_rt_axis_layout.scale_desc.coord.x = XgDefYrtScaleXCoord;
graphinfo->y_rt_axis_layout.scale_desc.coord.y = XgDefYrtScaleYCoord;
graphinfo->y_rt_axis_layout.scale_desc.dimen.x = 0;
graphinfo->y_rt_axis_layout.scale_desc.dimen.y = 0;

}

/******************* Create Menu Bar ***********************/
void Xg_createMenus(graph)
     XgGraphInfo
       *graph;
{
  int
    nargs;
  Arg
    args[10];
  XmString
    label;
  XgWidStruct
    *wid_addr;
  Widget
    main,
    pulldown,
    next_level,
    button;

nargs = 0;
XtSetArg(args[nargs], XmNscrollingPolicy, XmAPPLICATION_DEFINED); ++nargs;
graph->main = XmCreateMainWindow(graph->app_shell,"main",(ArgList) args,nargs);
XtManageChild(graph->main);

nargs = 0;
XtSetArg(args[nargs], XmNwidth, graph->width); ++nargs;
XtSetArg(args[nargs], XmNheight, graph->height); ++nargs;
XtSetArg(args[nargs], XmNresizeWidth, False); ++nargs;
XtSetArg(args[nargs], XmNresizeHeight, False); ++nargs;
graph->form = XmCreateForm(graph->main, "draw form", args, nargs);
XtManageChild(graph->form);

nargs = 0;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
graph->menu_bar = XmCreateMenuBar(graph->form, "menu_bar", args, nargs);
XtManageChild(graph->menu_bar);

/************* Create Menu Bar Buttons *************/

/***** help *****/
nargs = 0;    /* Menu Bar Button */
label = XmStringCreateLtoR("Help", XgCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetArg(args[nargs], XmNmnemonic, 'H'); ++nargs;
button = XmCreateCascadeButton(graph->menu_bar, "Help", args, nargs);
XtAddCallback(button, (String) XmNactivateCallback, Xg_main_help, graph);
XtManageChild(button);
XtFree(label);
XtSetArg(args[0], XmNmenuHelpWidget, button);
XtSetValues(graph->menu_bar, args, 1);

/************* pulldowns ****************/

/***********************************************************
*
*                         File Button
*
************************************************************/

pulldown = XmCreatePulldownMenu(graph->menu_bar, "File_pane", NULL, 0);

nargs = 0;   /* Menu Bar Button */
label = XmStringCreateLtoR("File", XgCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetArg(args[nargs], XmNsubMenuId, pulldown); ++nargs;
XtSetArg(args[nargs], XmNmnemonic, 'F'); ++nargs;
button = XmCreateCascadeButton(graph->menu_bar, "File", args, nargs);
XtManageChild(button);
XtFree(label);

  nargs = 0;   /* 1st level menu button */
  label = XmStringCreateLtoR("Load Graph", XgCharset);
  XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'L'); ++nargs;
  button = XmCreatePushButton(pulldown, "Load Graph", args, nargs);
  wid_addr = Xg_AllocStruct(XgLoad, (caddr_t) graph);
  XtAddCallback(button, XmNactivateCallback, Xg_load_save_graph, wid_addr);
  XtManageChild(button);
  XtFree(label);

  nargs = 0;   /* 1st level menu button */
  label = XmStringCreateLtoR("Save Graph", XgCharset);
  XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'S'); ++nargs;
  button = XmCreatePushButton(pulldown, "Save Graph", args, nargs);
  wid_addr = Xg_AllocStruct(XgSave, (caddr_t) graph);
  XtAddCallback(button, XmNactivateCallback, Xg_load_save_graph, wid_addr);
  XtManageChild(button);
  XtFree(label);

  nargs = 0;   /* 1st level menu button */
  label = XmStringCreateLtoR("Exit", XgCharset);
  XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'E'); ++nargs;
  button = XmCreatePushButton(pulldown, "Exit", args, nargs);
  XtAddCallback(button, XmNactivateCallback, Xg_create_exit_warning, graph);
  XtManageChild(button);
  XtFree(label);

/****************************************************************
*
*                             Edit Button
*
******************************************************************/

pulldown = XmCreatePulldownMenu(graph->menu_bar, "Edit_pane", NULL, 0);

nargs = 0;    /* Menu Bar Button */
label = XmStringCreateLtoR("Edit", XgCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetArg(args[nargs], XmNsubMenuId, pulldown); ++nargs;
XtSetArg(args[nargs], XmNmnemonic, 'E'); ++nargs;
button = XmCreateCascadeButton(graph->menu_bar, "Edit", args, nargs);
XtManageChild(button);
XtFree(label);

/*** Titles submenu ***/
next_level = XmCreatePulldownMenu(pulldown, "Text_pane", NULL, 0);

  nargs = 0;   /* 1st level menu button */
  XtSetArg(args[nargs], XmNsubMenuId, next_level); ++nargs;
  label = XmStringCreateLtoR("Titles", XgCharset);
  XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'T'); ++nargs;
  button = XmCreateCascadeButton(pulldown, "Titles", args, nargs);
  XtManageChild(button);
  XtFree(label);

     nargs = 0;     /* 2nd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'X'); ++nargs;
     button = XmCreatePushButton(next_level,"X-axis Description", args,nargs);
     wid_addr = Xg_AllocStruct(XgXaxisLabel, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg_do_edit_titles, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 2nd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'Y'); ++nargs;
     button = XmCreatePushButton(next_level, "Y-axis Left Desc",args,nargs);
     wid_addr = Xg_AllocStruct(XgYLaxisLabel, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg_do_edit_titles, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 2nd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'R'); ++nargs;
     button = XmCreatePushButton(next_level, "Y-axis Right Desc",args,nargs);
     wid_addr = Xg_AllocStruct(XgYRaxisLabel, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg_do_edit_titles, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 2nd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'T'); ++nargs;
     button = XmCreatePushButton(next_level, "Title", args, nargs);
     wid_addr = Xg_AllocStruct(XgTitle, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg_do_edit_titles, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 2nd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'S'); ++nargs;
     button = XmCreatePushButton(next_level, "Sub-title", args, nargs);
     wid_addr = Xg_AllocStruct(XgSubtitle, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg_do_edit_titles, wid_addr);
     XtManageChild(button);

     nargs = 0;     /* 2nd level menu button */
     XtSetArg(args[nargs], XmNmnemonic, 'L'); ++nargs;
     button = XmCreatePushButton(next_level, "Legend", args, nargs);
     wid_addr = Xg_AllocStruct(XgLegend, (caddr_t) graph);
     XtAddCallback(button, XmNactivateCallback, Xg_do_edit_titles, wid_addr);
     XtManageChild(button);

/*** Layout Submenu ***/
next_level = XmCreatePulldownMenu(pulldown, "Layout_pane", NULL, 0);

  nargs = 0;   /* 1st level menu button */
  XtSetArg(args[nargs], XmNsubMenuId, next_level); ++nargs;
  label = XmStringCreateLtoR("Layout", XgCharset);
  XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'L'); ++nargs;
  button = XmCreateCascadeButton(pulldown, "Layout", args, nargs);
  XtManageChild(button);
  XtFree(label);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'X'); ++nargs;
    button = XmCreatePushButton(next_level, "X-axis Labels", args, nargs);
    wid_addr = Xg_AllocStruct(XgXaxis, (caddr_t) graph);
    XtAddCallback(button, XmNactivateCallback, Xg_configure_axes, wid_addr);
    XtManageChild(button);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'Y'); ++nargs;
    button = XmCreatePushButton(next_level, "Y-axis L Labels", args, nargs);
    wid_addr = Xg_AllocStruct(XgYLaxis, (caddr_t) graph);
    XtAddCallback(button, XmNactivateCallback, Xg_configure_axes, wid_addr);
    XtManageChild(button);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'R'); ++nargs;
    button = XmCreatePushButton(next_level, "Y-axis R Labels", args, nargs);
    wid_addr = Xg_AllocStruct(XgYRaxis, (caddr_t) graph);
    XtAddCallback(button, XmNactivateCallback, Xg_configure_axes, wid_addr);
    XtManageChild(button);

/*** Data Submenu ***/
next_level = XmCreatePulldownMenu(pulldown, "Data_pane", NULL, 0);

  nargs = 0;   /* 1st level menu button */
  XtSetArg(args[nargs], XmNsubMenuId, next_level); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'D'); ++nargs;
  button = XmCreateCascadeButton(pulldown, "Data", args, nargs);
  XtManageChild(button);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'A'); ++nargs;
    button = XmCreatePushButton(next_level, "Add", args, nargs);
    XtAddCallback(button, XmNactivateCallback, Xg_start_add_file, graph);
    XtManageChild(button);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'D'); ++nargs;
    button = XmCreatePushButton(next_level, "Delete", args, nargs);
    wid_addr = Xg_AllocStruct(XgDSelDelete, (caddr_t) graph);
    XtAddCallback(button, XmNactivateCallback, Xg_select_data_set, wid_addr);
    XtManageChild(button);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'S'); ++nargs;
    button = XmCreatePushButton(next_level, "Save", args, nargs);
    wid_addr = Xg_AllocStruct(XgDSelSave, (caddr_t) graph);
    XtAddCallback(button, XmNactivateCallback, Xg_select_data_set, wid_addr);
    XtManageChild(button);

    nargs = 0;     /* 2nd level menu button */
    XtSetArg(args[nargs], XmNmnemonic, 'F'); ++nargs;
    button = XmCreatePushButton(next_level, "Format", args, nargs);
    wid_addr = Xg_AllocStruct(XgDSelFormat, (caddr_t) graph);
    XtAddCallback(button, XmNactivateCallback, Xg_select_data_set, wid_addr);
    XtManageChild(button);

/**************************************************************
*
*                          Options Button
*
****************************************************************/

pulldown = XmCreatePulldownMenu(graph->menu_bar, "Option_pane", NULL, 0);
XtManageChild(pulldown);

nargs = 0;   /* Menu Bar Button */
XtSetArg(args[nargs], XmNsubMenuId, pulldown); ++nargs;
XtSetArg(args[nargs], XmNmnemonic, 'O'); ++nargs;
button = XmCreateCascadeButton(graph->menu_bar, "Options", args, nargs);
XtManageChild(button);

  nargs = 0;   /* 1st level menu button */
  XtSetArg(args[nargs], XmNmnemonic, 'C'); ++nargs;
  button = XmCreatePushButton(pulldown, "Cursor On/Off", args, nargs);
  XtAddCallback(button, XmNactivateCallback, Xg_CursorToggle, graph);
  XtManageChild(button);

  nargs = 0;   /* 1st level menu button */
  XtSetArg(args[nargs], XmNmnemonic, 'P'); ++nargs;
  button = XmCreatePushButton(pulldown, "Print", args, nargs);
  XtAddCallback(button, XmNactivateCallback, Xg_Print, graph);
  XtManageChild(button);

}

/****************************************/
XgWidStruct * Xg_AllocStruct(client, graph)
     int
       client;
     caddr_t
       graph;
{
  XgWidStruct
    *wid_addr;
wid_addr = (XgWidStruct *) calloc(1, sizeof(XgWidStruct));
wid_addr->client = (caddr_t) client;
wid_addr->graph = graph;
return(wid_addr);
}

/*************************** End of File *****************************/
