/**********************************************************************
*
*                              XgMovie.h
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.0   4/93
*
*  Header file for XgMovie
*
***********************************************************************
*/

#define int2   short int
#define XgMClass       "XgMovie"
#define XgMDefFontDir  "/usr/lib/X11/fonts/misc/"
#define XgMDefFont     "6x13"

#define XgM_SCALE_MIN  0
/** use the 8191/8192 lines for IR 13 bit data, 4095/4096 for 12 bit visible */
#define XgM_SCALE_MAX  8191
#define XgM_ARRAY_MAX  8192
/*#define XgM_SCALE_MAX  4095
#define XgM_ARRAY_MAX  4096*/

#define XgM_MOVIE      101
#define XgM_SINGLE     102
#define XgM_FORWARD    103
#define XgM_REVERSE    104
#define XgM_FLIP_NORM  105
#define XgM_FLIP_REV   106
#define XgM_EXIT_KILL  107
#define XgM_EXIT_NOKILL 108
#define XgM_MINCELLS   256
#define XgM_MIN_GREY   128
#define XgM_SPARE      5
#define XgM_XColorMax  65535
#define XgM_SCL_WDTH   128
#define XgM_SCL_HT     10
#define XgM_INT_100    100
#define XgM_INT_200    200
#define XgM_INT_500    500
#define XgM_INT_1000   1000
#define XgM_SCALE_LIN  201
#define XgM_SCALE_LOG  202
#define XgM_SCALE_SQRT 203
#define XgM_SCALE_HIST 204
#define XgM_ZOOM_1     301
#define XgM_ZOOM_2     302
#define XgM_ZOOM_4     303
#define XgM_ZOOM_8     304
#define XgM_Pix_Bits   16
/** XgM_MINCELLS: minimum number of entries in colormap
    XgM_MIN_GREY: minimum number of grey scale colormap entries to use
    XgM_SPARE:    leaves a few cells for the window manager to use
    XgM_XColorMax: X uses this as system independent full scale color
    XgM_INT_xxx:  delay time in milliseconds between frames in movie
    XgM_SCALE_MAX: maximum value of data (e.g. 12 bits => 4095)
    XgM_ARRAY_MAX: XgM_SCALE_MAX+1 = number of levels
    XgM_SCL_WDTH: width of grey scale display
    XgM_SCL_HT:   height of grey scale display
**/

/********* help sources ************/
#define XgMMAIN_MENU           101

/******** error sources ************/
#define XgM_BAD_CALLOC         301
#define XgM_ZERO_WIDTH         302
#define XgM_HELP_FILE_TOO_BIG  303

/********** main movie structure *********/
typedef struct XgM_data
  {
    int
      nrows,
      ncols,
      nframes,
      xoff,    /** used only for printing coordinates **/
      yoff,
      ncells,
      base,
      nbits,
      pix_min,
      pix_max,
      grey_min,
      grey_max,
      screen_num,
      scaling,
      zoom_fac,
      flip,
      movie_single,
      direction,
      interval,
      kill_exit,
      rbrband_mode,
      overlay,
      x0,   /** starting image coordinates for rubber band **/
      y0,
      xp0,  /** starting display coordinates for rubber band **/
      yp0,
      xp1,  /** ending display coordinates for rubber band **/
      yp1,
      count;
    int2
      *pix_array;
    unsigned long
      overlay_pix,
      *pixels;
    char
      control_name[80],
      screen_name[80],
      filename[160];
    Colormap
      cmap;
    Pixmap
      grey_pix,
      *mv_img,
      icon_pixmap;
    GC
      gcxor,
      gc;
    XtAppContext
      theAppContext;
    Display
      *display;
    XVisualInfo
      *vis_info;
    Visual
      *visual;
    Widget
      ctrl_panel,   /** control panel app shell widget **/
      cursor_disp,  /** coordinates of cursor **/
      pixel_val,    /** pixel value at cursor **/
      min_bar,
      max_bar,
      min_box,
      max_box,
      grey_area,     /** grey scale display bar **/
      sbut_lin,      /** scaling buttons **/
      sbut_log,
      sbut_sqrt,
      sbut_hist,
      zbut_1,        /** zoom factor buttons **/
      zbut_2,
      zbut_4,
      zbut_8,
      fbut_norm,     /** flip orientation buttons **/
      fbut_rev,
      but_100,       /** time interval buttons **/
      but_200,
      but_500,
      but_1000,
      mv_scrn,       /** movie screen app shell widget **/
      frame_box,     /** frame number display **/
      frame_scroll,  /** frame scroll bar **/
      draw_area;     /** image display area **/
  } XgMovie_data;

/********* function declarations *************/
XgMovie_data * XgM_Initialize_Movie();
void XgM_Main_Entry(XgMovie_data * mv_data);
