/**********************************************************************
*
*                              XgMovieF.c
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.0   4/93
*
*  Program for displaying movie loop. This version reads frames from disk
*  as it goes. Also requires an 8 bit pseudocolor visual.
*
*  Input routine, called from here, is XgM_Read_Frame and must be defined
*  in calling module (by placing it there, this module remains source
*  independent)
*
***********************************************************************
*/
#define USE_PLOTS

#include "stdio.h"
#include "string.h"
#include "math.h"
#include "time.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "X11/cursorfont.h"
#include "Xm/Xm.h"
#include "Xm/Form.h"
#include "Xm/MainW.h"
#include "Xm/Frame.h"
#include "Xm/Label.h"
#include "Xm/CascadeB.h"
#include "Xm/PushB.h"
#include "Xm/ScrollBar.h"
#include "Xm/DrawingA.h"
#include "Xm/MessageB.h"
#include "Xm/RowColumn.h"
#include "Xm/ToggleB.h"

#include "etsheadr.h"
#include "XgMovieF.h"
#include "XgMovie.bmp"
#ifdef USE_PLOTS
#include "XgPlot.h"
#endif

#define XgM_Max_Help  250
#define XgMDefHelpDir   "/usr/local/bin/"
#define XgMDefHelpFile  "XgMovie.hlp"
/** green and red are good colors **/
#define XgMOverlayColor "green"

/********* function declarations *************/
void XgM_Create_Movie_Screen(XgMovie_data * mv_data);
void XgM_error_sys(int error_num, XgMovie_data * mv_data);
void XgM_Print();

/*** callbacks ***/
void XgM_help_sys();
void XgM_exit_program();
void XgM_Update_Coord();
void XgM_Clear_Coord();
void XgM_Exposure_Redraw();
void XgM_Gryscl_Exposure();
void XgM_Next_Frame();
void XgM_Move_Slider();
void XgM_Adjust_Cmap();
void XgM_Movie_On();
void XgM_Movie_Off();
void XgM_Forward();
void XgM_Reverse();
void XgM_Move_Frame();
void XgM_Adjust_Frame();
void XgM_ChangeScale();
void XgM_ChangeZoom();
void XgM_Flip_Frames();
void XgM_ChangeInt();
void XgM_Plot_Selection();
void XgM_Update_RbrBox();
void XgM_Create_Zoom();

/***********************************/

  XmStringCharSet XgMCharset = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
  char
    XgMFontDir[80],
    XgM_help_file[80];

/*************************************************************/
XgMovie_data * XgM_Initialize_Movie()
{
  XgMovie_data
    *mv_data;

mv_data = (XgMovie_data *) calloc(1, sizeof(XgMovie_data));
if (mv_data == NULL)
  {
    fprintf(stderr, "ERROR: Failed to calloc XgMovie_data struct. Aborted!\n");
    exit(1);
  }

mv_data->interval = XgM_INT_500;  /** default time in msec **/
mv_data->movie_single = XgM_MOVIE;
mv_data->direction = XgM_FORWARD;
mv_data->scaling = XgM_SCALE_LIN;
mv_data->zoom_fac = XgM_ZOOM_1;
mv_data->flip = XgM_FLIP_NORM;
mv_data->kill_exit = XgM_EXIT_KILL;
mv_data->rbrband_mode = FALSE;
mv_data->overlay = -1;
mv_data->count = 0;
strcpy(mv_data->control_name, "XgMcontrol");
strcpy(mv_data->screen_name, "XgMscreen");

return(mv_data);
}

/*************************************************************/
void XgM_Main_Entry
  (
   XgMovie_data * mv_data
   )
{
    int
      i,
      j,
      k,
      index,
      nargs,
      nbits,
      ncells,
      base,
      screen_num,
      color,
      min_value,
      max_value,
      nrows,
      ncols,
      nframes,
      npixels;
    double
      scale;
    char
      font_name[80],
      buf[160];
    long int
      fore_color,
      back_color;
    unsigned long
      gcmask,
      *pixels;
    Arg
      args[18];
    Widget
      button,
      frame1,
      frame2,
      cur_frame,
      pulldown,
      main_ctrl,
      form,
      form1,
      menu_bar,
      cursor_label,
      cursor_disp,
      min_box,
      max_box,
      slider_bar,
      grey_area,
      pixel_label,
      pixel_val,
      ctrl_panel;
    Pixmap
      grey_scale_pix;
    XFontStruct
      *font_struct;
    XmString
      label;
    XVisualInfo
      *vis_rtrn,
      vis_info;
    XColor
      overlay_rgb,
      overlay_hrdwr,
      *color_array;
    Colormap
      cmap;
    XGCValues
      gcvalues;

/***************** initial setup *************/
nrows = mv_data->nrows;
ncols = mv_data->ncols;
nframes = mv_data->nframes;
npixels = nrows * ncols;
mv_data->pix_array = (int2 *) calloc(npixels, sizeof(int2));
if (mv_data->pix_array == NULL)
  {
    fprintf(stderr, "ERROR: Failed to calloc pix_array. Aborted!\n");
    exit(1);
  }

/** find pixel min and max in data set **/
mv_data->pix_min = XgM_SCALE_MIN;
mv_data->pix_max = XgM_SCALE_MAX;
printf("Pixel minimum = %d, pixel maximum = %d\n",
       mv_data->pix_min, mv_data->pix_max);

mv_data->screen_num = DefaultScreen(mv_data->display);
sprintf(XgM_help_file, "%s%s", XgMDefHelpDir, XgMDefHelpFile);
strcpy(XgMFontDir, XgMDefFontDir);
font_struct = XLoadQueryFont(mv_data->display, XgMDefFont);
strcpy(font_name, XgMDefFont);

mv_data->icon_pixmap = XCreateBitmapFromData(mv_data->display,
	      RootWindow(mv_data->display, mv_data->screen_num),
	      bitmap_bits, bitmap_width, bitmap_height);

mv_data->grey_min = mv_data->pix_min;
mv_data->grey_max = mv_data->pix_max;

/************************************************************************
*
*                          Visuals & Colormap
*
*************************************************************************
*/

/****************** Find/Select Visual ************************/
vis_info.class = PseudoColor; /* must be adjustable */
vis_info.depth = 8; /** force character size data **/
vis_rtrn = XGetVisualInfo(mv_data->display, VisualClassMask | VisualDepthMask,
			  &vis_info, &nbits);
if (nbits == 0)
  {
    fprintf(stderr, "Unable to find a pseudocolor visual. Aborting!\n");
    exit(1);
  }

j = 0;
for (i=0; i<nbits; ++i)  /** find best visual **/
  {
/*    printf("%d: Visual with: depth=%d; class=%d; colormap_size=%d; id=%x; bprgb=%d\n",
	   i, vis_rtrn[i].depth, vis_rtrn[i].class, vis_rtrn[i].colormap_size,
	   vis_rtrn[i].visualid, vis_rtrn[i].bits_per_rgb);*/
/*    if (vis_rtrn[i].depth == 8) /** use depth = 8 **/
    if (vis_rtrn[i].depth > vis_rtrn[j].depth) /** find best depth **/
      j = i;
  }
vis_rtrn[0] = vis_rtrn[j];
printf(
   "\nSelected Visual %d with: depth=%d; class=%d; colormap_size=%d id=%0x\n",
       j, vis_rtrn[0].depth, vis_rtrn[0].class, vis_rtrn[0].colormap_size,
       vis_rtrn[0].visualid);

nbits = vis_rtrn[0].depth;
ncells = vis_rtrn[0].colormap_size;
if (ncells < XgM_MINCELLS)
  {
    fprintf(stderr, "Best visual has only %d color cells.\n", ncells);
    fprintf(stderr, "This program requires at least %d cells. Aborted\n",
	    XgM_MINCELLS);
    exit(1);
  }

pixels = (unsigned long *) calloc(ncells, sizeof(unsigned long));
if (pixels == NULL)
  {
    fprintf(stderr, "ERROR: Unable to calloc pixels array for colors.\n");
    exit(1);
  }
color_array = (XColor *) calloc(ncells, sizeof(XColor));
if (color_array == NULL)
  {
    fprintf(stderr, "ERROR: Unable to calloc color_array for colors.\n");
    exit(1);
  }

/***************** Create Colormap *******************/
cmap = XCreateColormap(mv_data->display,
		       RootWindow(mv_data->display, mv_data->screen_num),
		       vis_rtrn[0].visual, AllocNone);
if (cmap == DefaultColormap(mv_data->display, mv_data->screen_num))
  {
    fprintf(stderr, "Failed to create private colormap. Aborted!\n");
    exit(1);
  }
/*printf("Created private Colormap = 0x%x\n", cmap);*/

/**************** Setup Default Colors ***********************/
base = DisplayCells(mv_data->display, mv_data->screen_num);
i = base;
while ((XAllocColorCells(mv_data->display,
			DefaultColormap(mv_data->display, mv_data->screen_num),
			True, NULL, 0, pixels, i) == NULL) && (i > 5))
  {
    --i;
  }
if (i > 5)
  XFreeColors(mv_data->display,
	      DefaultColormap(mv_data->display, mv_data->screen_num),
	      pixels, i, 0);
base = base - i; /** this is the number of default cells in use **/
if ((ncells-base-XgM_SPARE < XgM_MIN_GREY) && (base > XgM_MIN_GREY-XgM_SPARE))
  base = XgM_MIN_GREY - XgM_SPARE;
printf("using %d cells from default colormap\n", base);

if (XAllocColorCells(mv_data->display, cmap, True,NULL,0,pixels, base) == NULL)
  {
    fprintf(stderr,
	    "Failed to allocate %d cells for standard colors. Aborted!\n",
	    base);
    exit(1);
  }
for (i=0; i<base; ++i)
  {
    color_array[i].pixel = pixels[i];
    color_array[i].flags = DoRed | DoGreen | DoBlue;
  }
XQueryColors(mv_data->display,
	     DefaultColormap(mv_data->display, mv_data->screen_num),
	     color_array, base);
XStoreColors(mv_data->display, cmap, color_array, base);

/*************** Allocate Grey Scale Cells *********************/
ncells = ncells - base - XgM_SPARE; /** only 256 distinct grey levels, but
				      this allows them to be allocated
				      easily to any portion of the range **/

if (XAllocColorCells(mv_data->display, cmap, True, NULL, 0, pixels, ncells)
    == NULL)
  {
    fprintf(stderr, "unable to allocate grey scale cells. Aborted.\n");
    exit(1);
  }
printf("allocated %d grey scale cells\n", ncells);

/*************** Set Initial Grey Scale Colors *********************/
scale = (double) XgM_XColorMax / (ncells - 1.);
for (i=0; i<ncells; ++i)
  {
    color = floor(scale * i + .5);
    color_array[i].pixel = pixels[i];
    color_array[i].red = color;
    color_array[i].green = color;
    color_array[i].blue = color;
    color_array[i].flags = DoRed | DoGreen | DoBlue;
  }
XStoreColors(mv_data->display, cmap, color_array, ncells);

/******************************/
mv_data->vis_info = &vis_rtrn[0];
mv_data->visual = vis_rtrn[0].visual;
mv_data->cmap = cmap;
mv_data->pixels = pixels;
mv_data->ncells = ncells;
mv_data->nbits = nbits;
mv_data->base = base;

/************************************************************************
*
*                   create control panel
*
**************************************************************************
*/
nargs = 0;
XtSetArg(args[nargs], XmNvisual, vis_rtrn); ++nargs;
XtSetArg(args[nargs], XmNcolormap, cmap); ++nargs;
XtSetArg(args[nargs], XmNdepth, nbits); ++nargs;
XtSetArg(args[nargs], XtNiconPixmap, mv_data->icon_pixmap); ++nargs;
ctrl_panel = XtAppCreateShell(mv_data->control_name, XgMClass,
			      applicationShellWidgetClass, mv_data->display,
			      args, nargs);
if (ctrl_panel == NULL)
  {
    fprintf(stderr, "Unable to create control panel. Aborting\n");
    exit(1);
  }
mv_data->ctrl_panel = ctrl_panel;

fore_color = XBlackPixelOfScreen(XtScreen(ctrl_panel));
back_color = XWhitePixelOfScreen(XtScreen(ctrl_panel));

/****** overlay color *******/
if (XLookupColor(mv_data->display, mv_data->cmap, XgMOverlayColor,
		 &overlay_rgb, &overlay_hrdwr) == 0)
  {
    fprintf(stderr,
	    "WARNING: could not find XgMOverlayColor = %s in the database\n",
	    XgMOverlayColor);
  }
if (XAllocColor(mv_data->display, mv_data->cmap, &overlay_hrdwr) != 0)
  {
    mv_data->overlay_pix = overlay_hrdwr.pixel;
  }
else
  {
    fprintf(stderr,
	    "Couldn't XAllocColor for XgMOverlayColor = %s. Using White.\n",
	    XgMOverlayColor);
    mv_data->overlay_pix = XWhitePixelOfScreen(XtScreen(ctrl_panel));
  }

nargs = 0;
XtSetArg(args[nargs], XmNscrollingPolicy, XmAPPLICATION_DEFINED); ++nargs;
main_ctrl = XmCreateMainWindow(ctrl_panel, "main", (ArgList) args, nargs);
XtManageChild(main_ctrl);

nargs = 0;
XtSetArg(args[nargs], XmNresizeWidth, False); ++nargs;
XtSetArg(args[nargs], XmNresizeHeight, False); ++nargs;
form = XmCreateForm(main_ctrl, "draw form", args, nargs);
XtManageChild(form);

/************* Create Menu Bar Buttons *************/
nargs = 0;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
menu_bar = XmCreateMenuBar(form, "menu_bar", args, nargs);
XtManageChild(menu_bar);

/***** help *****/
nargs = 0;    /* Menu Bar Button */
label = XmStringCreateLtoR("Help", XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetArg(args[nargs], XmNmnemonic, 'H'); ++nargs;
button = XmCreateCascadeButton(menu_bar, "Help", args, nargs);
XtAddCallback(button, (String) XmNactivateCallback,
	      XgM_help_sys, mv_data);
XtManageChild(button);
XtFree(label);
nargs = 0;
XtSetArg(args[nargs], XmNmenuHelpWidget, button); ++nargs;
XtSetValues(menu_bar, args, nargs);

/************* pulldowns ****************/
/***** File *****/
pulldown = XmCreatePulldownMenu(menu_bar, "File_pane", NULL, 0);

nargs = 0;   /* Exit Panel */
label = XmStringCreateLtoR("Exit", XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetArg(args[nargs], XmNsubMenuId, pulldown); ++nargs;
XtSetArg(args[nargs], XmNmnemonic, 'E'); ++nargs;
button = XmCreateCascadeButton(menu_bar, "Exit", args, nargs);
XtManageChild(button);
XtFree(label);

  nargs = 0;   /* Exit Button */
  label = XmStringCreateLtoR("Exit", XgMCharset);
  XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'E'); ++nargs;
  button = XmCreatePushButton(pulldown, "Exit", args, nargs);
  XtAddCallback(button, XmNactivateCallback, XgM_exit_program, mv_data);
  XtManageChild(button);
  XtFree(label);

/***** Options *****/
pulldown = XmCreatePulldownMenu(menu_bar, "print_pane", NULL, 0);

nargs = 0;   /* Print Panel */
label = XmStringCreateLtoR("Print", XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetArg(args[nargs], XmNsubMenuId, pulldown); ++nargs;
XtSetArg(args[nargs], XmNmnemonic, 'P'); ++nargs;
button = XmCreateCascadeButton(menu_bar, "Print", args, nargs);
XtManageChild(button);
XtFree(label);

  nargs = 0;   /* Print Button */
  label = XmStringCreateLtoR("Print", XgMCharset);
  XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
  XtSetArg(args[nargs], XmNmnemonic, 'P'); ++nargs;
  button = XmCreatePushButton(pulldown, "Print", args, nargs);
  XtAddCallback(button, XmNactivateCallback, XgM_Print, mv_data);
  XtManageChild(button);
  XtFree(label);

/**************** main info box *****************/

/************** cursor position ***************/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, menu_bar); ++nargs;
/*XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
cur_frame = XmCreateFrame(form, "increment frame", args, nargs);
XtManageChild(cur_frame);

nargs = 0;
XtSetArg(args[nargs], XmNresizeWidth, False); ++nargs;
XtSetArg(args[nargs], XmNresizeHeight, False); ++nargs;
form1 = XmCreateForm(cur_frame, "coord form", args, nargs);
XtManageChild(form1);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 5); ++nargs;
sprintf(buf, "Cursor: ");
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
cursor_label = XmCreateLabel(form1, "cursor label", args, nargs);
XtManageChild(cursor_label);
XtFree(label);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, menu_bar); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, cursor_label); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_IN); ++nargs;
button = XmCreateFrame(form1, "cursor frame", args, nargs);
XtManageChild(button);
nargs = 0;
sprintf(buf, "        ");
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
cursor_disp = XmCreateLabel(button, "cursor position", args, nargs);
XtManageChild(cursor_disp);
mv_data->cursor_disp = cursor_disp;
XtFree(label);

/*** pixel value display ***/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, button); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 5); ++nargs;
sprintf(buf, "Pixel Value: ");
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
pixel_label = XmCreateLabel(form1, "pixel label", args, nargs);
XtManageChild(pixel_label);
XtFree(label);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, button); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, pixel_label); ++nargs;
/*XtSetArg(args[nargs], XmNbottomAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNbottomOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_IN); ++nargs;
button = XmCreateFrame(form1, "pixel frame", args, nargs);
XtManageChild(button);
nargs = 0;
sprintf(buf, "     ");
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
pixel_val = XmCreateLabel(button, "pixel value", args, nargs);
XtManageChild(pixel_val);
mv_data->pixel_val = pixel_val;
XtFree(label);

/**************** Color Mapping ********************/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cur_frame); ++nargs;
/*XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
cur_frame = XmCreateFrame(form, "color map frame", args, nargs);
XtManageChild(cur_frame);

nargs = 0;
XtSetArg(args[nargs], XmNresizeWidth, False); ++nargs;
XtSetArg(args[nargs], XmNresizeHeight, False); ++nargs;
form1 = XmCreateForm(cur_frame, "coord form", args, nargs);
XtManageChild(form1);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
sprintf(buf, "GREY SCALE MAPPING");
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
cursor_label = XmCreateLabel(form1, "Mapping Label", args, nargs);
XtManageChild(cursor_label);
XtFree(label);

/** Minimum limit **/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cursor_label); ++nargs;
/*XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 5); ++nargs;
sprintf(buf, "MINIMUM");
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
cursor_label = XmCreateLabel(form1, "Minimum Label", args, nargs);
XtManageChild(cursor_label);
XtFree(label);

min_value = mv_data->pix_min;
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cursor_label); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_IN); ++nargs;
frame1 = XmCreateFrame(form1, "min frame", args, nargs);
XtManageChild(frame1);
nargs = 0;
sprintf(buf, "%4d", min_value);
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
min_box = XmCreateLabel(frame1, "min value", args, nargs);
XtManageChild(min_box);
mv_data->min_box = min_box;
XtFree(label);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cursor_label); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, frame1); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNminimum, mv_data->pix_min); ++nargs; /*250 is kludge*/
XtSetArg(args[nargs], XmNmaximum, mv_data->pix_max + 250); ++nargs;
XtSetArg(args[nargs], XmNsliderSize, 10); ++nargs;
XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
/*XtSetArg(args[nargs], XmNshowArrows, FALSE); ++nargs;*/
XtSetArg(args[nargs], XmNvalue, min_value); ++nargs;
slider_bar = XmCreateScrollBar(form1, "min scroll", args, nargs);
XtManageChild(slider_bar);
mv_data->min_bar = slider_bar;
XtAddCallback(slider_bar, XmNdragCallback, XgM_Move_Slider, mv_data);
XtAddCallback(slider_bar, XmNvalueChangedCallback, XgM_Adjust_Cmap, mv_data);

/** Maximum limit **/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, frame1); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 5); ++nargs;
sprintf(buf, "MAXIMUM");
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
frame1 = XmCreateLabel(form1, "Maximum Label", args, nargs);
XtManageChild(frame1);
XtFree(label);

max_value = mv_data->pix_max;
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, frame1); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_IN); ++nargs;
frame2 = XmCreateFrame(form1, "max frame", args, nargs);
XtManageChild(frame2);
nargs = 0;
sprintf(buf, "%4d", max_value);
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
max_box = XmCreateLabel(frame2, "max value", args, nargs);
XtManageChild(max_box);
mv_data->max_box = max_box;
XtFree(label);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, frame1); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNleftWidget, frame2); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNminimum, mv_data->pix_min); ++nargs; /*250 is kludge*/
XtSetArg(args[nargs], XmNmaximum, mv_data->pix_max + 250); ++nargs;
XtSetArg(args[nargs], XmNsliderSize, 10); ++nargs;
XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
/*XtSetArg(args[nargs], XmNshowArrows, FALSE); ++nargs;*/
XtSetArg(args[nargs], XmNvalue, max_value-10); ++nargs;
slider_bar = XmCreateScrollBar(form1, "max scroll", args, nargs);
XtManageChild(slider_bar);
mv_data->max_bar = slider_bar;
XtAddCallback(slider_bar, XmNdragCallback, XgM_Move_Slider, mv_data);
XtAddCallback(slider_bar, XmNvalueChangedCallback, XgM_Adjust_Cmap, mv_data);

/** grey scale display box **/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, frame2); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftOffset, 5); ++nargs;
/*XtSetArg(args[nargs], XmNbottomAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNbottomOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNwidth, XgM_SCL_WDTH); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
frame2 = XmCreateFrame(form1, "scale display frame", args, nargs);
XtManageChild(frame2);

nargs = 0;
/*XtSetArg(args[nargs], XmNbackground, back_color); ++nargs;*/
XtSetArg(args[nargs], XmNwidth, XgM_SCL_WDTH); ++nargs;
XtSetArg(args[nargs], XmNheight, XgM_SCL_HT); ++nargs;
grey_area = XmCreateDrawingArea(frame2, "grey area", args, nargs);
XtManageChild(grey_area);
mv_data->grey_area = grey_area;

/***************** Scaling ********************************/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cur_frame); ++nargs;
/*XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
cur_frame = XmCreateFrame(form, "scaling frame", args, nargs);
XtManageChild(cur_frame);

nargs = 0;
XtSetArg(args[nargs], XmNresizeWidth, False); ++nargs;
XtSetArg(args[nargs], XmNresizeHeight, False); ++nargs;
form1 = XmCreateForm(cur_frame, "scaling form", args, nargs);
XtManageChild(form1);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
sprintf(buf, "Scaling");
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
cursor_label = XmCreateLabel(form1, "Scaling Label", args, nargs);
XtManageChild(cursor_label);
XtFree(label);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cursor_label); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
XtSetArg(args[nargs], XmNnumColumns, 2); ++nargs; /* select 2 rows */
frame2 = XmCreateRadioBox(form1, "radio box", args, nargs);
XtManageChild(frame2);

nargs = 0;
if (mv_data->scaling == XgM_SCALE_LIN)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "Linear", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_ChangeScale, mv_data);
mv_data->sbut_lin = button;
XtManageChild(button);
nargs = 0;
if (mv_data->scaling == XgM_SCALE_LOG)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "Log", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_ChangeScale, mv_data);
mv_data->sbut_log = button;
XtManageChild(button);
nargs = 0;
if (mv_data->scaling == XgM_SCALE_SQRT)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "Sqrt", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_ChangeScale, mv_data);
mv_data->sbut_sqrt = button;
XtManageChild(button);
nargs = 0;
if (mv_data->scaling == XgM_SCALE_HIST)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "HistEq", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_ChangeScale, mv_data);
mv_data->sbut_hist = button;
XtManageChild(button);

/***************** Zoom Factor ****************************/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cur_frame); ++nargs;
/*XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
cur_frame = XmCreateFrame(form, "zoom frame", args, nargs);
XtManageChild(cur_frame);

nargs = 0;
XtSetArg(args[nargs], XmNresizeWidth, False); ++nargs;
XtSetArg(args[nargs], XmNresizeHeight, False); ++nargs;
form1 = XmCreateForm(cur_frame, "zoom form", args, nargs);
XtManageChild(form1);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
sprintf(buf, "Zoom Factor");
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
cursor_label = XmCreateLabel(form1, "Zoom Label", args, nargs);
XtManageChild(cursor_label);
XtFree(label);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cursor_label); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
/*XtSetArg(args[nargs], XmNnumColumns, 2); ++nargs; /* select 2 rows */
frame2 = XmCreateRadioBox(form1, "zoom box", args, nargs);
XtManageChild(frame2);

nargs = 0;
if (mv_data->zoom_fac == XgM_ZOOM_1)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "1X", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_ChangeZoom, mv_data);
mv_data->zbut_1 = button;
XtManageChild(button);
nargs = 0;
if (mv_data->zoom_fac == XgM_ZOOM_2)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "2X", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_ChangeZoom, mv_data);
mv_data->zbut_2 = button;
XtManageChild(button);
nargs = 0;
if (mv_data->zoom_fac == XgM_ZOOM_4)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "4X", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_ChangeZoom, mv_data);
mv_data->zbut_4 = button;
XtManageChild(button);
nargs = 0;
if (mv_data->zoom_fac == XgM_ZOOM_8)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "8X", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_ChangeZoom, mv_data);
mv_data->zbut_8 = button;
XtManageChild(button);

/***************** Vertical Flip **************************/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cur_frame); ++nargs;
/*XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
cur_frame = XmCreateFrame(form, "flip frame", args, nargs);
XtManageChild(cur_frame);
nargs = 0;
XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
frame2 = XmCreateRadioBox(cur_frame, "flip box", args, nargs);
XtManageChild(frame2);

nargs = 0;
if (mv_data->flip == XgM_FLIP_NORM)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "Normal", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_Flip_Frames, mv_data);
mv_data->fbut_norm = button;
XtManageChild(button);
nargs = 0;
if (mv_data->flip == XgM_FLIP_REV)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "V. Flip", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_Flip_Frames, mv_data);
mv_data->fbut_rev = button;
XtManageChild(button);

/***************** movie/single radio box *****************/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cur_frame); ++nargs;
/*XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
cur_frame = XmCreateFrame(form, "radio frame", args, nargs);
XtManageChild(cur_frame);
nargs = 0;
frame2 = XmCreateRadioBox(cur_frame, "radio box", args, nargs);
XtManageChild(frame2);
nargs = 0;
if (mv_data->movie_single == XgM_MOVIE)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "Movie Loop", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_Movie_On, mv_data);
XtManageChild(button);
nargs = 0;
if (mv_data->movie_single == XgM_SINGLE)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "Single Frame", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_Movie_Off, mv_data);
XtManageChild(button);

/***************** Direction radio box *****************/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cur_frame); ++nargs;
/*XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
cur_frame = XmCreateFrame(form, "direction frame", args, nargs);
XtManageChild(cur_frame);
nargs = 0;
XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
/*XtSetArg(args[nargs], XmNnumColumns, 2); ++nargs; /* select 2 rows */
frame2 = XmCreateRadioBox(cur_frame, "direction box", args, nargs);
XtManageChild(frame2);

nargs = 0;
if (mv_data->direction == XgM_FORWARD)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "Forward", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_Forward, mv_data);
XtManageChild(button);
nargs = 0;
if (mv_data->direction == XgM_REVERSE)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "Reverse", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_Reverse, mv_data);
XtManageChild(button);

/***************** Time increment radio box *****************/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cur_frame); ++nargs;
/*XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;*/
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
cur_frame = XmCreateFrame(form, "increment frame", args, nargs);
XtManageChild(cur_frame);

nargs = 0;
XtSetArg(args[nargs], XmNresizeWidth, False); ++nargs;
XtSetArg(args[nargs], XmNresizeHeight, False); ++nargs;
form1 = XmCreateForm(cur_frame, "increment form", args, nargs);
XtManageChild(form1);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
sprintf(buf, "Time Interval (sec)");
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
cursor_label = XmCreateLabel(form1, "Interval Label", args, nargs);
XtManageChild(cursor_label);
XtFree(label);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, cursor_label); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
/*XtSetArg(args[nargs], XmNnumColumns, 2); ++nargs; /* select 2 rows */
frame2 = XmCreateRadioBox(form1, "radio box", args, nargs);
XtManageChild(frame2);

nargs = 0;
if (mv_data->interval == XgM_INT_100)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, ".1", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_ChangeInt, mv_data);
mv_data->but_100 = button;
XtManageChild(button);
nargs = 0;
if (mv_data->interval == XgM_INT_200)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, ".2", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_ChangeInt, mv_data);
mv_data->but_200 = button;
XtManageChild(button);
nargs = 0;
if (mv_data->interval == XgM_INT_500)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, ".5", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_ChangeInt, mv_data);
mv_data->but_500 = button;
XtManageChild(button);
nargs = 0;
if (mv_data->interval == XgM_INT_1000)
  { XtSetArg(args[nargs], XmNset, True); ++nargs; }
XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
button = XmCreateToggleButton(frame2, "1", args, nargs);
XtAddCallback(button, XmNarmCallback, XgM_ChangeInt, mv_data);
mv_data->but_1000 = button;
XtManageChild(button);

XtRealizeWidget(ctrl_panel);

/***************** Create GCs ***********************/
gcvalues.function = GXcopy;
gcvalues.line_style = LineSolid;
gcmask = GCFunction | GCLineStyle;
mv_data->gc = XCreateGC(mv_data->display, XtWindow(grey_area),
			gcmask, &gcvalues);
if (mv_data->gc == NULL)
  {
    fprintf(stderr, "Failure to create Graphics Context\n");
    exit(1);
  }

gcvalues.function = GXinvert;   /** have to use this instead of GXxor **/
gcvalues.line_style = LineSolid;
gcmask = GCFunction | GCLineStyle;
/*gcvalues.line_style = LineOnOffDash;*/
mv_data->gcxor = XCreateGC(mv_data->display, XtWindow(grey_area),
			   gcmask, &gcvalues);
if (mv_data->gcxor == NULL)
  {
    fprintf(stderr, "Failure to create Xor Graphics Context\n");
    exit(1);
  }

/************** Create Grey Scale Display Pixmap *****************/
grey_scale_pix = XCreatePixmap(mv_data->display, XtWindow(grey_area),
			       XgM_SCL_WDTH, XgM_SCL_HT, nbits);
if (grey_scale_pix == NULL)
  {
    fprintf(stderr,
	    "Failure to allocate grey_scale_pix with XCreatePixmap.\n");
    exit(1);
  }
mv_data->grey_pix = grey_scale_pix;

i = XgM_SCL_HT - 1;  /** create grey scale display **/
scale = (double) (ncells - 1)/ (XgM_SCL_WDTH - 1);
for (j=0; j<XgM_SCL_WDTH; ++j)
  {
    color = floor(j * scale + .5);
    XSetForeground(mv_data->display, mv_data->gc, pixels[color]);
    XDrawLine(mv_data->display, grey_scale_pix, mv_data->gc, j, 0, j, i);
  }
XCopyArea(mv_data->display, grey_scale_pix, XtWindow(grey_area),
	  mv_data->gc, 0, 0, XgM_SCL_WDTH, XgM_SCL_HT, 0, 0);
XtAddEventHandler(grey_area, ExposureMask, FALSE, XgM_Gryscl_Exposure,
		  mv_data);

XgM_Create_Movie_Screen(mv_data);

/***************** Finish-up; Enter MainLoop *************/
XtAppAddTimeOut(mv_data->theAppContext, mv_data->interval, XgM_Next_Frame,
		mv_data);
}

/********************************************************************
*
*                        create movie screen
*
*********************************************************************
*/
void XgM_Create_Movie_Screen(mv_data)
     XgMovie_data * mv_data;
{
  int
    i,
    k,
    nargs,
    nrows,
    ncols,
    nframes,
    npixels,
    nbits,
    ncells,
    zoom_fac,
    width,
    height;
  char
    buf[160];
  long int
    fore_color,
    back_color;
  Cursor
    crosshair;
  XColor
    xhair_fore,
    xhair_back,
    xhair_null;
  Display
    *display;
  Arg
    args[18];
  XmString
    label;
  Widget
    mv_scrn,
    main_dspl,
    frame_box,
    draw_area,
    form,
    frame1,
    frame2,
    slider_bar;
  XWindowAttributes
    win_attr;
  XSizeHints
    size_hints;

/***** setup ******/
nrows = mv_data->nrows;
ncols = mv_data->ncols;
nframes = mv_data->nframes;
ncells = mv_data->ncells;
nbits = mv_data->nbits;
npixels = nrows * ncols;
display = mv_data->display;

switch (mv_data->zoom_fac)
  {
  case XgM_ZOOM_1:
    zoom_fac = 1;
    break;
  case XgM_ZOOM_2:
    zoom_fac = 2;
    break;
  case XgM_ZOOM_4:
    zoom_fac = 4;
    break;
  case XgM_ZOOM_8:
    zoom_fac = 8;
    break;
  default:
    fprintf(stderr, "Internal Error: Bad Zoom Factor. Aborted\n");
    exit(1);
    break;
  }

width = ncols * zoom_fac;
height = nrows * zoom_fac;

/****************** Allocate Movie Pixmaps *************/
mv_data->mv_img = XCreatePixmap(display, DefaultRootWindow(display),
				   width, height, nbits);
  if (mv_data->mv_img == NULL)
    {
      fprintf(stderr,
	      "Failure to allocate movie_image with XCreatePixmap.\n");
      exit(1);
    }

XgM_Read_Frame(mv_data);
/*XgM_Fill_Pixmaps(mv_data);*/

/********* create display window app shell **********/
nargs = 0;
XtSetArg(args[nargs], XmNvisual, mv_data->vis_info); ++nargs;
XtSetArg(args[nargs], XmNcolormap, mv_data->cmap); ++nargs;
XtSetArg(args[nargs], XmNdepth, nbits); ++nargs;
XtSetArg(args[nargs], XtNiconPixmap, mv_data->icon_pixmap); ++nargs;
mv_scrn = XtAppCreateShell(mv_data->screen_name, XgMClass,
			    applicationShellWidgetClass, display, args, nargs);
if (mv_scrn == NULL)
  {
    fprintf(stderr, "Unable to create Movie Screen. Aborting\n");
    exit(1);
  }
mv_data->mv_scrn = mv_scrn;

fore_color = XBlackPixelOfScreen(XtScreen(mv_scrn));
back_color = XWhitePixelOfScreen(XtScreen(mv_scrn));

nargs = 0;
XtSetArg(args[nargs], XmNscrollingPolicy, XmAPPLICATION_DEFINED); ++nargs;
main_dspl = XmCreateMainWindow(mv_scrn, "main", (ArgList) args, nargs);
XtManageChild(main_dspl);

nargs = 0;
/*XtSetArg(args[nargs], XmNwidth, width); ++nargs;
XtSetArg(args[nargs], XmNheight, height); ++nargs;*/
XtSetArg(args[nargs], XmNresizeWidth, False); ++nargs;
XtSetArg(args[nargs], XmNresizeHeight, False); ++nargs;
form = XmCreateForm(main_dspl, "draw form", args, nargs);
XtManageChild(form);

/**************** Frame Number Display ***************/
nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
sprintf(buf, "%s\nFrame Number", mv_data->filename);
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
frame1 = XmCreateLabel(form, "frame label", args, nargs);
XtManageChild(frame1);
XtFree(label);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, frame1); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_IN); ++nargs;
frame2 = XmCreateFrame(form, "frame frame", args, nargs);
XtManageChild(frame2);
nargs = 0;
sprintf(buf, "%4d", mv_data->count);
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
frame_box = XmCreateLabel(frame2, "frame number", args, nargs);
XtManageChild(frame_box);
mv_data->frame_box = frame_box;
XtFree(label);

nargs = 0;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, frame1); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNrightWidget, frame2); ++nargs;
XtSetArg(args[nargs], XmNminimum, 0); ++nargs;
XtSetArg(args[nargs], XmNmaximum, mv_data->nframes); ++nargs;
XtSetArg(args[nargs], XmNvalue, mv_data->count); ++nargs;
XtSetArg(args[nargs], XmNsliderSize, 1); ++nargs;
XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
/*XtSetArg(args[nargs], XmNshowArrows, FALSE); ++nargs;*/
slider_bar = XmCreateScrollBar(form, "frame scroll", args, nargs);
XtManageChild(slider_bar);
mv_data->frame_scroll = slider_bar;
XtAddCallback(slider_bar, XmNdragCallback, XgM_Move_Frame, mv_data);
XtAddCallback(slider_bar, XmNvalueChangedCallback, XgM_Adjust_Frame, mv_data);

/**************** display area ************/
nargs = 0;
XtSetArg(args[nargs], XmNwidth, width); ++nargs;
XtSetArg(args[nargs], XmNheight, height); ++nargs;
XtSetArg(args[nargs], XmNborderWidth, 0); ++nargs;
XtSetArg(args[nargs], XmNbackground, back_color); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, frame2); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNbottomAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
draw_area = XmCreateDrawingArea(form, "draw area", args, nargs);
XtManageChild(draw_area);
mv_data->draw_area = draw_area;
XtAddEventHandler(draw_area, PointerMotionMask, FALSE, XgM_Update_Coord,
		  mv_data);
XtAddEventHandler(draw_area, LeaveWindowMask, FALSE, XgM_Clear_Coord,
		  mv_data);
XtAddEventHandler(draw_area, ButtonPressMask, FALSE, XgM_Plot_Selection,
		  mv_data);
XtAddEventHandler(draw_area, PointerMotionMask, FALSE, XgM_Update_RbrBox,
		  mv_data);
XtAddEventHandler(draw_area, ButtonReleaseMask, FALSE, XgM_Create_Zoom,
		  mv_data);
XtAddEventHandler(draw_area, ExposureMask, FALSE, XgM_Exposure_Redraw,
		  mv_data);

XtRealizeWidget(mv_scrn);
XCopyArea(mv_data->display, mv_data->mv_img,
	  XtWindow(mv_data->draw_area), mv_data->gc,
	  0, 0, mv_data->ncols*zoom_fac, mv_data->nrows*zoom_fac, 0, 0);

/******* cross hair cursor *******/
crosshair = XCreateFontCursor(display, XC_crosshair);
XLookupColor(display, mv_data->cmap, "red", &xhair_null, &xhair_fore);
XLookupColor(display, mv_data->cmap, "black", &xhair_null, &xhair_back);
XRecolorCursor(display, crosshair, &xhair_fore, &xhair_back);
/*XRecolorCursor(display, crosshair, &xhair_fore, &xhair_fore);*/
XDefineCursor(display, XtWindow(draw_area), crosshair);

/** don't allow user to resize this window **/
XGetWindowAttributes(display, XtWindow(mv_scrn), &win_attr);
size_hints.min_width = win_attr.width;
size_hints.min_height = win_attr.height;
size_hints.max_width = win_attr.width;
size_hints.max_height = win_attr.height;
size_hints.flags = PMinSize | PMaxSize;
XSetWMNormalHints(display, XtWindow(mv_scrn), &size_hints);
}

/***********************************************************************
*
*                           Help & Error System
*
************************************************************************/

/****************************/
void XgM_help_sys(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    rslt,
    count,
    nargs;
  char
    buf[80];
  FILE
    *fp;
  Arg
    args[5];
  XmString
    okaymsg,
    title,
    help_message[XgM_Max_Help];  /** make sure this is big enough **/
  Widget
    help_box,
    button;
  XgMovie_data
    *mv_data;

mv_data = (XgMovie_data *) client_data;

fp = NULL;
fp = fopen(XgM_help_file, "r");
if (fp == NULL)
  {
    sprintf(buf, "%s", XgMDefHelpFile);
    fp = fopen(buf, "r");
    if (fp == NULL)
      {
	sprintf(buf, "Unable to find Help File: %s", XgMDefHelpFile);
	help_message[0]
	  = XmStringCreateLtoR(buf, XgMCharset);
	count = 1;
      }
  }

if (fp != NULL)
  {
    count = 0;
    rslt = fscanf(fp, "%[^\n]", buf);
    getc(fp);
    while (rslt != EOF)
      {
	help_message[count] = XmStringCreateLtoR(buf, XgMCharset);
	++count;
	buf[0] = '\0';   /** for blank lines **/
	rslt = fscanf(fp, "%[^\n]", buf);
	getc(fp);
	if (count >= XgM_Max_Help)
	  {
	    XgM_error_sys(XgM_HELP_FILE_TOO_BIG, mv_data);
	    rslt = EOF;
	  }
      }
    fclose(fp);
  }

title = XmStringCreateLtoR("XgM: Help", XgMCharset);
okaymsg = XmStringCreateLtoR("I get it", XgMCharset);
nargs = 0;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNlistItemCount, count); ++nargs;
XtSetArg(args[nargs], XmNlistItems, help_message); ++nargs;
XtSetArg(args[nargs], XmNlistVisibleItemCount, 12); ++nargs;
/*XtSetArg(args[nargs], XmNtextColumns, 40); ++nargs;*/
XtSetArg(args[nargs], XmNokLabelString, okaymsg); ++nargs;
/*XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;*/
help_box = (Widget) XmCreateSelectionDialog(mv_data->ctrl_panel, "XgM: Help",
					    args, nargs);

button = (Widget) XmSelectionBoxGetChild(help_box, XmDIALOG_CANCEL_BUTTON);
XtUnmanageChild(button);
button = (Widget) XmSelectionBoxGetChild(help_box, XmDIALOG_HELP_BUTTON);
XtUnmanageChild(button);
button = (Widget) XmSelectionBoxGetChild(help_box, XmDIALOG_APPLY_BUTTON);
XtUnmanageChild(button);
button = (Widget) XmSelectionBoxGetChild(help_box, XmDIALOG_LIST_LABEL);
XtUnmanageChild(button);
button = (Widget) XmSelectionBoxGetChild(help_box, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(button);
button = (Widget) XmSelectionBoxGetChild(help_box, XmDIALOG_TEXT);
XtUnmanageChild(button);

for (rslt=0; rslt<count; ++rslt)
  XtFree(help_message[rslt]);
XtFree(okaymsg);
XtFree(title);
XtManageChild(help_box);
}

/******************* error system *************************/
void XgM_error_sys(error_type, mv_data)
     int
       error_type;
     XgMovie_data
       *mv_data;
{
  int
    nargs;
  Arg
    args[10];
  XmString
    message,
    title,
    ok_message;
  Widget
    error_box,
    button;

switch (error_type)
  {
  case XgM_ZERO_WIDTH:
    message = XmStringCreateLtoR("WARNING: Can't zoom zero width/height area",
				 XgMCharset);
    break;
  case XgM_HELP_FILE_TOO_BIG:
    message = XmStringCreateLtoR("WARNING: Help file too big. Increase\n\
  XgM_Max_Help in XgMovie & recompile.\n\
  Rest of help has been ignored.", XgMCharset);
    break;
  case XgM_BAD_CALLOC:
    message = XmStringCreateLtoR("ERROR: unable to calloc data for plot.",
				 XgMCharset);
    break;
  default:
    message = XmStringCreateLtoR("Internal Error: No such error!", XgMCharset);
    break;
  }

ok_message = XmStringCreateLtoR("Continue", XgMCharset);
title = XmStringCreateLtoR("XgM: Error System", XgMCharset);

nargs = 0;
XtSetArg(args[nargs], XmNmessageString, message); ++nargs;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNokLabelString, ok_message); ++nargs;
error_box = XmCreateErrorDialog(mv_data->ctrl_panel, "XgM: Error System",
				args, nargs);

button = XmMessageBoxGetChild(error_box, XmDIALOG_HELP_BUTTON);
XtUnmanageChild(button);
button = XmMessageBoxGetChild(error_box, XmDIALOG_CANCEL_BUTTON);
XtUnmanageChild(button);
XtManageChild(error_box);
XtFree(message);
XtFree(title);
XtFree(ok_message);
}

/*********** program exit ****************/
void XgM_exit_program(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i;
  XWindowAttributes
    win_attr;
  XgMovie_data
    *mv_data;

mv_data = (XgMovie_data *) client_data;
printf("\n");
/*XGetWindowAttributes(mv_data->display, XtWindow(mv_data->grey_area),
  &win_attr);
printf("********** grey_area attributes **********\n");
printf("depth=%d; Visual=0x%x; cmap=0x%x\n", win_attr.depth,
       XVisualIDFromVisual(win_attr.visual), win_attr.colormap);*/
if (mv_data->kill_exit == XgM_EXIT_KILL)
  {
    mv_data->movie_single = 0; /** to avoid a callback **/
    free(mv_data->pix_array);
    free(mv_data->pixels);
    XtDestroyWidget(mv_data->mv_scrn);
    XtDestroyWidget(mv_data->ctrl_panel);
    XFreePixmap(mv_data->display, mv_data->mv_img);
    free(mv_data);
    printf("terminating XgMovie\n");
    exit(0);
  }
else  /** exit movie but don't kill the process **/
  {
    mv_data->movie_single = 0; /** to avoid a callback **/
    free(mv_data->pix_array);
    free(mv_data->pixels);
    XtDestroyWidget(mv_data->mv_scrn);
    XtDestroyWidget(mv_data->ctrl_panel);
    XFreePixmap(mv_data->display, mv_data->mv_img);
    free(mv_data);
  }
}

/**********************************************************
*
*                      Callbacks
*
***********************************************************
*/

/**************** Cursor Positioning ***********************/
void XgM_Update_Coord(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    x,
    y,
    zoom_fac,
    nargs;
  char
    buf[80];
  XgMovie_data
    *mv_data;
  Arg
    args[10];
  XEvent
    *mouse_ev;
  Widget
    cursor_disp;
  XmString
    label;

mv_data = (XgMovie_data *) client_data;
mouse_ev = (XEvent *) call_data;
switch (mv_data->zoom_fac)
  {
  case XgM_ZOOM_1:
    zoom_fac = 1;
    break;
  case XgM_ZOOM_2:
    zoom_fac = 2;
    break;
  case XgM_ZOOM_4:
    zoom_fac = 4;
    break;
  case XgM_ZOOM_8:
    zoom_fac = 8;
    break;
  default:
    fprintf(stderr, "Internal Error: Bad Zoom Factor. Aborted\n");
    exit(1);
    break;
  }

nargs = 0;
x = mouse_ev->xmotion.x;
if (mv_data->flip == XgM_FLIP_NORM)
  y = mv_data->nrows*zoom_fac - mouse_ev->xmotion.y - 1;
else
  y = mouse_ev->xmotion.y;
x = floor((double) x / zoom_fac);
y = floor((double) y / zoom_fac);
sprintf(buf, "%4d %4d", x + mv_data->xoff, y + mv_data->yoff);
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetValues(mv_data->cursor_disp, args, nargs);
XtFree(label);
if (mv_data->movie_single == XgM_SINGLE)
  {
    nargs = 0;
    sprintf(buf, "%5d", mv_data->pix_array[y * mv_data->ncols + x]);
    label = XmStringCreateLtoR(buf, XgMCharset);
    XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
    XtSetValues(mv_data->pixel_val, args, nargs);
    XtFree(label);
  }
}
/***************************************/
void XgM_Clear_Coord(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    nargs;
  char
    buf[80];
  XgMovie_data
    *mv_data;
  Arg
    args[10];
  XEvent
    *mouse_ev;
  Widget
    cursor_disp;
  XmString
    label;

mv_data = (XgMovie_data *) client_data;
mouse_ev = (XEvent *) call_data;

nargs = 0;
sprintf(buf, "         ");
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetValues(mv_data->cursor_disp, args, nargs);
XtFree(label);

nargs = 0;
sprintf(buf, "     ");
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetValues(mv_data->pixel_val, args, nargs);
XtFree(label);
}

/******* Exposures **********/
void XgM_Exposure_Redraw(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;

{
  int
    zoom_fac;
  XgMovie_data
    *mv_data;

mv_data = (XgMovie_data *) client_data;
if (mv_data->zoom_fac == XgM_ZOOM_1)
  zoom_fac = 1;
else if (mv_data->zoom_fac == XgM_ZOOM_2)
  zoom_fac = 2;
else if (mv_data->zoom_fac == XgM_ZOOM_4)
  zoom_fac = 4;
else if (mv_data->zoom_fac == XgM_ZOOM_8)
  zoom_fac = 8;
else
  {
    fprintf(stderr, "ERROR: bad zoom factor. Aborted\n");
    exit(1);
  }

if (mv_data->movie_single == XgM_SINGLE)
  XCopyArea(mv_data->display, mv_data->mv_img,
	    XtWindow(mv_data->draw_area), mv_data->gc,
	    0, 0, mv_data->ncols*zoom_fac, mv_data->nrows*zoom_fac, 0, 0);
}

/*******************/
void XgM_Gryscl_Exposure(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;

{
  int
    zoom_fac;
  XgMovie_data
    *mv_data;

mv_data = (XgMovie_data *) client_data;
if (mv_data->zoom_fac == XgM_ZOOM_1)
  zoom_fac = 1;
else if (mv_data->zoom_fac == XgM_ZOOM_2)
  zoom_fac = 2;
else if (mv_data->zoom_fac == XgM_ZOOM_4)
  zoom_fac = 4;
else if (mv_data->zoom_fac == XgM_ZOOM_8)
  zoom_fac = 8;
else
  {
    fprintf(stderr, "ERROR: bad zoom factor. Aborted\n");
    exit(1);
  }

if (mv_data->movie_single == XgM_SINGLE)
  XCopyArea(mv_data->display, mv_data->grey_pix, XtWindow(mv_data->grey_area),
	    mv_data->gc, 0, 0, XgM_SCL_WDTH, XgM_SCL_HT, 0, 0);
  }

/**************** Row, Column, or Histogram Plotting ***********************/
void XgM_Plot_Selection(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i,
    x,
    y,
    index,
    nrows,
    ncols,
    npixels,
    min,
    max,
    delta,
    nbins,
    val,
    zoom_fac;
  double
    scale,
    *data;
  char
    buf[80];
  XgMovie_data
    *mv_data;
  XEvent
    *mouse_ev;
  Widget
    graph_widget;

#ifdef USE_PLOTS
  XgGraphInfo
    *graph;
#endif

mv_data = (XgMovie_data *) client_data;
mouse_ev = (XEvent *) call_data;

nrows = mv_data->nrows;
ncols = mv_data->ncols;
npixels = nrows * ncols;
min = mv_data->pix_min;
max = mv_data->pix_max;
delta = max - min + 1;
if (mv_data->zoom_fac == XgM_ZOOM_1)
  zoom_fac = 1;
else if (mv_data->zoom_fac == XgM_ZOOM_2)
  zoom_fac = 2;
else if (mv_data->zoom_fac == XgM_ZOOM_4)
  zoom_fac = 4;
else if (mv_data->zoom_fac == XgM_ZOOM_8)
  zoom_fac = 8;
else
  {
    fprintf(stderr, "Internal Error: Bad Zoom Factor. Aborted\n");
    exit(1);
  }

/*** find location of pointer when pressed ***/
x = mouse_ev->xmotion.x;
if (mv_data->flip == XgM_FLIP_NORM)
  y = nrows*zoom_fac - mouse_ev->xmotion.y - 1;
else
  y = mouse_ev->xmotion.y;
x = floor((double) x / zoom_fac);
y = floor((double) y / zoom_fac);
index = mv_data->count * npixels;
index = 0;  /** for read frame mode, only have current frame in memory **/

/************ Start Rubber band for Zoom Area *****************/
if (mouse_ev->xbutton.state & ShiftMask)
  {
    mv_data->rbrband_mode = TRUE;
    mv_data->x0 = x;
    mv_data->y0 = y;
/**/printf("Start coordinates = (%d %d)\n", x, y);
    x = mouse_ev->xmotion.x;
    y = mouse_ev->xmotion.y;
    mv_data->xp0 = x;
    mv_data->yp0 = y;
    mv_data->xp1 = x;
    mv_data->yp1 = y;
    XDrawRectangle(mv_data->display, XtWindow(mv_data->draw_area),
		   mv_data->gcxor, x, y, 0, 0);

  }

#ifdef USE_PLOTS
/********* x-y plots ****************/
if (mouse_ev->xbutton.state & ControlMask)
  {
    graph = XgCreateStruct();
    graph->num_sets = 1;
    graph->kill_flag = XgNoKillOnExit;
    graph->dataset[0].line_status = XgSolidLine;
    graph->dataset[0].point_type = XgPoint;
    graph->xaxis_layout.nsig_fig = XgSig2;
    graph->xaxis_layout.increment = XgLast5;
    graph->xaxis_layout.label_type = XgNo_label;
    graph->xaxis_layout.user_set_flag = XgUinc | XgUnsig | XgUscale;
    graph->yaxis_layout.nsig_fig = XgSig1;
    graph->yaxis_layout.increment = XgLast1;
    graph->yaxis_layout.label_type = XgNo_label;
    graph->yaxis_layout.user_set_flag = XgUinc | XgUnsig | XgUscale;

    if (mouse_ev->xbutton.button == Button1)  /** plot row **/
      {
	data = (double *) calloc(ncols*2, sizeof(double));
	if (data == NULL)
	  {
	    XgM_error_sys(XgM_BAD_CALLOC, mv_data);
	    return;
	  }
	for (i=0; i<ncols; ++i)
	  {
	    *(data + 2*i) = (double) i + mv_data->xoff;
	    *(data + 2*i + 1) = (double) mv_data->pix_array[index +y*ncols +i];
	  }
	graph->dataset[0].num_points = ncols;
	sprintf(graph->title_desc.text, "File %s", mv_data->filename);
	sprintf(graph->subtitle_desc.text, "Frame %d; Row %d", mv_data->count,
		y + mv_data->yoff);
	sprintf(graph->x_axis_desc.text, "X Pixel Location");
	sprintf(graph->y_axis_desc.text, "Pixel Value");
	sprintf(graph->dataset[0].legend_text, "Row %d data", y+mv_data->yoff);
      }

    if (mouse_ev->xbutton.button == Button2)  /** plot histogram **/
      {
	if (delta < 500)
	  nbins = delta;
	else
	  nbins = 500;
	scale = (double) (nbins - 1.) / (max - min);
	data = (double *) calloc(nbins*2, sizeof(double));
	if (data == NULL)
	  {
	    XgM_error_sys(XgM_BAD_CALLOC, mv_data);
	    return;
	  }
	for (i=0; i<nbins; ++i)
	  {
	    *(data + 2*i) = (double) i / scale + min;
	    *(data + 2*i + 1) = 0.;
	  }

	for (i=0; i<npixels; ++i)
	  {
	    val = floor((mv_data->pix_array[index + i] - min) * scale + .5);
	    *(data + 2*val + 1) += 1.;
	  }

	graph->dataset[0].num_points = nbins;
	sprintf(graph->title_desc.text, "File %s", mv_data->filename);
	sprintf(graph->subtitle_desc.text, "Histogram of Frame %d",
		mv_data->count);
	sprintf(graph->x_axis_desc.text, "Pixel Value");
	sprintf(graph->y_axis_desc.text, "Number of Occurances");
	sprintf(graph->dataset[0].legend_text, "Frame %d Histogram",
		mv_data->count);
      }

    if (mouse_ev->xbutton.button == Button3)  /** plot column **/
      {
	data = (double *) calloc(nrows*2, sizeof(double));
	if (data == NULL)
	  {
	    XgM_error_sys(XgM_BAD_CALLOC, mv_data);
	    return;
	  }
	for (i=0; i<nrows; ++i)
	  {
	    *(data + 2*i) = (double) i + mv_data->yoff;
	    *(data + 2*i + 1) = (double) mv_data->pix_array[index +i*ncols +x];
	  }
	graph->dataset[0].num_points = nrows;
	sprintf(graph->title_desc.text, "File %s", mv_data->filename);
	sprintf(graph->subtitle_desc.text, "Frame %d; Column %d",
		mv_data->count,
		x + mv_data->xoff);
	sprintf(graph->x_axis_desc.text, "Y Pixel Location");
	sprintf(graph->y_axis_desc.text, "Pixel Value");
	sprintf(graph->dataset[0].legend_text, "Column %d data",
		x + mv_data->xoff);
      }

    graph->dataset[0].darray = data;
    graph->dataset[0].file_type = Xg_Not_File;
    graph->app_shell = XtAppCreateShell("Pltxsh", XgClass,
				    applicationShellWidgetClass,
				    mv_data->display, NULL, 0);
    XgCreateGraph(graph);
  }
#endif

}

/***************** Main Movie Loop ***********************/
void XgM_Next_Frame(client_data, id)
     caddr_t
       client_data;
     XtIntervalId
       *id;
{
  int
    zoom_fac,
    nargs;
  char
    buf[80];
  XgMovie_data
    *mv_data;
  Arg
    args[3];
  XmString
    label;

nargs = 0;
mv_data = (XgMovie_data *) client_data;
if (mv_data->movie_single == 0)   /** probably a cancelled zoom **/
  return;

switch (mv_data->zoom_fac)
  {
  case XgM_ZOOM_1:
    zoom_fac = 1;
    break;
  case XgM_ZOOM_2:
    zoom_fac = 2;
    break;
  case XgM_ZOOM_4:
    zoom_fac = 4;
    break;
  case XgM_ZOOM_8:
    zoom_fac = 8;
    break;
  default:
    fprintf(stderr, "Internal Error: Bad Zoom Factor. Aborted\n");
    exit(1);
    break;
  }

if (mv_data->movie_single == XgM_MOVIE)
  {
    if (mv_data->direction == XgM_FORWARD)
      {
	++mv_data->count;
	if (mv_data->count >= mv_data->nframes)
	  mv_data->count = 0;
      }
    else  /** reverse **/
      {
	--mv_data->count;
	if (mv_data->count < 0)
	  mv_data->count = mv_data->nframes-1;
      }
    XgM_Read_Frame(mv_data);
  }

XCopyArea(mv_data->display, mv_data->mv_img,
	  XtWindow(mv_data->draw_area), mv_data->gc,
	  0, 0, mv_data->ncols*zoom_fac, mv_data->nrows*zoom_fac, 0, 0);
XCopyArea(mv_data->display, mv_data->grey_pix,
	  XtWindow(mv_data->grey_area), mv_data->gc,
	  0, 0, XgM_SCL_WDTH, XgM_SCL_HT, 0, 0);
sprintf(buf, "%4d", mv_data->count);
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetValues(mv_data->frame_box, args, nargs);
XtFree(label);
nargs = 0;
XtSetArg(args[nargs], XmNvalue, mv_data->count); ++nargs;
XtSetValues(mv_data->frame_scroll, args, nargs);

if (mv_data->movie_single == XgM_MOVIE)
  {
    XtAppAddTimeOut(mv_data->theAppContext, mv_data->interval,
		    XgM_Next_Frame, mv_data);
  }
}

/************ Slider Bars Moved - Update Colormap ****************/
void XgM_Move_Slider(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgMovie_data
    *mv_data;
  int
    nargs,
    incr,
    slider_size,
    not_used,
    min_value,
    max_value;
  char
    buf[80];
  Arg
    args[3];
  XmString
    label;
mv_data = (XgMovie_data *) client_data;

nargs = 0;
XmScrollBarGetValues(mv_data->min_bar, &min_value, &slider_size,
		     &incr, &not_used);
if (min_value > mv_data->grey_max)
  min_value = mv_data->grey_max;
/*printf("min_value now %d\n", min_value);*/
sprintf(buf, "%4d", min_value);
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetValues(mv_data->min_box, args, nargs);
XtFree(label);

nargs = 0;
XmScrollBarGetValues(mv_data->max_bar, &max_value, &slider_size,
		     &incr, &not_used);
if (max_value > mv_data->grey_max)
  max_value = mv_data->grey_max;
max_value += slider_size;
/*printf("max_value now %d\n", max_value);*/
sprintf(buf, "%4d", max_value);
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetValues(mv_data->max_box, args, nargs);
XtFree(label);
}
/*******************/
void XgM_Adjust_Cmap(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgMovie_data
    *mv_data;
  int
    i,
    nargs,
    color,
    delta,
    slider_size,
    incr,
    page,
    min_value,
    max_value;
  double
    scale1,
    scale2;
  char
    buf[80];
  XColor
    *color_array;
  Arg
    args[3];
  XmString
    label;

mv_data = (XgMovie_data *) client_data;
color_array = (XColor *) calloc(mv_data->ncells, sizeof(XColor));
if (color_array == NULL)
  {
    fprintf(stderr, "ERROR: Failure to calloc color_array in Adjust_Cmap\n");
    exit(1);
  }

nargs = 0;
XmScrollBarGetValues(mv_data->min_bar, &min_value, &slider_size,
		     &incr, &page);
/*printf("min_value=%d, slider_size=%d, inc=%d, pageinc=%d\n",
       min_value, slider_size, incr, page);*/
if (min_value > mv_data->grey_max)
  min_value = mv_data->grey_max;
sprintf(buf, "%4d", min_value);
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetValues(mv_data->min_box, args, nargs);
XtFree(label);

nargs = 0;
XmScrollBarGetValues(mv_data->max_bar, &max_value, &slider_size,
		     &incr, &page);
/*printf("max_value=%d, slider_size=%d, inc=%d, pageinc=%d\n",
       max_value, slider_size, incr, page);*/
max_value += slider_size;
if (max_value > mv_data->grey_max)
  max_value = mv_data->grey_max;
sprintf(buf, "%4d", max_value);
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetValues(mv_data->max_box, args, nargs);
XtFree(label);

delta = fabs((double) mv_data->grey_max - mv_data->grey_min);
if (delta == 0)
  scale1 = 0.;
else
  scale1 = (double) (mv_data->ncells - 1.) / delta;
/* convert grey scale to color cell indices */
min_value = floor((double) (min_value - mv_data->grey_min) * scale1 + .5);
max_value = floor((double) (max_value - mv_data->grey_min) * scale1 + .5);

if ((max_value - min_value) == 0) /** compute grey scale slope **/
  scale2 = 0.;
else
  scale2 = (double) XgM_XColorMax / (max_value - min_value);
for (i=0; i<mv_data->ncells; ++i)  /** define color mapping **/
  {
    if (min_value < max_value)
      {
	if (i < min_value)
	  color = 0;
	else if (i > max_value)
	  color = XgM_XColorMax;
	else
	  color = scale2 * (i - min_value);
      }
    else  /** max_value < min_value **/
      {
	if (i > min_value)
	  color = 0;
	else if (i < max_value)
	  color = XgM_XColorMax;
	else
	  color = scale2 * (i - max_value) + XgM_XColorMax;
      }
    color_array[i].pixel = mv_data->pixels[i];
    color_array[i].red = color;
    color_array[i].blue = color;
    color_array[i].green = color;
    color_array[i].flags = DoRed | DoGreen | DoBlue;
  }
XStoreColors(mv_data->display, mv_data->cmap, color_array, mv_data->ncells);

}

/************ Frame Bar Moved - Update Frame Display ****************/
void XgM_Move_Frame(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgMovie_data
    *mv_data;
  int
    nargs,
    incr,
    slider_size,
    not_used,
    frame_value;
  char
    buf[80];
  Arg
    args[3];
  XmString
    label;
mv_data = (XgMovie_data *) client_data;

nargs = 0;
XmScrollBarGetValues(mv_data->frame_scroll, &frame_value, &slider_size,
		     &incr, &not_used);
sprintf(buf, "%4d", frame_value);
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetValues(mv_data->frame_box, args, nargs);
XtFree(label);
}
/*******************/
void XgM_Adjust_Frame(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgMovie_data
    *mv_data;
  int
    nargs,
    zoom_fac,
    not_used,
    slider_size,
    frame_value;
  char
    buf[80];
  Arg
    args[3];
  XmString
    label;

mv_data = (XgMovie_data *) client_data;
switch (mv_data->zoom_fac)
  {
  case XgM_ZOOM_1:
    zoom_fac = 1;
    break;
  case XgM_ZOOM_2:
    zoom_fac = 2;
    break;
  case XgM_ZOOM_4:
    zoom_fac = 4;
    break;
  case XgM_ZOOM_8:
    zoom_fac = 8;
    break;
  default:
    fprintf(stderr, "Internal Error: Bad Zoom Factor. Aborted\n");
    exit(1);
    break;
  }

nargs = 0;
XmScrollBarGetValues(mv_data->frame_scroll, &frame_value, &slider_size,
		     &not_used, &not_used);
mv_data->count = frame_value;
sprintf(buf, "%4d", frame_value);
label = XmStringCreateLtoR(buf, XgMCharset);
XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
XtSetValues(mv_data->frame_box, args, nargs);
XtFree(label);
XgM_Read_Frame(mv_data);
XCopyArea(mv_data->display, mv_data->mv_img,
	  XtWindow(mv_data->draw_area), mv_data->gc,
	  0, 0, mv_data->ncols*zoom_fac, mv_data->nrows*zoom_fac, 0, 0);
}

/***************** Movie/Single Toggle **********************/
void XgM_Movie_On(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgMovie_data
    *mv_data;
mv_data = (XgMovie_data *) client_data;
mv_data->movie_single = XgM_MOVIE;
/*mv_data->count = 0;*/
XtAppAddTimeOut(mv_data->theAppContext, mv_data->interval,
		XgM_Next_Frame, mv_data);
}
/*****************/
void XgM_Movie_Off(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgMovie_data
    *mv_data;
mv_data = (XgMovie_data *) client_data;
mv_data->movie_single = XgM_SINGLE;
}

/***************** Forward/Reverse Toggle **********************/
void XgM_Forward(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgMovie_data
    *mv_data;
mv_data = (XgMovie_data *) client_data;
mv_data->direction = XgM_FORWARD;
}
/*****************/
void XgM_Reverse(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgMovie_data
    *mv_data;
mv_data = (XgMovie_data *) client_data;
mv_data->direction = XgM_REVERSE;
}

/********** Time Increment *******/
void XgM_ChangeInt(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  XgMovie_data
    *mv_data;
mv_data = (XgMovie_data *) client_data;

if (w == mv_data->but_100)
  mv_data->interval = XgM_INT_100;
else if (w == mv_data->but_200)
  mv_data->interval = XgM_INT_200;
else if (w == mv_data->but_500)
  mv_data->interval = XgM_INT_500;
else if (w == mv_data->but_1000)
  mv_data->interval = XgM_INT_1000;
else
  fprintf(stderr, "internal error. Bad time interval selection!\n");
}

/********** Zoom Factor *******/
void XgM_ChangeZoom(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    k,
    zoom_fac;
  XgMovie_data
    *mv_data;

mv_data = (XgMovie_data *) client_data;

if (w == mv_data->zbut_1)
  {
    mv_data->zoom_fac = XgM_ZOOM_1;
    zoom_fac = 1;
  }
else if (w == mv_data->zbut_2)
  {
    mv_data->zoom_fac = XgM_ZOOM_2;
    zoom_fac = 2;
  }
else if (w == mv_data->zbut_4)
  {
    mv_data->zoom_fac = XgM_ZOOM_4;
    zoom_fac = 4;
  }
else if (w == mv_data->zbut_8)
  {
    mv_data->zoom_fac = XgM_ZOOM_8;
    zoom_fac = 8;
  }
else
  fprintf(stderr, "internal error. Bad zoom factor selection!\n");

XFreePixmap(mv_data->display, mv_data->mv_img);
XtDestroyWidget(mv_data->mv_scrn);
XgM_Create_Movie_Screen(mv_data);

if (mv_data->movie_single == XgM_SINGLE)
  XCopyArea(mv_data->display, mv_data->mv_img,
	    XtWindow(mv_data->draw_area), mv_data->gc,
	    0, 0, mv_data->ncols*zoom_fac, mv_data->nrows*zoom_fac, 0, 0);

}

/********** Scaling *******/
void XgM_ChangeScale(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    zoom_fac;
  XgMovie_data
    *mv_data;

mv_data = (XgMovie_data *) client_data;
if (mv_data->zoom_fac == XgM_ZOOM_1)
  zoom_fac = 1;
else if (mv_data->zoom_fac == XgM_ZOOM_2)
  zoom_fac = 2;
else if (mv_data->zoom_fac == XgM_ZOOM_4)
  zoom_fac = 4;
else if (mv_data->zoom_fac == XgM_ZOOM_8)
  zoom_fac = 8;
else
  {
    fprintf(stderr, "ERROR: bad zoom factor. Aborted\n");
    exit(1);
  }

if (w == mv_data->sbut_lin)
  mv_data->scaling = XgM_SCALE_LIN;
else if (w == mv_data->sbut_log)
  mv_data->scaling = XgM_SCALE_LOG;
else if (w == mv_data->sbut_sqrt)
  mv_data->scaling = XgM_SCALE_SQRT;
else if (w == mv_data->sbut_hist)
  mv_data->scaling = XgM_SCALE_HIST;
else
  fprintf(stderr, "internal error. Bad scaling selection!\n");

XgM_Fill_Pixmaps(mv_data);
if (mv_data->movie_single == XgM_SINGLE)
  XCopyArea(mv_data->display, mv_data->mv_img,
	    XtWindow(mv_data->draw_area), mv_data->gc,
	    0, 0, mv_data->ncols*zoom_fac, mv_data->nrows*zoom_fac, 0, 0);
}

/********** Flip Frames Vertically **********/
void XgM_Flip_Frames(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    zoom_fac;
  XgMovie_data
    *mv_data;

mv_data = (XgMovie_data *) client_data;
if (mv_data->zoom_fac == XgM_ZOOM_1)
  zoom_fac = 1;
else if (mv_data->zoom_fac == XgM_ZOOM_2)
  zoom_fac = 2;
else if (mv_data->zoom_fac == XgM_ZOOM_4)
  zoom_fac = 4;
else if (mv_data->zoom_fac == XgM_ZOOM_8)
  zoom_fac = 8;
else
  {
    fprintf(stderr, "ERROR: bad zoom factor. Aborted\n");
    exit(1);
  }

if (w == mv_data->fbut_norm)
  mv_data->flip = XgM_FLIP_NORM;
else if (w == mv_data->fbut_rev)
  mv_data->flip = XgM_FLIP_REV;

XgM_Fill_Pixmaps(mv_data);
if (mv_data->movie_single == XgM_SINGLE)
  XCopyArea(mv_data->display, mv_data->mv_img,
	    XtWindow(mv_data->draw_area), mv_data->gc,
	    0, 0, mv_data->ncols*zoom_fac, mv_data->nrows*zoom_fac, 0, 0);
}

/*********** Fill Pixmaps with properly scaled iamge data ******/
void XgM_Fill_Pixmaps(mv_data)
     XgMovie_data
       *mv_data;
{
  int
    i,
    j,
    k,
    i1,
    j1,
    nargs,
    nrows,
    ncols,
    nframes,
    ncells,
    npixels,
    width,
    height,
    color,
    zoom_fac,
    delta,
    *lookup;
  unsigned long
    *pixels;
  double
    scale;
  char
    *char_data;
  int2
    *pix_array;
  XImage
    *ximage;
  Arg
    args[5];

/***********************/
nrows = mv_data->nrows;
ncols = mv_data->ncols;
nframes = mv_data->nframes;
ncells = mv_data->ncells;
npixels = nrows * ncols;
pix_array = mv_data->pix_array;
delta = mv_data->pix_max - mv_data->pix_min + 1;
pixels = mv_data->pixels;
switch (mv_data->zoom_fac)
  {
  case XgM_ZOOM_1:
    zoom_fac = 1;
    break;
  case XgM_ZOOM_2:
    zoom_fac = 2;
    break;
  case XgM_ZOOM_4:
    zoom_fac = 4;
    break;
  case XgM_ZOOM_8:
    zoom_fac = 8;
    break;
  default:
    fprintf(stderr, "Internal Error: Bad Zoom Factor. Aborted\n");
    exit(1);
    break;
  }
width = ncols * zoom_fac;
height = nrows * zoom_fac;

char_data = (char *) calloc(width * height, sizeof(char));
if (char_data == NULL)
  {
    fprintf(stderr, "Failure to allocate char_data for byte array.\n");
    exit(1);
  }
ximage = XCreateImage(mv_data->display, mv_data->visual, mv_data->nbits,
		      ZPixmap, 0, char_data, width, height, 8,0);
if (ximage == NULL)
  {
    fprintf(stderr, "Failure to allocate ximage with XCreateImage.\n");
    exit(1);
  }
lookup = (int *) calloc(delta, sizeof(int));
if (lookup == NULL)
  {
    fprintf(stderr, "Failure to allocate lookup in XgM_Fill_Pixmaps.\n");
    exit(1);
  }

nargs = 0;
XtSetArg(args[nargs], XmNminimum, mv_data->pix_min); ++nargs;
XtSetArg(args[nargs], XmNmaximum, mv_data->pix_max + 250); ++nargs;
mv_data->grey_min = mv_data->pix_min;
mv_data->grey_max = mv_data->pix_max;
XtSetValues(mv_data->min_bar, args, nargs);
XtSetValues(mv_data->max_bar, args, nargs);

switch (mv_data->scaling)
  {
  case XgM_SCALE_LIN:
    scale = (double) (mv_data->ncells - 1) / (delta-1);
    for (i=0; i<delta; ++i)
      lookup[i] = floor(scale * i + .5);
    break;
  case XgM_SCALE_LOG:
    scale = (double) (mv_data->ncells - 1) / log((double) delta);
    for (i=0; i<delta; ++i)
      lookup[i] = floor(scale * log((double) i+1) + .5);
    break;
  case XgM_SCALE_SQRT:
    scale = (double) (mv_data->ncells - 1) / sqrt((double) delta-1);
    for (i=0; i<delta; ++i)
      lookup[i] = floor(scale * sqrt((double) i) + .5);
    break;
  case XgM_SCALE_HIST:
    for (i=0; i<delta; ++i)
      lookup[i] = 0;
    for (i=0; i<npixels; ++i)    /** form histogram **/
      {
	if (*(pix_array + i) != mv_data->overlay)
	  {
	    ++lookup[*(pix_array + i) - mv_data->pix_min];
	  }
      }
    for (i=1; i<delta; ++i)      /** form cumulative distribution function **/
      lookup[i] += lookup[i-1];
    scale = (double) (mv_data->ncells - 1) / lookup[delta-1];
    for (i=0; i<delta; ++i)      /** normalize to number of color cells **/
      lookup[i] = floor(scale * lookup[i] + .5);
    break;
  default:
    fprintf(stderr, "Error: Invalid scaling type. Aborted!\n");
    exit(1);
    break;
  }

if (mv_data->flip == XgM_FLIP_NORM)
  {
    for (i=0; i<nrows; ++i)
      {
	for (j=0; j<ncols; ++j)
	  {
	    if (*(pix_array + i*ncols + j) != mv_data->overlay)
	      {
		if (*(pix_array + i*ncols + j) > mv_data->pix_max)
		  *(pix_array + i*ncols + j) = mv_data->pix_max;
		if (*(pix_array + i*ncols + j) < mv_data->pix_min)
		  *(pix_array + i*ncols + j) = mv_data->pix_min;
	      }
	    for (i1=0; i1<zoom_fac; ++i1)
	      {
		for (j1=0; j1<zoom_fac; ++j1)
		  {
		    if (*(pix_array + i*ncols + j) == mv_data->overlay)
		      {
			*(char_data + (height-1 - (i*zoom_fac + i1))*width
			  + j*zoom_fac + j1)
			  = mv_data->overlay_pix;
		      }
		    else
		      {
			*(char_data + (height-1 - (i*zoom_fac + i1))*width
			  + j*zoom_fac + j1)
			  = pixels[lookup[*(pix_array + i*ncols + j)
					  - mv_data->pix_min]];
		      }
		  } /** end of j1 loop **/
	      } /** end of i1 loop **/
	  } /** end of j loop over columns **/
      } /** end of i loop over rows **/
  } /** end normal orientation **/
else  /** vertical flip **/
  {
    for (i=0; i<nrows; ++i)
      {
	for (j=0; j<ncols; ++j)
	  {
	    if (*(pix_array + i*ncols + j) != mv_data->overlay)
	      {
		if (*(pix_array + i*ncols + j) > mv_data->pix_max)
		  *(pix_array + i*ncols + j) = mv_data->pix_max;
		if (*(pix_array + i*ncols + j) < mv_data->pix_min)
		  *(pix_array + i*ncols + j) = mv_data->pix_min;
	      }
	     for (i1=0; i1<zoom_fac; ++i1)
	       {
		 for (j1=0; j1<zoom_fac; ++j1)
		   {
		     if (*(pix_array + i*ncols + j) == mv_data->overlay)
		       {
			 *(char_data + (i*zoom_fac + i1)*width
			   + j*zoom_fac + j1)
			   = mv_data->overlay_pix;
		       }
		     else
		       {
			 *(char_data + (i*zoom_fac +i1)*width
			   + j*zoom_fac + j1)
			   = pixels[lookup[*(pix_array + i*ncols + j)
					   - mv_data->pix_min]];
		       }
		   } /** end of j1 loop **/
	       } /** end of i1 loop **/
	  } /** end of j loop over columns **/
      } /** end of i loop over rows **/
  } /** end vert flip orientation **/
XPutImage(mv_data->display, mv_data->mv_img, mv_data->gc, ximage,
	  0, 0, 0, 0, ncols*zoom_fac, nrows*zoom_fac);

XDestroyImage(ximage);
free(char_data);
free(lookup);
XgM_Adjust_Cmap(NULL, mv_data, NULL);
}

/***************** Create Zoomed Area ***************/
void XgM_Update_RbrBox(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i,
    x,
    y,
    minx,
    maxx,
    miny,
    maxy;
  XgMovie_data
    *mv_data;
  XEvent
    *mouse_ev;

mv_data = (XgMovie_data *) client_data;
if (mv_data->rbrband_mode == FALSE)  /** not in rubberband mode **/
  return;
mouse_ev = (XEvent *) call_data;

/*** find location of pointer when pressed ***/
x = mouse_ev->xmotion.x;
y = mouse_ev->xmotion.y;

/** erase old line **/
minx = (mv_data->xp0 > mv_data->xp1) ? mv_data->xp1 : mv_data->xp0;
maxx = (mv_data->xp0 < mv_data->xp1) ? mv_data->xp1 : mv_data->xp0;
miny = (mv_data->yp0 > mv_data->yp1) ? mv_data->yp1 : mv_data->yp0;
maxy = (mv_data->yp0 < mv_data->yp1) ? mv_data->yp1 : mv_data->yp0;
XDrawRectangle(mv_data->display, XtWindow(mv_data->draw_area),
	       mv_data->gcxor, minx, miny, maxx - minx, maxy - miny);

mv_data->xp1 = x;
mv_data->yp1 = y;

/** draw new line **/
minx = (mv_data->xp0 > mv_data->xp1) ? mv_data->xp1 : mv_data->xp0;
maxx = (mv_data->xp0 < mv_data->xp1) ? mv_data->xp1 : mv_data->xp0;
miny = (mv_data->yp0 > mv_data->yp1) ? mv_data->yp1 : mv_data->yp0;
maxy = (mv_data->yp0 < mv_data->yp1) ? mv_data->yp1 : mv_data->yp0;
XDrawRectangle(mv_data->display, XtWindow(mv_data->draw_area),
	       mv_data->gcxor, minx, miny, maxx - minx, maxy - miny);

}

/********************/
void XgM_Create_Zoom(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i,
    j,
    k,
    x,
    y,
    minx,
    maxx,
    miny,
    maxy,
    index,
    nrows,
    ncols,
    nframes,
    npixels,
    nr,
    nc,
    npix,
    ndx,
    zoom_fac;
  double
    scale,
    *data;
  char
    buf[80];
  XEvent
    *mouse_ev;
  XgMovie_data
    *mv_data,
    *new_mov;

mv_data = (XgMovie_data *) client_data;
if (mv_data->rbrband_mode == FALSE)  /** not in rubberband mode **/
  return;
mouse_ev = (XEvent *) call_data;

/******** remove rubberband line **********/
mv_data->rbrband_mode = FALSE;
x = mouse_ev->xmotion.x;
y = mouse_ev->xmotion.y;

/** erase old line **/
minx = (mv_data->xp0 > mv_data->xp1) ? mv_data->xp1 : mv_data->xp0;
maxx = (mv_data->xp0 < mv_data->xp1) ? mv_data->xp1 : mv_data->xp0;
miny = (mv_data->yp0 > mv_data->yp1) ? mv_data->yp1 : mv_data->yp0;
maxy = (mv_data->yp0 < mv_data->yp1) ? mv_data->yp1 : mv_data->yp0;
XDrawRectangle(mv_data->display, XtWindow(mv_data->draw_area),
	       mv_data->gcxor, minx, miny, maxx - minx, maxy - miny);

mv_data->xp1 = x;
mv_data->yp1 = y;

/*********** setup zoomed area ***********/
nrows = mv_data->nrows;
ncols = mv_data->ncols;
nframes = mv_data->nframes;
npixels = nrows * ncols;
if (mv_data->zoom_fac == XgM_ZOOM_1)
  zoom_fac = 1;
else if (mv_data->zoom_fac == XgM_ZOOM_2)
  zoom_fac = 2;
else if (mv_data->zoom_fac == XgM_ZOOM_4)
  zoom_fac = 4;
else if (mv_data->zoom_fac == XgM_ZOOM_8)
  zoom_fac = 8;
else
  {
    fprintf(stderr, "Internal Error: Bad Zoom Factor. Aborted\n");
    exit(1);
  }

/*** find location of pointer when pressed ***/
x = mouse_ev->xmotion.x;
if (mv_data->flip == XgM_FLIP_NORM)
  y = nrows*zoom_fac - mouse_ev->xmotion.y - 1;
else
  y = mouse_ev->xmotion.y;
x = floor((double) x / zoom_fac);
y = floor((double) y / zoom_fac);
if (x < 0)
  x = 0;
if (x >= ncols)
  x = ncols - 1;
if (y < 0)
  y = 0;
if (y >= nrows)
  y = nrows - 1;
printf("  end coordinates = (%d %d)\n", x, y);

/********* get size and origin of subregion *******/
nr = fabs((double) y - mv_data->y0) + 1;
nc = fabs((double) x - mv_data->x0) + 1;
npix = nr * nc;
if ((nr == 1) || (nc == 1))
  {
    XgM_error_sys(XgM_ZERO_WIDTH, mv_data);
    return;
  }
if (x < mv_data->x0)
  mv_data->x0 = x;
if (y < mv_data->y0)
  mv_data->y0 = y;
printf("  nc x nr = %d x %d, and origin is (%d, %d)\n", nc, nr, mv_data->x0,
       mv_data->y0);

/*******************************************************/
if (mouse_ev->xbutton.button == Button1)  /** zoom area **/
  {

    /****** zoom ******/
    new_mov = XgM_Initialize_Movie();
    new_mov->kill_exit = XgM_EXIT_NOKILL;
    new_mov->ets_struct = mv_data->ets_struct;
    new_mov->nrows = nr;
    new_mov->ncols = nc;
    new_mov->nframes = nframes;
    new_mov->xoff = mv_data->x0 + mv_data->xoff;
    new_mov->yoff = mv_data->y0 + mv_data->yoff;
    new_mov->overlay = mv_data->overlay;
    sprintf(new_mov->filename, "%s", mv_data->filename);
    strcpy(new_mov->control_name, "Zoom Ctrl");
    strcpy(new_mov->screen_name, "Zoom Scrn");
    new_mov->theAppContext = mv_data->theAppContext;
    new_mov->display = mv_data->display;
    new_mov->count = mv_data->count;
    new_mov->movie_single = XgM_SINGLE;

    XgM_Main_Entry(new_mov);
  }  /** end of left button selection for zoom **/

else if (mouse_ev->xbutton.button == Button2)  /** hang user function here **/
  {
  }

else if (mouse_ev->xbutton.button == Button3)  /** draw box **/
  {
    minx = (mv_data->xp0 > mv_data->xp1) ? mv_data->xp1 : mv_data->xp0;
    maxx = (mv_data->xp0 < mv_data->xp1) ? mv_data->xp1 : mv_data->xp0;
    miny = (mv_data->yp0 > mv_data->yp1) ? mv_data->yp1 : mv_data->yp0;
    maxy = (mv_data->yp0 < mv_data->yp1) ? mv_data->yp1 : mv_data->yp0;
    XDrawRectangle(mv_data->display, XtWindow(mv_data->draw_area),
		   mv_data->gc, minx, miny, maxx - minx, maxy - miny);
  }

}

/*************** Print ***************/
#define XgM_PSlevels 256
void XgM_Print(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  FILE
    *fp;
  time_t
    time_sec;
  int
    i,
    j,
    page,
    incr,
    slider_size,
    min_value,
    max_value,
    delta,
    color,
    lookup[XgM_PSlevels],
    index,
    width,
    height,
    wdum,
    zoom_fac,
    text_font_size;
  double
    scale1,
    scale2;
  float
    scale,
    page_long_dim,
    page_short_dim;
  char
    buf[160],
    text_font_name[80];
  XImage
    *ximage;
  XgMovie_data
    *mv_data;

mv_data = (XgMovie_data *) client_data;
time(&time_sec);
page_long_dim = 11.;
page_short_dim = 8.5;
sprintf(buf, "XgMovie: File: %s, frame %d; Date: %s",
	mv_data->filename, mv_data->count, ctime(&time_sec));
buf[strlen(buf)-1] = '\0';  /** remove \n inserted by ctime **/
switch (mv_data->scaling)
  {
  case XgM_SCALE_LIN:
    strcat(buf, "; Scaling: Linear");
    break;
  case XgM_SCALE_LOG:
    strcat(buf, "; Scaling: Logarithmic");
    break;
  case XgM_SCALE_SQRT:
    strcat(buf, "; Scaling: Square Root");
    break;
  case XgM_SCALE_HIST:
    strcat(buf, "; Scaling: Histogram Equalization");
    break;
  default:
    break;
  }

text_font_size = 8;
strcpy(text_font_name, "Helvetica");

switch (mv_data->zoom_fac)
  {
  case XgM_ZOOM_1:
    zoom_fac = 1;
    break;
  case XgM_ZOOM_2:
    zoom_fac = 2;
    break;
  case XgM_ZOOM_4:
    zoom_fac = 4;
    break;
  case XgM_ZOOM_8:
    zoom_fac = 8;
    break;
  default:
    fprintf(stderr, "Internal Error: Bad Zoom Factor. Aborted!");
    exit(1);
    break;
  }
width = mv_data->ncols * zoom_fac;
height = mv_data->nrows * zoom_fac;
wdum = width;
/** damned if I know why, seems to be an even/odd width thing? **/
if (width/2. - floor((double) width/2.) > .1)
  if ((mv_data->xoff == 0) && (mv_data->yoff == 0))
    wdum += zoom_fac;

/****** grey scale mapping *****/
XmScrollBarGetValues(mv_data->min_bar, &min_value, &slider_size,
		     &incr, &page);
if (min_value > mv_data->grey_max)
  min_value = mv_data->grey_max;

XmScrollBarGetValues(mv_data->max_bar, &max_value, &slider_size,
		     &incr, &page);
max_value += slider_size;
if (max_value > mv_data->grey_max)
  max_value = mv_data->grey_max;

delta = fabs((double) mv_data->grey_max - mv_data->grey_min);
if (delta == 0)
  scale1 = 0.;
else
  scale1 = (double) (XgM_PSlevels-1) / delta;

/* convert grey scale to color cell indices */
min_value = floor((double) (min_value - mv_data->grey_min) * scale1 + .5);
max_value = floor((double) (max_value - mv_data->grey_min) * scale1 + .5);

if ((max_value - min_value) == 0) /** compute grey scale slope **/
  scale2 = 0.;
else
  scale2 = (double) (XgM_PSlevels - 1) / (max_value - min_value);

for (i=0; i<XgM_PSlevels; ++i)  /** define grey scale mapping **/
  {
    if (min_value < max_value)
      {
	if (i < min_value)
	  color = 0;
	else if (i > max_value)
	  color = XgM_PSlevels - 1;
	else
	  color = scale2 * (i - min_value);
      }
    else  /** max_value < min_value **/
      {
	if (i > min_value)
	  color = 0;
	else if (i < max_value)
	  color = XgM_PSlevels - 1;
	else
	  color = scale2 * (i - max_value) + (XgM_PSlevels - 1);
      }
    lookup[i] = color;
  }

/******* header ********/
fp = fopen("XgMov.psc", "w");
if (fp == NULL)
  return;

fprintf(fp, "%%!PS-Adobe-1.0\n");
fprintf(fp, "%%%%Creator: XgMovie program\n");
fprintf(fp, "%%%%Title: Image\n");
fprintf(fp, "%%%%Creation Date: %s", ctime(&time_sec));
fprintf(fp, "%%%%BoundingBox: 0 0 611 791\n");
fprintf(fp, "%%%%End Comments\n");
fprintf(fp, "%%%%Pages: 1\n");
fprintf(fp, "%%%%End Prolog\n");
fprintf(fp, "%%%%Page: 1 1\n\n");
fprintf(fp, "%% For EPS usage:\n");
fprintf(fp, "%%   1) Add EPSF-1.2 at end of first line\n");
fprintf(fp, "%%   2) Switch BoundingBox coordinates to 0 0 791 611\n");
/*fprintf(fp, "%%   3) Comment out the two lines indicated just below\n");*/
fprintf(fp, "%%   4) remove showpage at end of this file\n");

/********* coordinate system **********/
fprintf(fp, "/inch {72 mul} def\n");
/*fprintf(fp, "%% comment out the next two lines for EPS usage\n");
fprintf(fp, "%f inch 0 inch translate\n", page_short_dim);
fprintf(fp, "90 rotate\n");*/

/************** labels **************/
fprintf(fp, "\n%%  Print Labels\n\n");
fprintf(fp, "/%s findfont %d scalefont setfont\n", text_font_name,
	text_font_size);
fprintf(fp, "1 inch 1 inch moveto (%s) show\n", buf);    /** title **/
/** might want to add annotation, zoom region **/

/********** print image ************/
fprintf(fp, "\n%%  Create Image\n\n");
scale = (page_short_dim - 1.) / width;
if (scale * height > (page_long_dim - 1.5))
  {
    scale = (page_long_dim - 1.5) / height;
    fprintf(fp, "%f inch .5 inch translate\n",
	    (page_short_dim - scale * width) / 2.);
  }
else
  {
    fprintf(fp, ".5 inch %f inch translate\n",
	    (page_long_dim - scale * height) / 2.);
  }
fprintf(fp, "gsave %f inch %f inch scale\n", (float) scale * width,
	(float) scale * height);

fprintf(fp, "/picstr %d string def\n", width);
fprintf(fp, "%d %d 8 [%d 0 0 %d 0 %d] \n",
	width, height, width, -height, height);
fprintf(fp, "{currentfile picstr readhexstring pop} image\n\n");

ximage = XGetImage(mv_data->display, mv_data->mv_img, 0, 0, width, height,
		   255, ZPixmap);

for (i=0; i<height; ++i)
  {
    index = i * (wdum);
    for (j=0; j<width; ++j)
      {
	/** get pixel from pixmap, write it in hex **/
	color = *(ximage->data + index + j);
	if (color < 0)  /** kludge for alphas **/
	  color += 256;
	color = lookup[color];
	fprintf(fp, "%02x", color);
      }
    fprintf(fp, "\n");
  }

fprintf(fp, "\ngrestore\n");
fprintf(fp, "\n0 0 moveto 0 %f inch rlineto\n", (float) scale * height);
fprintf(fp, " %f inch 0 rlineto", (float) scale * width);
fprintf(fp, " 0 %f inch rlineto closepath stroke\n", (float) -scale * height);

/*********** grey scale bar **********/
fprintf(fp, "\n%%  Create Grey Scale Bar\n\n");
fprintf(fp, "0 -.10 inch translate\n");
fprintf(fp, "gsave %f inch .10 inch scale\n", (float) scale * width);
fprintf(fp, "256 1 8 [256 0 0 1 0 0] \n");

fprintf(fp, "{<");
for (i=0; i<256; ++i)
  {
    fprintf(fp, "%02x", lookup[i]);
  }
fprintf(fp, ">} image grestore\n");

fprintf(fp, "\n0 0 moveto 0 .1 inch rlineto\n");
fprintf(fp, " %f inch 0 rlineto", (float) scale * width);
fprintf(fp, " 0 -.1 inch rlineto closepath stroke\n");

/************ finish up **************/
XDestroyImage(ximage);
fprintf(fp, "\nshowpage\n");
fclose(fp);
}
