/************************************************************************
*
*                             XgPlot.h
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*             Main (global) header file for Xg plotting package
*
************************************************************************
*/
#define XgPlot
#define DEV_COORD Dimension
#define XgClass          "XgPlot"
#define XgNSETS          20   /* total allowed number of data sets to plot */
#define XgNWID           25

#define XgKillOnExit     101
#define XgNoKillOnExit   102
#define XgNo_label       101
#define XgSci_label      102
#define XgEng_label      103

#define XgPoint          101
#define XgCircle         102
#define XgSquare         103
#define XgBar            104
#define XgTriangle       105
#define XgPlus           106
#define XgEx             107

#define XgNoLine         201
#define XgSolidLine      202

#define XgSig0           300
#define XgSig1           301
#define XgSig2           302
#define XgSig3           303

#define XgLast0          400
#define XgLast1          401
#define XgLast2          402
#define XgLast5          405

#define XgSize0          500
#define XgSize3          503
#define XgSize4          504
#define XgSize5          505

#define XgLinear         601
#define XgLogarithm      602
#define XgProbability    603

#define XgDivNone        701
#define XgDivMajorTic    702
#define XgDivMinorTic    703
#define XgDivMajorLin    704
#define XgDivLin_Tic     705
#define XgDivMinorLin    706

#define XgLeft           801
#define XgRight          802
#define XgFilled         803
#define XgOpen           804
#define XgErrorOn        805
#define XgErrorOff       806
#define XgNoErrorPts     807
#define XgHasErrorPts    808
#define XgInLegend       809
#define XgNotInLegend    810
#define XgLegendOn       811
#define XgLegendOff      812

#define Xg_Add_Normal    1
#define Xg_Add_Pair      2
#define Xg_Add_Single    3
#define Xg_Not_File      4
#define Xg_Has_ErrorBars 5

/***** bit masks *****/
#define XgUNothing       0
#define XgUmin           1
#define XgUmax           2
#define XgUinc           4
#define XgUnsig          8
#define XgUscale         16

/****** defaults ******/
#define XgDefPointType        XgCircle
#define XgDefLineType         XgNoLine
#define XgDefPtSize           XgSize3
#define XgDefFillOpen         XgOpen
#define XgDefErrorBarStatus   XgErrorOn
#define XgDefLegendStatus     XgInLegend
#define XgDefLegendOnOff      XgLegendOff
#define XgDefMinXSize         100
#define XgDefMinYSize         100
#define XgDefXtextXCoord      50
#define XgDefXtextYCoord      98
#define XgDefYtextXCoord      2
#define XgDefYtextYCoord      50
#define XgDefYrtTextXCoord    95
#define XgDefYrtTextYCoord    50
#define XgDefTitletextXCoord  50
#define XgDefTitletextYCoord  2
#define XgDefSubtextXCoord    50
#define XgDefSubtextYCoord    7
#define XgDefLegendXCoord     20
#define XgDefLegendYCoord     20
#define XgDefXscaleXCoord     90
#define XgDefXscaleYCoord     0
#define XgDefYscaleXCoord     90
#define XgDefYscaleYCoord     5
#define XgDefYrtScaleXCoord   90
#define XgDefYrtScaleYCoord   10
#define XgDefSigFig           XgSig2
#define XgDefInc              XgLast2
#define XgDefScale            XgLinear
#define XgDefDivMark          XgDivMajorTic
#define XgDefLabelType        XgSci_label
#define XgDefFont             "6x13"
#define XgDefFontDir          "/usr/lib/X11/fonts/misc/"
#define XgDefDataDescXCoord   50
#define XgDefDataDescYCoord   50
#define XgDefHelpDir          "/usr/local/bin/"
#define XgDefHelpFile         "XgPlot.hlp"
#define Xg_Max_Help           250

/*******************************/
typedef struct Xgcoord
  {
    double
      x,                 /* normalized x coordinate (0 - 1) */
      y;                 /* normalized y coordinate (0 - 1) */
  } XgCoord;

typedef struct Xgfontinfo  /* Font name and XFontStruct for labels, etc. */
  {
    char
      name[80];
    XFontStruct
      *font_struct;
    XmFontList
      font_list;
  } XgFontInfo;

typedef struct Xgdesc    /* text label description info */
  {
    char text[80];        /* text description */
    XgCoord coord;        /* location of desc */
    XgCoord dimen;        /* width & height of desc */
    XgFontInfo font_info; /* font info */
    long int color;       /* color */
  } XgDesc;

typedef struct Xgaxis    /* Layout Prescription for each axis */
  {
    double
      min,               /* minimum value along this axis */
      max,               /* maximum value along this axis */
      sclmin,            /* scaled minimum value along this axis */
      sclmax;            /* scaled maximum value along this axis */
    XgDesc
      scale_desc;        /* location of axis scaling info */
    int
      prob_mindx,        /* index into XgProb_array for Probability labels */
      prob_maxdx,        /* index into XgProb_array for Probability labels */
      nsig_fig,          /* number of significant figures in labels */
      increment,         /* increment of least sig. figure between labels */
      div_marks,         /* tick marks and/or lines at axis division points */
      scaling,           /* scaling of axis: linear, log, probability, etc */
      label_type;        /* Scaling rules: none, scientific, ... */
    char
      user_set_flag;     /* which values are defaults or user set */
  } XgAxis;

typedef struct Xgdatainfo /* One structure per data set to display */
  {
    double
      *err_array,        /* pointer to start of Dx1 error data (dy) */
      *darray;           /* pointer to start of Dx2 data (x, y) */
    int
      num_points,        /* number of data points in this data set */
      error_pts,         /* flag to indicate if error data present */
      error_bars,        /* flag to draw error bars with data points */
      point_type,        /* type of point to display for this data set */
      point_size,        /* size in pixels for Circle or Square */
      filled_open,       /* draw point as filled or open circle, square, etc */
      line_status,       /* Connect data points with line, or no line */
      file_type,         /* Normal, just pairs, or only single values */
      left_right,        /* Plot using left or right y-axis scaling */
      in_legend;         /* include this data set in the legend */
    char
      legend_text[80],   /* description for legend */
      filename[80];      /* file name for data set */
    XgDesc
      description;       /* text description for data set */
  } XgDataInfo;

typedef struct Xglegend    /* Legend layout info */
  {
    int use_legend;       /* turn legend display on/off */
    XgDesc
      description;        /* legend desc, font, location */
  } XgLegend;

typedef struct Xgwidstruct
  {
    caddr_t
      client,
      graph;
  } XgWidStruct;

/**************** Main Data structure for Xg graphs *******************/
typedef struct Xggraphinfo
  {
    char
      filename[80];      /* filename for this graph */
    int
      num_sets,          /* number of data sets */
      kill_flag;         /* indicates whether to kill process when exiting */
    XgAxis
      xaxis_layout,      /* layout prescription for x-axis */
      yaxis_layout,      /* layout prescription for y-axis */
      y_rt_axis_layout;  /* layout prescription for right hand y-axis */
    XgDataInfo
      dataset[XgNSETS];  /* Data & info for individual data sets */
    XgDesc
      x_axis_desc,       /* position of x-axis text (normalized coord) */
      y_axis_desc,       /* position of y-axis text (normalized coord) */
      y_rt_axis_desc,    /* position of right y-axis text (normalized coord) */
      title_desc,        /* position of title text (normalized coord) */
      subtitle_desc;     /* position of subtitle text (normalized coord) */
    XgLegend
      legend;            /* legend */
    long int
      axis_color,        /* x,y-axes, ticks, numbers, & scaling color */
      background_color,  /* overall graph background color */
      cursor_color;      /* cursor color */

    /************ realization variables *************/
    XtAppContext
      theAppContext;
    Display
      *display;
    GC
      theGC,
      gcxor;
    Widget
      app_shell,
      main,
      form,
      menu_bar,
      draw_area;
    int
      rubber_band,   /** TRUE if in rubber band drag mode **/
      rbr_xoff,      /** coordinates for rubber band box **/
      rbr_yoff,
      rbr_dx,
      rbr_dy,
      rbr_xp0,
      rbr_yp0,
      ftype,     /** file type (used when adding files) **/
      err_bar,   /** does new file have error bar data? **/
      redraw,
      nxdraw,    /** draw area dimensions **/
      nydraw,
      width,     /** initial graph dimensions **/
      height,
      use_right,      /** there are datasets attached to right y-axis **/
      cursor_status,  /** cursor variables **/
      cursWidth,
      cursHeight,
      cursDset,
      cursPos,
      cur_xo,    /** pointer initial position **/
      cur_yo;
    DEV_COORD
      label_dx,  /** label box dimensions **/
      label_dy,
      label_xo,  /** label box initial position **/
      label_yo,
      box_xo,    /** label box previous/current position **/
      box_yo,
      mb_dy;     /** menu_bar height **/
    double       /** world (data) coordinates for mapping to device **/
      sw_xll,
      sw_xur,
      sw_yll,
      sw_yur,
      sw_yll_rt,
      sw_yur_rt;
    Pixmap
      savePixmap,
      cursPixmap,
      yLabelPixmap;
    char
      helpFile[80],
      fontDir[80],
      temp_text[80],
      temp_font[80];
    Widget
      temp_widget,
      but1,
      but2,
      but3,
      but4,
      but5;
    XgCoord
      temp_coord;
    XgAxis
      temp_axis;
    XgDataInfo
      temp_data;
    XgWidStruct
      wid_array[XgNWID];

  } XgGraphInfo;

/********* external definitions ************/
int XgCreateGraph
  (
   XgGraphInfo * graphinfo
  );

XgGraphInfo * XgCreateStruct(); /* Allocate and initialize XgGraphInfo
				       data structure to default values */

void XgReadGrf  /** read a graph description file **/
  (
   FILE *fp,
   XgGraphInfo * graph
   );
