/************************************************************************
*
*                             XgPlot_loc.h
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*        Header file with local (internal) function prototypes and defines
*
************************************************************************
*/

/*************** help sources ****************/
#define XgHELP_TITLES        101
#define XgHELP_DATA_SELECT   102
#define XgHELP_ADD_FILE      104
#define XgHELP_FONTS         105
#define XgHELP_LOAD_SAVE     107
#define XgHELP_LAYOUT        108
#define XgHELP_DATA_OPTIONS  106
#define XgHELP_SAVE_FILE     109

#define XgEDIT_LABEL    103
#define XgCOORDS        109
#define XgCOLOR_HELP    110
#define XgFILE_TYPE     111
#define XgDATA_DESC     112

/************* error sources ****************/
#define XgERR_NO_FILE         101
#define XgERR_DATA_CALLOC     102
#define XgERR_TOO_MANY_FILES  103
#define XgERR_NO_DELETE_LAST  104
#define XgERR_LAST_Y_LEFT     105
#define XgNOT_IMPLEMENTED     106
#define XgERR_NO_FONT         107
#define XgERR_BAD_COORDS      108
#define XgERR_BAD_MINMAX      109
#define XgERR_BAD_LOG         110
#define XgERR_BAD_PROBABILITY 111
#define XgERR_DATA_NOT_FILE   112
#define Xg_HELP_FILE_TOO_BIG  113

/****************** general defines of current selections ***********/
#define XgXaxisLabel  1
#define XgYLaxisLabel 2
#define XgYRaxisLabel 4
#define XgTitle       8
#define XgSubtitle    16
#define XgLegend      32
#define XgXScale      64
#define XgYLScale     128
#define XgYRScale     256
#define XgData        512
#define XgDSelDelete  32
#define XgDSelFormat  64
#define XgXaxis       128
#define XgYLaxis      256
#define XgYRaxis      512
#define XgDSelSave    1024
#define XgLoad        1
#define XgSave        2
#define XgX_pos       4
#define XgY_pos       8
#define XgMin         0
#define XgMax         1
#define XgCursOn      10
#define XgCursOff     11
#define XgFILE_MAX    1024
#define DEV_COORD     Dimension

/** define the following for y-axis probability labels based on erf(x/sqrt(2).
    Don't define it for labels based on erf(x) **/
#define ERF_X_DIV_ROOT2

/** XgNPROBENT should be one less than the number of XgProb_array entries **/
#define XgNPROB_ENT  14

#ifndef XgInit
extern XmStringCharSet XgCharset;
extern double XgProb_array[XgNPROB_ENT];
#endif

/************** function declarations *********/

/*** XgInit.c ***/
/* XgGraphInfo * XgCreateStruct();    declared in XgPlot.h */
/* int XgCreateGraph(XgGraphInfo * graphinfo);   declared in XgPlot.h */
void Xg_initializeStruct(XgGraphInfo * graph);
int Xg_finishInitialize(XgGraphInfo * graph);
void Xg_createMenus(XgGraphInfo * graph);
XgWidStruct * Xg_AllocStruct(int, caddr_t);

/*** XgHelpErrMisc.c ***/
void Xg_main_help();
void Xg_help_sys();
void Xg_error_sys(int, XgGraphInfo *);
void Xg_cancel_CB();
int Xg_Getline(FILE * fp, char * buf);
void Xg_create_exit_warning();
void Xg_exit_graph();

/*** XgLdSvGr.c ***/
/* void XgReadGrf(FILE *fp, XgGraphInfo * graph);    declared in XgPlot.h */
void Xg_load_save_graph();
void Xg_save_graph();
void Xg_load_graph();

/********* XgTitles.c **********/
void Xg_do_edit_titles();
void Xg_get_temp_text();
void Xg_choose_new_font();
void Xg_get_temp_font();
void Xg_get_temp_coord();
void Xg_set_label_vals();
void Xg_titles_defaults();
void Xg_toggle_legend();

/*********** XgAxis.c ***********/
void Xg_configure_axes();
void Xg_toggle_axes();
void Xg_minmax();
void Xg_set_axes();
void Xg_default_axes();

/********** XgData_Add.c ***********/
void Xg_start_add_file();
void Xg_toggle_ftype();
void Xg_add_file();
void Xg_read_datafile(FILE *fp, XgGraphInfo * graph, int ds_num);

/********** XgData_Edit.c *********/
void Xg_select_data_set();
void Xg_delete_dataset();
void Xg_save_dataset();
void Xg_save_file();
void Xg_edit_dataset();
void Xg_get_temp_legend();
void Xg_toggle_data();
void Xg_set_data_vals();
void Xg_set_data_defaults();

/******** XgDrawGraph.c ***********/
void Xg_createGraph(XgGraphInfo * graph);
GC Xg_createGC(XgGraphInfo *);
void Xg_drawGraph();
void Xg_drag_start();
void Xg_drag_update();
void Xg_drag_finish();

/********** XgSclData.c ***********/
void Xg_scale_data(XgGraphInfo * graph);
double Xg_inverf(double p);

/********** XgDrawLabels.c ***********/
void Xg_drawLabels(XgGraphInfo * graph);

/********** XgDrawAxes.c ***********/
void Xg_drawAxes(XgGraphInfo * graph);

/********** XgDrawDivs.c ***********/
void Xg_drawDivMarks(XgGraphInfo * graph);

/********** XgDrawData.c ***********/
void Xg_plotData(XgGraphInfo * graph);

/********** XgDrawLegend.c ***********/
void Xg_drawLegend(XgGraphInfo * graph);

/******* XgMapCoord.c ********/
void Xg_Setworld(double xll, double yll, double xur, double yur,
		 XgGraphInfo * graph, int left_right);
void Xg_Mapwtod(double xw, double yw, DEV_COORD * xd, DEV_COORD * yd,
	       XgGraphInfo * graph, int left_right);
void Xg_Mapdtow(int xd, int yd, double * xw, double * yw,
		XgGraphInfo * graph, int left_right);

/********** XgPrint.c **********/
void Xg_Print();

/********* XgCursor.c *********/
void Xg_CursorToggle();
void Xg_initcurs(XgGraphInfo * graph);
void Xg_delcurs(XgGraphInfo * graph);
void Xg_oncurs(XgGraphInfo * graph);
void Xg_processArrow();
