/************************************************************************
*
*                             XgPrint.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 4.2   10/93
*
*        Postscript Output
*
*   Hardcoded values that can be easily changed & recompiled:
*    * font type, size
*    * line widths
*    * page size & borders
*
************************************************************************
*/

#include "stdio.h"
#include "math.h"
#include "string.h"
#include "time.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"
#define XgDATA_LINE_WIDTH .2
#define XgSCALE_LINE_WIDTH .2
#define XgMAJOR_DIV_LINE_WIDTH .2
#define XgMINOR_DIV_LINE_WIDTH .2
#define Xg_LegendPSOffset 20

/*************** Postscript Output File *******************/
void Xg_Print(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  FILE
    *fp;
  XgGraphInfo
    *graph;
  long int
    time_sec;
  DEV_COORD
    x1d,
    y1d,
    x2d,
    y2d,
    dxd,
    dyd;
  int
    i,
    j,
    k,
    nx,
    ny,
    de,
    exp,
    len,
    len1,
    num_points,
    left_right,
    scaling,
    minx,
    miny,
    maxx,
    maxy,
    x_font_size,
    y_font_size,
    title_font_size,
    subtitle_font_size,
    desc_font_size,
    legend_font_size,
    scale_font_size;
  double
    min,
    max,
    dminx,
    dminy,
    dmaxx,
    dmaxy,
    x,
    y,
    x1,
    y1,
    y2,
    err,
    x0,
    y0,
    dx,
    z,
    inc;
  float
    fx0,
    fy0,
    fx,
    fy,
    fdx,
    fdy,
    dimy,
    size,
    scalex,
    scaley,
    borderx,
    bordery,
    page_long_dim,
    page_short_dim;
  char
    *chrptr,
    format[10],
    buf2[80],
    buf[80],
    x_font_name[80],
    y_font_name[80],
    title_font_name[80],
    subtitle_font_name[80],
    desc_font_name[80],
    legend_font_name[80],
    scale_font_name[80];

/********** initialize variables ***********/
graph = (XgGraphInfo *) client_data;
time(&time_sec);
x_font_size = 16;
y_font_size = 16;
title_font_size = 20;
subtitle_font_size = 18;
desc_font_size = 14;
legend_font_size = 12;
scale_font_size = 14;
page_long_dim = 11.;
page_short_dim = 8.5;
borderx = 0.;         /** border (in inches) between page edge and graph **/
bordery = 0.;
strcpy(x_font_name, "Helvetica-Bold");
strcpy(y_font_name, "Helvetica-Bold");
strcpy(title_font_name, "Helvetica-Bold");
strcpy(subtitle_font_name, "Helvetica-Bold");
strcpy(desc_font_name, "Helvetica");
strcpy(legend_font_name, "Helvetica");
strcpy(scale_font_name, "Helvetica");

fp = fopen("XgPlt.psc", "w");
if (fp == NULL)
  return;

/******* header ********/
fprintf(fp, "%%!PS-Adobe-1.0\n");
fprintf(fp, "%%%%Creator: XgPlot program\n");
fprintf(fp, "%%%%Title: Graph\n");
fprintf(fp, "%%%%Creation Date: %s", ctime(&time_sec));
fprintf(fp, "%%%%BoundingBox: 0 0 611 791\n");
fprintf(fp, "%%%%End Comments\n");
fprintf(fp, "%%%%Pages: 1\n");
fprintf(fp, "%%%%End Prolog\n");
fprintf(fp, "%%%%Page: 1 1\n\n");
fprintf(fp, "%% For EPS usage:\n");
fprintf(fp, "%%   1) Add EPSF-1.2 at end of first line\n");
fprintf(fp, "%%   2) Switch BoundingBox coordinates to 0 0 791 611\n");
fprintf(fp, "%%   3) Comment out the two lines indicated just below\n");
fprintf(fp, "%%   4) remove showpage at end of this file\n\n");

/********* coordinate system **********/
fprintf(fp, "/inch {72 mul} def\n");
fprintf(fp, "%% comment out the next two lines for EPS usage\n");
fprintf(fp, "%f inch 0 inch translate\n", page_short_dim);
fprintf(fp, "90 rotate\n");
fprintf(fp, "%f inch %f inch translate\n", borderx, bordery);
page_long_dim -= 2*borderx;
page_short_dim -= 2*bordery;

/************** labels **************/
fprintf(fp, "\n%%  Draw Labels\n\n");

if (strlen(graph->x_axis_desc.text) > 0)
  {
    fprintf(fp, "/%s findfont %d scalefont setfont\n",x_font_name,x_font_size);
    fprintf(fp, "%f inch %f inch moveto\n",     /** x-axis **/
	    (graph->x_axis_desc.coord.x / 100. * page_long_dim),
	    ((1. - graph->x_axis_desc.coord.y / 100.) * page_short_dim));
    fprintf(fp, "(%s) dup stringwidth pop neg 2 div %d rmoveto show\n",
	    graph->x_axis_desc.text, x_font_size);
  }

if (strlen(graph->y_axis_desc.text) > 0)
  {
    fprintf(fp, "/%s findfont %d scalefont setfont\n",y_font_name,y_font_size);
    fprintf(fp, "%f inch %f inch moveto\n",     /** left y-axis **/
	    (graph->y_axis_desc.coord.x / 100. * page_long_dim),
	    (graph->y_axis_desc.coord.y / 100. * page_short_dim));
    fprintf(fp, "gsave 90 rotate\n");
    fprintf(fp, "(%s) dup stringwidth pop neg 2 div %d rmoveto show\n",
	    graph->y_axis_desc.text, -y_font_size);
    fprintf(fp, "grestore\n");
  }

if (graph->use_right == TRUE)
  {
    if (strlen(graph->y_rt_axis_desc.text) > 0)
      {
	fprintf(fp, "%f inch %f inch moveto\n",     /** right y-axis **/
		(graph->y_rt_axis_desc.coord.x / 100. * page_long_dim),
		(graph->y_rt_axis_desc.coord.y / 100. * page_short_dim));
	fprintf(fp, "gsave -90 rotate\n");
	fprintf(fp, "(%s) dup stringwidth pop neg 2 div %d rmoveto show\n",
		graph->y_rt_axis_desc.text, -y_font_size);
	fprintf(fp, "grestore\n");
      }
  }

if (strlen(graph->title_desc.text) > 0)
  {
    fprintf(fp, "/%s findfont %d scalefont setfont\n", title_font_name,
	    title_font_size);
    fprintf(fp, "%f inch %f inch moveto\n",     /** title **/
	    (graph->title_desc.coord.x / 100. * page_long_dim),
	    ((1. - graph->title_desc.coord.y / 100.) * page_short_dim));
    fprintf(fp, "(%s) dup stringwidth pop neg 2 div %d rmoveto show\n",
	    graph->title_desc.text, -title_font_size);
  }

if (strlen(graph->subtitle_desc.text) > 0)
  {
    fprintf(fp, "/%s findfont %d scalefont setfont\n", subtitle_font_name,
	    subtitle_font_size);
    fprintf(fp, "%f inch %f inch moveto\n",  /** sub-title **/
	    (graph->subtitle_desc.coord.x / 100. * page_long_dim),
	    ((1. - graph->subtitle_desc.coord.y / 100.) * page_short_dim));
    fprintf(fp, "(%s) dup stringwidth pop neg 2 div %d rmoveto show\n\n",
	    graph->subtitle_desc.text, -subtitle_font_size);
  }

for (j=0; j<graph->num_sets; ++j)  /** data descriptions **/
  {
    if (strlen(graph->dataset[j].description.text) > 0)
      {
	fprintf(fp, "/%s findfont %d scalefont setfont\n", desc_font_name,
		desc_font_size);
	fprintf(fp, "%f inch %f inch moveto\n",
		(graph->dataset[j].description.coord.x / 100. * page_long_dim),
		((1. - graph->dataset[j].description.coord.y / 100.)
		 * page_short_dim));
	fprintf(fp, "(%s) show\n", graph->dataset[j].description.text);
/*	fprintf(fp, "(%s) dup stringwidth pop neg 2 div %d rmoveto show\n",
		graph->dataset[j].description.text, -desc_font_size);*/
      }

  }

/************ legend ***************/
fprintf(fp, "\n%%    Draw Legend\n\n");

j = 0;
len = -1;
k = -1;
for (i=0; i<graph->num_sets; ++i)
  {
    if (graph->dataset[i].in_legend == XgInLegend)
      {  /** find longest string **/
	++j;
	len1 = strlen(graph->dataset[i].legend_text);
	if (len1 > len)
	  {
	    len = len1;
	    k = i;
	  }
      }
  }
dimy = (j+2.) * legend_font_size;

if ((j > 0) && (graph->legend.use_legend == XgLegendOn))
  {
    fprintf(fp, "%lg setlinewidth newpath\n", XgDATA_LINE_WIDTH);
    fprintf(fp, "/%s findfont %d scalefont setfont\n", legend_font_name,
	    legend_font_size);
    fx0 = graph->legend.description.coord.x / 100. * page_long_dim;
    fy0 = (1. - graph->legend.description.coord.y / 100.) * page_short_dim;
    strcpy(buf, graph->dataset[k].legend_text);
    if (graph->use_right == TRUE)
      strcat(buf, "  (L) ");
    else
      strcat(buf, " ");
    fprintf(fp, "(%s) stringwidth pop %d add\n", buf, Xg_LegendPSOffset);

    fprintf(fp, "%f inch %f inch moveto\n", fx0, fy0);
    fprintf(fp, "dup 0 rlineto\n");             /** top **/
    fprintf(fp, "0 %f rlineto\n", -dimy);       /** right **/
    fprintf(fp, "dup neg 0 rlineto\n");         /** bottom **/
    fprintf(fp, "closepath stroke\n");

    fprintf(fp, "%f inch %f inch moveto\n", fx0, fy0);
    fprintf(fp, "2 div %d rmoveto\n", -legend_font_size);
    fprintf(fp, "(%s) dup stringwidth pop neg 2 div 0 rmoveto show\n",
	    graph->legend.description.text);

    j = 1;
    size = 5;
    fdx = Xg_LegendPSOffset / 2.;
    fdy = legend_font_size / 4.;
    for (i=0; i<graph->num_sets; ++i)
      {
	if (graph->dataset[i].in_legend == XgInLegend)
	  {
	    ++j;
	    fx = Xg_LegendPSOffset;
	    fy = -(j+.5) * legend_font_size;
	    sprintf(buf2, "%s ", graph->dataset[i].legend_text);
	    fprintf(fp, "%f inch %f inch moveto\n", fx0, fy0);
	    fprintf(fp, "%f %f rmoveto (%s) show\n", fx, fy, buf2);
	    if (graph->use_right == TRUE)
	      {
		if (graph->dataset[i].left_right == XgLeft)
		  strcpy(buf2, "(L) ");
		else if (graph->dataset[i].left_right == XgRight)
		  strcpy(buf2, "(R) ");
		fprintf(fp, "%f inch %f inch moveto\n", fx0, fy0);
		fprintf(fp, "(%s) stringwidth pop %d add\n",
			buf, Xg_LegendPSOffset);
		fprintf(fp , "%f rmoveto\n", fy);
		fprintf(fp, "(%s) dup stringwidth pop neg 0 rmoveto show\n",
			buf2);
	      }  /** end of (L) or (R) label **/

	    fprintf(fp, "newpath %f inch %f add %f inch %f add\n",
		    fx0, fdx, fy0, fy + fdy);
	    switch (graph->dataset[i].point_type)
	      {
	      case XgPoint:
		fprintf(fp, "%f 0 360 arc fill\n", size/10.);
		break;
	      case XgCircle:
		if (graph->dataset[i].filled_open == XgFilled)
		  fprintf(fp, "%f 0 360 arc fill\n", size/2.);
		else if (graph->dataset[i].filled_open == XgOpen)
		  fprintf(fp, "%f 0 360 arc stroke\n", size/2.);
		break;
	      case XgSquare:
		fprintf(fp, "moveto ");
		if (graph->dataset[i].filled_open == XgFilled)
		  {
		    fprintf(fp, "%f %f rmoveto\n", -size/2., -size/2.);
		    fprintf(fp, "%f 0 rlineto 0 %f rlineto\n", size, size);
		    fprintf(fp, "%f 0 rlineto closepath fill\n", -size);
		  }
		else if (graph->dataset[i].filled_open == XgOpen)
		  {
		    fprintf(fp, "%f %f rmoveto\n", -size/2., -size/2.);
		    fprintf(fp, "%f 0 rlineto 0 %f rlineto\n", size, size);
		    fprintf(fp, "%f 0 rlineto closepath stroke\n", -size);
		  }
		break;
	      case XgTriangle:
		fprintf(fp, "moveto ");
		if (graph->dataset[i].filled_open == XgFilled)
		  {
		    fprintf(fp, "%f %f rmoveto\n", -size/2., -size/2.);
		    fprintf(fp, "%f 0 rlineto %f %f rlineto\n",
			    size, -size/2, size);
		    fprintf(fp, "closepath fill\n");
		  }
		else if (graph->dataset[i].filled_open == XgOpen)
		  {
		    fprintf(fp, "%f %f rmoveto\n", -size/2., -size/2.);
		    fprintf(fp, "%f 0 rlineto %f %f rlineto\n",
			    size, -size/2, size);
		    fprintf(fp, "closepath stroke\n");
		  }
		break;
	      case XgPlus:
		fprintf(fp, "moveto %f 0 rmoveto\n", -size/2.);
		fprintf(fp, "%f 0 rlineto stroke\n", size);
		fprintf(fp, "%f inch %f add %f inch %f add moveto\n",
		    fx0, fdx, fy0, fy + fdy);
		fprintf(fp, "0 %f rmoveto 0 %f rlineto stroke\n",-size/2,size);
		break;
	      case XgEx:
		fprintf(fp, "moveto %f %f rmoveto\n", (float) -size/2.,
			(float) -size/2.);
		fprintf(fp, "%f %f rlineto stroke\n", size, size);
		fprintf(fp, "%f inch %f add %f inch %f add moveto\n",
		    fx0, fdx, fy0, fy + fdy);
		fprintf(fp, "%f %f rmoveto %f %f rlineto stroke\n",
			-size/2., size/2., size, -size);
		break;
	      case XgBar:
		fprintf(fp, "moveto %f %f rmoveto\n", -size/2, -size/2);
		fprintf(fp, "4 0 rlineto 0 6 rlineto\n");
		fprintf(fp, "-4 0 rlineto closepath fill\n");
		break;
	      default:
		break;
	      }

	    if (graph->dataset[i].line_status == XgSolidLine)
	      {
		fprintf(fp, "newpath %f inch %f add %f inch %f add moveto\n",
			fx0, fdx, fy0, fy + fdy);
		fprintf(fp, "%f %f rmoveto %f %f rlineto stroke\n",
			-size, size/4, size*2., -size/2.);
	      }

	  }  /** end of dataset is in legend **/
      }  /** end of loop over datasets for legend text **/
  }  /** end of print legend **/

/************* axes box **************/
fprintf(fp, "\n%%    Draw Axes box\n\n");

fprintf(fp, "%lg setlinewidth newpath\n", XgSCALE_LINE_WIDTH);
nx = graph->nxdraw;
ny = graph->nydraw;
scalex = page_long_dim * 72. / (float) nx;
scaley = page_short_dim * 72. / (float) ny;
fprintf(fp, "%f %f scale\n", scalex, scaley);
Xg_Mapwtod(graph->xaxis_layout.sclmin, graph->yaxis_layout.sclmin,
	  &x1d, &y1d, graph, XgLeft);
Xg_Mapwtod(graph->xaxis_layout.sclmin, graph->yaxis_layout.sclmax,
	  &x2d, &y2d, graph, XgLeft);
y1d = ny - y1d;
y2d = ny - y2d;
fprintf(fp, "%d %d moveto %d %d lineto\n", x1d, y1d, x2d, y2d);
Xg_Mapwtod(graph->xaxis_layout.sclmax, graph->yaxis_layout.sclmax,
	  &x1d, &y1d, graph, XgLeft);
y1d = ny - y1d;
fprintf(fp, "%d %d lineto\n", x1d, y1d);
Xg_Mapwtod(graph->xaxis_layout.sclmax, graph->yaxis_layout.sclmin,
	  &x1d, &y1d, graph, XgLeft);
y1d = ny - y1d;
fprintf(fp, "%d %d lineto closepath stroke\n\n", x1d, y1d);

/************* scales ************/
fprintf(fp, "\n%%    Draw Axis Scales\n\n");

fprintf(fp, "/%s findfont %d scalefont setfont\n", scale_font_name,
	(int) (scale_font_size / scalex));

/****** x axis numbering *******/
if (graph->xaxis_layout.scaling == XgLogarithm)
  {
    y = graph->yaxis_layout.sclmin;
    y1 = graph->yaxis_layout.sclmax;
    min = graph->xaxis_layout.sclmin;
    max = graph->xaxis_layout.sclmax;
    for (x=min; x<=max; ++x)
      {   /** tick marks across bottom **/
	Xg_Mapwtod(x, y, &x1d, &y1d, graph, XgLeft);
	y1d = ny - y1d;
	fprintf(fp, "%d %d moveto 0 -5 rlineto stroke\n", x1d, y1d);

	z = x;
	i = x;
	sprintf(buf2, "10^%d", i);
	if ((max <= 5) && (min >= -4))
	  {
	    z = pow(10., x);
	    sprintf(buf2, "%lg", z);
	  }
	fprintf(fp, "%d %d moveto (%s) dup stringwidth\n", x1d, y1d-5, buf2);
	fprintf(fp, "pop neg 2 div %d rmoveto show\n", 
		(int) (-scale_font_size / scalex));
      }
  }  /** end x-axis Logarithm **/

if (graph->xaxis_layout.scaling == XgLinear)
  {
    max = fabs(graph->xaxis_layout.sclmax);    /** x axis scale factor **/
    min = fabs(graph->xaxis_layout.sclmin);
    if (max > min)
      sprintf(buf, "%le", 0.9999 * max);
    else  /** the .9999 avoids a problem if max is an exact power of 10 **/
      sprintf(buf, "%le", 0.9999 * min);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
      ++chrptr;
    sscanf(chrptr, "%d", &exp);

    de = 0;
    if (graph->xaxis_layout.label_type == XgEng_label)
      {
	while (exp/3. != floor(exp/3.))
	  {
	    --exp;
	    ++de;
	  }
      }
    
    if ( (graph->xaxis_layout.label_type == XgSci_label)
	|| (graph->xaxis_layout.label_type == XgEng_label) )
      {
	sprintf(buf, "x data = graph * 10^%d", exp);
	x = graph->xaxis_layout.scale_desc.coord.x * nx / 100.;
	y = graph->xaxis_layout.scale_desc.coord.y * ny / 100.;
	if (exp != 0)
	  {
	    fprintf(fp, "%lg (%s) stringwidth pop add %d gt\n", x, buf, nx);
	    fprintf(fp, "  { %f %f moveto\n", nx, ny - y);
	    fprintf(fp, "    (%s) dup stringwidth\n", buf);
	    fprintf(fp, "    pop neg %d rmoveto show }\n",
		    (int) (-2. * scale_font_size / scalex));
	    fprintf(fp, "  { %f %f moveto\n", x, ny - y);
	    fprintf(fp, "    0 %d rmoveto (%s) show } ifelse\n",
		    (int) (-2. * scale_font_size / scalex), buf);
	  }

	j = graph->xaxis_layout.nsig_fig - XgSig0 - 1 - de;
	if (j < 0)
	  j = 0;
	sprintf(format, "%%%d.%dlf", graph->xaxis_layout.nsig_fig-XgSig0 +1,j);
      }  /** end of no scientific notation labels **/

    if (graph->xaxis_layout.label_type == XgNo_label)
      {
	sprintf(format, "%%lg");
	if ((exp == 0) && (graph->xaxis_layout.nsig_fig == XgSig2))
	  sprintf(format, "%%3.1lf");  /** e.g. 8.9, 9.0 **/
	if ((exp == 0) && (graph->xaxis_layout.nsig_fig == XgSig3))
	  sprintf(format, "%%4.2lf");  /** e.g. 8.95, 9.00 **/
	if ((exp == 1) && (graph->xaxis_layout.nsig_fig == XgSig3))
	  sprintf(format, "%%4.1lf");  /** e.g. 89.5, 90.0 **/
	if (exp < 0)   /** e.g. 0.055, 0.060 **/
	  {
	    i = -exp + graph->xaxis_layout.nsig_fig - XgSig0 - 1;
	    sprintf(format, "%%%d.%dlf", i+2, i);
	  }
      }

    y = graph->yaxis_layout.sclmin;
    y1 = graph->yaxis_layout.sclmax;
    min = graph->xaxis_layout.sclmin;
    max = graph->xaxis_layout.sclmax;
    inc = pow(10., (double) ((exp + de)
			     - (graph->xaxis_layout.nsig_fig - XgSig0-1)));
    inc = (double) (graph->xaxis_layout.increment - XgLast0) * inc;

    for (x=min; x<=1.0005*max; x+=inc)
      {   /** tick marks across bottom **/
	Xg_Mapwtod(x, y, &x1d, &y1d, graph, XgLeft);
	y1d = ny - y1d;
	fprintf(fp, "%d %d moveto 0 -5 rlineto stroke\n", x1d, y1d);
	z = x;
	if ( (graph->xaxis_layout.label_type == XgSci_label)
	    || (graph->xaxis_layout.label_type == XgEng_label) )
	  z = z / pow(10., (double) exp);
	else
	  if (fabs(z) < inc / 10.)
	    z = 0.;
	sprintf(buf, format, z);
	sscanf(buf, "%s", buf2);
	fprintf(fp, "%d %d moveto (%s) dup stringwidth\n", x1d, y1d-5, buf2);
	fprintf(fp, "pop neg 2 div %d rmoveto show\n", 
		(int) (-scale_font_size / scalex));
      }
  }  /** end x-axis Linear **/

/******* y-axis numbering *******/
if (graph->yaxis_layout.scaling == XgLogarithm)
  {
    x = graph->xaxis_layout.sclmin;
    x1 = graph->xaxis_layout.sclmax;
    min = graph->yaxis_layout.sclmin;
    max = graph->yaxis_layout.sclmax;
    for (y=min; y<=max; ++y)
      {  /** tick marks down left side **/
	Xg_Mapwtod(x, y, &x1d, &y1d, graph, XgLeft);
	y1d = ny - y1d;
	fprintf(fp, "%d %d moveto -5 0 rlineto stroke\n", x1d, y1d);

	z = y;
	i = y;
	sprintf(buf2, "10^%d", i);
	if ((max <= 5) && (min >= -4))
	  {
	    z = pow(10., y);
	    sprintf(buf2, "%lg", z);
	  }
	fprintf(fp, "%d %d moveto (%s) dup stringwidth\n", x1d-6, y1d, buf2);
	fprintf(fp, "pop neg %d rmoveto show\n", 
		(int) (-scale_font_size / scalex / 2.));
      }
  }  /** end y-axis Logarithm **/

if (graph->yaxis_layout.scaling == XgProbability)
  {
    x = graph->xaxis_layout.sclmin;
    x1 = graph->xaxis_layout.sclmax;
    min = graph->yaxis_layout.prob_mindx;
    max = graph->yaxis_layout.prob_maxdx;
    for (i=min; i<=max; ++i)
      {
	y1 = Xg_inverf(XgProb_array[i]);
	/** tick marks down left side **/
	Xg_Mapwtod(x, y1, &x1d, &y1d, graph, XgLeft);
	y1d = ny - y1d;
	fprintf(fp, "%d %d moveto -5 0 rlineto stroke\n", x1d, y1d);
	sprintf(buf2, "%lg", XgProb_array[i]);
	fprintf(fp, "%d %d moveto (%s) dup stringwidth\n", x1d-6, y1d, buf2);
	fprintf(fp, "pop neg %d rmoveto show\n", 
		(int) (-scale_font_size / scalex / 2.));
      }
  }  /** end y-axis Probability **/

if (graph->yaxis_layout.scaling == XgLinear)
  {
    max = fabs(graph->yaxis_layout.sclmax);   /** y axis scale factor **/
    min = fabs(graph->yaxis_layout.sclmin);
    if (max > min)
      sprintf(buf, "%le", 0.9999 * max);
    else  /** the .9999 avoids a problem if max is an exact power of 10 **/
      sprintf(buf, "%le", 0.9999 * min);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
      ++chrptr;
    sscanf(chrptr, "%d", &exp);

    de = 0;
    if (graph->yaxis_layout.label_type == XgEng_label)
      {
	while (exp/3. != floor(exp/3.))
	  {
	    --exp;
	    ++de;
	  }
      }

    if ( (graph->yaxis_layout.label_type == XgSci_label)
	|| (graph->yaxis_layout.label_type == XgEng_label) )
      {
	sprintf(buf, "y data = graph * 10^%d", exp);
	x = graph->yaxis_layout.scale_desc.coord.x * nx / 100.;
	y = graph->yaxis_layout.scale_desc.coord.y * ny / 100.;
	if (exp != 0)
	  {
	    fprintf(fp, "%lg (%s) stringwidth pop add %d gt\n", x, buf, nx);
	    fprintf(fp, "  { %f %f moveto\n", nx, ny - y);
	    fprintf(fp, "    (%s) dup stringwidth\n", buf);
	    fprintf(fp, "    pop neg %d rmoveto show }\n",
		    (int) (-2. * scale_font_size / scalex));
	    fprintf(fp, "  { %f %f moveto\n", x, ny - y);
	    fprintf(fp, "    0 %d rmoveto (%s) show } ifelse\n",
		    (int) (-2. * scale_font_size / scalex), buf);
	  }

	j = graph->yaxis_layout.nsig_fig - XgSig0 - 1 - de;
	if (j < 0)
	  j = 0;
	sprintf(format, "%%%d.%dlf", graph->yaxis_layout.nsig_fig-XgSig0+1, j);
      }  /** end sci. not. label **/

    if (graph->yaxis_layout.label_type == XgNo_label)
      {
	sprintf(format, "%%lg");
	if ((exp == 0) && (graph->yaxis_layout.nsig_fig == XgSig2))
	  sprintf(format, "%%3.1lf");  /** e.g. 8.9, 9.0 **/
	if ((exp == 0) && (graph->yaxis_layout.nsig_fig == XgSig3))
	  sprintf(format, "%%4.2lf");  /** e.g. 8.95, 9.00 **/
	if ((exp == 1) && (graph->yaxis_layout.nsig_fig == XgSig3))
	  sprintf(format, "%%4.1lf");  /** e.g. 89.5, 90.0 **/
	if (exp < 0)   /** e.g. 0.055, 0.060 **/
	  {
	    i = -exp + graph->yaxis_layout.nsig_fig - XgSig0 - 1;
	    sprintf(format, "%%%d.%dlf", i+2, i);
	  }
      }

    x = graph->xaxis_layout.sclmin;
    x1 = graph->xaxis_layout.sclmax;
    min = graph->yaxis_layout.sclmin;
    max = graph->yaxis_layout.sclmax;
    inc = pow(10., (double) ((exp + de)
			     - (graph->yaxis_layout.nsig_fig - XgSig0-1)));
    inc = (double) (graph->yaxis_layout.increment - XgLast0) * inc;

    for (y=min; y<=1.0005*max; y+=inc)
      {  /** tick marks down left side **/
	Xg_Mapwtod(x, y, &x1d, &y1d, graph, XgLeft);
	y1d = ny - y1d;
	fprintf(fp, "%d %d moveto -5 0 rlineto stroke\n", x1d, y1d);

	z = y;
	if ( (graph->yaxis_layout.label_type == XgSci_label)
	    || (graph->yaxis_layout.label_type == XgEng_label) )
	  z = z / pow(10., (double) exp);
	else
	  if (fabs(z) < inc / 10.)
	    z = 0.;
	sprintf(buf, format, z);
	sscanf(buf, "%s", buf2);
	fprintf(fp, "%d %d moveto (%s) dup stringwidth\n", x1d-6, y1d, buf2);
	fprintf(fp, "pop neg %d rmoveto show\n", 
		(int) (-scale_font_size / scalex / 2.));
      }
  }  /** end y-axis Linear **/

/*********** Right y-axis numbering ***********/
if (graph->use_right == TRUE)
  {
    if (graph->y_rt_axis_layout.scaling == XgLogarithm)
      {
	x = graph->xaxis_layout.sclmin;
	x1 = graph->xaxis_layout.sclmax;
	min = graph->y_rt_axis_layout.sclmin;
	max = graph->y_rt_axis_layout.sclmax;
	for (y=min; y<=max; ++y)
	  {  /** tick marks down right side **/
	    Xg_Mapwtod(x1, y, &x1d, &y1d, graph, XgRight);
	    y1d = ny - y1d;
	    fprintf(fp, "%d %d moveto 5 0 rlineto stroke\n", x1d, y1d);
	    
	    z = y;
	    i = y;
	    sprintf(buf2, "10^%d", i);
	    if ((max <= 5) && (min >= -4))
	      {
		z = pow(10., y);
		sprintf(buf2, "%lg", z);
	      }
	    fprintf(fp, "%d %d moveto (%s) show\n", x1d+6,
		    y1d - (int) (scale_font_size / scaley / 2.), buf2);
	  }
      }  /** end y-axis Logarithm **/

    if (graph->y_rt_axis_layout.scaling == XgProbability)
      {
	x = graph->xaxis_layout.sclmin;
	x1 = graph->xaxis_layout.sclmax;
	min = graph->y_rt_axis_layout.prob_mindx;
	max = graph->y_rt_axis_layout.prob_maxdx;
	for (i=min; i<=max; ++i)
	  {
	    y1 = Xg_inverf(XgProb_array[i]);
	    /** tick marks down right side **/
	    Xg_Mapwtod(x1, y1, &x1d, &y1d, graph, XgRight);
	    y1d = ny - y1d;
	    fprintf(fp, "%d %d moveto 5 0 rlineto stroke\n", x1d, y1d);
	    sprintf(buf2, "%lg", XgProb_array[i]);
	    fprintf(fp, "%d %d moveto (%s) show\n", x1d+6,
		    y1d - (int) (scale_font_size / scaley / 2.), buf2);
	  }
      }  /** end y-axis Probability **/

    if (graph->y_rt_axis_layout.scaling == XgLinear)
      {
	max = fabs(graph->y_rt_axis_layout.sclmax); /** y axis scale factor **/
	min = fabs(graph->y_rt_axis_layout.sclmin);
	if (max > min)
	  sprintf(buf, "%le", 0.9999 * max);
	else  /** the .9999 avoids a problem if max is an exact power of 10 **/
	  sprintf(buf, "%le", 0.9999 * min);
	chrptr = strchr(buf, 'e');
	++chrptr;
	if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
	  ++chrptr;
	sscanf(chrptr, "%d", &exp);

	de = 0;
	if (graph->y_rt_axis_layout.label_type == XgEng_label)
	  {
	    while (exp/3. != floor(exp/3.))
	      {
		--exp;
		++de;
	      }
	  }

	if ( (graph->y_rt_axis_layout.label_type == XgSci_label)
	    || (graph->y_rt_axis_layout.label_type == XgEng_label) )
	  {
	    sprintf(buf, "y(r) data = graph * 10^%d", exp);
	    x = graph->y_rt_axis_layout.scale_desc.coord.x * nx / 100.;
	    y = graph->y_rt_axis_layout.scale_desc.coord.y * ny / 100.;
	    if (exp != 0)
	      {
		fprintf(fp, "%lg (%s) stringwidth pop add %d gt\n", x, buf,nx);
		fprintf(fp, "  { %f %f moveto\n", nx, ny - y);
		fprintf(fp, "    (%s) dup stringwidth\n", buf);
		fprintf(fp, "    pop neg %d rmoveto show }\n",
			(int) (-2. * scale_font_size / scalex));
		fprintf(fp, "  { %f %f moveto\n", x, ny - y);
		fprintf(fp, "    0 %d rmoveto (%s) show } ifelse\n",
			(int) (-2. * scale_font_size / scalex), buf);
	      }

	    j = graph->y_rt_axis_layout.nsig_fig - XgSig0 - 1 - de;
	    if (j < 0)
	      j = 0;
	    sprintf(format, "%%%d.%dlf",
		    graph->y_rt_axis_layout.nsig_fig - XgSig0 + 1, j);
	  }  /** end sci. not. label **/

	if (graph->y_rt_axis_layout.label_type == XgNo_label)
	  {
	    sprintf(format, "%%lg");
	    if ((exp == 0) && (graph->y_rt_axis_layout.nsig_fig == XgSig2))
	      sprintf(format, "%%3.1lf");  /** e.g. 8.9, 9.0 **/
	    if ((exp == 0) && (graph->y_rt_axis_layout.nsig_fig == XgSig3))
	      sprintf(format, "%%4.2lf");  /** e.g. 8.95, 9.00 **/
	    if ((exp == 1) && (graph->y_rt_axis_layout.nsig_fig == XgSig3))
	      sprintf(format, "%%4.1lf");  /** e.g. 89.5, 90.0 **/
	    if (exp < 0)   /** e.g. 0.055, 0.060 **/
	      {
		i = -exp + graph->y_rt_axis_layout.nsig_fig - XgSig0 - 1;
		sprintf(format, "%%%d.%dlf", i+2, i);
	      }
	  }

	x = graph->xaxis_layout.sclmin;
	x1 = graph->xaxis_layout.sclmax;
	min = graph->y_rt_axis_layout.sclmin;
	max = graph->y_rt_axis_layout.sclmax;
	inc = pow(10., (double) ((exp + de)
			    - (graph->y_rt_axis_layout.nsig_fig - XgSig0-1)));
	inc = (double) (graph->y_rt_axis_layout.increment - XgLast0) * inc;

	for (y=min; y<=1.0005*max; y+=inc)
	  {  /** tick marks down right side **/
	    Xg_Mapwtod(x1, y, &x1d, &y1d, graph, XgRight);
	    y1d = ny - y1d;
	    fprintf(fp, "%d %d moveto 5 0 rlineto stroke\n", x1d, y1d);

	    z = y;
	    if ( (graph->y_rt_axis_layout.label_type == XgSci_label)
		|| (graph->y_rt_axis_layout.label_type == XgEng_label) )
	      z = z / pow(10., (double) exp);
	    else
	      if (fabs(z) < inc / 10.)
		z = 0.;
	    sprintf(buf, format, z);
	    sscanf(buf, "%s", buf2);
	    fprintf(fp, "%d %d moveto (%s) show\n", x1d+6,
		    y1d - (int) (scale_font_size / scaley / 2.), buf2);
	  }
      }  /** end y-axis Linear **/
  }  /** end of Right y axis scaling **/

/*********** div marks *************/
fprintf(fp, "\n%%    Draw Division Marks\n\n");

fprintf(fp, "newpath %lg setlinewidth\n", XgSCALE_LINE_WIDTH);
nx = graph->nxdraw;
ny = graph->nydraw;

/******* x axis div marks *********/
if (graph->xaxis_layout.scaling == XgLogarithm)
  {
    min = graph->xaxis_layout.sclmin;
    max = graph->xaxis_layout.sclmax;
    inc = 1.;
  }
else if (graph->xaxis_layout.scaling == XgLinear)
  {
    /** get exp **/
    max = fabs(graph->xaxis_layout.sclmax);
    min = fabs(graph->xaxis_layout.sclmin);
    if (max > min)
      sprintf(buf, "%le", 0.9999 * max);
    else  /** the .9999 avoids a problem if max is an exact power of 10 **/
      sprintf(buf, "%le", 0.9999 * min);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
      ++chrptr;
    sscanf(chrptr, "%d", &exp);

    de = 0;
    if (graph->xaxis_layout.label_type == XgEng_label)
      {
	while (exp/3. != floor(exp/3.))
	  {
	    --exp;
	    ++de;
	  }
      }

    min = graph->xaxis_layout.sclmin;
    max = graph->xaxis_layout.sclmax;
    inc = pow(10., (double) ((exp + de)
			     - (graph->xaxis_layout.nsig_fig - XgSig0-1)));
    inc = (double) (graph->xaxis_layout.increment - XgLast0) * inc;
  }

y = graph->yaxis_layout.sclmin;
y1 = graph->yaxis_layout.sclmax;

for (x=min; x<max; x+=inc)
  {
    if ( ( (graph->xaxis_layout.div_marks == XgDivMajorTic)
	  || (graph->xaxis_layout.div_marks == XgDivMinorTic) )
	&& (x>min) && (x<max))
      {
	/** tick marks across bottom **/
	Xg_Mapwtod(x, y, &x1d, &y1d, graph, XgLeft);
	y1d = ny - y1d;
	fprintf(fp, "%d %d moveto 0 10 rlineto stroke\n", x1d, y1d);
	/** tick marks across top **/
	Xg_Mapwtod(x, y1, &x1d, &y1d, graph, XgLeft);
	y1d = ny - y1d;
	fprintf(fp, "%d %d moveto 0 -10 rlineto stroke\n", x1d, y1d);
      }  /** end of Major Div Tics **/

    if ( ( (graph->xaxis_layout.div_marks == XgDivMajorLin)
	  || (graph->xaxis_layout.div_marks == XgDivMinorLin)
	  || (graph->xaxis_layout.div_marks == XgDivLin_Tic) )
	&& (x>min) && (x<max))
      {
	fprintf(fp, "%lg setlinewidth\n", XgMAJOR_DIV_LINE_WIDTH);
	Xg_Mapwtod(x, y, &x1d, &y1d, graph, XgLeft);
	Xg_Mapwtod(x, y1, &x2d, &y2d, graph, XgLeft);
	y1d = ny - y1d;
	y2d = ny - y2d;
	fprintf(fp, "%d %d moveto %d %d lineto stroke\n", x1d, y1d, x2d, y2d);
	fprintf(fp, "%lg setlinewidth\n", XgSCALE_LINE_WIDTH);
      }  /** end of Major Div Lines **/

    if ((graph->xaxis_layout.div_marks == XgDivMinorTic)
	  || (graph->xaxis_layout.div_marks == XgDivLin_Tic))
      {
	if (graph->xaxis_layout.scaling == XgLogarithm)
	  {
	    dx = pow(10., x);
	    for (i=2; i<10; ++i)
	      {
		z = log10(i*dx);
		Xg_Mapwtod(z, y, &x1d, &y1d, graph, XgLeft);
		y1d = ny - y1d;
		fprintf(fp, "%d %d moveto 0 5 rlineto stroke\n", x1d, y1d);
		Xg_Mapwtod(z, y1, &x1d, &y1d, graph, XgLeft);
		y1d = ny - y1d;
		fprintf(fp, "%d %d moveto 0 -5 rlineto stroke\n",x1d,y1d);
	      }
	  }  /** end of XgLogarithm **/
	else if (graph->xaxis_layout.scaling == XgLinear)
	  {
	    if (graph->xaxis_layout.increment == XgLast2)
	      j = 2;
	    else if (graph->xaxis_layout.increment == XgLast5)
	      j = 5;
	    else if (graph->xaxis_layout.increment == XgLast1)
	      j = 10;
	    dx = inc / (double) j;
	    for (i=1; i<j; ++i)
	      {
		z = x + dx*i;
		if ((z > min) && (z < max))
		  {
		    Xg_Mapwtod(z, y, &x1d, &y1d, graph, XgLeft);
		    y1d = ny - y1d;
		    fprintf(fp, "%d %d moveto 0 5 rlineto stroke\n", x1d, y1d);
		    Xg_Mapwtod(z, y1, &x1d, &y1d, graph, XgLeft);
		    y1d = ny - y1d;
		    fprintf(fp, "%d %d moveto 0 -5 rlineto stroke\n", x1d,y1d);
		  }
	      }
	  }  /** end of Linear **/
      }  /** end of Minor Div Tics **/

    if (graph->xaxis_layout.div_marks == XgDivMinorLin)
      {
	fprintf(fp, "[8 8] 0 setdash %lg setlinewidth\n",
		XgMINOR_DIV_LINE_WIDTH);
	if (graph->xaxis_layout.scaling == XgLogarithm)
	  {
	    dx = pow(10., x);
	    for (i=2; i<10; ++i)
	      {
		z = log10(i*dx);
		Xg_Mapwtod(z, y, &x1d, &y1d, graph, XgLeft);
		Xg_Mapwtod(z, y1, &x2d, &y2d, graph, XgLeft);
		y1d = ny - y1d;
		y2d = ny - y2d;
		fprintf(fp, "%d %d moveto %d %d lineto stroke\n",
			x1d, y1d, x2d, y2d);
	      }
	  }
	else if (graph->xaxis_layout.scaling == XgLinear)
	  {
	    if (graph->xaxis_layout.increment == XgLast2)
	      j = 2;
	    else if (graph->xaxis_layout.increment == XgLast5)
	      j = 5;
	    else if (graph->xaxis_layout.increment == XgLast1)
	      j = 10;
	    dx = inc / (double) j;
	    for (i=1; i<j; ++i)
	      {
		z = x + dx*i;
		if ((z > min) && (z < max))
		  {
		    Xg_Mapwtod(z, y, &x1d, &y1d, graph, XgLeft);
		    Xg_Mapwtod(z, y1, &x2d, &y2d, graph, XgLeft);
		    y1d = ny - y1d;
		    y2d = ny - y2d;
		    fprintf(fp, "%d %d moveto %d %d lineto stroke\n",
			    x1d, y1d, x2d, y2d);
		  }
	      }
	  }  /** end of Linear **/
	fprintf(fp, "[] 0 setdash %lg setlinewidth\n",
		XgSCALE_LINE_WIDTH);
      }  /** end of Minor Div Lines **/
  }  /** end of x loop **/

/******* y axis div marks *********/
if (graph->yaxis_layout.scaling == XgLogarithm)
  {
    min = graph->yaxis_layout.sclmin;
    max = graph->yaxis_layout.sclmax;
    inc = 1.;
  }
else if (graph->yaxis_layout.scaling == XgProbability)
  {
    min = graph->yaxis_layout.prob_mindx;
    max = graph->yaxis_layout.prob_maxdx;
    inc = 1.;
  }
else if (graph->yaxis_layout.scaling == XgLinear)
  {
    /** get exp **/
    max = fabs(graph->yaxis_layout.sclmax);
    min = fabs(graph->yaxis_layout.sclmin);
    if (max > min)
      sprintf(buf, "%le", 0.9999 * max);
    else  /** the .9999 avoids a problem if max is an exact power of 10 **/
      sprintf(buf, "%le", 0.9999 * min);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
      ++chrptr;
    sscanf(chrptr, "%d", &exp);

    de = 0;
    if (graph->yaxis_layout.label_type == XgEng_label)
      {
	while (exp/3. != floor(exp/3.))
	  {
	    --exp;
	    ++de;
	  }
      }

    min = graph->yaxis_layout.sclmin;
    max = graph->yaxis_layout.sclmax;
    inc = pow(10., (double) ((exp + de)
			     - (graph->yaxis_layout.nsig_fig - XgSig0-1)));
    inc = (double) (graph->yaxis_layout.increment - XgLast0) * inc;
  }

x = graph->xaxis_layout.sclmin;
x1 = graph->xaxis_layout.sclmax;

for (y=min; y<max; y+=inc)
  {
    y1 = y;
    if (graph->yaxis_layout.scaling == XgProbability)
      y1 = Xg_inverf(XgProb_array[(int) y]);

    if ( ( (graph->yaxis_layout.div_marks == XgDivMajorTic)
	  || (graph->yaxis_layout.div_marks == XgDivMinorTic) )
	&& (y>min) && (y<max))
      {
	/** tick marks down left **/
	Xg_Mapwtod(x, y1, &x1d, &y1d, graph, XgLeft);
	y1d = ny - y1d;
	fprintf(fp, "%d %d moveto 10 0 rlineto stroke\n", x1d, y1d);
	/** tick marks down right **/
	if (graph->use_right == FALSE)
	  {
	    Xg_Mapwtod(x1, y1, &x1d, &y1d, graph, XgLeft);
	    y1d = ny - y1d;
	    fprintf(fp, "%d %d moveto -10 0 rlineto stroke\n", x1d, y1d);
	  }
      }  /** end of Major Div Tics **/

    if ( ( (graph->yaxis_layout.div_marks == XgDivMajorLin)
	  || (graph->yaxis_layout.div_marks == XgDivMinorLin)
	  || (graph->yaxis_layout.div_marks == XgDivLin_Tic) )
	&& (y>min) && (y<max))
      {
	fprintf(fp, "%lg setlinewidth\n", XgMAJOR_DIV_LINE_WIDTH);
	Xg_Mapwtod(x, y1, &x1d, &y1d, graph, XgLeft);
	Xg_Mapwtod(x1, y1, &x2d, &y2d, graph, XgLeft);
	y1d = ny - y1d;
	y2d = ny - y2d;
	fprintf(fp, "%d %d moveto %d %d lineto stroke\n", x1d, y1d, x2d, y2d);
	fprintf(fp, "%lg setlinewidth\n", XgSCALE_LINE_WIDTH);
      }  /** end of Major Div Lines **/

    if ((graph->yaxis_layout.div_marks == XgDivMinorTic)
	  || (graph->yaxis_layout.div_marks == XgDivLin_Tic))
      {
	if (graph->yaxis_layout.scaling == XgLogarithm)
	  {
	    dx = pow(10., y);
	    for (i=2; i<10; ++i)
	      {
		z = log10(i*dx);
		Xg_Mapwtod(x, z, &x1d, &y1d, graph, XgLeft);
		y1d = ny - y1d;
		fprintf(fp, "%d %d moveto 5 0 rlineto stroke\n", x1d, y1d);
		if (graph->use_right == FALSE)
		  {
		    Xg_Mapwtod(x1, z, &x1d, &y1d, graph, XgLeft);
		    y1d = ny - y1d;
		    fprintf(fp, "%d %d moveto -5 0 rlineto stroke\n",x1d,y1d);
		  }
	      }
	  }  /** end of XgLogarithm **/
	else if (graph->yaxis_layout.scaling == XgLinear)
	  {
	    if (graph->yaxis_layout.increment == XgLast2)
	      j = 2;
	    else if (graph->yaxis_layout.increment == XgLast5)
	      j = 5;
	    else if (graph->yaxis_layout.increment == XgLast1)
	      j = 10;
	    dx = inc / (double) j;
	    for (i=1; i<j; ++i)
	      {
		z = y + dx*i;
		if ((z > min) && (z < max))
		  {
		    Xg_Mapwtod(x, z, &x1d, &y1d, graph, XgLeft);
		    y1d = ny - y1d;
		    fprintf(fp, "%d %d moveto 5 0 rlineto stroke\n", x1d, y1d);
		    if (graph->use_right == FALSE)
		      {
			Xg_Mapwtod(x1, z, &x1d, &y1d, graph, XgLeft);
			y1d = ny - y1d;
			fprintf(fp, "%d %d moveto -5 0 rlineto stroke\n",
				x1d, y1d);
		      }
		  }
	      }
	  }  /** end of Linear **/
      }  /** end of Minor Div Tics **/

    if (graph->yaxis_layout.div_marks == XgDivMinorLin)
      {
	fprintf(fp, "[8 8] 0 setdash %lg setlinewidth\n",
		XgMINOR_DIV_LINE_WIDTH);
	if (graph->yaxis_layout.scaling == XgLogarithm)
	  {
	    dx = pow(10., y);
	    for (i=2; i<10; ++i)
	      {
		z = log10(i*dx);
		Xg_Mapwtod(x, z, &x1d, &y1d, graph, XgLeft);
		Xg_Mapwtod(x1, z, &x2d, &y2d, graph, XgLeft);
		y1d = ny - y1d;
		y2d = ny - y2d;
		fprintf(fp, "%d %d moveto %d %d lineto stroke\n",
			x1d, y1d, x2d, y2d);
	      }
	  }
	else if (graph->yaxis_layout.scaling == XgLinear)
	  {
	    if (graph->yaxis_layout.increment == XgLast2)
	      j = 2;
	    else if (graph->yaxis_layout.increment == XgLast5)
	      j = 5;
	    else if (graph->yaxis_layout.increment == XgLast1)
	      j = 10;
	    dx = inc / (double) j;
	    for (i=1; i<j; ++i)
	      {
		z = y + dx*i;
		if ((z > min) && (z < max))
		  {
		    Xg_Mapwtod(x, z, &x1d, &y1d, graph, XgLeft);
		    Xg_Mapwtod(x1, z, &x2d, &y2d, graph, XgLeft);
		    y1d = ny - y1d;
		    y2d = ny - y2d;
		    fprintf(fp, "%d %d moveto %d %d lineto stroke\n",
			    x1d, y1d, x2d, y2d);
		  }
	      }
	  }  /** end of Linear **/
	fprintf(fp, "[] 0 setdash %lg setlinewidth\n",
		XgSCALE_LINE_WIDTH);
      }  /** end of Minor Div Lines **/
  }  /** end of y loop **/

/******* right y axis div marks *********/
if (graph->use_right == TRUE)
  {
    if (graph->y_rt_axis_layout.scaling == XgLogarithm)
      {
	min = graph->y_rt_axis_layout.sclmin;
	max = graph->y_rt_axis_layout.sclmax;
	inc = 1.;
      }
    else if (graph->y_rt_axis_layout.scaling == XgProbability)
      {
	min = graph->y_rt_axis_layout.prob_mindx;
	max = graph->y_rt_axis_layout.prob_maxdx;
	inc = 1.;
      }
    else if (graph->y_rt_axis_layout.scaling == XgLinear)
      {
	/** get exp **/
	max = fabs(graph->y_rt_axis_layout.sclmax);
	min = fabs(graph->y_rt_axis_layout.sclmin);
	if (max > min)
	  sprintf(buf, "%le", 0.9999 * max);
	else  /** the .9999 avoids a problem if max is an exact power of 10 **/
	  sprintf(buf, "%le", 0.9999 * min);
	chrptr = strchr(buf, 'e');
	++chrptr;
	if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
	  ++chrptr;
	sscanf(chrptr, "%d", &exp);

	de = 0;
	if (graph->y_rt_axis_layout.label_type == XgEng_label)
	  {
	    while (exp/3. != floor(exp/3.))
	      {
		--exp;
		++de;
	      }
	  }

	min = graph->y_rt_axis_layout.sclmin;
	max = graph->y_rt_axis_layout.sclmax;
	inc = pow(10., (double) ((exp + de)
			     - (graph->y_rt_axis_layout.nsig_fig - XgSig0-1)));
	inc = (double) (graph->y_rt_axis_layout.increment - XgLast0) * inc;
      }

    x = graph->xaxis_layout.sclmin;
    x1 = graph->xaxis_layout.sclmax;

    for (y=min; y<max; y+=inc)
      {
	y1 = y;
	if (graph->y_rt_axis_layout.scaling == XgProbability)
	  y1 = Xg_inverf(XgProb_array[(int) y]);

	if ( ( (graph->y_rt_axis_layout.div_marks == XgDivMajorTic)
	      || (graph->y_rt_axis_layout.div_marks == XgDivMinorTic) )
	    && (y>min) && (y<max))
	  {
	    /** tick marks down left **/
	    Xg_Mapwtod(x1, y1, &x1d, &y1d, graph, XgRight);
	    y1d = ny - y1d;
	    fprintf(fp, "%d %d moveto -10 0 rlineto stroke\n", x1d, y1d);
	    /** tick marks down right **/
	    if (graph->use_right == FALSE)
	      {
		Xg_Mapwtod(x, y1, &x1d, &y1d, graph, XgRight);
		y1d = ny - y1d;
		fprintf(fp, "%d %d moveto 10 0 rlineto stroke\n", x1d, y1d);
	      }
	  }  /** end of Major Div Tics **/

	if ( ( (graph->y_rt_axis_layout.div_marks == XgDivMajorLin)
	      || (graph->y_rt_axis_layout.div_marks == XgDivMinorLin)
	      || (graph->y_rt_axis_layout.div_marks == XgDivLin_Tic) )
	    && (y>min) && (y<max))
	  {
	    fprintf(fp, "%lg setlinewidth\n", XgMAJOR_DIV_LINE_WIDTH);
	    Xg_Mapwtod(x, y1, &x1d, &y1d, graph, XgRight);
	    Xg_Mapwtod(x1, y1, &x2d, &y2d, graph, XgRight);
	    y1d = ny - y1d;
	    y2d = ny - y2d;
	    fprintf(fp, "%d %d moveto %d %d lineto stroke\n", x1d,y1d,x2d,y2d);
	    fprintf(fp, "%lg setlinewidth\n", XgSCALE_LINE_WIDTH);
	  }  /** end of Major Div Lines **/

	if ((graph->y_rt_axis_layout.div_marks == XgDivMinorTic)
	    || (graph->y_rt_axis_layout.div_marks == XgDivLin_Tic))
	  {
	    if (graph->y_rt_axis_layout.scaling == XgLogarithm)
	      {
		dx = pow(10., y);
		for (i=2; i<10; ++i)
		  {
		    z = log10(i*dx);
		    Xg_Mapwtod(x1, z, &x1d, &y1d, graph, XgRight);
		    y1d = ny - y1d;
		    fprintf(fp, "%d %d moveto -5 0 rlineto stroke\n",x1d,y1d);
		    if (graph->use_right == FALSE)
		      {
			Xg_Mapwtod(x, z, &x1d, &y1d, graph, XgRight);
			y1d = ny - y1d;
			fprintf(fp, "%d %d moveto 5 0 rlineto stroke\n",
				x1d, y1d);
		      }
		  }
	      }  /** end of XgLogarithm **/
	    else if (graph->y_rt_axis_layout.scaling == XgLinear)
	      {
		if (graph->y_rt_axis_layout.increment == XgLast2)
		  j = 2;
		else if (graph->y_rt_axis_layout.increment == XgLast5)
		  j = 5;
		else if (graph->y_rt_axis_layout.increment == XgLast1)
		  j = 10;
		dx = inc / (double) j;
		for (i=1; i<j; ++i)
		  {
		    z = y + dx*i;
		    if ((z > min) && (z < max))
		      {
			Xg_Mapwtod(x1, z, &x1d, &y1d, graph, XgRight);
			y1d = ny - y1d;
			fprintf(fp, "%d %d moveto -5 0 rlineto stroke\n",
				x1d, y1d);
			if (graph->use_right == FALSE)
			  {
			    Xg_Mapwtod(x, z, &x1d, &y1d, graph, XgRight);
			    y1d = ny - y1d;
			    fprintf(fp, "%d %d moveto 5 0 rlineto stroke\n",
				    x1d, y1d);
			  }
		      }
		  }
	      }  /** end of Linear **/
	  }  /** end of Minor Div Tics **/

	if (graph->y_rt_axis_layout.div_marks == XgDivMinorLin)
	  {
	    fprintf(fp, "[8 8] 0 setdash %lg setlinewidth\n",
		    XgMINOR_DIV_LINE_WIDTH);
	    if (graph->y_rt_axis_layout.scaling == XgLogarithm)
	      {
		dx = pow(10., y);
		for (i=2; i<10; ++i)
		  {
		    z = log10(i*dx);
		    Xg_Mapwtod(x, z, &x1d, &y1d, graph, XgRight);
		    Xg_Mapwtod(x1, z, &x2d, &y2d, graph, XgRight);
		    y1d = ny - y1d;
		    y2d = ny - y2d;
		    fprintf(fp, "%d %d moveto %d %d lineto stroke\n",
			    x1d, y1d, x2d, y2d);
		  }
	      }
	    else if (graph->y_rt_axis_layout.scaling == XgLinear)
	      {
		if (graph->y_rt_axis_layout.increment == XgLast2)
		  j = 2;
		else if (graph->y_rt_axis_layout.increment == XgLast5)
		  j = 5;
		else if (graph->y_rt_axis_layout.increment == XgLast1)
		  j = 10;
		dx = inc / (double) j;
		for (i=1; i<j; ++i)
		  {
		    z = y + dx*i;
		    if ((z > min) && (z < max))
		      {
			Xg_Mapwtod(x, z, &x1d, &y1d, graph, XgRight);
			Xg_Mapwtod(x1, z, &x2d, &y2d, graph, XgRight);
			y1d = ny - y1d;
			y2d = ny - y2d;
			fprintf(fp, "%d %d moveto %d %d lineto stroke\n",
				x1d, y1d, x2d, y2d);
		      }
		  }
	      }  /** end of Linear **/
	    fprintf(fp, "[] 0 setdash %lg setlinewidth\n",
		    XgSCALE_LINE_WIDTH);
	  }  /** end of Minor Div Lines **/
      }  /** end of y loop **/
  }  /** end of right y axis **/

/************* data ****************/
fprintf(fp, "\n%%    Draw Data Sets\n\n");

fprintf(fp, "%lg setlinewidth newpath\n", XgDATA_LINE_WIDTH);
dminx = graph->xaxis_layout.sclmin;
dmaxx = graph->xaxis_layout.sclmax;

for (j=0; j<graph->num_sets; ++j)
  {
    num_points = graph->dataset[j].num_points;
    size = (graph->dataset[j].point_size - XgSize0) / 2.;
    if (graph->dataset[j].point_size == XgSize5)  /** make it BIG **/
      size = (graph->dataset[j].point_size - XgSize0);
    left_right = graph->dataset[j].left_right;
    if (left_right == XgLeft)
      {
	scaling = graph->yaxis_layout.scaling;
	dminy = graph->yaxis_layout.sclmin;
	dmaxy = graph->yaxis_layout.sclmax;
      }
    else if (left_right == XgRight)
      {
	scaling = graph->y_rt_axis_layout.scaling;
	dminy = graph->y_rt_axis_layout.sclmin;
	dmaxy = graph->y_rt_axis_layout.sclmax;
      }
    else
      scaling = -1;

    /**** draw points ****/
    for (i=0; i<num_points; ++i)
      {
	x = *(graph->dataset[j].darray + 2*i);
	y = *(graph->dataset[j].darray + 2*i + 1);
	if (graph->xaxis_layout.scaling == XgLogarithm)
	  {
	    if (x > 0.)
	      x = log10(x);
	    else
	      x = dminx - 1;
	  }
	if (scaling == XgLogarithm)
	  {
	    if (y > 0.)
	      y = log10(y);
	    else
	      y = dminy - 1;
	  }
	if (scaling == XgProbability)
	  {
	    if ((y >= 0.) && (y <= 1.))
	      y = Xg_inverf(y);
	    else
	      y = dminy - 1;
	  }

	if ( ((x >= dminx) && (x <= dmaxx))
	    && ((y >= dminy) && (y <= dmaxy)) )
	  {
	    Xg_Mapwtod(x, y, &x1d, &y1d, graph, left_right);
	    y1d = ny - y1d;
	    switch (graph->dataset[j].point_type)
	      {
	      case XgPoint:
		fprintf(fp, "%d %d %f 0 360 arc fill\n", x1d, y1d, size/10.);
		break;
	      case XgCircle:
		if (graph->dataset[j].filled_open == XgFilled)
		  fprintf(fp, "%d %d %f 0 360 arc fill\n", x1d, y1d, size/2.);
		else if (graph->dataset[j].filled_open == XgOpen)
		  fprintf(fp, "%d %d %f 0 360 arc stroke\n", x1d, y1d,size/2.);
		break;
	      case XgSquare:
		if (graph->dataset[j].filled_open == XgFilled)
		  {
		    fprintf(fp, "%f %f moveto\n", x1d-size/2., y1d-size/2.);
		    fprintf(fp, "%f 0 rlineto 0 %f rlineto\n", size, size);
		    fprintf(fp, "%f 0 rlineto closepath fill\n", -size);
		  }
		else if (graph->dataset[j].filled_open == XgOpen)
		  {
		    fprintf(fp, "%f %f moveto\n", x1d-size/2., y1d-size/2.);
		    fprintf(fp, "%f 0 rlineto 0 %f rlineto\n", size, size);
		    fprintf(fp, "%f 0 rlineto closepath stroke\n", -size);
		  }
		break;
	      case XgTriangle:
		if (graph->dataset[j].filled_open == XgFilled)
		  {
		    fprintf(fp, "%f %f moveto\n", x1d-size/2., y1d-size/2.);
		    fprintf(fp, "%f 0 rlineto %d %f lineto\n",
			    size, x1d, y1d+size/2.);
		    fprintf(fp, "closepath fill\n");
		  }
		else if (graph->dataset[j].filled_open == XgOpen)
		  {
		    fprintf(fp, "%f %f moveto\n", x1d-size/2., y1d-size/2.);
		    fprintf(fp, "%f 0 rlineto %d %f lineto\n",
			    size, x1d, y1d+size/2.);
		    fprintf(fp, "closepath stroke\n");
		  }
		break;
	      case XgPlus:
		fprintf(fp, "%f %f moveto\n", (float) x1d-size/2.,(float) y1d);
		fprintf(fp, "%f 0 rlineto stroke\n", (float) size);
		fprintf(fp, "%f %f moveto\n", (float) x1d,(float) y1d-size/2.);
		fprintf(fp, "0 %f rlineto stroke\n", (float) size);
		break;
	      case XgEx:
		fprintf(fp, "%f %f moveto\n", (float) x1d-size/2.,
			(float) y1d-size/2.);
		fprintf(fp, "%f %f lineto stroke\n", (float) x1d+size/2.,
			(float) y1d+size/2.);
		fprintf(fp, "%f %f moveto\n", (float) x1d-size/2.,
			(float) y1d+size/2.);
		fprintf(fp, "%f %f lineto stroke\n", (float) x1d+size/2.,
			(float) y1d-size/2.);
		break;
	      case XgBar:
		fx = (graph->dataset[j].point_size - XgSize0 - 2) * 4.;
		Xg_Mapwtod(graph->xaxis_layout.sclmin,
			  graph->yaxis_layout.sclmin, &x2d, &y2d,graph,XgLeft);
		y2d = ny - y2d;
		fprintf(fp, "%f %d moveto\n", (float) x1d-fx/2., y1d);
		fprintf(fp, "%f 0 rlineto 0 %d rlineto\n", fx, -(y1d-y2d));
		fprintf(fp, "%f 0 rlineto closepath fill\n", -fx);
		break;
	      default:
		break;
	      }
	  }  /** end of check-bounds **/
      }  /** end of loop over points **/

    /********* draw error bars *********/
    if ( (graph->dataset[j].error_pts == XgHasErrorPts)
	&& (graph->dataset[j].error_bars == XgErrorOn) )
      {
	for (i=0; i<num_points; ++i)
	  {
	    x = *(graph->dataset[j].darray + 2*i);
	    y = *(graph->dataset[j].darray + 2*i + 1);
	    err = *(graph->dataset[j].err_array + i);
	    y1 = y + err;
	    y2 = y - err;
	    if (graph->xaxis_layout.scaling == XgLogarithm)
	      {
		if (x > 0.)
		  x = log10(x);
		else
		  x = dminx - 1;
	      }
	    if (scaling == XgLogarithm)
	      {
		if (y > 0.)
		  y = log10(y);
		else
		  y = dminy - 1;
	      }
	    if (scaling == XgProbability)
	      {
		if ((y >= 0.) && (y <= 1.))
		  y = Xg_inverf(y);
		else
		  y = dminy - 1;
	      }

	    if ( ((x >= dminx) && (x <= dmaxx)) /** basic point is in graph **/
		&& ((y >= dminy) && (y <= dmaxy)) )
	      {
		Xg_Mapwtod(x, y, &x1d, &y1d, graph, left_right);
		y1d = ny - y1d;
		if (scaling == XgLogarithm)
		  {
		    if (y1 > 0.)
		      y1 = log10(y1);
		    else
		      y1 = dminy - 1;
		    if (y2 > 0.)
		      y2 = log10(y2);
		    else
		      y2 = dminy - 1;
		  }
		if (scaling == XgProbability)
		  {
		    if ((y1 >= 0.) && (y1 <= 1.))
		      y1 = Xg_inverf(y1);
		    else
		      y1 = dminy - 1;
		    if ((y2 >= 0.) && (y2 <= 1.))
		      y2 = Xg_inverf(y2);
		    else
		      y2 = dminy - 1;
		  }
		/******** upper extension ********/
		if ((y1 >= dminy) && (y1 <= dmaxy))
		  {
		    Xg_Mapwtod(x, y1, &x2d, &y2d, graph, left_right);
		    y2d = ny - y2d;
		    fprintf(fp, "%f %f moveto\n", (float) x1d, (float) y1d);
		    fprintf(fp, "%f %f lineto stroke\n",
			    (float) x2d, (float) y2d);
		    fprintf(fp, "%f %f moveto\n", (float) x2d-3, (float) y2d);
		    fprintf(fp, "6 0 rlineto stroke\n");
		  }

		/******** lower extension ********/
		if ((y2 >= dminy) && (y2 <= dmaxy))
		  {
		    Xg_Mapwtod(x, y2, &x2d, &y2d, graph, left_right);
		    y2d = ny - y2d;
		    fprintf(fp, "%f %f moveto\n", (float) x1d, (float) y1d);
		    fprintf(fp, "%f %f lineto stroke\n",
			    (float) x2d, (float) y2d);
		    fprintf(fp, "%f %f moveto\n", (float) x2d-3, (float) y2d);
		    fprintf(fp, "6 0 rlineto stroke\n");
		  }

	      }  /** end of basic point is valid **/
	  }  /** end of i-loop over data points **/
      }  /** end of if error_pts && error_bars **/

    /********** draw lines **************/
    if (graph->dataset[j].line_status == XgSolidLine)
      {
	fprintf(fp, "newpath\n");
	for (i=0; i<num_points-1; ++i)
	  {
	    k = i + 1;
	    x = *(graph->dataset[j].darray + 2*i);
	    y = *(graph->dataset[j].darray + 2*i + 1);
	    x1 = *(graph->dataset[j].darray + 2*k);
	    y1 = *(graph->dataset[j].darray + 2*k + 1);
	    if (graph->xaxis_layout.scaling == XgLogarithm)
	      {
		if ((x > 0.) && (x1 > 0.))
		  {
		    x = log10(x);
		    x1 = log10(x1);
		  }
		else
		  x = dminx - 1;
	      }
	    if (scaling == XgLogarithm)
	      {
		if ((y > 0.) && (y1 > 0.))
		  {
		    y = log10(y);
		    y1 = log10(y1);
		  }
		else
		  y = dminy - 1;
	      }
	    if (scaling == XgProbability)
	      {
		if ((y >= 0.) && (y <= 1.) && (y1 >= 0.) && (y1 <= 1.))
		  {
		    y = Xg_inverf(y);
		    y1 = Xg_inverf(y1);
		  }
		else
		  y = dminy - 1;
	      }

	    if ( ((x >= dminx) && (x <= dmaxx) && (y >= dminy) && (y <= dmaxy))
		&& ((x1 >= dminx) && (x1 <= dmaxx)
		    && (y1 >= dminy) && (y1 <= dmaxy)) )
	      {
		Xg_Mapwtod(x1, y1, &x2d, &y2d, graph, left_right);
		y2d = ny - y2d;
		Xg_Mapwtod(x, y, &x1d, &y1d, graph, left_right);
		y1d = ny - y1d;
		fprintf(fp, "%d %d moveto %d %d lineto stroke\n",
			x2d, y2d, x1d, y1d);
	      }
	  }  /** end of loop over points **/
	fprintf(fp, "\n");
      }  /** end of Line Draw **/
  }  /** end of loop over data sets **/

/******* finish up ********/
fprintf(fp, "showpage\n");
fclose(fp);
}
