/**********************************************************************
*
*                              decirant.c
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.6   5/93
*
*  decode ir camera frame annotation data
*
***********************************************************************
*/
#include "stdio.h"
#include "string.h"
#include "math.h"
#include "etsheadr.h"

#ifdef sun
#define NON_ANSI
#endif

/*************** decode frame annotation data ****************/
#ifdef NON_ANSI
void decode_ir(nrows, ncols, pix_array, annot_rtrn)
     int
       nrows,
       ncols;
     int2
       * pix_array;
     IR_ANNOT_1
       * annot_rtrn;
#else
void decode_ir
  (
   int nrows,
   int ncols,
   int2 * pix_array,
   IR_ANNOT_1 * annot_rtrn
   )
#endif
{
  int
    i,
    j,
    index,
    avg_val,
    avg_exp,
    day,
    hour,
    min,
    sec,
    msec;
  char
    irig_buf[10];

/** 319x244 data uses rows 237, 238. 320x244 data uses rows 236, 237 **/
index = 236 * ncols;
i = *(pix_array + index);            /** frame number **/
j = i & 0x0FFF;
index = 237 * ncols;
i = *(pix_array + index);            /** frame number **/
annot_rtrn->frame_seq_num = i & 0x0FFF;
/*if (j != annot_rtrn->frame_seq_num)
  {
    fprintf(stderr, "WARNING: even/odd fields not aligned!\n");
    fprintf(stderr, "   values are %d and %d\n",
	    j, annot_rtrn->frame_seq_num);
  }*/
i = *(pix_array + index + 6);        /** focal plane BCD temp (deg K) **/
sprintf(irig_buf, "%3x", i & 0x0FFF);
sscanf(irig_buf, "%lf", &annot_rtrn->fp_temp);

/*#define DECODE_HERE*/

#ifdef DECODE_HERE
/*** decode using sprintf/sscanf & buffers ***/
i = *(pix_array + index + 1);        /** day of year **/
sprintf(irig_buf, "%3x", i & 0x0FFF);
sscanf(irig_buf, "%3d", &day);
i = *(pix_array + index + 2);        /** hour & intg_time in 30ths sec **/
sprintf(irig_buf, "%2x", i & 0x00FF);
sscanf(irig_buf, "%2d", &hour);
/** use 0x0700 if bit 12 really not valid **/
j = ((i & 0x0F00) >> 8) << 1;
annot_rtrn->intg_time = pow(2., (double) j) / 30.;
i = *(pix_array + index + 3);        /** min & gain setting **/
sprintf(irig_buf, "%2x", i & 0x00FF);
sscanf(irig_buf, "%2d", &min);
/** use 0x0300 if bit 11 & 12 really not valid **/
j = (i & 0x0F00) >> 8;
annot_rtrn->gain = j;
i = *(pix_array + index + 4);        /** sec & exponent for frame avg **/
sprintf(irig_buf, "%2x", i & 0x00FF);
sscanf(irig_buf, "%2d", &sec);
avg_exp = (i & 0x0F00) >> 8;
i = *(pix_array + index + 5);        /** millisec **/
sprintf(irig_buf, "%3x", i & 0x0FFF);
sscanf(irig_buf, "%3d", &msec);
i = *(pix_array + index + 7);        /** frame avg value (in elects) **/
avg_val = i & 0x0FFF;

annot_rtrn->bias = avg_val * pow(10., (double) avg_exp-2);
annot_rtrn->mjd = day + 48987;
annot_rtrn->msec_time = msec + ((hour*60 + min)*60 + sec)*1000;
#else

/*** decode by direct bit manipulation ***/
i = *(pix_array + index + 5);    /** msec **/
irig_buf[0] = i & 0x00FF;
irig_buf[1] = (i & 0X0F00) >> 8;
i = *(pix_array + index + 4);    /** sec & exponent for frame avg **/
irig_buf[2] = i & 0x00FF;
avg_exp = (i & 0x0F00) >> 8;
i = *(pix_array + index + 3);    /** min & gain setting **/
irig_buf[3] = i & 0x00FF;
/** use 0x0300 if bit 11 & 12 really not valid **/
j = (i & 0x0F00) >> 8;
annot_rtrn->gain = j;
i = *(pix_array + index + 2);    /** hour & intg_time in 30ths sec **/
irig_buf[4] = i & 0x00FF;
/** use 0x0700 if bit 12 really not valid **/
j = ((i & 0x0F00) >> 8) << 1;
annot_rtrn->intg_time = pow(2., (double) j) / 30.;
i = *(pix_array + index + 1);    /** day **/
irig_buf[5] = i & 0x00FF;
irig_buf[6] = (i & 0x0F00) >> 8;
decode_IRIG_time(irig_buf, &annot_rtrn->mjd, &annot_rtrn->msec_time);
i = *(pix_array + index + 7);        /** frame avg value (in elects) **/
avg_val = i & 0x0FFF;
annot_rtrn->bias = avg_val * pow(10., (double) avg_exp-2);
#endif

/********** display results **********/
#ifdef MIN_DIAG
printf("  Frame=%d; mjd=%d; time=%d\n", annot_rtrn->frame_seq_num,
       annot_rtrn->mjd, annot_rtrn->msec_time);

printf("  intg_time=%g sec; gain=%g; temp=%g K; avg_val=%g elect\n",
       annot_rtrn->intg_time, annot_rtrn->gain,
       annot_rtrn->fp_temp, annot_rtrn->bias);
#endif

}

/***********************************/
/** for 1983, the mjd offset is given below **/
#define MJD_93     48987

void decode_IRIG_time
  (
   char * irig_time,
   int * mjd,
   int * msec_time
   )
{
  int
    mjd_offset,
    day,
    hour,
    min,
    sec,
    msec,
    usec;
  char
    *i_c;  /** irig character (bit) array **/

/*******************************************************************
*
*                   IRIG bit ordering (BCD)
*

d200 d100 | d80 d40 d20 d10 | d8 d4 d2 d1 | NU NU h20 h10 | h8 h4 h2 h1 |
 49   48  | 47  46  45  44    43 42 41 40 | 39 38 37  36    35 34 33 32 |
  1    0  |  7   6   5   4     3  2  1  0 |  7  6  5   4     3  2  1  0 |
 Byte 6   |            Byte 5             |            Byte 4

NU m40 m20 m10 | m8 m4 m2 m1 | NU s40 s20 s10 | s8 s4 s2 s1 |
31 30  29  28    27 26 25 24 | 23 22  21  20    19 18 17 16 |
 7  6   5   4     3  2  1  0 |  7  6   5   4     3  2  1  0 |
          Byte 3             |            Byte 2

NU NU NU NU ms800 ms400 ms200 ms100 | ms80 ms40 ms20 ms10 | ms8 ms4 ms2 ms1 |
15 14 13 12  11    10     9     8   |  7    6    5    4      3   2   1   0  |
 7  6  5  4   3     2     1     0   |  7    6    5    4      3   2   1   0  |
                Byte 1              |                Byte 0                 |

*******************************************************************
*/

mjd_offset = MJD_93;
i_c = (char *) irig_time;

day = (i_c[6] & 0x03)*100 + ((i_c[5] & 0xF0) >> 4)*10 + (i_c[5] & 0x0F);
hour = ((i_c[4] & 0x30) >> 4)*10 + (i_c[4] & 0x0F);
min = ((i_c[3] & 0x70) >> 4)*10 + (i_c[3] & 0x0F);
sec = ((i_c[2] & 0x70) >> 4)*10 + (i_c[2] & 0x0F);
msec = (i_c[1] & 0x0F)*100 + ((i_c[0] & 0xF0) >> 4)*10 + (i_c[0] & 0x0F);

printf("day=%d; hour=%d; min=%d; sec=%d; msec=%d\r",
       day, hour, min, sec, msec);
fflush(stdout);
min += hour * 60;
sec += min * 60;
msec += sec * 1000;

*mjd = day + mjd_offset;
*msec_time = msec;
}
