/**********************************************************************
*
*                              filefunc.c
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.6   5/92
*
*  Program for performing I/O operations on standard ETS data files.
*
***********************************************************************
*/
#include "stdio.h"
#include "string.h"
#include "etsheadr.h"

#ifdef sun
#define NON_ANSI
#endif

/* define IS_IEEE_MACHINE for 680X0 and all RISC machines (Sun, SGI, etc.),
   not for DEC or Intel */
#ifndef _I386
#ifndef _DEC
#ifndef __alpha
#define IS_IEEE_MACHINE
#endif
#endif
#endif

/***************** open data file *********************/
#ifdef NON_ANSI
void fileopen(datafile)
    ETS_STRUCT
      *datafile;
#else
void fileopen
  (
   ETS_STRUCT * datafile
  )
#endif
{
    char
      *header_char,
      sbyte,
      inp_frmt[10],
      tbuf[300];
/*************** check basics for proper definitions, etc. ************/
#ifdef IS_IEEE_MACHINE
    datafile->this_machine = IEEE_MACHINE;
#else
    datafile->this_machine = NON_IEEE_MACHINE;
#endif
if (strlen(datafile->filename) == 0)  /* no predefined name */
  printf("file header is %d bytes long, according to etsheadr.h\n",
	 sizeof(datafile->file_header));
if (HEADER_SIZE != sizeof(datafile->file_header))
  {
    fprintf(stderr, "**** HEADER_SIZE = %d but sizeof(file_header) = %d\n",
      HEADER_SIZE, sizeof(datafile->file_header));
  }
if (sizeof(int2) != 2)
  {
    fprintf(stderr, "**** int2 size defined wrong in etsheadr.h! Aborting\n");
    exit(1);
  }
if (sizeof(int4) != 4)
  {
    fprintf(stderr, "**** int4 size defined wrong in etsheadr.h! Aborting\n");
    exit(1);
  }
if (strlen(datafile->filename) == 0)  /* no predefined name */
  {
    if (datafile->this_machine == IEEE_MACHINE)
      printf("This is an IEEE format machine, i.e. RISC, 680x0, etc.\n");
    else
      printf("This is a non-IEEE format machine such as DEC or 80x86\n");
  }
header_char = (char *) &datafile->file_header;
sprintf(inp_frmt, "%%%dc", HEADER_SIZE);  /* should be "%4096c" */
/*************************** open file *************/
do
  {
    if (strlen(datafile->filename) == 0)  /* no predefined name */
      {
	printf("enter file name: ");
	scanf("%[^\n]", datafile->filename);
	getchar();
	printf("data file: %s\n\n", datafile->filename);
      }
    datafile->dat_file = fopen(datafile->filename, "r");
    if (datafile->dat_file == NULL)
      {
	fprintf(stderr, "Smooth move, wing nut! Now you've really goobered it up!\n");
	fprintf(stderr, "Unable to open file %s, try again\n",
		datafile->filename);
	datafile->filename[0] = '\0';  /* reset for loop back & try again */
/*	fprintf(stderr, "You leave me no choice but to BARF OUT. Bye bye\n\n");
	exit(1);*/
      }
  }
while (datafile->dat_file == NULL);
fscanf(datafile->dat_file, inp_frmt, header_char);
datafile->current_frame = 0;
/************ determine byte ordering *****************/
if (datafile->file_header.big_little == REVERSE_ORDER)
  datafile->data_order = REVERSE_ORDER;
else
  datafile->data_order = PROPER_ORDER;
if (datafile->file_header.big_little == DEF_VAX)
  if (datafile->this_machine == IEEE_MACHINE)
    datafile->data_order = REVERSE_ORDER;
  else
    datafile->data_order = PROPER_ORDER;
/************* swap bytes if necessary ********/
if (datafile->data_order == REVERSE_ORDER)
   /* swap bytes if IEEE machine reading non-IEEE data or vice-versa */
  {
    printf("swapping header bytes\n");
    swab(&header_char[0], tbuf, 6);  /* version & bits_per_pixel */
    memcpy(&header_char[0], tbuf, 6);
    sbyte = header_char[26];          /* chipnum */
    header_char[26] = header_char[27];
    header_char[27] = sbyte;
    sbyte = header_char[28];          /* ext_hdr_len swap (4 bytes) */
    header_char[28] = header_char[31];
    header_char[31] = sbyte;
    sbyte = header_char[29];
    header_char[29] = header_char[30];
    header_char[30] = sbyte;
    swab(&header_char[106], tbuf, 16);  /* row_start through frame_type */
    memcpy(&header_char[106], tbuf, 16);
  }
/*********************************/
if (datafile->file_header.frames_good == 0)  /** for bozoids **/
  {
    datafile->file_header.frames_good = datafile->file_header.frames_requested;
    printf(
    "WARNING! frames_good was zero, so set frames_good = frames_requested\n");
  }
}

/******************* read a frame of data ********************/
#ifdef NON_ANSI
int2 * readframe(datafile, frame_num)
     ETS_STRUCT
       *datafile;
     int
       frame_num; /* if 0, then next frame, else get given frame # */
#else
int2 * readframe
  (
   ETS_STRUCT * datafile,
   int frame_num   /* if 0, then next frame, else get given frame_num */
  )
#endif
{
    int2
      ncols,
      nrows,
      nframes,
      *pix_array;
    int
      i,
      j,
      k,
      maxbytes;
    char
      inp_frmt[10],
      hdr_frmt[10],
      header_rec[HEADER_SIZE],
      *input_data;
ncols = datafile->file_header.row_length;  /* Note that # pix/row = # cols */
nrows = datafile->file_header.col_length;
nframes = datafile->file_header.frames_good;
maxbytes = sizeof(int2) * ncols;
input_data = (char *) calloc(maxbytes, sizeof(char));
if (input_data == NULL)
  {
    fprintf(stderr, "calloc failure for input_data\n");
    exit(1);
  }
sprintf(inp_frmt, "%%%dc", maxbytes); /* should be, e.g. "%840c" */
pix_array = (int2 *) calloc(nrows*ncols, sizeof(int2));
if (pix_array == NULL)
  {
   fprintf(stderr, "Failure to malloc pix_array in readframe in filefunc.c\n");
   fprintf(stderr, "aborting ...\n");
   exit(1);
  }
/******************* read in next frame *********************/
if (frame_num == 0)
  {
  if (datafile->current_frame >= nframes)
    {
      return(NULL);
    }
  else
    {
      ++datafile->current_frame;
      for (i=0; i<nrows; ++i)
	{
	  fscanf(datafile->dat_file, inp_frmt, input_data);  /* read a line */
	  if (datafile->data_order == REVERSE_ORDER)
	    swab(input_data, (pix_array + i*ncols), maxbytes); /* swap bytes */
	  else
	    memcpy((pix_array + i*ncols), input_data, maxbytes);/*no swapping*/
	} /* end i-loop for nrows */
    } /* end, still in good frame region */
  } /* (end if frame_num == 0) */
else /* move to  frame_num */
  {
    fseek(datafile->dat_file,
	  (frame_num-1)*nrows*ncols*sizeof(int2) + HEADER_SIZE, 0);
    datafile->current_frame = frame_num;  /* base 1 */
    /**** okay, now read in the frame ****/
    for (i=0; i<nrows; ++i)
      {
	  fscanf(datafile->dat_file, inp_frmt, input_data);  /* read a line */
	  if (datafile->data_order == REVERSE_ORDER)
	    swab(input_data, (pix_array + i*ncols), maxbytes); /* swap bytes */
	  else
	    memcpy((pix_array + i*ncols), input_data, maxbytes);/*no swapping*/
      } /* end frame */
  } /* end single frame processing */

free(input_data);
return(pix_array);
}

/****************** create a file ******************/
#ifdef NON_ANSI
void filecreate(datafile)
     ETS_STRUCT
       *datafile;
#else
void filecreate
  (
   ETS_STRUCT * datafile
  )
#endif
{
  int
    i;
  char
    sbyte,
    tbuf[300],
    *header_char;
header_char = (char *) &datafile->file_header;
datafile->file_header.major_version = MAJOR_VER;
datafile->file_header.minor_version = MINOR_VER;
datafile->file_header.big_little = PROPER_ORDER; /* will be swapped if needed*/
/*strcpy(datafile->file_header.operator, "filecreate() on PS/2");*/
/************* swap bytes if necessary ********/
if (datafile->data_order == REVERSE_ORDER)
  {
    printf("swapping header bytes\n");
    swab(&header_char[0], tbuf, 6);  /* version & bits_per_pixel */
    memcpy(&header_char[0], tbuf, 6);
    sbyte = header_char[26];          /* chipnum */
    header_char[26] = header_char[27];
    header_char[27] = sbyte;
    sbyte = header_char[28];          /* ext_hdr_len swap (4 bytes) */
    header_char[28] = header_char[31];
    header_char[31] = sbyte;
    sbyte = header_char[29];
    header_char[29] = header_char[30];
    header_char[30] = sbyte;
    swab(&header_char[106], tbuf, 16);  /* row_start through frame_type */
    memcpy(&header_char[106], tbuf, 16);
  }
/*********************************/
datafile->dat_file = fopen(datafile->filename, "w");
if (datafile->dat_file == NULL)
  {
    fprintf(stderr, "failed to open output file: %s. Aborted!\n",
	    datafile->filename);
    exit(1);
  }
for (i=0; i<HEADER_SIZE; ++i)
/*  fprintf(datafile->dat_file, "%c", *(header_char + i));*/
  putc(*(header_char + i), datafile->dat_file);
/************* unswap bytes if done earlier ********/
if (datafile->data_order == REVERSE_ORDER)
  {
    printf("unswapping header bytes\n");
    swab(&header_char[0], tbuf, 6);  /* version & bits_per_pixel */
    memcpy(&header_char[0], tbuf, 6);
    sbyte = header_char[26];          /* chipnum */
    header_char[26] = header_char[27];
    header_char[27] = sbyte;
    sbyte = header_char[28];          /* ext_hdr_len swap (4 bytes) */
    header_char[28] = header_char[31];
    header_char[31] = sbyte;
    sbyte = header_char[29];
    header_char[29] = header_char[30];
    header_char[30] = sbyte;
    swab(&header_char[106], tbuf, 16);  /* row_start through frame_type */
    memcpy(&header_char[106], tbuf, 16);
  }
}

/****************** write a frame of data ***************/
#ifdef NON_ANSI
void writeframe(datafile, pix_array)
     ETS_STRUCT
       *datafile;
     int2
       *pix_array;
#else
void writeframe
  (
   ETS_STRUCT * datafile,      /* header structure to save image with */
   int2 * pix_array            /* 2-D array for image pixel data */
  )
#endif
{
    int
      ncols,
      nrows;
    int
      i,
      j,
      maxbytes;
    char
      out_frmt[10],
      *output_data;
ncols = datafile->file_header.row_length; /* Note that # pix/row = # cols */
nrows = datafile->file_header.col_length;
maxbytes = sizeof(int2) * ncols;
output_data = (char *) calloc(maxbytes, sizeof(char));
if (output_data == NULL)
  {
    fprintf(stderr, "calloc failure for output_data\n");
    exit(1);
  }
sprintf(out_frmt, "%%%dc", maxbytes); /* should be, e.g. "%840c" */
for (i=0; i<nrows; ++i)
  {
    if (datafile->data_order == REVERSE_ORDER)
      swab((pix_array + i*ncols), output_data, maxbytes);  /* swap bytes */
    else
      memcpy(output_data, (pix_array + i*ncols), maxbytes); /* no swapping */
    for (j=0; j<maxbytes; ++j)
      putc(*(output_data + j), datafile->dat_file);
/*    fprintf(datafile->dat_file, out_frmt, output_data);  /* write a line */
  }
free(output_data);
}

/****************** close file *********************/
#ifdef NON_ANSI
void fileclose(dat_file)
     FILE
       *dat_file;
#else
void fileclose
  (
   FILE * dat_file
  )
#endif
{
fclose(dat_file);
}

/************* display header stuff **********/
#ifdef NON_ANSI
void showhead(datafile)
    ETS_STRUCT
      *datafile;
#else
void showhead
  (
   ETS_STRUCT * datafile
  )
#endif
{
  int
    i;
  char
    tbuf[300];
if (datafile->this_machine == IEEE_MACHINE)
  printf("This is an IEEE format machine\n");
else if (datafile->this_machine == NON_IEEE_MACHINE)
  printf("This is a NON_IEEE format machine\n");
else /** UNKNOWN_MACHINE **/
  printf("This is an unknown format machine\n");
if (datafile->data_order == PROPER_ORDER)
  printf("    and the data was generated on the same format machine\n");
else
  printf("    and the data was generated on a reverse format machine\n");
printf("Header version =        %d.%d\n", datafile->file_header.major_version,
       datafile->file_header.minor_version);
printf("number of bits/pixel =  %d\n", datafile->file_header.bits_per_pixel);
strncpy(tbuf, datafile->file_header.operator, 20);
tbuf[20] = '\0';
printf("operator =              >%s<\n", tbuf);
printf("chipnum =               %d\n", datafile->file_header.chipnum);
printf("ext_hdr_len =           %d\n", datafile->file_header.ext_hdr_len);
strncpy(tbuf, datafile->file_header.start_time, 23);
tbuf[23] = '\0';
printf("start_time is           >%s<\n", tbuf);
strncpy(tbuf, datafile->file_header.end_time, 23);
tbuf[23] = '\0';
printf("end_time is             >%s<\n", tbuf);
printf("row_start =             %d\n", datafile->file_header.row_start);
printf("row_length =            %d\n", datafile->file_header.row_length);
printf("col_start =             %d\n", datafile->file_header.col_start);
printf("col_length =            %d\n", datafile->file_header.col_length);
printf("frames_requested =      %d\n", datafile->file_header.frames_requested);
printf("frames_good =           %d\n", datafile->file_header.frames_good);
printf("big_little =            %d\n", datafile->file_header.big_little);
printf("frame_type =            %d\n", datafile->file_header.frame_type);
printf("Data Description: >%s<\n", datafile->file_header.description);
printf("Processing History: >%s<\n", datafile->file_header.proc_hist);
/****************** description ***************/
/*printf("\ndescription:\n");
for (i=0; i<390; ++i)
  {
    printf("%c", datafile->file_header.description[i]);
  }
printf("\n");*/
#ifdef JUNK
/****************** camera status area ***************/
printf("\ncamera status area data:\n");
for (i=0; i<512; ++i)
  {
    printf("%c", datafile->file_header.camera_status[i]);
  }
printf("\n");
/****************** utility status area ***************/
printf("\n utility status area data:\n");
for (i=0; i<512; ++i)
  {
    printf("%c", datafile->file_header.utility_status[i]);
  }
printf("\n");
/****************** telescope mount status area ***************/
printf("\ntelescope status area data:\n");
for (i=0; i<512; ++i)
  {
    printf("%c", datafile->file_header.los_status[i]);
  }
printf("\n");
#endif
/*********************************/
printf("<<<<<<<<<<<<<< end of header for %s >>>>>>>>>>>>>>>>>>\n",
       datafile->filename);
}

/******************* read an fft frame of data ********************/
#ifdef NON_ANSI
FFTDAT * read_fft_frame(datafile, frame_num)
     ETS_STRUCT
       *datafile;
     int
       frame_num; /* if 0, then next frame, else get given frame # */
#else
FFTDAT * read_fft_frame
  (
   ETS_STRUCT * datafile,
   int frame_num          /* if 0, then next frame, else get given frame # */
  )
#endif
{
    int2
      ncols,
      nrows,
      nframes;
    int
      i,
      j,
      k,
      maxbytes;
    FFTDAT
      *pix_array;
    char
      inp_frmt[10],
      hdr_frmt[10],
      header_rec[HEADER_SIZE],
      *input_data;
ncols = datafile->file_header.row_length;  /* Note that # pix/row = # cols */
nrows = datafile->file_header.col_length;
if (datafile->file_header.frame_type == FOURIER_AMP)
    ncols *= 2;
nframes = datafile->file_header.frames_good;
maxbytes = sizeof(FFTDAT) * ncols;
input_data = (char *) calloc(maxbytes, sizeof(char));
if (input_data == NULL)
  {
    fprintf(stderr, "calloc failure for input_data\n");
    exit(1);
  }
sprintf(inp_frmt, "%%%dc", maxbytes); /* should be, e.g. "%840c" */
pix_array = (FFTDAT *) calloc(nrows*ncols, sizeof(FFTDAT));
if (pix_array == NULL)
  {
   fprintf(stderr,
	   "Failure to malloc pix_array in read_fft_frame in filefunc.c\n");
   fprintf(stderr, "aborting ...\n");
   exit(1);
  }
/******************* read in next frame *********************/
if (frame_num == 0)
  {
  if (datafile->current_frame >= nframes)
    {
      return(NULL);
    }
  else
    {
      ++datafile->current_frame;
      for (i=0; i<nrows; ++i)
	{
	  fscanf(datafile->dat_file, inp_frmt, input_data);  /* read a line */
	  if (datafile->data_order == REVERSE_ORDER)
	    swab(input_data, (pix_array + i*ncols), maxbytes); /* swap bytes */
	  else
	    memcpy((pix_array + i*ncols), input_data, maxbytes);/*no swapping*/
	} /* end i-loop for nrows */
    } /* end, still in good frame region */
  } /* (end if frame_num == 0) */
else /* find frame_num */  /* could make this smarter, to skip ahead in ok */
  {
    fseek(datafile->dat_file,
	  (frame_num-1)*nrows*ncols*sizeof(FFTDAT)*2 + HEADER_SIZE, 0);
    datafile->current_frame = frame_num;  /* base 1 */
    /**** okay, now read in the frame ****/
    for (i=0; i<nrows; ++i)
      {
	  fscanf(datafile->dat_file, inp_frmt, input_data);  /* read a line */
	  if (datafile->data_order == REVERSE_ORDER)
	    swab(input_data, (pix_array + i*ncols), maxbytes); /* swap bytes */
	  else
	    memcpy((pix_array + i*ncols), input_data, maxbytes);/*no swapping*/
      } /* end frame */
  } /* end single frame processing */

free(input_data);
return(pix_array);
}

/****************** write an fft frame of data ***************/
#ifdef NON_ANSI
void write_fft_frame(datafile, pix_array)
     ETS_STRUCT
       *datafile;
     FFTDAT
       *pix_array;
#else
void write_fft_frame
  (
   ETS_STRUCT * datafile,        /* header structure to save image with */
   FFTDAT * pix_array            /* 2-D array for image pixel data */
  )
#endif
{
    int2
      ncols,
      nrows;
    int
      i,
      j,
      maxbytes;
    char
      out_frmt[10],
      *output_data;
ncols = datafile->file_header.row_length;  /* Note that # pix/row = # cols */
nrows = datafile->file_header.col_length;
if (datafile->file_header.frame_type == FOURIER_AMP) /* real & img parts */
    ncols *= 2;
maxbytes = sizeof(FFTDAT) * ncols;
output_data = (char *) calloc(maxbytes, sizeof(char));
if (output_data == NULL)
  {
    fprintf(stderr, "calloc failure for output_data\n");
    exit(1);
  }
sprintf(out_frmt, "%%%dc", maxbytes); /* should be, e.g. "%840c" */
for (i=0; i<nrows; ++i)
  {
    if (datafile->data_order == REVERSE_ORDER)
      swab((pix_array + i*ncols), output_data, maxbytes);  /* swap bytes */
    else
      memcpy(output_data, (pix_array + i*ncols), maxbytes); /* no swapping */
    for (j=0; j<maxbytes; ++j)
      putc(*(output_data + j), datafile->dat_file);
/*    fprintf(datafile->dat_file, out_frmt, output_data);  /* write a line */
  }
free(output_data);
}

/******************* read frame of 1 byte data ********************/
#ifdef NON_ANSI
int1 * read_int1_frame(datafile, frame_num)
     ETS_STRUCT
       *datafile;
     int
       frame_num; /* if 0, then next frame, else get given frame # */
#else
int1 * read_int1_frame
  (
   ETS_STRUCT * datafile,
   int frame_num   /* if 0, then next frame, else get given frame_num */
  )
#endif
{
    int2
      ncols,
      nrows,
      nframes;
    int1
      *pix_array;
    int
      i,
      j,
      k,
      maxbytes;
    char
      inp_frmt[10],
      hdr_frmt[10],
      header_rec[HEADER_SIZE],
      *input_data;
ncols = datafile->file_header.row_length;  /* Note that # pix/row = # cols */
nrows = datafile->file_header.col_length;
nframes = datafile->file_header.frames_good;
maxbytes = sizeof(int1) * ncols;
input_data = (char *) calloc(maxbytes, sizeof(char));
if (input_data == NULL)
  {
    fprintf(stderr, "calloc failure for input_data\n");
    exit(1);
  }
sprintf(inp_frmt, "%%%dc", maxbytes); /* should be, e.g. "%840c" */
pix_array = (int1 *) calloc(nrows*ncols, sizeof(int1));
if (pix_array == NULL)
  {
   fprintf(stderr,
	   "Failure to malloc pix_array in read_int1_frame in filefunc.c\n");
   fprintf(stderr, "aborting ...\n");
   exit(1);
  }
/******************* read in next frame *********************/
if (frame_num == 0)
  {
  if (datafile->current_frame >= nframes)
    {
      return(NULL);
    }
  else
    {
      ++datafile->current_frame;
      for (i=0; i<nrows; ++i)
	{
	  fscanf(datafile->dat_file, inp_frmt, input_data);  /* read a line */
	  memcpy((pix_array + i*ncols), input_data, maxbytes);
	} /* end i-loop for nrows */
    } /* end, still in good frame region */
  } /* (end if frame_num == 0) */
else /* find frame_num */  /* could make this smarter, to skip ahead in ok */
  {
    fseek(datafile->dat_file,
	  (frame_num-1)*nrows*ncols*sizeof(int1) + HEADER_SIZE, 0);
    datafile->current_frame = frame_num;  /* base 1 */
    /**** okay, now read in the frame ****/
    for (i=0; i<nrows; ++i)
      {
	  fscanf(datafile->dat_file, inp_frmt, input_data);  /* read a line */
	  memcpy((pix_array + i*ncols), input_data, maxbytes);/*no swapping*/
      } /* end frame */
  } /* end single frame processing */

free(input_data);
return(pix_array);
}

/****************** write a frame of 1-byte data ***************/
#ifdef NON_ANSI
void write_int1_frame(datafile, pix_array)
     ETS_STRUCT
       *datafile;
     int1
       *pix_array;
#else
void write_int1_frame
  (
   ETS_STRUCT * datafile,        /* header structure to save image with */
   int1 * pix_array              /* 2-D array for image pixel data */
  )
#endif
{
    int2
      ncols,
      nrows;
    int
      i,
      j,
      maxbytes;
    char
      out_frmt[10],
      *output_data;
ncols = datafile->file_header.row_length;  /* Note that # pix/row = # cols */
nrows = datafile->file_header.col_length;
maxbytes = sizeof(int1) * ncols;
output_data = (char *) calloc(maxbytes, sizeof(char));
if (output_data == NULL)
  {
    fprintf(stderr, "calloc failure for output_data\n");
    exit(1);
  }
sprintf(out_frmt, "%%%dc", maxbytes); /* should be, e.g. "%840c" */
for (i=0; i<nrows; ++i)
  {
    memcpy(output_data, (pix_array + i*ncols), maxbytes);
    for (j=0; j<maxbytes; ++j)
      putc(*(output_data + j), datafile->dat_file);
/*    fprintf(datafile->dat_file, out_frmt, output_data);  /* write a line */
  }
free(output_data);
}

/******************* read frame of float data ********************/
#ifdef NON_ANSI
float * read_float_frame(datafile, frame_num)
     ETS_STRUCT
       *datafile;
     int
       frame_num; /* if 0, then next frame, else get given frame # */
#else
float * read_float_frame
  (
   ETS_STRUCT * datafile,
   int frame_num   /* if 0, then next frame, else get given frame_num */
  )
#endif
{
    int2
      ncols,
      nrows,
      nframes;
    float
      *pix_array;
    int
      i,
      j,
      k,
      ndx,
      maxbytes;
    char
      tmp_char,
      inp_frmt[10],
      hdr_frmt[10],
      header_rec[HEADER_SIZE],
      *input_data;
ncols = datafile->file_header.row_length;  /* Note that # pix/row = # cols */
nrows = datafile->file_header.col_length;
nframes = datafile->file_header.frames_good;
maxbytes = sizeof(float) * ncols;
input_data = (char *) calloc(maxbytes, sizeof(char));
if (input_data == NULL)
  {
    fprintf(stderr, "calloc failure for input_data\n");
    exit(1);
  }
sprintf(inp_frmt, "%%%dc", maxbytes); /* should be, e.g. "%840c" */
pix_array = (float *) calloc(nrows*ncols, sizeof(float));
if (pix_array == NULL)
  {
   fprintf(stderr,
	   "Failure to malloc pix_array in read_float_frame in filefunc.c\n");
   fprintf(stderr, "aborting ...\n");
   exit(1);
  }
/******************* read in next frame *********************/
if (frame_num == 0)
  {
  if (datafile->current_frame >= nframes)
    {
      return(NULL);
    }
  else
    {
      ++datafile->current_frame;
      for (i=0; i<nrows; ++i)
	{
	  fscanf(datafile->dat_file, inp_frmt, input_data);  /* read a line */
	  if (datafile->data_order == REVERSE_ORDER)
	    {  /** dcba ==> abcd **/
	      for (j=0; j<ncols; ++j)
		{
		  ndx = j * 4;
		  tmp_char = *(input_data + ndx);  /** swap d and a **/
		  *(input_data + ndx) = *(input_data + ndx + 3);
		  *(input_data + ndx + 3) = tmp_char;
		  tmp_char = *(input_data + ndx + 1);  /** swap c and b **/
		  *(input_data + ndx + 1) = *(input_data + ndx + 2);
		  *(input_data + ndx + 2) = tmp_char;
		}
	    }
	  memcpy((pix_array + i*ncols), input_data, maxbytes);/*no swapping*/
	} /* end i-loop for nrows */
    } /* end, still in good frame region */
  } /* (end if frame_num == 0) */
else /* find frame_num */  /* could make this smarter, to skip ahead in ok */
  {
    fseek(datafile->dat_file,
	  (frame_num-1)*nrows*ncols*sizeof(float) + HEADER_SIZE, 0);
    datafile->current_frame = frame_num;  /* base 1 */
    /**** okay, now read in the frame ****/
    for (i=0; i<nrows; ++i)
      {
	  fscanf(datafile->dat_file, inp_frmt, input_data);  /* read a line */
	  if (datafile->data_order == REVERSE_ORDER)
	    {  /** dcba ==> abcd **/
	      for (j=0; j<ncols; ++j)
		{
		  ndx = j * 4;
		  tmp_char = *(input_data + ndx);  /** swap d and a **/
		  *(input_data + ndx) = *(input_data + ndx + 3);
		  *(input_data + ndx + 3) = tmp_char;
		  tmp_char = *(input_data + ndx + 1);  /** swap c and b **/
		  *(input_data + ndx + 1) = *(input_data + ndx + 2);
		  *(input_data + ndx + 2) = tmp_char;
		}
	    }
	  memcpy((pix_array + i*ncols), input_data, maxbytes);/*no swapping*/
#ifdef JUNK
	  if (datafile->data_order == REVERSE_ORDER)
	    swab(input_data, (pix_array + i*ncols), maxbytes); /* swap bytes */
	  else
	    memcpy((pix_array + i*ncols), input_data, maxbytes);/*no swapping*/
#endif
      } /* end frame */
  } /* end single frame processing */

free(input_data);
return(pix_array);
}

/****************** write a frame of float data ***************/
#ifdef NON_ANSI
void write_float_frame(datafile, pix_array)
     ETS_STRUCT
       *datafile;
     float
       *pix_array;
#else
void write_float_frame
  (
   ETS_STRUCT * datafile,        /* header structure to save image with */
   float * pix_array             /* 2-D array for image pixel data */
  )
#endif
{
    int2
      ncols,
      nrows;
    int
      i,
      j,
      maxbytes;
    char
      out_frmt[10],
      *output_data;
ncols = datafile->file_header.row_length;  /* Note that # pix/row = # cols */
nrows = datafile->file_header.col_length;
maxbytes = sizeof(float) * ncols;
output_data = (char *) calloc(maxbytes, sizeof(char));
if (output_data == NULL)
  {
    fprintf(stderr, "calloc failure for output_data\n");
    exit(1);
  }
sprintf(out_frmt, "%%%dc", maxbytes); /* should be, e.g. "%840c" */
for (i=0; i<nrows; ++i)
  {
    memcpy(output_data, (pix_array + i*ncols), maxbytes);
    for (j=0; j<maxbytes; ++j)
      putc(*(output_data + j), datafile->dat_file);
/*    fprintf(datafile->dat_file, out_frmt, output_data);  /* write a line */
  }
free(output_data);
}
