/**********************************************************************
*
*                              plt3d.c
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.2   5/92
*
*  Program for reading standard ETS data files and displaying header
*    info. Also shows pixel values if desired. Other file formats can be
*    supported by changes to filefunc.c, etsheadr.h and the add_file function
*    in Xg3dcon.c
*
***********************************************************************
*/
#include "stdio.h"
#include "string.h"
#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xg3d.h"
#include "etsheadr.h"

main(argc, argv)
     int
       argc;
     char
       **argv;
{
  struct fileinfo
    datafile;
  char
    buf[80],
    *header_char;
  int2
    *pix_array;
  int
    i,
    j,
    k,
    desc_flag,
    index,
    frame_num;
  FILE
    *fp;
  double
    scale_fac;
  Widget
    graph;
  Display
    *display;
  XtAppContext
    theAppContext;
  Xg3dGraphInfo
    graphinfo;
/***********************************/
Xg3dInitializeStruct(&graphinfo);
header_char = (char *) &datafile.file_header;
datafile.filename[0] = '\0';  /* make sure we don't open garbage */
desc_flag = 0;
for (i=1; i<argc; ++i) /* find file name if given */
  {
    /** if this is - option, skip it & possibly next arg **/
    if (!strcmp(argv[i], "-h")) ;
    else if (!strncmp(argv[i], "-", 1))
      ++i;
    else /** assume this is file name **/
      strcpy(datafile.filename, argv[i]);
  }
frame_num = 1;
scale_fac = 0;
for (i=1; i<argc; ++i)
  {
    if (!strcmp(argv[i], "-g"))   /** description file **/
      {
	sscanf(argv[i+1], "%s", buf);
	fp = fopen(buf, "r");
	if (fp == NULL)
	  {
	    fprintf(stderr, "Can't open description file. Aborting!\n");
	    exit(0);
	  }
	desc_flag = 1;
	graphinfo.num_sets = 1;  /* minimal defaults */
	Xg3dReadGrf(fp, &graphinfo);
	fclose(fp);
      }
    if (!strcmp(argv[i], "-f"))
      {
	sscanf(argv[i+1], "%d", &frame_num);
      }
    if (!strcmp(argv[i], "-s"))
      {
	sscanf(argv[i+1], "%le", &scale_fac);
	printf("plt3d: Selected scale factor %lg\n", scale_fac);
      }
    if (!strncmp(argv[i], "-h", 2)) /** check for help **/
      {
	plt3dhlp();
	exit(0);
      }
  }
if (desc_flag == 0)
  {
    fileopen(&datafile);
    /*showhead(&datafile);*/
    if ((frame_num < 1) || (frame_num > datafile.file_header.frames_good))
      frame_num = 1;   /* he blew it! */
    printf("plt3d: Selected frame number %d\n", frame_num);
    /****************** get frame *******************/
#ifdef TEXT
    do
      {
	printf("There are %d frames in this file.\n",
	       datafile.file_header.frames_good);
	printf("Enter frame number (base 1): ");
	scanf("%d", &frame_num);
      }
    while ((frame_num < 1) || (frame_num > datafile.file_header.frames_good));
#endif
    pix_array = readframe(&datafile, frame_num);
    /**********************************/
    graphinfo.num_sets = 1;
    graphinfo.dataset[0].frame_num = frame_num;
    graphinfo.dataset[0].darray = pix_array;
    graphinfo.dataset[0].nx = datafile.file_header.row_length;
    graphinfo.dataset[0].ny = datafile.file_header.col_length;
    strcpy(graphinfo.dataset[0].filename, datafile.filename);
    strcpy(graphinfo.x_axis_text, datafile.filename);
    strcpy(graphinfo.y_axis_text, "Intensity");
    strcpy(graphinfo.title_text, "Surface Plotter");
    sprintf(buf, "Image Data: frame %d", frame_num);
    strcpy(graphinfo.subtitle_text, buf);
    if (scale_fac > 0.)
      graphinfo.scale_fac = scale_fac;
  }
/********************************/
XtToolkitInitialize();
theAppContext = XtCreateApplicationContext();
display = XtOpenDisplay(theAppContext, NULL, argv[0], "Xg3dPlot", NULL, 0,
			(Cardinal *) &argc, argv);
if (display == NULL)
  {
    fprintf(stderr, "Unable to open requested display. Aborting\n");
    exit(0);
  }
graph = XtAppCreateShell("Imager", Xg3dClass,
			    applicationShellWidgetClass, display, NULL, 0);
graphinfo.app_shell = graph;
Xg3dCreateGraph(&graphinfo);
fileclose(datafile.dat_file);
XtAppMainLoop(theAppContext);
/****************** finish up *********************/
free(pix_array);
}

/********************/
plt3dhlp()
{
fprintf(stderr, "plt3d command line help:\n");
fprintf(stderr, " usage: plt3d [options]\n");
fprintf(stderr, "      where options are one or more of the following:\n");
fprintf(stderr, "   any of the standard Xt command line options\n");
fprintf(stderr, "   [(a data file name)] indicating startup data to plot\n");
fprintf(stderr, "   [-f (frame number)]  default is frame 1\n");
fprintf(stderr, "   [-s (scale factor)] default is 0.1\n");
fprintf(stderr, "   [-g (description file name)] use a stored layout\n");
fprintf(stderr, "   Data is expected to be in 'ETS' standard format as\n");
fprintf(stderr, "   defined in the etsheadr.h file\n");
}
