/**********************************************************************
*
*                              showfile.c
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.6   5/92
*
*  Program for reading standard ETS data files and displaying header
*    info. Also shows pixel values if desired.
*
***********************************************************************
*/
#include "stdio.h"
#include "string.h"
#include "etsheadr.h"

main(argc, argv)
     int
       argc;
     char
       **argv;
{
  ETS_STRUCT
    datafile;
  char
    *header_char,
    ans[5],
    outbuf[10];
  int2
    *pix_array;
  int
    i,
    j,
    k,
    index,
    start,
    finish,
    frame_num;
  double
    max,
    pwr;
header_char = (char *) &datafile.file_header;
datafile.filename[0] = '\0';  /* make sure we don't open garbage */
for (i=1; i<argc; ++i) /* find file name if given */
  {
    if (!strncmp(argv[i], "-", 1)) /** if this is - option, skip next arg **/
      ++i;
    else /** assume this is file name **/
      {
	strcpy(datafile.filename, argv[i]);
	i = argc;
      }
  }
fileopen(&datafile);
showhead(&datafile);
/************* other header stuff *******************/
printf("\ndo you want to print the camera status? ");
scanf("%s", ans);
if (ans[0] == 'q')
  {
    fileclose(datafile.dat_file);
    exit(0);
  }
if (ans[0] == 'y')
  {
    for (i=0; i<512; ++i)
      if (datafile.file_header.camera_status[i] == 27)
	datafile.file_header.camera_status[i] = ' ';
    datafile.file_header.camera_status[511] = '\0';
    printf("%s\n", datafile.file_header.camera_status);
  }
printf("do you want to print the utility status? ");
scanf("%s", ans);
if (ans[0] == 'y')
  {
    for (i=0; i<512; ++i)
      if (datafile.file_header.utility_status[i] == 27)
	datafile.file_header.utility_status[i] = ' ';
    datafile.file_header.utility_status[511] = '\0';
    printf("%s\n", datafile.file_header.utility_status);
  }
printf("do you want to print the telescope Line-Of-Sight status? ");
scanf("%s", ans);
if (ans[0] == 'y')
  {
    for (i=0; i<512; ++i)
      {
	if (datafile.file_header.los_status[i] == 27)
	  datafile.file_header.los_status[i] = ' ';
	if (datafile.file_header.los_status[i] == 0)
	  datafile.file_header.los_status[i] = ' ';
      }
    datafile.file_header.los_status[511] = '\0';
    printf("%s\n", datafile.file_header.los_status);
  }
/****************** header byte dump ******************/
printf("do you want a header byte dump (y or n): ");
scanf("%s", ans);
if (ans[0] == 'q')
  {
    fileclose(datafile.dat_file);
    exit(0);
  }
if (ans[0] == 'y')
  {
    for (i=0; i<273; ++i)
      {
	for (j=0; j<15; ++j)  /* print text */
	  {
	    if ((header_char[15*i + j] > 31) && (header_char[15*i + j] < 127))
	      printf("    %c", header_char[15*i + j]);
	    else printf("     ");
	  }
	printf("\n");
	for (j=0; j<15; ++j)  /* print byte value */
	  {
	    printf("%5d", header_char[15*i + j]);
	  }
	printf("\n");
	for (j=0; j<15; ++j)  /* print index in file */
	  {
	    printf("%5d", 15*i+j);
	  }
	printf("\n\n");
      }
  }
/****************** pixel dump *******************/
printf("do you want to dump the pixels (y or n)? ");
scanf("%s", ans);
if (ans[0] == 'y')
  {
    printf("do you want to dump in decimal or hex (d or h)? ");
    scanf("%s", ans);
    if (ans[0] == 'd')
      strcpy(outbuf, "%d ");
    else
      strcpy(outbuf, "%04X ");
    do
      {
	printf("There are %d frames in this file.\n",
	       datafile.file_header.frames_good);
	printf("Enter frame number (base 1) to dump (or 0 for all): ");
	scanf("%d", &frame_num);
      }
    while ((frame_num < 0) || (frame_num > datafile.file_header.frames_good));
    if (frame_num == 0)
      {
	start = 0;
	finish = datafile.file_header.frames_good;
      }
    else
      {
	start = 0;
	finish = 1;
      }
    for (k=start; k<finish; ++k)
      {
	pix_array = readframe(&datafile, frame_num);
	printf("***** frame %d\n", datafile.current_frame);
	max = 0.;
	pwr = 0.;
	for (i=0; i<datafile.file_header.col_length; ++i)
	  for (j=0; j<datafile.file_header.row_length; ++j)
	    {
	      index = i*datafile.file_header.row_length + j;
	      if (*(pix_array + index) > max)
		max = *(pix_array + index);
	      pwr += (double) *(pix_array + index)
		* (double) *(pix_array + index);
	    }
	printf("max = %lg & pwr = %lg\n\n", max, pwr);
	for (i=0; i<datafile.file_header.col_length; ++i)
	  {
	    printf("** frame %d: row %d\n", datafile.current_frame, i);
	    for (j=0; j<datafile.file_header.row_length; ++j)
	      {
		printf(outbuf,
		       *(pix_array + i*datafile.file_header.row_length + j));
	      }
	    printf("\n");
	  } /* end frame */
	free(pix_array);
      } /* end all frames (k-loop) */
  } /* end pixel dump */
/****************** finish up *********************/
printf("\ndone\n");
fileclose(datafile.dat_file);
}
