/**********************************************************************
*
*                                  xgmf.c
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.0   4/93
*
*  Shell Program for XgMovieF. Can read either ETS or SLBD format image data.
*
***********************************************************************
*/

#include "stdio.h"
#include "string.h"
#include "math.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"

#include "etsheadr.h"
#include "XgMovieF.h"

int
  image_format;

/*************************************************************/
main(argc, argv)
     int
       argc;
     char
       **argv;
{
  ETS_STRUCT
    datafile;
  XgMovie_data
    *mv_data;
  int
    i,
    nrows,
    ncols,
    nframes;
  char
    buf[160];
  Display
    *display;
  XtAppContext
    theAppContext;

/***************** initial setup *************/
image_format = 0;
datafile.filename[0] = '\0';  /* make sure we don't open garbage */
for (i=1; i<argc; ++i) /* find file name if given */
  {
    if (!strncmp(argv[i], "-h", 2))  /** display basic usage message **/
      {
	XgM_Command_Help();
	exit(0);
      }
    else if (!strncmp(argv[i], "-s", 2))  /** select SLBD format data **/
      {
	image_format = 1;
	printf("Selected SLBD format data\n");
      }
    else if (!strncmp(argv[i], "-", 1)) /*if this is - option, skip next arg */
      ++i;
    else /** assume this is file name **/
      {
	strcpy(datafile.filename, argv[i]);
      }
  }
if (image_format == 1)
  slbdopen(&datafile);
else
  fileopen(&datafile);
/*showhead(&datafile);*/
printf("Data Description: %s\n", datafile.file_header.description);
printf("Processing History: %s\n", datafile.file_header.proc_hist);
nrows = datafile.file_header.col_length;
ncols = datafile.file_header.row_length;
nframes = datafile.file_header.frames_good;

printf("xgm: There are %d frames of %d x %d (width x height) data.\n",
       nframes, ncols, nrows);

/************ connect to X-Windows & start movie ***************/
theAppContext = NULL;
display = NULL;

XtToolkitInitialize();
theAppContext = XtCreateApplicationContext();
if (theAppContext == NULL)
  {
    fprintf(stderr, "Unable to create application context. Aborting\n");
    exit(1);
  }
display = XtOpenDisplay(theAppContext, NULL, argv[0], "XgMovie", NULL, 0,
			(Cardinal *) &argc, argv);
if (display == NULL)
  {
    fprintf(stderr, "Unable to open requested display. Aborting\n");
    exit(1);
  }

mv_data = XgM_Initialize_Movie();
mv_data->nrows = nrows;
mv_data->ncols = ncols;
mv_data->nframes = nframes;
mv_data->xoff = 0;
mv_data->yoff = 0;
mv_data->ets_struct = &datafile;
strcpy(mv_data->filename, datafile.filename);
mv_data->theAppContext = theAppContext;
mv_data->display = display;

XgM_Main_Entry(mv_data);
XtAppMainLoop(theAppContext);
}

/*********************************************/
XgM_Command_Help()
{
fprintf(stderr, "xgmf command line help:\n");
fprintf(stderr, " usage: xgm [options]\n");
fprintf(stderr, "      where options may be one or more of the following:\n");
fprintf(stderr, "   any of the standard Xt command line options\n");
fprintf(stderr, "   [(an image file name)] indicating the image file to display\n");
fprintf(stderr, "     The image file format is described in etsheadr.h\n");
fprintf(stderr, "   [-s] read SLBD format image data instead of ETS\n");
fprintf(stderr, "   [-h] this message\n");
fprintf(stderr, "  This program requires an 8-bit pseudocolor (adjustable)\n");
fprintf(stderr, "    visual to run. You should also have enough memory to\n");
fprintf(stderr, "    hold the entire movie data set in memory.\n");
}

/*************** read a frame into memory ***********/
/** this function is here, so XgMovieF.c remains source independent **/

void XgM_Read_Frame(mv_data) /* all external communication is localized here */
     XgMovie_data
       *mv_data;
{
  int
    i,
    j,
    nrows,
    ncols,
    xoff,
    yoff,
    nc,
    nr;
  int2
    *temp_array,
    *pix_array;
  IR_ANNOT_1
    annotation;

nrows = mv_data->nrows;
ncols = mv_data->ncols;
pix_array = mv_data->pix_array;
xoff = mv_data->xoff;
yoff = mv_data->yoff;
nc = mv_data->ets_struct->file_header.row_length;
nr = mv_data->ets_struct->file_header.col_length;

if (image_format == 1)
  temp_array = read_slbd_frame(mv_data->ets_struct, mv_data->count+1);
else
  temp_array = readframe(mv_data->ets_struct, mv_data->count+1);

for (i=0; i<nrows; ++i)
  {
    for (j=0; j<ncols; ++j)
      {
	*(pix_array + i*ncols + j) = *(temp_array + (yoff+i)*nc + (xoff+j));
      }
  }

annotation.frame_seq_num = mv_data->ets_struct->current_frame;
annotation.intg_time = .033;
annotation.mjd = 1;
annotation.msec_time = annotation.frame_seq_num * .033 * 1000;

/*decode_ir(nr, nc, temp_array, &annotation);*/
free(temp_array);
XgM_Fill_Pixmaps(mv_data);
/*printf("Frame = %d; mjd = %d; msec = %d\r", annotation.frame_seq_num,
       annotation.mjd, annotation.msec_time);
fflush(stdout);*/
}
